/* lA.c : linear algebra routines for utility programs */
/* (Almost) all routines assume znoc = group->nontips on entry and ensure
   this on exit. */

#include "module.h"
#include "module_decls.h"
#include "meataxe.h"

static long *logarithm;

/******************************************************************************/
void initializeZechLogarithms(void)
{
  long i;
  FEL f;
  logarithm = (long *) malloc(zfl << LONGSH);
  if (!logarithm) AllocationError("initializeZechLogarithms");
  for (i = 0, f = F_ONE; i < zfl; i++, f *= zgen)
    logarithm[zftoi(f)] = i;
  return;
}

/******************************************************************************/
long ZechLogarithm(FEL f)
{
  if (f == F_ZERO) OtherError("ZechLogarithm: 0 invalid input");
  if (!logarithm) initializeZechLogarithms();
  return logarithm[zftoi(f)];
}

/******************************************************************************/
char *zftos(FEL f, char *gen)
/* Return value must be used at once.  Don't write, don't free. */
{
  static char buffer[MAXLINE];
  if (f == F_ZERO)
    sprintf(buffer, "0*%s", gen);
  else
    sprintf(buffer, "%s^%ld", gen, ZechLogarithm(f));
  return buffer;
}

/******************************************************************************/
void matclean(matrix_t *dirt, matrix_t *brush, long *piv)
/* Sets znoc = dirt->noc */
{
  PTR row;
  long i;
  zsetlen(dirt->noc);
  if (brush->nor == 0) return;
  for (i = 0, row = dirt->d; i < dirt->nor; i++, zsteprow(&row))
    zcleanrow(row, brush->d, brush->nor, piv);
  return;
}

/******************************************************************************/
static void unfoldRow(PTR ptrDest, PTR ptrSrc, long r, FEL *buffer)
{
  long i,j;
  PTR ptr,byteptr;
  int idx;
  long nontips = znoc;
  for (i=0, ptr=ptrSrc; i < r; i++, zsteprow(&ptr))
  { byteptr = ptr;
    idx = 0; 
    for (j=0; j < nontips; j++)
      buffer[i*nontips + j] = zextract_step(&byteptr, &idx);
  }
  zsetlen(r * nontips);
  byteptr = ptrDest;
  idx = 0;
  for (i=0; i < r * nontips; i++)
    zinsert_step(&byteptr, &idx, buffer[i]);
  zsetlen(nontips);
  return;
}

/******************************************************************************/
static void foldedRow(PTR ptrDest, PTR ptrSrc, long r, FEL *buffer)
{
  long i,j;
  PTR ptr,byteptr;
  int idx;
  long nontips = znoc;
  zsetlen(r * nontips);
  byteptr = ptrSrc;
  idx = 0;
  for (i=0; i < r * nontips; i++)
    buffer[i] = zextract_step(&byteptr, &idx);
  zsetlen(nontips);
  for (i=0, ptr=ptrDest; i < r; i++, zsteprow(&ptr))
  { byteptr = ptr;
    idx = 0;
    for (j=0; j < nontips; j++)
      zinsert_step(&byteptr, &idx, buffer[i*nontips + j]);
  }
  return;
}

/******************************************************************************/
matrix_t *unfoldMatrix(matrix_t *matSrc, long r)
{
  long nontips = znoc;
  FEL *buffer = (FEL *) malloc(r * nontips * sizeof(FEL));
  long s = matSrc->nor / r;
  long i;
  PTR ptrSrc, ptrDest;
  matrix_t *matDest = matalloc(zfl, s, nontips * r);
  if (!buffer || !matDest) AllocationError("unfoldMatrix");
  zsetlen(nontips);
  for (i=0, ptrSrc=matSrc->d, ptrDest=matDest->d; i < s; i++,
    zsetlen(nontips * r), zsteprow(&ptrDest),
    zsetlen(nontips), zadvance(&ptrSrc, r))
    unfoldRow(ptrDest, ptrSrc, r, buffer);
  free(buffer);
  return matDest;
}

/******************************************************************************/
matrix_t *foldedMatrix(matrix_t *matSrc, long r)
{
  long nontips = znoc;
  FEL *buffer = (FEL *) malloc(r * nontips * sizeof(FEL));
  long s = matSrc->nor;
  PTR ptrSrc, ptrDest;
  long i;
  matrix_t *matDest = matalloc(zfl, s * r, nontips);
  if (!buffer || !matDest) AllocationError("unfoldMatrix");
  for (i=0, ptrSrc=matSrc->d, ptrDest=matDest->d; i < s; i++,
    zsetlen(nontips * r), zsteprow(&ptrSrc),
    zsetlen(nontips), zadvance(&ptrDest, r))
    foldedRow(ptrDest, ptrSrc, r, buffer);
  free(buffer);
  return matDest;
}

/******************************************************************************/
matrix_t *allImages(group_t *group, matrix_t *genIms, long r)
{
  long nontips = group->nontips;
  long s = genIms->nor / r;
  PTR ptrSrc, ptrDest, ptrRoot;
  long block, i, j;
  matrix_t *mat = matalloc(zfl, r * s * nontips, nontips);
  if (!mat) AllocationError("allImages");
  for (block = 0, ptrSrc = genIms->d, ptrDest = mat->d; block < s;
    block++, zadvance(&ptrSrc, r), zadvance(&ptrDest, r * nontips))
    memcpy(ptrDest, ptrSrc, zsize(r));
  for (block = 0, ptrRoot = mat->d; block < s;
    block++, zadvance(&ptrRoot, r * nontips))
    for (i = 1; i < nontips; i++)
    {
      path_t *w = group->root + i;
      ptrDest = ptrPlus(ptrRoot, i * r);
      ptrSrc = ptrPlus(ptrRoot, r * w->parent->index);
      for (j = 0; j < r; j++)
        zmaprow(ptrPlus(ptrSrc, j), group->action[w->lastArrow]->d, nontips,
          ptrPlus(ptrDest, j));
    }
  return mat;
}

/******************************************************************************/
matrix_t *allImagesOfUnfolded(group_t *group, matrix_t *genIms)
{
  long nontips = group->nontips;
  long r = genIms->noc / nontips;
  matrix_t *mat = foldedMatrix(genIms, r);
  matrix_t *allIms = allImages(group, mat, r);
  matrix_t *result = unfoldMatrix(allIms, r);
  matfree(mat); matfree(allIms);
  return result;
}

/******************************************************************************/
void makeFullMatrixOfDifferential(group_t *group, char *nameD, char *nameDest,
long r)
{
  long nontips = group->nontips;
  matrix_t *genIms = matload(nameD);
  matrix_t *matSrc = allImages(group, genIms, r);
  matrix_t *matDest = unfoldMatrix(matSrc, r);
  matsave(matDest, nameDest);
  zsetlen(nontips);
  matfree(matSrc); matfree(genIms); matfree(matDest);
  return;
}

/******************************************************************************/

matrix_t *fullMatrixOfDifferential(group_t *group, char *nameD, long r)
{
  matrix_t *genIms, *matSrc, *matDest;
  long nontips = group->nontips;
  genIms = matload(nameD);
  if (!genIms) OtherError("fullMatrixOfDifferential: loading");
  matSrc = allImages(group, genIms, r);
  matDest = unfoldMatrix(matSrc, r);
  zsetlen(nontips);
  matfree(matSrc); matfree(genIms);
  return matDest;
}

/******************************************************************************/
void makeUnfoldedMatrix(group_t *group, char *nameSrc, char *nameDest, long r)
{
  long nontips = group->nontips;
  matrix_t *matSrc = matload(nameSrc);
  matrix_t *matDest = unfoldMatrix(matSrc, r);
  matsave(matDest, nameDest);
  zsetlen(nontips);
  matfree(matSrc); matfree(matDest);
  return;
}

/******************************************************************************/
matrix_t *vectorsSpanningSubmoduleRadical(group_t *group, matrix_t *submod,
  module_t *module)
{
  long dimM = module->dim;
  long nor = submod->nor;
  matrix_t *rad = matalloc(zfl, nor * dimM, dimM);
  PTR ptrSrc, ptrDest;
  long a, row;
  if (!rad) AllocationError("vectorsSpanningSubmoduleRadical");
  for (a = 0; a < group->arrows; a++)
    for (row = 0; row < nor; row++)
    {
      ptrSrc = ptrPlus(submod->d, row);
      ptrDest = ptrPlus(rad->d, a * nor + row);
      zmaprow(ptrSrc, module->action[a]->d, dimM, ptrDest);
    }
  return rad;
}

/******************************************************************************/
matrix_t *vectorsSpanningSubfreeRadical(group_t *group, matrix_t *ker)
{
  long nontips = group->nontips;
  long nor = ker->nor;
  long r = ker->noc / nontips;
  matrix_t *foldedKer = foldedMatrix(ker, r);
  matrix_t *foldedMat = matalloc(zfl, nor * r * group->arrows, nontips);
  matrix_t *mat;
  PTR ptrSrc, ptrDest;
  long a, row;
  if (!foldedMat) AllocationError("vectorsSpanningSubfreeRadical");
  matsave(foldedKer, "foldK.bin"); zsetlen(nontips);
  for (a = 0; a < group->arrows; a++)
    for (row = 0; row < nor * r; row++)
    {
      ptrSrc = ptrPlus(foldedKer->d, row);
      ptrDest = ptrPlus(foldedMat->d, a * nor * r + row);
      zmaprow(ptrSrc, group->action[a]->d, nontips, ptrDest);
    }
  matsave(foldedMat, "foldedVSR.bin"); zsetlen(nontips);
  mat = unfoldMatrix(foldedMat, r);
  matsave(mat, "unfoldedVSR.bin"); zsetlen(nontips);
  matfree(foldedKer); matfree(foldedMat);
  return mat;
}

/******************************************************************************/
matrix_t *basisForRadical(group_t *group, matrix_t *mod, long *piv)
{
  long nontips = group->nontips;
  long r = mod->noc / nontips;
  matrix_t *rad = vectorsSpanningSubfreeRadical(group, mod);
  zsetlen(r * nontips);
  zmkechelon(rad->d, rad->nor, piv);
  rad->nor = piv[0];
  zsetlen(nontips);
  return rad;
}

/******************************************************************************/
boolean isHeadOfModule(group_t *group, matrix_t *mod, matrix_t *head)
{
  long nontips = group->nontips;
  long rd;
  long hd = head->nor;
  matrix_t *rad;
  long *piv = (long *) malloc((mod->nor * group->arrows + 1) << LONGSH);
  if (!piv) AllocationError("minGeneratorsLA");
  rad = basisForRadical(group, mod, piv);
  rd = rad->nor;
  matsave(rad, "rad.bin");
  matclean(head, rad, piv); /* Executes zsetlen(head->noc) */
  matfree(rad);
  matsave(head, "cleaned.bin");
  zmkechelon(head->d, hd, piv);
  zsetlen(nontips);
  head->nor = piv[0];
  free(piv);
  printf("Dim of head: actual=%ld, claimed=%ld, image of claimed=%ld\n",
    mod->nor - rd, hd, head->nor);
  return (hd == head->nor && hd == mod->nor - rd) ? true : false;
}

/******************************************************************************/
void minGeneratorsLA(group_t *group, matrix_t *ker)
{
  long nontips = group->nontips;
  matrix_t *rad;
  long *piv = (long *) malloc((ker->nor * group->arrows + 1) << LONGSH);
  if (!piv) AllocationError("minGeneratorsLA");
  rad = basisForRadical(group, ker, piv);
  matsave(rad, "rad.bin");
  matclean(ker, rad, piv); /* Executes zsetlen(ker->noc) */
  matfree(rad);
  matsave(ker, "cleaned.bin");
  zmkechelon(ker->d, ker->nor, piv);
  zsetlen(nontips);
  ker->nor = piv[0];
  free(piv);
  return;
}

/******************************************************************************/
void verifyInKernel(group_t *group, char *nameD, char *nameV)
{
  matrix_t *diffl = matload(nameD);
  matrix_t *ker = matalloc(zfl, diffl->nor, diffl->nor);
  matrix_t *rows = matload(nameV);
  long *piv = (long *) malloc((diffl->nor + 1) << LONGSH);
  long n;
  if (!diffl || !ker || !piv || !rows) AllocationError("minGeneratorsOfKernel");
  zsetlen(diffl->noc);
  printf("Hello\n");
  ker->nor = znullsp(diffl->d, diffl->nor, piv, ker->d);
  matfree(diffl);
  printf("nullity: %ld\n", ker->nor);
  matsave(ker, "vIKnull.bin");
  matclean(rows, ker, piv); /* Executes zsetlen(rows->noc) */
  matfree(ker);
  n = zmkechelon(rows->d, rows->nor, piv);
  matfree(rows);
  free(piv);
  zsetlen(group->nontips);
  printf("Number of lin. indep. vectors outside kernel: %ld\n", n);
  return;
}

/******************************************************************************/
void verifyHeadOfKernel(group_t *group, char *nameD, char *nameH)
{
  matrix_t *diffl = matload(nameD);
  matrix_t *ker = matalloc(zfl, diffl->nor, diffl->nor);
  matrix_t *head = matload(nameH);
  long *piv = (long *) malloc((diffl->nor + 1) << LONGSH);
  if (!diffl || !ker || !piv) AllocationError("verifyHeadOfKernel");
  zsetlen(diffl->noc);
  printf("Hello\n");
  ker->nor = znullsp(diffl->d, diffl->nor, piv, ker->d);
  printf("nullity: %ld\n", ker->nor);
  matsave(ker, "null.bin");
  zsetlen(group->nontips);
  free(piv);
  matfree(diffl);
  isHeadOfModule(group, ker, head);
  matfree(ker);
  matfree(head);
  return;
}

/******************************************************************************/
void minGeneratorsOfKernel(group_t *group, char *nameSrc, char *nameDest)
{
  matrix_t *diffl = matload(nameSrc);
  matrix_t *ker = matalloc(zfl, diffl->nor, diffl->nor);
  matrix_t *projCover;
  long *piv = (long *) malloc((diffl->nor + 1) << LONGSH);
  if (!diffl || !ker || !piv) AllocationError("minGeneratorsOfKernel");
  zsetlen(diffl->noc);
  printf("Hello\n");
  ker->nor = znullsp(diffl->d, diffl->nor, piv, ker->d);
  printf("nullity: %ld\n", ker->nor);
  matsave(ker, "null.bin");
  zsetlen(group->nontips);
  free(piv);
  matfree(diffl);
  minGeneratorsLA(group, ker);
  matsave(ker, "nextD.bin"); zsetlen(group->nontips);
  projCover = allImagesOfUnfolded(group, ker);
  matfree(ker);
  printf("Saving: fl=%ld nor=%ld noc=%ld\n", projCover->fl, projCover->nor,
    projCover->noc);
  matsave(projCover, nameDest);
  zsetlen(group->nontips);
  matfree(ker);
  return;
}

/******************************************************************************/
static void actOnRow(group_t *group, PTR src, long a, PTR dest, long r,
  PTR tmpSrc, PTR tmpDest, FEL *buffer)
{
  long i;
  zsetlen(group->nontips);
  foldedRow(tmpSrc, src, r, buffer);
  for (i = 0; i < r; i++)
    zmaprow(ptrPlus(tmpSrc, i), group->action[a]->d, group->nontips,
      ptrPlus(tmpDest, i));
  unfoldRow(dest, tmpDest, r, buffer);
  return;
}

/******************************************************************************/
static module_t *constructSubmodule(group_t *group, matrix_t *submod,
  long *piv)
{
  module_t *module = allocateModule(group, submod->nor);
  long arrows = group->arrows;
  long nontips = group->nontips;
  long a, i;
  long r = submod->noc / nontips;
  FEL *buffer = (FEL *) malloc(submod->noc * sizeof(FEL));
  PTR dest, src, image, tmpSrc, tmpDest;
  zsetlen(submod->noc);
  image = zalloc(1);
  zsetlen(nontips);
  tmpSrc = zalloc(r);
  tmpDest = zalloc(r);
  if (!buffer || !image || !tmpSrc || !tmpDest)
    AllocationError("constructSubmodule");
  for (a = 0; a < arrows; a++)
    for (i = 0; i < submod->nor; i++)
    {
      zsetlen(submod->noc); src = ptrPlus(submod->d, i); zsetlen(nontips);
      actOnRow(group, src, a, image, r, tmpSrc, tmpDest, buffer);
      zsetlen(submod->nor);
      dest = ptrPlus(module->action[a]->d, i);
      zsetlen(submod->noc);
      zcleanrow2(image, submod->d, submod->nor, piv, dest);
      zsetlen(nontips);
    }
  free(tmpDest); free(tmpSrc); free(buffer); free(image);
  return module;
}

/******************************************************************************/
long *loewyLayers(group_t *group, module_t *module)
{
  long nontips = group->nontips;
  long dimM = module->dim;
  matrix_t *rad;
  matrix_t *this = matid(zfl, dimM);
  long *radpiv = (long *) malloc((dimM + 1) << LONGSH);
  long *thispiv = (long *) malloc((dimM + 1) << LONGSH);
  long *loewyLayer = (long *) malloc((nontips + 1) << LONGSH);
  long loewyLayers = 0;
  if (!this || !radpiv || !thispiv || !loewyLayer)
    AllocationError("loewyLayers");
  zsetlen(nontips);
  while (this->nor > 0)
  {
    rad = vectorsSpanningSubmoduleRadical(group, this, module);
    rad->nor = zmkechelon(rad->d, rad->nor, radpiv);
    matclean(this, rad, radpiv);
    this->nor = zmkechelon(this->d, this->nor, thispiv);
    loewyLayer[++loewyLayers] = this->nor;
    matfree(this);
    this = rad;
  }
  matfree(this); free(thispiv); free(radpiv);
  loewyLayer[0] = loewyLayers;
  return loewyLayer;
}

/******************************************************************************/
void HellerModule(group_t *group, matrix_t *fullDiffl)
{
  long nontips = group->nontips;
  long r = fullDiffl->noc / nontips;
  long s = fullDiffl->nor / nontips;
  long i;
  module_t *hellerModule, *hellerDual;
  matrix_t *this = matdup(fullDiffl);
  matrix_t *Omega = matalloc(zfl, s * nontips, r * nontips);
  long *radpiv = (long *) malloc((s * nontips + 1) << LONGSH);
  long *thispiv = (long *) malloc((s * nontips + 1) << LONGSH);
  long *hellerpiv = (long *) malloc((s * nontips + 1) << LONGSH);
  long *loewyLayer = (long *) malloc((nontips + 1) << LONGSH);
  long *socleLayer;
  if (!this || !Omega || !radpiv || !thispiv || !hellerpiv || !loewyLayer)
    AllocationError("HellerModule");
  Omega->nor = 0; hellerpiv[0] = 0; loewyLayer[0] = 0;
  zsetlen(nontips);
  while (this->nor > 0)
  {
    matrix_t *rad;
    rad = vectorsSpanningSubfreeRadical(group, this);
    zsetlen(rad->noc);
    rad->nor = zmkechelon(rad->d, rad->nor, radpiv);
    matclean(this, rad, radpiv);
    this->nor = zmkechelon(this->d, this->nor, thispiv);
    if (this->nor != 0)
    {
      memcpy(ptrPlus(Omega->d, Omega->nor), this->d, zsize(this->nor));
      for (i=1; i <= this->nor; i++)
        hellerpiv[Omega->nor + i] = thispiv[i];
      hellerpiv[0] += this->nor;
      Omega->nor += this->nor;
    }
    loewyLayer[++loewyLayer[0]] = this->nor;
    matfree(this);
    this = rad;
    zsetlen(nontips);
  }
  matfree(this); free(thispiv); free(radpiv);
  printf("Heller module has %ld Loewy layers.  Dimensions:", loewyLayer[0]);
  for (i = 1; i <= loewyLayer[0]; i++)
    printf(" %ld", loewyLayer[i]);
  printf("\n");
  free(loewyLayer);
  hellerModule = constructSubmodule(group, Omega, hellerpiv);
  matfree(Omega); free(hellerpiv);
  hellerDual = newModuleRecord();
  dualModule(group, hellerModule, hellerDual);
  socleLayer = loewyLayers(group, hellerDual);
  printf("Heller module has %ld Socle layers.  Dimensions:", socleLayer[0]);
  for (i = 1; i <= socleLayer[0]; i++)
    printf(" %ld", socleLayer[i]);
  printf("\n");
  free(socleLayer);
  freeModule(hellerModule); freeModule(hellerDual);
  return;
}
