/* module.c : routines for non-subfree modules */

#include "module.h"

/******************************************************************************/
static matrix_t *matneg(matrix_t *src)
{
  matrix_t *dest = matdup(src);
  FEL m_one = zneg(F_ONE);
  long i;
  PTR row;
  if (!dest) AllocationError("matneg");
  if (zchar == 2) return dest;
  for (i = 0, row = dest->d; i < dest->nor; i++, zsteprow(&row))
    zmulrow(row, m_one);
  return dest;
}

/******************************************************************************/
module_t *newModuleRecord(void)
{
  module_t *module = (module_t *) malloc(sizeof(module_t));
  if (!module) AllocationError("newModuleRecord");
  module->stem = NULL;
  module->dim = 0;
  module->action = NULL;
  return module;
}

/******************************************************************************/
module_t *namedModuleRecord(char *MStem)
{
  module_t *module = newModuleRecord();
  module->stem = djg_strdup(MStem);
  return module;
}

/******************************************************************************/
static void innerAllocateModuleAction(group_t *group, module_t *module,
  PTR base)
{
  matrix_t **action, *mats;
  PTR ptr;
  long arrows = group->arrows;
  long dimM = module->dim;
  long i;
  mats = (matrix_t *) malloc(arrows * sizeof(matrix_t));
  action = (matrix_t **) malloc (arrows * sizeof(matrix_t *));
  if (!mats || !action) AllocationError("innerAllocateModuleAction");
  for (i = 0, ptr = base; i < arrows; i++, zadvance(&ptr, dimM))
  {
    action[i] = mats + i;
    action[i]->fl = zfl;
    action[i]->noc = dimM;
    action[i]->nor = dimM;
    action[i]->d = ptr;
  }
  module->action = action;
  return;
}

/******************************************************************************/
static void allocateModuleAction(group_t *group, module_t *module)
{
  PTR base;
  zsetlen(module->dim);
  base = zalloc(group->arrows * module->dim);
  if (!base) AllocationError("allocateModuleAction");
  innerAllocateModuleAction(group, module, base);
  return;
}

/******************************************************************************/
module_t *allocateModule(group_t *group, long dimM)
{
  module_t *module = newModuleRecord();
  module->dim = dimM;
  allocateModuleAction(group, module);
  return module;
}

/******************************************************************************/
module_t *modulePlusOne(group_t *group, module_t *module)
{
  long dimM = module->dim;
  module_t *result = allocateModule(group, dimM);
  matrix_t *In = matid(zfl, dimM);
  matrix_t *A;
  long a;
  for (a = 0; a < group->arrows; a++)
  {
    A = matdup(module->action[a]);
    matadd(A, In);
    memcpy(result->action[a]->d, A->d, zsize(dimM));
    matfree(A);
  }
  matfree(In);
  return result;
}

/******************************************************************************/
void moduleMinusOne(group_t *group, module_t *src, module_t *dest)
{
  long dimM = src->dim;
  matrix_t *In = matid(zfl, dimM);
  matrix_t *m_In = matneg(In);
  matrix_t *A;
  long a;
  dest->dim = dimM;
  allocateModuleAction(group, dest);
  for (a = 0; a < group->arrows; a++)
  {
    A = matdup(src->action[a]);
    matadd(A, m_In);
    memcpy(dest->action[a]->d, A->d, zsize(dimM));
    matfree(A);
  }
  matfree(In); matfree(m_In);
  return;
}

/******************************************************************************/
void dualModule(group_t *group, module_t *src, module_t *dest)
{
  long dimM = src->dim;
  matrix_t *In = matid(zfl, dimM);
  matrix_t *m_In = matneg(In);
  matrix_t *A, *Ainv, *Atr;
  long a;
  dest->dim = dimM;
  allocateModuleAction(group, dest);
  for (a = 0; a < group->arrows; a++)
  {
    A = matdup(src->action[a]);
    matadd(A, In);
    Ainv = matinv(A);
    matfree(A);
    matadd(Ainv, m_In);
    Atr = mattr(Ainv);
    matfree(Ainv);
    memcpy(dest->action[a]->d, Atr->d, zsize(dimM));
    matfree(Atr);
  }
  matfree(In); matfree(m_In);
  return;
}

/******************************************************************************/
void loadModuleAction(group_t *group, module_t *module)
{
  matrix_t *bigmat;
  long old_zfl = zfl;
  char name[MAXLINE];
  PTR base;
  long dimM;
  if (!module->stem) OtherError("loadModuleAction: module nameless");
  strext(name, module->stem, ".mod");
  bigmat = matload(name); /* sets zfl, znoc to required values */
  if (!bigmat)
    OtherError("loadModule: loading bigmat");
  dimM = bigmat->noc;
  module->dim = dimM;
  if (bigmat->nor != group->arrows * dimM)
    OtherError("loadModule: nor != arrows * dimM");
  if (zfl != old_zfl)
    OtherError("loadModule: fields differ");
  base = bigmat->d;
  free(bigmat);
  innerAllocateModuleAction(group, module, base);
  return;
}

/******************************************************************************/
void saveModule(group_t *group, module_t *module)
{
  char name[MAXLINE];
  matrix_t bigmat;
  if (!module->stem) OtherError("saveModule: module nameless");
  strext(name, module->stem, ".mod");
  bigmat.fl = zfl; bigmat.noc = module->dim;
  bigmat.nor = group->arrows * module->dim;
  bigmat.d = module->action[0]->d;
  matsave(&bigmat, name);
  return;
}

/******************************************************************************/
void freeModule(module_t *module)
{
  if (module->action)
  {
    free(module->action[0]->d);
    free(module->action[0]);
    free(module->action);
  }
  if (module->stem) free(module->stem);
  free(module);
  return;
}
