#*****************************************************************************
#
#    Sage Package "Modular Cohomology Rings of Finite p-Groups"
#
#    Copyright (C) 2009 Simon A. King  <simon.king@uni-jena.de> and
#                       David J. Green <david.green@uni-jena.de>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 2 or later (at your choice)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
"""
Modular Cohomology Rings of Finite p-Groups

AUTHORS:

- Simon King  <simon.king@uni-jena.de>
- David Green <david.green@uni-jena.de>

ALGORITHM:

Our package uses Singular and Gap.

NOTE:

    Our package requires the `Small Groups <http://www-public.tu-bs.de:8080/~hubesche/small.html>`_
    library of Hans Ulrich Besche, Bettina Eick and Eamonn O'Brien. If necessary, you can
    install it in Sage by ``install_package('database_gap')``.

The package "pGroupCohomology" comprises :func:`CohomologyRing`, a constructor for 
the cohomology ring of a finite `p`-Group `G` that does caching etc. behind the 
scenes. The package also comprises five modules
:mod:`~pGroupCohomology.cohomology`, :mod:`~pGroupCohomology.cochain`,
:mod:`~pGroupCohomology.resolution`, :mod:`~pGroupCohomology.mtx` and
:mod:`~pGroupCohomology.dickson`. 

In the first section of our introduction, we demonstrate the usage of our
package, mainly by examples. It starts with a synopsis of the functionality. 
Then we go into detail and illustrate various ways of defining a cohomology 
ring, computing the ring structure, and working in the cohomology ring.
Last, we discuss how to save and load a cohomology ring on a local file 
system and how to export and import it to a different platform.

In the second section, we give an overview on the theoretical background of
our package. The computation involves certain special subgroups. On the
one hand, restrictions to the subgroups play a role in chosing the ring
generators. On the other hand, they are used to construct so-called Dickson
classes, that are essential for determining whether the incremental
construction of the cohomology ring was successful (Dave Benson's criterion).
We illustrate this with a step-by-step example.

An overview of the various options that can  be used is given in the documentation
of :func:`CohomologyRing`.

1. EXAMPLES
===========

Synopsis
--------

Our package is devoted to the cohomology rings of finite `p`-groups (i.e., of groups
of prime power order) with coefficients in `\\mathbb F_p` (the finite field with `p`
elements). We expect the groups to be given either by their address in the 
Small Groups Library, or as a group in Sage's Gap interface. In the latter case,
it is required that the first items on the list of generators of `G` form a minimal
generating set. If this is not the case, a ``ValueError`` is raised.

In the near future, our package shall include a relational data base of cohomology
rings. Currently, the package is shipped with the cohomology rings for all groups of
order 64; the data are stored in folders and can simply be retrieved by calling
:func:`~pGroupCohomology.CohomologyRing` with appropriate keys.
These data are common to all users of one Sage installation, thus we refer to it as the
*public data base*. 

In addition, we provide the cohomology rings for all groups of order 128 and for
all but six groups of order 243 in the web. Unless the user requires a 
recomputation from scratch, these data will be automatically downloaded.

Any user can construct new data bases; we refer to them as *private data base*.
There is a default location for a private data base, namely in a
sub-directory of ``DOT_SAGE``. But any location may be chosen
as well, provided that the user has write permissions.

If the ring is not in the cache and unless the user requests a different 
behavious, the program looks for pre-computed cohomology rings in the 
following default places, in order:

1. In the public data base, which is in ``SAGE_ROOT+'/local/pGroupCohomology/db/'``.
2. In the currently activated private data base.
3. In the currently activated cohomology internet source; if the data can 
   be downloaded, they will be unpacked in the currently activated private 
   data base.

If all this fails, a new ring is initialized. This is first tried in the 
public data base. If this is impossible (e.g., since the user does not have 
write permission), then the currently activated private data base is used.

Creating a cohomology ring
--------------------------

We describe here how to describe the `p`-group whose cohomology ring shall be computed.
There are various options to influence the algorithm and the use of data bases;
these options are described in the documentation of :func:`~pGroupCohomology.CohomologyRing`.

... using the SmallGroups library
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

In our examples, we choose a temporary directory for our private data base. Of 
course, in reality, a permanent directory would be chosen for the data bases. We
do so in order to avoid a conflict with an existing private data base. For the 
same reason, we only address *existing* entries in the public data base. 
::

    sage: from pGroupCohomology import CohomologyRing
    sage: H0 = CohomologyRing(8,3)

Since the cohomology ring of ``SmallGroup(8,3)``, which is the Dihedral Group of
order 8, is in the public data base shipped with our package, it is simply loaded
and already completely computed. Note that we also have a list of group names and
group descriptions -- hence, ``H0`` knows that it belongs to the Dihedral Group of
order 8::

    sage: H0
    H^*(D8; GF(2))
    sage: print H0
    Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
    <BLANKLINE>
    Computation complete
    Minimal list of generators:
    [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
     b_1_0, a 1-Cochain in H^*(D8; GF(2)),
     b_1_1, a 1-Cochain in H^*(D8; GF(2))]
    Minimal list of algebraic relations:
    [b_1_0*b_1_1]

It is also possible to convert the cohomology ring into a quotient ring in the
Singular interface::

    sage: singular(H0)
    //   characteristic : 2
    //   number of vars : 3
    //        block   1 : ordering M
    //                  : names    c_2_2 b_1_0 b_1_1
    //                  : weights      2     1     1
    //                  : weights     -1     0     0
    //                  : weights      0    -1     0
    //        block   2 : ordering C
    // quotient ring from ideal
    _[1]=b_1_0*b_1_1

In Sage, parent structures should be unique. And indeed, cohomology rings of 
*the same* group in *the same* data base are unique::

    sage: H0 is CohomologyRing(8,3)
    True
    sage: H0 is loads(dumps(H0))
    True

We set up a private data base in a temporary directory, and show that indeed 
two cohomology rings are only identical if their data are stored in the same 
place::

    sage: tmp = tmp_filename()
    sage: CohomologyRing.set_user_db(tmp)
    sage: H1 = CohomologyRing.user_db(8,3, websource=False)
    sage: H1 is H0
    False

Since we used the option ``websource=False``, it was not attempted to download
the cohomology ring from the web. Hence, ``H1`` is not completely computed yet,
and therefore it is not even equal to ``H0``::

    sage: H0 == H1
    False

We now compute ``H1`` (see below for the theoretical background) and verify that
then ``H0`` is equal (though not identical) to ``H1``::

    sage: H1.make()
    sage: H0 == H1
    True

... using a group in the Gap interface
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

Above, we mentioned that two cohomology rings are the same only if the corresponding
groups are the same. Indeed, if we represent the Dihedral Group of order 8 in a 
different way then we obtain a different ring. Note that, when referring to a group
in the Gap interface, we need to provide a name for that group, in one or the other
way. That name also determines the location of data storage, and thus if two different
names are provided then the corresponding cohomology rings are not the same::

    sage: G = gap('DihedralGroup(8)')
    sage: H2 = CohomologyRing.user_db(G,GroupName='DihedralA')
    sage: G.SetName('"DihedralB"')
    sage: H3 = CohomologyRing.user_db(G)
    sage: H2 is H3
    False

Of course, when ``H2`` and ``H3`` are completely computed, they are
isomorphic to ``H1``, since they belong to isomorphic groups. However, they do
not have the same ring presentation. This is because the presentation of the 
cohomology ring depends on the choice of a minimal generating set of the group;
and the minimal generating sets of ``gap('SmallGroup(8,3)')`` and of 
``gap('DihedralGroup(8)')`` are essentially different::

    sage: H2.make()
    sage: H3.make()
    sage: H2 == H3
    True
    sage: H2 == H1
    False

Indeed, the minimal relations of ``H1`` and ``H2`` look different::

    sage: print H1
    Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
    <BLANKLINE>
    Computation complete
    Minimal list of generators:
    [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
     b_1_0, a 1-Cochain in H^*(D8; GF(2)),
     b_1_1, a 1-Cochain in H^*(D8; GF(2))]
    Minimal list of algebraic relations:
    [b_1_0*b_1_1]
    sage: print H2
    Cohomology ring of DihedralA with coefficients in GF(2)
    <BLANKLINE>
    Computation complete
    Minimal list of generators:
    [c_2_2, a 2-Cochain in H^*(DihedralA; GF(2)),
     b_1_0, a 1-Cochain in H^*(DihedralA; GF(2)),
     b_1_1, a 1-Cochain in H^*(DihedralA; GF(2))]
    Minimal list of algebraic relations:
    [b_1_1^2+b_1_0*b_1_1]


Computations in a cohomology ring
---------------------------------

Up to now, we implemented the computation of four ring theoretic 
invariants, namely the dimension, the depth, the a-invariants and the 
Poincare series::

    sage: H2.dimension()
    2
    sage: H2.depth()
    2
    sage: H2.a_invariants()
    [-Infinity, -Infinity, -2]
    sage: H2.poincare_series()
    1/(t^2 - 2*t + 1)

Of course, they are the same for both presentations of the cohomology ring
of the dihedral group of order 8::

    sage: H1.dimension()
    2
    sage: H1.depth()
    2
    sage: H1.a_invariants()
    [-Infinity, -Infinity, -2]
    sage: H1.poincare_series()
    1/(t^2 - 2*t + 1)

Actually we implemented two essentially different ways of computating the
Poincare series. The method :meth:`~pGroupCohomology.cohomology.COHO.poincare_without_parameters`
is usually much slower than :meth:`~pGroupCohomology.cohomology.COHO.poincare_series`, but of course the results are the
same::

    sage: H2.poincare_without_parameters()
    1/(t^2 - 2*t + 1)

After computing the ring structure, generators and relations of the
cohomology ring are present. Generator number zero is the multiplicative
unit of the underlying field. The other generators are represented by 
cochains (:class:`~pGroupCohomology.cochain.COCH`)::

    sage: H1.gens()
    [1, c_2_2, a 2-Cochain in H^*(D8; GF(2)), b_1_0, a 1-Cochain in H^*(D8; GF(2)), b_1_1, a 1-Cochain in H^*(D8; GF(2))]

The relations are given as strings::

    sage: H1.rels()
    ['b_1_0*b_1_1']

The relations can be interpreted as elements of the cohomology ring,
and indeed they vanish::

    sage: print H1(H1.rels()[0])
    2-Cochain in H^*(D8; GF(2)),
    represented by
    [0 0 0]

We can also compute the nil radical. It is given in the Singular interface,
as an ideal in a free graded commututive ring. In particular, all elements
of the relation ideal of the cohomology ring are in the nil radical. For
the dihedral group of order 8, the nil radical is in fact equal to the
relation ideal::

    sage: H1.nil_radical()
    b_1_0*b_1_1
    sage: H2.nil_radical()
    b_1_1^2+b_1_0*b_1_1

As we have seen before, the presentation of the cohomology ring depends
on the choice of a minimal generating set of the corresponding finite
`p`-group. :meth:`~pGroupCohomology.cohomology.COHO.group` returns the 
permutation group in the gap interface that was used to set up the cohomology ring::

    sage: G1 = H1.group()
    sage: G1
    Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] )
    sage: G2 = H2.group()
    sage: G2
    Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] )

Apparently, ``G1`` and ``G2`` are different permutation groups. But in fact 
they are isomorphic. Note that we could get a group isomorphism by Gap's 
function ``IsomorphismGroups``. But the result is not unique. In order to have
reproducible doc tests, we provide an explicit isomorphism::

    sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,5)(2,6)(3,8)(4,7), (1,8)(2,7)(3,6)(4,5) ] )')
    sage: phi.IsInjective()
    true
    sage: phi.IsSurjective()
    true

Of course, the group isomorphism induces an isomorphism of the 
corresponding cohomology rings. After ensuring that ``phi`` is printed
nicely, we obtain the induced map::

    sage: phi.SetName('"phi"')
    sage: phi_star = H2.hom(phi,H1)
    sage: phi_star
    phi^*

Indeed, we can verify that the image of the relation of ``H2`` is zero
in ``H1``::

    sage: H2.rels()
    ['b_1_1^2+b_1_0*b_1_1']
    sage: phi_star(H2('b_1_1'))^2 + phi_star(H2('b_1_0'))*phi_star(H2('b_1_1'))
    (phi^*(b_1_1))**2+(phi^*(b_1_0))*(phi^*(b_1_1)), a 2-Cochain in H^*(D8; GF(2))
    sage: print phi_star(H2('b_1_1'))^2 + phi_star(H2('b_1_0'))*phi_star(H2('b_1_1'))
    2-Cochain in H^*(D8; GF(2)),
    represented by
    [0 0 0]

We can also compute the inverse of ``phi_star``, compose the two maps, and see that it 
is the identity map::

    sage: phi_star_inv = phi_star^(-1)
    sage: phi_star_inv
    (phi^*)^(-1)
    sage: [(phi_star_inv*phi_star)(X) == X for X in H2.gens()]
    [True, True, True, True]
    sage: [(phi_star*phi_star_inv)(X) == X for X in H1.gens()]
    [True, True, True, True]

It is possible to convert an induced homomorphism into a map of quotient rings in the
Singular interface. This allows for working with cohomology ring elements of very high
degrees. However, it is always needed to take care of the ``basering`` in Singular::

    sage: S_phi_star = singular(phi_star)
    sage: singular(H2).set_ring()
    sage: I = singular.ideal(['c_2_2^50','b_1_0^50','b_1_1^50'])
    sage: singular(H1).set_ring()
    sage: S_phi_star(I)
    b_1_1^100+c_2_2^2*b_1_1^96+c_2_2^16*b_1_1^68+c_2_2^18*b_1_1^64+c_2_2^32*b_1_1^36+c_2_2^34*b_1_1^32+c_2_2^48*b_1_1^4+c_2_2^50,
    b_1_1^50+b_1_0^2*b_1_1^48+b_1_0^16*b_1_1^34+b_1_0^18*b_1_1^32+b_1_0^32*b_1_1^18+b_1_0^34*b_1_1^16+b_1_0^48*b_1_1^2+b_1_0^50,
    b_1_1^50
        
Note that Singular does not do automatic reduction in quotient rings. So, eventually we
do the reductions explicitly, in two ways, with the same result::

    sage: S_phi_star(I).reduce(singular('ideal(0)'))
    b_1_1^100+c_2_2^2*b_1_1^96+c_2_2^16*b_1_1^68+c_2_2^18*b_1_1^64+c_2_2^32*b_1_1^36+c_2_2^34*b_1_1^32+c_2_2^48*b_1_1^4+c_2_2^50,
    b_1_1^50+b_1_0^50,
    b_1_1^50
    sage: singular(H2).set_ring()
    sage: I = I.reduce(singular('ideal(0)'))
    sage: singular(H1).set_ring()
    sage: S_phi_star(I)
    b_1_1^100+c_2_2^2*b_1_1^96+c_2_2^16*b_1_1^68+c_2_2^18*b_1_1^64+c_2_2^32*b_1_1^36+c_2_2^34*b_1_1^32+c_2_2^48*b_1_1^4+c_2_2^50,
    b_1_1^50+b_1_0^2*b_1_1^48+b_1_0^16*b_1_1^34+b_1_0^18*b_1_1^32+b_1_0^32*b_1_1^18+b_1_0^34*b_1_1^16+b_1_0^48*b_1_1^2+b_1_0^50,
    b_1_1^50+b_1_0*b_1_1^49+b_1_0^16*b_1_1^34+b_1_0^17*b_1_1^33+b_1_0^32*b_1_1^18+b_1_0^33*b_1_1^17+b_1_0^48*b_1_1^2+b_1_0^49*b_1_1
    sage: S_phi_star(I).reduce(singular('ideal(0)'))
    b_1_1^100+c_2_2^2*b_1_1^96+c_2_2^16*b_1_1^68+c_2_2^18*b_1_1^64+c_2_2^32*b_1_1^36+c_2_2^34*b_1_1^32+c_2_2^48*b_1_1^4+c_2_2^50,
    b_1_1^50+b_1_0^50,
    b_1_1^50

Data storage and data relocation
--------------------------------

Modular cohomology rings of finite `p`-groups can be very complicated. When computing
the ring structure, many data pieces are needed --- but not all pieces at the same 
time. Therefore we save the various pieces in separate files, and reload them when
they are needed. In some examples, the memory and time consumption of saving *all*
data in a single file would not be reasonable. Hence, 'saving' a cohomology ring to
some file ``f`` means that ``f`` only contains the information needed to find the
files in which the various data pieces can be found.

This method is fast, but has a clear disadvantage: If you want to transfer your 
cohomology computations to a different computer, it does not suffice to copy one 
file for one cohomology ring. *All* data pieces must be copied. Moreover, when the 
data are in a different location, then the information stored in ``f`` would point 
to non-existing paths. We tried to address these problems in the least inconvenient 
way, as explained in the following sub-sections.

Data storage
^^^^^^^^^^^^

We do some new computations from scratch, using a temporary directory.
::
  
    sage: from pGroupCohomology import CohomologyRing
    sage: tmp = tmp_filename()
    sage: CohomologyRing.set_user_db(tmp)
    sage: H = CohomologyRing.user_db(8,3)
    sage: H.make()

By default, when computing a cohomology ring ``H``, the result is automatically 
saved in a file ``H.autosave_name()``. Cohomology rings are unique parent 
structures, and therefore reloading from that file only creates a new reference
to  ``H``::

    sage: K = load(H.autosave_name())
    sage: K is H
    True
    sage: loads(dumps(H)) is H
    True

By the command ``CohomologyRing.set_user_db(tmp)``, all data in the user
data base are stored in sub-folders of the folder ``tmp``. The names of 
these folders are composed as follows.

 1. The basic folder ``tmp`` is known to ``H`` by the attribute ``root``. 
    :func:`~pGroupCohomology.CohomologyRing` normalizes the given folder name ``tmp``
    so that it does not contain double slashes. In order to not break the following
    doc test, we remove double slashes from ``tmp``.
    ::

        sage: tmp = tmp.replace('//','/')
        sage: H.root == tmp+'/'
        True

 2. Any cohomology ring has been assigned a "stem name" that determines
    the names of sub-folders and file names associated with the ring. If the
    cohomology ring belongs to a group in the Small Groups Library, its stem
    name is formed using the address in the Small Groups Library. Otherwise,
    it is obtained from the group name used in the definition of the cohomology 
    ring::

        sage: H = CohomologyRing.user_db(8,3)
        sage: G = gap('DihedralGroup(8)')
        sage: K = CohomologyRing.user_db(G, GroupName = 'DihedralA')
        sage: G.SetName('"DihedralB"')
        sage: L = CohomologyRing.user_db(G)
        sage: H.GStem
        '8gp3'
        sage: K.GStem
        'DihedralA'
        sage: L.GStem
        'DihedralB'

 3. Data belonging to a cohomology ring can be found in a folder that is 
    stored as the attribute "gps_folder", and is composed by the root and
    the stem name::

        sage: H.gps_folder == H.root+H.GStem+'/'
        True

Data relocation
^^^^^^^^^^^^^^^

If you wish to transfer these data to a different location, you may do
so. But please make sure that ``H.gps_folder`` keeps its name (only the
parent directory is allowed to change!), and that all files in ``H.gps_folder``
and its sub-folders are preserved. Then,  by ``CohomologyRing.set_user_db``,
one can activate the private data base in the new location, and reload the ring.
In order to demonstrate that it really is the same ring, we provide ``H``
with an additional attribute before saving and moving the data::

    sage: H.setprop('info', 'The cohomology data were moved')
    sage: save(H,H.autosave_name())
    sage: tmp2 = tmp_filename()
    sage: import os
    sage: os.rename(tmp,tmp2)
    sage: CohomologyRing.set_user_db(tmp2)
    sage: M = CohomologyRing.user_db(8,3)
    sage: print M
    Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
    <BLANKLINE>
    Computation complete
    Minimal list of generators:
    [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
     b_1_0, a 1-Cochain in H^*(D8; GF(2)),
     b_1_1, a 1-Cochain in H^*(D8; GF(2))]
    Minimal list of algebraic relations:
    [b_1_0*b_1_1]
    sage: M.info
    'The cohomology data were moved'

2. Theoretical Background
=========================

Elementary abelian subgroups and choice of generators
-----------------------------------------------------

Let `G` be a non-abelian finite `p`-group that is not a generalised quaternion
group. Let `S_1` be its greatest central elementary abelian subgroup, and let
`S_2,...,S_n` be representatives for the conjugacy classes of maximal elementary
abelian subgroups of `G`.

Our approach relies on knowing these subgroups, to which we refer as
*special subgroups*. When defining a cohomology ring, the special
subgroups are computed using Gap functions written by David Green.

In our example, `G` is the dihedral group of order 8, which is the third
group of order 8 in the Small Groups Library. We set up the cohomology ring
in a temporary directory. The special subgroups can be retrieved as follows::

    sage: from pGroupCohomology import CohomologyRing
    sage: tmp = tmp_filename()
    sage: CohomologyRing.set_user_db(tmp)
    sage: H = CohomologyRing.user_db(8,3, websource=False)
    sage: H.subgroups()
    {(4, 2): H^*(SmallGroup(4,2); GF(2)), (2, 1): H^*(SmallGroup(2,1); GF(2))}

Hence, ``H.subgroups()`` is a dictionary, whose keys correspond to elementary 
abelian groups in the SmallGroups library, and whose values are the
corresponding cohomology rings.

David Green provided an algorithm for the computation of minimal 
projective resolutions of finite `p`-groups, using non-commutative
Groebner bases. He implemented it in C-programs, and we provide a
Cython wrapper in :mod:`pGroupCohomology.resolution`.

Knowing a resolution out to degree `d` allows for computing a 
minimal set of cohomology generators out to degree `d` and a minimal set of
algebraic relations of degree up to `d` for these generators. It pays
off to choose the generators in a special way, as follows:

1. A cohomology class of `G` is nilpotent if and only if its 
   restrictions to `S_2,...,S_n` are nilpotent. Since the cohomology
   rings of elementary abelian groups are either polynomial rings
   or the tensor product of a polynomial ring with an outer algebra
   (if `p` is odd), it is easy to test whether a cohomology class of `G`
   is nilpotent, even if the whole cohomology ring is not known yet.
   We choose as many as possible new nilpotent generators in degree `d`.
2. After step 1, continue with choosing as many as possible new generators 
   in degree `d` with nilpotent restriction to `S_1`.
3. After step 2, include the remaining new generators in degree 
   `d`. These are Duflot regular.

If `z` is the rank of the center of `G` then there are exactly `z` 
Duflot regular generators, which was proved by `Nick Kuhn <http://de.arxiv.org/abs/math/0612133>`_.
David Green introduced a monomial ordering on the cohomology ring of `G` that, in the first place, 
depends on the degree of a monomial, but also depends on the number of nilpotent respectively
Duflot regular generators occuring in the monomial. It turns out that this monomial ordering
magically simplifies many computations. We now compute the cohomology ring ``H`` out to
degree 2, and show the generators and relations.
::

    sage: H.make(2)
    sage: H.gens()
    [1,
     c_2_2, a 2-Cochain in H^*(D8; GF(2)),
     b_1_0, a 1-Cochain in H^*(D8; GF(2)),
     b_1_1, a 1-Cochain in H^*(D8; GF(2))]
    sage: H.rels()
    ['b_1_0*b_1_1']

We look at the generators in more detail. Generator number ``i`` (``i`` an 
integer) of ``H`` can be obtained by ``H.gen(i)`` or by ``H.i`` (the latter only 
if ``i`` is a *number*, not just a *variable* assigned to a 
number). Note that ``H.0`` always returns the multiplicative unit in the 
underlying base field, which is ``GF(2)`` in this case.
::

    sage: H.0
    1
    sage: print H.1
    2-Cochain in H^*(D8; GF(2)),
    represented by
    [0 0 1]
    rdeg = 1
    ydeg = 0
    sage: print H.2
    1-Cochain in H^*(D8; GF(2)),
    represented by
    [1 0]
    rdeg = 0
    ydeg = 0
    sage: print H.3
    1-Cochain in H^*(D8; GF(2)),
    represented by
    [0 1]
    rdeg = 0
    ydeg = 0

``rdeg`` and ``ydeg`` are notions needed in the definition of David 
Green's monomial ordering. ``H.1.rdeg()==1`` means that the generator 
``H.1`` is Duflot regular. We now compute the restrictions. The 
restriction maps can be obtained as follows::

    sage: H.restriction_maps()
    {1: [(2, 1),
         Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(2,1); GF(2))],
     2: [(4, 2),
         Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))],
     3: [(4, 2),
         Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))]}

Hence, it is a dictionary. The keys correspond to the number 
of a special subgroup and yield a pair formed by the SmallGroups
identifier of that subgroup and the restriction map. It can be 
applied to a generator in multiplicative or functional notation::

    sage: print H.restriction_maps()[1][1]*H.1
    2-Cochain in H^*(SmallGroup(2,1); GF(2)),
    represented by
    [1]
    sage: print H.restriction_maps()[1][1](H.1)
    2-Cochain in H^*(SmallGroup(2,1); GF(2)),
    represented by
    [1]

As required, the restriction of the Duflot regular generator ``H.1`` to
`S_1` is not nilpotent. Actually, for `p=2`, the restriction of a cohomology class to some
elementary abelian subgroup is nilpotent if and only if the restriction is zero.

The list of generators restricted to any subgroup and expressed as a
polynomial in the generators of the subgroup's cohomology ring can be 
obtained by :meth:`~pGroupCohomology.cohomology.COHO.restrict_to_subgroup`::

    sage: H.restrict_to_subgroup(2)
    [c_1_0*c_1_1+c_1_0^2, a 2-Cochain in H^*(SmallGroup(4,2); GF(2)),
     c_1_1, a 1-Cochain in H^*(SmallGroup(4,2); GF(2)),
     0, a 1-Cochain in H^*(SmallGroup(4,2); GF(2))]

Note that the choice of generators and relations depends on how the 
group was originally defined, as we have seen above.

Dave Benson's completeness criterion
------------------------------------

Dickson classes
^^^^^^^^^^^^^^^

One key problem is to detect whether all generators and 
relations are already known. In the case of an abelian group, it
is known that a computation out to degree 2 will always suffice.
In the case of generalized quaternion groups, it is known that a
computation out to degree 4 will always suffice. In the general
case, we solve the termination problem according to a method
introduced by Dave Benson, slightly modified by David Green and
Simon King.

Let `r` be the `p`-rank of `G`, and let `z` be the rank of the 
center of `G`. Using Dickson invariants, we define `r-z` cohomology
classes `d_{1,j},...,d_{r-z,j}` of `S_j` (`j=2,...,n`) in the
polynomial part of the cohomology ring of `S_j`, restricting to zero
on `S_1` (recall that `S_2,...S_n` are maximal elementary abelian 
subgroups of `G`, and `S_1` is a common subgroup, namely the greatest
central elementary abelian subgoup). 

It is known that there exist cohomology classes `D_1,...,D_{r-z}` 
of `G` such that `D_i` simultaneously restricts to 
`d_{i,2},...,d_{i,n}` or to their `p^{k_i}`-th power for some `k_1,...,k_{r-z}`.
We briefly refer to these classes as *Dickson classes* of `G`
respectively of `S_j`. By default, we try to factorise the simultanous 
lifts of the `d_{i,j}`, and let `D_i` be the smallest non-constant 
factor, in order to keep the degrees low. In the case of the dihedral 
group, we have `n=2` and the Dickson class of `S_2` is of degree one.
 
If the Dickson classes of `G` have already been found, there is an 
attribute ``Dickson``, which is a list of strings providing a polynomial
representation of the Dickson classes. There are two widely different
ways to find the Dickson classes, see below.

The dihedral group of order 8 has two non-conjugate maximal elementary abelian 
subgroups of order 4. The restriction maps to these subgroups can be
obtained with ``H.restriction_maps()[2]`` and ``H.restriction_maps()[3]``. The restriction
map to the greatest central elementary abelian subgroup can be obtained 
with ``H.restriction_maps()[1]``.

We verify that the Dickson class of ``H`` really restricts to the Dickson 
class of its two special subgroups of order 4, and that the restriction
to the greatest central elementary abelian subgroup is zero.
Note that the Dickson classes of the subgroups are not available if
the optional parameter ``useElimination=True`` is used (see below). In any
case, the Dickson classes of the subgroups can be constructed using the
method :meth:`~pGroupCohomology.cohomology.COHO.dickson_in_subgroup`::

    sage: H.dickson_in_subgroup((4,2))
    sage: H.subgpDickson[4,2]
    [c_1_1, a 1-Cochain in H^*(SmallGroup(4,2); GF(2))]

During ``H.make(2)``, it was successfully tried to lift it to a Dickson 
class in ``H``, using :meth:`~pGroupCohomology.cohomology.COHO.lift_dickson`. 
The class is represented by a string::

    sage: H.Dickson
    ['b_1_0 + b_1_1']

There is an alternative way of lifting the Dickson classes. While
:meth:`~pGroupCohomology.cohomology.COHO.lift_dickson` works with linear algebra, 
:meth:`~pGroupCohomology.cohomology.COHO.find_dickson` uses
elimination. The first method requires that the restriction maps to the
maximal elementary abelian subgroups are known out to the degree in which
the Dickson classes live; this degree can be very high, making this method
unfeasible. Hence, if the degree of the Dickson classes is greater than 18, 
the elimination method is used by default. Using the optional parameter
``useElimination=True``, the elimination method can be chosen explicitly
and can be explicitly avoided by ``useElimination=False``.

In the case of the dihedral group of order 8, both methods work, and in fact
they yield the same result (which is not necessarily the case)::

    sage: H.Dickson = []
    sage: H.find_dickson()
    sage: H.Dickson
    ['b_1_0 + b_1_1']

We verify that the element ``H('b_1_1+b_1_0)`` restricts to zero in the
greatest central elementary abelian subgroup, and restricts to the Dickson
class in the two maximal elementary abelian subgroups::

    sage: print H.restriction_maps()[1][1](H('b_1_1+b_1_0'))
    1-Cochain in H^*(SmallGroup(2,1); GF(2)),
    represented by
    [0]
    sage: H.subgpDickson[4,2][0] == H.restriction_maps()[2][1](H('b_1_1+b_1_0'))
    True
    sage: H.subgpDickson[4,2][0] == H.restriction_maps()[3][1](H('b_1_1+b_1_0'))
    True

It follows from results of Dave Benson that the Duflot regular generators 
of `H^*(G)` together with the Dickson classes of `G` form a filter-regular 
system of parameters for `H^*(G)`. Hence, in our example, we already know 
that if the cohomology computation were complete then 
``['c_2_2','b_1_0+b_1_1']`` would give rise to a filter regular system 
of parameters. Filter regularity can be checked as follows::

    sage: H.is_filter_regular(['c_2_2','b_1_0+b_1_1'], [2,1])
    True

Now, one considers the so-called (raw) filter-degree type. After 
calling :meth:`~pGroupCohomology.cohomology.COHO.is_filter_regular`, raw 
filter degree type and filter degree
type are available by
::

    sage: H.rfdt
    [-1, -1, 1]
    sage: H.filter_degree_type()
    [-1, -2, -2]

The filter degree type together with the degrees of the filter regular 
system of parameters and the rank of the center of `G` give rise to a 
degree bound:

**Theorem [Benson]**

Assume that `R_n` is an approximation of `H^\\ast(G)` out to degree 
`n` and assume that there is a filter regular HSOP for `R_n` in degrees
at least two. Let `f_1,...,f_{r+1}` be the filter degree type and let 
`d_1,...,d_r` be the degrees of the filter regular HSOP. If

  `n > \\max(0, f_1+1,...,f_r+r) - r + d_1+ ... +d_r` 

then `R_n` is isomorphic to the cohomology ring.

Benson conjectures that the filter degree type for any modular cohomology
ring of a finite p-group is of the form `-1,-2,...,-r,-r`. Note that 
the strict inequality in the theorem can be improved to a non-strict 
inequality if the center of `G` is of rank at least two.

How do we apply that theorem? Apparently, the key problem is to find
a filter regular HSOP for the approximation `R_n` of the cohomology 
ring. The point is that the Duflot regular generators together with
the Dickson classes form a filter regular HSOP for `H^\\ast(G)`, and
that we are able to compute Duflot regular generators and Dickson
classes in `R_n`. 

1. Test whether the Duflot regular generators together with the Dickson
   classes form a filter regular HSOP for `R_n`. If it fails, then
   certainly `R_n` is not isomorphic to the cohomology ring. So, we
   need to compute an approximation in higher degrees.
2. If they do form a filter regular HSOP for `R_n` then we can compute
   the filter degree type and check whether the inequality in Benson's
   theorem is satisfied.
3. If it *is* satisfied, then `R_n` is isomorphic to the cohomology
   ring, and we can stop. Otherwise, we can not prove that `R_n` is
   isomorphic to the cohomology ring, and thus we must continue in
   higher degrees.

What does it mean, in our example? We have verified that we have 
a filter regular HSOP for the approximation in degree two. But looking
at the degrees of the parameters, Benson's criterion yields the degree 
bound 3. However, it turns out that there is no further generators or 
relations for ``H`` in degree 3. Hence, Benson's criterion is not 
perfect, as it sometimes applies only in higher degrees than necessary.

In fact, in many examples there exist filter-regular systems of
parameters having smaller degrees than the one provided by Duflot 
regular generators and Dickson classes. An existence proof is based 
on a Lemma that is formulated in the following section, where we also 
provide an example.

Since the filter degree type is independent of the choice of a filter-regular
system of parameters, we can use the Dickson classes for computing the 
filter-degree type Then, we prove the existence of a filter regular HSOP
in smaller degrees. We do not need this HSOP explicitly --- we only need
the degrees (which comes out of the existence proof) and the filter-degree
type (which can be computed using the Dickson classes).

A step-by-step example
----------------------

The purpose of this section is to show shat happens behind the scenes when invoking
:meth:`~pGroupCohomology.cohomology.COHO.make`. We use group number 32 of order 64, since this allows to illustrate
various aspects of the computation.
::

    sage: H = CohomologyRing.user_db(64,32,websource=False)

We compute the ring structure out to degree 6. Since we did not use the optional
parameter ``useElimination=True``, some of the Dickson classes of `G` will be 
found by the linear algebra method. For the last one, this would be impossible
by the linear algebra method, as this Dickson class is of degree 7. Note that
instead of repeating ``H.next()`` six times, one could also do ``H.make(6)``.
::

    sage: H.next()
    sage: H.next()
    sage: H.next()
    sage: H.next()
    sage: H.next()
    sage: H.next()
    sage: H.Dickson
    ['b_1_1^4 + b_2_1^2 + b_2_2^2 + b_2_3^2 + b_1_1*b_3_6',
     'b_2_2^2*b_1_1^2 + b_2_3^2*b_1_1^2 + b_1_1^3*b_3_6 + b_2_1*b_2_3^2 + b_2_2*b_2_3^2 + b_2_2*b_1_1*b_3_6',
     None]

Although the third Dickson class is in degree 7 and we know the ring structure
only out to degree 6, it is possible to express the third Dickson class as a
polynomial in the generators of our approximation of the cohomology ring. This
fact can not be seen with the linear algebra method, but it can be seen using
elimination, which is done in :meth:`~pGroupCohomology.cohomology.COHO.find_dickson`.
Note that, by default, it is tried to factorise the lift of the Dickson classes,
potentially after a modification by some nilpotent element (see 
:meth:`~pGroupCohomology.cohomology.COHO.factorise`). By this little trick, we 
actually obtain a parameter in degree one::

    sage: H.find_dickson()
    sage: H.Dickson
    ['b_1_1^4 + b_2_1^2 + b_2_2^2 + b_2_3^2 + b_1_1*b_3_6',
     'b_2_2^2*b_1_1^2 + b_2_3^2*b_1_1^2 + b_1_1^3*b_3_6 + b_2_2*b_2_3^2 + b_2_2*b_1_1*b_3_6 + b_3_5^2',
     'b_1_1']

Since the rank of the center of the group is one, there is exactly one Duflot regular generator.
We have already found it, namely in degree 4::

    sage: print H.5
    4-Cochain in H^*(SmallGroup(64,32); GF(2)),
    represented by
    [0 0 0 0 0 0 0 0 0 0 0 1]
    rdeg = 1
    ydeg = 0
    
Now, as we have the Dickson classes and the Duflot regular generator, does it make sense to try
and apply Benson's criterion? It does not, since there is one 'boring' generator in degree 5::

    sage: H.gens()
    [1,
     b_2_1, a 2-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_2_2, a 2-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_2_3, a 2-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_4_10, a 4-Cochain in H^*(SmallGroup(64,32); GF(2)),
     c_4_11, a 4-Cochain in H^*(SmallGroup(64,32); GF(2)),
     a_1_0, a 1-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_1_1, a 1-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_3_4, a 3-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_3_5, a 3-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_3_6, a 3-Cochain in H^*(SmallGroup(64,32); GF(2)),
     b_5_17, a 5-Cochain in H^*(SmallGroup(64,32); GF(2))]

We expect that  there will be a relation at least in degree 2*5=10. This heuristical lower bound
for the necessary degree of computations can be computed by a method::

    sage: H.expect_last_relation()
    10

So, we continue out to degree 10. Indeed, we find a relation in that degree,
but there are no further generators. Now it makes sense to test whether the 
Duflot regular generator and the lifts of the Dickson classes form a filter 
regular homogeneous system of parameters::

    sage: H.next()
    sage: H.next()
    sage: H.next()
    sage: H.next()
    sage: H.lastRel
    10
    sage: H.is_filter_regular(['c_4_11']+H.Dickson, [4,4,6,1])
    True

The Filter Degree Type is in accordance with Benson's Strong Regularity Conjecture::
    
    sage: H.filter_degree_type()
    [-1, -2, -3, -4, -4]

The original form of Benson's criterion would only apply in degree 18, which is the sum of the
degrees of the filter-regular HSOP, minus the dimension, plus 1 since the center has rank less
than two. However, we are able to improve the bound.

**Lemma:** 
Let `p_1,...,p_r` be a filter-regular homogeneous system of parameters for the
approximation `R_n` of a cohomology ring. Assume that the quotient of `R_n` modulo `p_1,...,p_m`
and modulo all monomials of some degree `d` is finite. Then, there is a filter-regular
homogeneous system of parameters for `R_n`, formed by `p_1,...,p_m` and `r-m` elements of
degree `d`.

This lemma is used in :meth:`~pGroupCohomology.cohomology.COHO.potential_degree_bound`::

    sage: H.potential_degree_bound(['c_4_11']+H.Dickson, [4,4,6,7])
    7

That means: *If* ``['c_4_11']+H.Dickson`` with degrees ``[4,4,6,7]`` provides a filter-regular
HSOP of ``H`` of an appropriate filter degree type and if the ring structure is known at least
out to degree 7, then the computation is complete. Since we know that we have a
filter-regular HSOP of a good filter degree type, we are done.

We remark that we only need to know the mere *existence* of a filter-regular HSOP in low degrees.
It is not needed to construct it explicitly. However, we do so here, just for illustration.
The attribute ``WhatFRS`` tells us that we can replace the last three parameters (which are
of degree 4, 6, and 7) by elements in degree 2::

    sage: H.WhatFRS
    (3, 2)

Since usually we would only work with a single filter-regular HSOP and since the test for
filter-regularity is expensive, we cache the result. So, before testing another HSOP, we
have to clear the cache. Then we see whether the three generators in degree two can be used
as parameters::

    sage: H.delprop('lastTested')
    sage: H.is_filter_regular(['c_4_11','b_2_1','b_2_2','b_2_3'], [4,2,2,2])
    False

This is not the case, so, we try some other elements::

    sage: H.delprop('lastTested')
    sage: H.is_filter_regular(['c_4_11', 'b_2_1+b_2_2', 'b_2_1+b_2_3', 'b_2_2+b_2_3'], [4,2,2,2])
    False
    sage: H.delprop('lastTested')
    sage: H.is_filter_regular(['c_4_11', 'b_2_1+b_2_2', 'b_1_1^2+b_2_3', 'b_2_2+b_2_3'], [4,2,2,2])
    True

So, indeed, it is possible (but not needed) to explicitly provide three filter regular parameters
in degree two, in addition to the Duflot regular generator.

"""

from sage.all import SAGE_ROOT, DOT_SAGE, load
from sage.rings.integer import Integer
from sage.interfaces.singular import singular
from pGroupCohomology.cohomology import COHO
from pGroupCohomology.resolution import OPTION
import re

###########
## Cohomology rings will be unique parent structures
from weakref import WeakValueDictionary
_cache = WeakValueDictionary({})


##########
## A little regular expression that transforms any string in a valid GStem

GStemMaker = re.compile(r'[^0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ]')

class CohomologyRingFactory:
    r"""
    A factory for creating modular cohomology rings of finite p-groups as unique parent structures

    Please use :func:`~pGroupCohomology.CohomologyRing`, which is an instance of this class, and
    is provided with a documentation of the available options.
    
    TESTS::
    
        sage: from pGroupCohomology import CohomologyRing
        sage: H0 = CohomologyRing(8,3)
        sage: print H0
        Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
        <BLANKLINE>
        Computation complete
        Minimal list of generators:
        [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
         b_1_0, a 1-Cochain in H^*(D8; GF(2)),
         b_1_1, a 1-Cochain in H^*(D8; GF(2))]
        Minimal list of algebraic relations:
        [b_1_0*b_1_1]
    
    """
    def __call__ (self, *args, **kwds):
        """
        Create the modular cohomology ring of a finite p-groups

        Of course, isomorphic p-Groups have isomorphic cohomology rings.
        However, the presentation of the cohomology rings as obtained
        by our package depends on the choice of a minimal generating set
        of the p-group. 
        
        If a `p`-group `G` is given by its position in the SmallGroups library,
        then this position, perhaps together with a custom name provided by
        the user, forms a unique key for the cohomology ring.
        
        If `G` is given as a group in the Gap interface, then it is required that
        the first items on the list of generators of `G` forms a minimal generating
        set. If this is not the case, an error is raised. We transform `G` into
        a permutation group whose generators correspond to a minimal generating set
        of `G`. The description of that permutation group, perhaps together with a
        custom name, forms a unique key for the cohomology ring.
        
        The unique key also depends on the chosen folders containing data of
        the ring.

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: H0 = CohomologyRing(8,3) # indirect doc test
            sage: print H0
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            sage: tmp_root = tmp_filename()
            sage: H1 = CohomologyRing(8,3,root=tmp_root)
            
        """
        # Basic idea: 
        # The key shall both be a unique pointer to the data in the file 
        # system and a descriptor of the group-with-minimal-generators.
        # Hence, it is the root directory plus the stem name plus [either
        # the position in the SmallGroups library or a permutation group
        # presentation].
        # The GroupName and other properties are extra arguments

        from pGroupCohomology.resolution import OPTION
        from sage.all import gap
        if len(args)>2 or len(args)==0:
            raise ValueError, "The p-Group must be described by one or two parameters"

        # If cohomology options are required, they are provided now.
        # Note that these are valid for any subsequent computations with
        # any cohomology ring: The options are not associated with the
        # ring that we are returning below.
        if kwds.has_key('options'):
            from pGroupCohomology.cohomology import COHO_option
            if isinstance(kwds['options'], basestring):
                COHO_option(kwds['options'])
            else:
                COHO_option(*kwds['options'])

        if len(args) == 2:
            q = Integer(args[0])
            if not q.is_prime_power():
                raise ValueError, "First parameter must be a prime power"
            n = Integer(args[1])
            max_n = Integer(gap('NumberSmallGroups(%d)'%q))
            if not 0<n<=max_n:
                raise ValueError, "Second parameter must be between 1 and %d"%max_n
            KEY = (q,n)
            GStem = '%dgp%d'%(q,n)      # this should be chosen in an automated way
            GroupName = kwds.get('GroupName', None)  # this may be provided by the user.
                                                    # If the name is None, then it will be fixed
                                                    # in COHO.__init__
        else:
            H0 = args[0]
            if not hasattr(H0,'parent'):
                raise TypeError, "We expected a group defined in Gap"
            GAP = H0.parent()
            
            ####
            ## make up the (stem) name
            try:
                GroupName = GAP.eval('Name(%s)'%(H0.name()))[1:-1]
            except RuntimeError:
                GroupName = None
            GroupName = kwds.get('GroupName', GroupName)
            GStem = kwds.get('GStem', GroupName)    # here, a unique short descriptor should be given -- up to the user,
                                                    # since we don't see a clear automated procedure of choice.
            if (GStem is None):
                raise ValueError, "'GroupName' must be provided"
            if not isinstance(GStem, basestring):
                raise TypeError, "'GroupName' must be a string"
            if GroupName is None:
                GroupName = GStem

            ## Now the name is fixed (and can be any string).
            ## The GStem must be usable as a valid file name:
            GStem = GStemMaker.sub('_',GStem)            

            ####
            ## process the gap input:
            #   1. good group order?
            #   2. minimal generators given?
            #      if not, find them!
            #   3. transform into permutation group
            q = Integer(GAP.eval('Order(%s)'%(H0.name())))
            if not q.is_prime_power():
                raise ValueError, "The group must be of prime power order"
                n = -1

            ## Try to make up minimal generators
            if kwds.get('minimal_generators'): # or GAP.eval('RankPGroup(%s)'%(H0.name())) == GAP.eval('Length(GeneratorsOfGroup(%s))'%(H0.name())):
                # We already have minimal generators (or the user thinks so...), and then we are supposed to use exactly *these* generators!
                Hfinal = H0
            else:
                from pGroupCohomology.resolution import OPTION
                if OPTION.opts['prot']:
                    print "\n"+"Trying to find minimal generators for", H0
                PhiP = H0.admissibleGroup()
                if repr(PhiP)!='fail':
                    Hfinal = PhiP.Range()
                else:
                    raise ValueError, "The first generators of %s must form a minimal generating set"%(H0)
                #HP = GAP('Group(Image(IsomorphismPermGroup(%s),GeneratorsOfGroup(%s)))'%(H0.name(),H0.name()))
                #try:
                ## Hopefully gap can find minimal generating sets for permutation groups...
                #Hfinal = GAP('Subgroup(%s,MinimalGeneratingSet(%s))'%(HP.name(),HP.name()))
                #
                ## If not, here might be a rescue:
                #except:
                #    try:
                #        Hpc = H0.parent()('PcGroupCode(CodePcGroup(%s),%d)'%(H0.name(),q))
                #        H = H0.parent()('Subgroup(%s,MinimalGeneratingSet(%s))'%(Hpc.name(),Hpc.name()))
                #    except:
                #        raise RuntimeError, "Unable to find a minimal generating set"

            ####
            ## Finally, make up a description of the group which preserves the generators
            ## -- unless the user wants a different key (which may fail badly, but this is not our problem... :)
            if kwds.has_key('GroupDefinition'): 
                KEY = (kwds['GroupDefinition'],)
            else:
                #if not GAP('IsPermGroup(%s)'%(Hfinal.name())):
                KEY = (GAP.eval('Group(List([1..Length(GeneratorsOfGroup(%s))], x -> Image(IsomorphismPermGroup(%s),GeneratorsOfGroup(%s)[x])))'%(Hfinal.name(),Hfinal.name(),Hfinal.name())),)
                #else:
                #    KEY = (repr(Hfinal),)

        #########
        ## Do we have the non-commutative case? If Singular is less than 3-1-0, raise an error.
        if q%2:  
            SingVer = tuple([int(x) for x in singular.eval('system("version")')])
            if SingVer < (3,1,0):
                raise NotImplementedError, ("""There is only Singular %s installed.
We need at least Singular 3-1-0 in the non-commutative Case."""%('-'.join(len(SingVer)*['%d'])))%SingVer

        #########
        ## Determine the location of data storage
        root_explicit = kwds.get('root')
        if root_explicit is not None:
            if len(root_explicit):
                if root_explicit[-1]!='/':
                    root_explicit = root_explicit + '/'
            while '//' in root_explicit:
                root_explicit = root_explicit.replace('//','/')
        from pGroupCohomology.cohomology import COHO
        root_public_db = COHO.public_db #SAGE_ROOT+'/local/pGroupCohomology/db/'
        root_user_db = COHO.user_db #DOT_SAGE+'pGroupCohomology/db/'

        # Now, KEY either provides the coordinates (q,n) of a group in the
        # small groups library, or is of the form (s,) with a string s such
        # that gap(s) defines a group with a minimal generating set.
        # It can be hashed.
        # Moreover the stem name (GStem) is set up, and we may have 
        # a different GroupName (or None).
        # Together with the root, GStem uniquely determines the position
        # of the group data in the file system.
        #
        # The following combined key comprises both and is used to cache
        # the cohomology ring resp. to automatically reload it from disc.

        OUT = None
        if not kwds.get('websource'): # if a websource is provided, it will be used.
            if root_explicit is None:
                CacheKey = (KEY, root_public_db+GStem+'/dat/State')
                if _cache.has_key(CacheKey):
                    OUT = _cache[CacheKey]
                else:
                    CacheKey = (KEY, root_user_db+GStem+'/dat/State')
                    if _cache.has_key(CacheKey):
                        OUT = _cache[CacheKey]
            else:
                CacheKey = (KEY, root_explicit+GStem+'/dat/State')
                if _cache.has_key(CacheKey):
                    OUT = _cache[CacheKey]
            if OUT is not None:    # check if the singular subprocess still runs
                try:
                    foobar = OUT.GenS.parent().eval(OUT.GenS.name())
                except RuntimeError:
                    OUT = None
                    _cache.__delitem__(CacheKey)
            if OUT is not None:
                return OUT
            
            # It was not in the cache. So, we try to look it up on disc.
            from pGroupCohomology.cohomology import COHO        
            if root_explicit is not None:
                try:
                    root = root_explicit
                    OUT = load(root_explicit+GStem+'/H'+GStem)
                except:
                    OUT = None
            else:
                try:
                    root = root_public_db
                    OUT = load(root_public_db+GStem+'/H'+GStem)
                except:
                    try:
                        root = root_user_db
                        OUT = load(root_user_db+GStem+'/H'+GStem)
                    except:
                        OUT = None

        if OUT is None:  # Unless the user forbids it, try to obtain it from some web source
            if kwds.get('websource')!=False:
                try:
                    root = root_explicit or root_user_db
                    if isinstance(kwds.get('websource'), basestring):
                        OUT = self.web_db(GStem, websource=kwds.get('websource'), root=root)
                    else:
                        OUT = self.web_db(GStem, root=root)
                except:
                    pass

        # Now put the result (if it was obtained yet!) into the cache
        if OUT is not None:
            StateData = root+GStem+'/dat/State'
            while '//' in StateData:
                StateData = StateData.replace('//','/')
            CacheKey = (KEY, StateData)
            
            if not isinstance(OUT, COHO):
                raise ValueError, 'The data stored at %s are no Cohomology data!'%(root+GStem+'/')
            if OUT.root != root:
                OUT.setprop('_need_new_root',1)
            if OUT._key != CacheKey:
                if (len(CacheKey[0])==1) and (repr(OUT.group().canonicalIsomorphism(OUT.group().parent()(CacheKey[0][0]))) != 'fail'):
                    print 'Warning: Stored cohomology data have a different group description, but they seem to be equivalent'
                    return OUT
                else:
                    raise ValueError, 'The Cohomology data stored at %s belong to a different group!'%(root+GStem+'/')
            if OUT is not _cache[CacheKey]: # Should not happen, since COHO_unpickle also does caching,
                                            # and even if data moved on disk then they are inserted to the cache
                                            # when being updated.
                raise RuntimeError, "Cache of Cohomology Rings is broken"
            return OUT
        

        #########################
        ## We can't help, the ring must be constructed from scratch.
        # Fancy optional parameters, such as 'auto', 'GroupDescr'
        # etc., will be used here, in ``extras``
        extras = {}
        for k in kwds.items():
            extras[k[0]] = k[1]
        extras['GroupName'] = GroupName
        extras['GStem'] = GStem
        #extras['key'] = CacheKey
        if len(KEY)==1:
            from sage.interfaces.gap import gap
            extras['gap_input'] = q # we must specify the group order
            if root_explicit is not None:
                extras['root'] = root_explicit
                CacheKey = (KEY, root_explicit+GStem+'/dat/State')
                extras['key'] = CacheKey
                OUT = COHO(gap(KEY[0]), **extras)
            else:
                try:
                    CacheKey = (KEY, root_public_db+GStem+'/dat/State')
                    extras['key'] = CacheKey
                    extras['root'] = root_public_db
                    OUT = COHO(gap(KEY[0]), **extras)
                except:
                    if OPTION.opts['prot']:
                        print 'No write permission for the cohomology data base'
                        print '-- using private data base at', COHO.user_db
                    CacheKey = (KEY, root_user_db+GStem+'/dat/State')
                    extras['key'] = CacheKey
                    extras['root'] = root_user_db
                    OUT = COHO(gap(KEY[0]), **extras)                
        else:
            if root_explicit is not None:
                extras['root'] = root_explicit
                CacheKey = (KEY, root_explicit+GStem+'/dat/State')
                extras['key'] = CacheKey
                OUT = COHO(KEY[0],KEY[1], **extras)
            else:
                try:
                    CacheKey = (KEY, root_public_db+GStem+'/dat/State')
                    extras['key'] = CacheKey
                    extras['root'] = root_public_db
                    OUT = COHO(KEY[0],KEY[1], **extras)
                except:
                    if OPTION.opts['prot']:
                        print 'No write permission for the cohomology data base'
                        print '-- using private data base at', COHO.user_db
                    CacheKey = (KEY, root_user_db+GStem+'/dat/State')
                    extras['key'] = CacheKey
                    extras['root'] = root_user_db
                    OUT = COHO(KEY[0],KEY[1], **extras)
        if OUT._key != CacheKey:
            raise RuntimeError, "Keys for caching are corrupted"
        #_cache[CacheKey] = OUT # not necessary, since COHO.__init__ inserts into the cache
        return OUT

    def set_user_db(self, s = None):
        """
        Define the location of a user-dependent cohomology data base

        INPUT:

        ``s``, a string providing a folder name, or ``None``.

        If ``s`` is a string, a cohomology data base in the folder ``s`` will be activated
        as the user's private cohomology data base. If it is ``None``, a private data base
        in a default location will be activated.

        NOTE:

        If necessary, the folder will be created as soon as data from ``s`` are requested.

        EXAMPLES:

        We create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)

        Note that the given path ``tmp_root`` may contain double slashes.
        :func:`~pGroupCohomology.CohomologyRing` replaces them by single slashes::

            sage: tmp_root = tmp_root.replace('//','/')
            sage: H.root.startswith(tmp_root)
            True

        """
        if s is None:
            s = DOT_SAGE+'/pGroupCohomology/db/'
        if not isinstance(s,basestring):
            raise TypeError, "String (pathname) expected"
        while '//' in s:
            s = s.replace('//','/')
        COHO.user_db = s

    def user_db(self,*args, **kwds):
        """
        Retrieve a cohomology ring from a user-dependent cohomology data base

        EXAMPLES:

        We create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. We use the optional parameter
        ``websource=False`` in order to ensure that it is not tried to download a completely
        computed cohomology ring from the web.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computed up to degree 0
            Minimal list of generators:
            []
            Minimal list of algebraic relations:
            []

        """
        kwds['root'] = COHO.user_db
        return self(*args, **kwds)

    def set_web_db(self, s = None):
        """
        Define the location of a web source for cohomology rings

        INPUT:

        ``s``, a string providing a folder name, or ``None``.

        If ``s`` is a string, then cohomology rings will be searched in a web source
        named by ``s``. If it is ``None``, the web source is reset to some default location.
        
        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit. 
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_web_db('http://sage.math.washington.edu/home/SimonKing/Cohomology/db/')
            sage: H=CohomologyRing.web_db('8gp3', root=tmp_root)
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
        
        """
        from pGroupCohomology.cohomology import COHO
        if s is None:
            s = 'http://sage.math.washington.edu/home/SimonKing/Cohomology/db/'
        if isinstance(s,basestring):
            if not s.endswith('/'):
                s = s+'/'
            COHO.web_db = s
        else:
            raise ValueError, "String expected"    

        
    def web_db(self, GStem, websource = None,  root = None):
        """
        Import a cohomology ring from a web source.

        NOTE:

        Usually this function would not be directly used. It is automatically called
        by :func:`~pGroupCohomology.CohomologyRing` if a cohomology ring can not be found
        in a local folder.

        INPUT:

        - ``GStem``, a string so that ``GStem+'.tar.gz'`` can be found in the web source
        - ``websource``: If ``None`` (default), a default location (respectively the
          location provided by :meth:`~pGroupCohomology.CohomologyRingFactory.set_web_db`)
          is chosen. Otherwise, it should be a web address.
        - ``root``: If ``None`` (default), the data are unpacked into the user's currently
          activated private cohomology data base. Otherwise, it provides a directory into
          which the downloaded cohomology data will be unpacked.

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit.
        We use a protocol mode, so that it is visible what happens behind the
        scenes.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: from pGroupCohomology.resolution import OPTION
            sage: OPTION('prot')
            <BLANKLINE>
            sage: H=CohomologyRing.web_db('8gp3', websource='http://sage.math.washington.edu/home/SimonKing/Cohomology/db/', root=tmp_root)
            Downloading http://sage.math.washington.edu/home/SimonKing/Cohomology/db/8gp3.tar.gz
            Extracting archive file at ...
            The state of this cohomology ring is expected to be provided at .../8gp3/dat/State.sobj

            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]

        In order to not break the other doc tests, we now switch the protocol mode off.
        ::
        
            sage: H.option('noprot')
              
        """
        if root is None:
            root = COHO.user_db
        if not root.endswith('/'):
            root = root + '/'
        while '//' in root:
            root = root.replace('//','/')
            
        if websource is None:
            websource = COHO.web_db
        if not websource.endswith('/'):
            websource = websource + '/'
        
        # First step: Download the tar file from the web and unpack it to root
        import urllib
        import tarfile
        from pGroupCohomology.resolution import OPTION
        if OPTION.opts['prot']:
            print "Downloading "+websource + GStem + '.tar.gz'
        (f, h) = urllib.urlretrieve(websource + GStem + '.tar.gz')
        if OPTION.opts['prot']:
            print "Extracting archive file at "+f
        T = tarfile.open(f)
        T.extractall(path=root)
        
        ## Second step: load the cohomology ring and return it
        from pGroupCohomology.resolution import OPTION
        OPTION.opts['@use_this_root@'] = root
        try:
            OUT = load(root + GStem + '/H'+GStem)
            r = OUT.root # this line may imply that the downloaded data are changed on disk in order to make them fit
        except:
            raise RuntimeError, "Unable to import cohomology ring from "+websource + GStem + '.tar.gz'
        return OUT


CohomologyRing = CohomologyRingFactory()
CohomologyRing.__doc__ = r"""
Constructor for modular cohomology rings of finite p-groups

This Constructor is an instance of :class:`~pGroupCohomology.CohomologyRingFactory`.

The constructor can be called directly. Then, it is first checked whether the
completely computed cohomology ring of the given group is part of some data
base, or whether it can be downloaded. If this is not the case, a new cohomology
ring is created, being part of the public or a user defined data base.

Using :meth:`~pGroupCohomology.CohomologyRingFactory.set_user_db`, the location of the
user defined data base can be determined. By :meth:`~pGroupCohomology.CohomologyRingFactory.user_db`,
one can explicitly ask for taking data from the user defined data base. The input
formats for calling :func:`~pGroupCohomology.CohomologyRing` and for calling
:meth:`~pGroupCohomology.CohomologyRingFactory.user_db` are the same.

INPUT:

**Parameters describing the group**

- a finite `p`-group `G` , either

  * given by a prime power ``q`` and a positive number ``n``, determining an entry
    of the SmallGroups library, or
  * given as an object in the Gap interface  
- ``GroupName`` (optional string): a name for the group. If the 
  group `G` is given in the Gap interface and if it is not provided with a 
  custom name (using Gap's ``SetName``) then ``GroupName`` *must* be provided.
- ``GroupDescr`` (optional string): a description of the group. This can be any
  string, and is used when printing the cohomology ring or creating a web-site for it.

**Parameters describing the data base**

- ``root``: A string that, when given, provides a base folder for the data files. This
  folder is created, if necessary.
- ``websource``: If it is ``False``, it is not attempted to download data from some data
  base in the web. If it is a string providing the location of a data base in the web,
  then it is attempted to download the data from there. If ``websource`` is not given
  then first it is tried to look up data in the local file system, and if this fails then
  it is attempted to download the data from some default location in the web.

**Parameters modifying the algorithm**

- ``useElimination`` (optional, default ``None``): If ``True``, the 
  elimination method is used for trying to construct the Dickson classes.
  If ``False``, the linear algebra method is used for that purpose. By default,
  the linear algebra method is chosen, unless there is a Dickson class in degree 
  greater than 18.
- ``DicksonExp`` (optional integer, default = 3): If the elimination method for finding
  the Dickson classes is used, it is needed to set a bound for the power to which
  the Dickson classes might be raised: If `G` is a `p`-group and `n` is the given
  ``DicksonExp``, then the Dickson classes of the elementary abelian sub-groups of `G`
  are raised to the power of `p^0,p^1,...,p^n` before trying to simultaneously lift 
  them to `G`. We do not know any example in which the default value would not suffice.
- ``useFactorization`` (optional boolean, default True): Try to replace the Dickson
  classes of `G` by their minimal non-constant factors. This may simplify some 
  computations.
- ``auto`` (optional integer, default = 2 for abelian groups, and = 4 otherwise):
  a quick but potentially memory consuming method for lifting chain maps will be 
  used in degree at most ``auto``. Up to group orders around 500, the default value
  seems to be heuristically best.
- ``useSlimgb`` / ``useStd`` (optinal boolean): Use Singular's ``slimgb`` (resp. ``std``)
  for computing the Groebner basis of the relation ideal. If not specified, Singular's 
  ``groebner`` method is chosen, which uses a heuristics to find the best algorithm
  for the computation of the Groebner basis.

**Global options**

``options`` (optional string or list of strings): set/unset global options.

There are various options, e.g. concerning protocol output, that apply to *all*
cohomology rings when set for one ring. Each option is set by a string, and unset
by prepending ``'no'`` to that string.

  **Available options**

  * ``'prot'`` [not default], display protocol
  * ``'timing'`` [not default], display timing
  * ``'useMTX'`` [default], use :class:`~pGroupCohomology.mtx.MTX` matrices for
    linear algebra over finite fields, when computing the ring structure. Note that
    the resolutions will always be computed using the Aachen
    `C MeatAxe <http://www.math.rwth-aachen.de/homes/MTX/>`_. By consequence,
    if ``useMTX`` is turned off, much time is spent on the conversion of different
    matrix types.
  * ``'save'`` [default], save intermediate results
  * ``'sparse'`` [default], remove temporarily unneeded data from memory 
  * ``'liftlist'`` [not default], try to lift a whole bunch of cochains at once

The options can also be (un)set later, by using the method 
:meth:`~pGroupCohomology.cohomology.COHO.option`.

The options ``'prot'`` and ``'timing'`` are useful to see the progress of lengthy 
computations. 

* If ``'useMTX'`` is unset, the computation time usually dramatically increases.
  So, we strongly advise to not unset ``'useMTX'``.
* If ``'save'`` is set, the ring will be saved after each computed degree. This 
  will come in handy if a long computation was interrupted. Note, however, that 
  many data, including a minimal projective resolution, will *always* be saved 
  on disk.
* Using the option ``'sparse'`` not only allows for computation of rather big 
  examples; it turns out that it improves the computation time in small 
  examples. So, it is strongly advised to not unset ``'sparse'``.
* With the option ``'liftlist'``, some examples are faster, others are slower. 
  On the groups of order 64, it is slightly faster to not use this option, 
  therefore our default is to not use it.

"""
