#*****************************************************************************
#
#    Cohomology Ring Elements and Maps
#
#    Copyright (C) 2009 Simon A. King <simon.king@uni-jena.de>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 2 or later (at your choice)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
"""
Elements of Cohomology Rings and Induced Homomorphisms

AUTHORS:

- Simon King <simon.king@uni-jena.de>

"""

import sys
import os

## Sage generalities
import sage
import sage.all
from sage.all import cputime
from sage.all import walltime
from sage.all import copy
from sage.all import deepcopy
from sage.all import save
from sage.all import load
from sage.all import SAGE_DB
from sage.all import SAGE_ROOT
from sage.all import tmp_filename

## Sage Rings etc.
from sage.rings.integer import Integer
from sage.rings.finite_field import FiniteField as GF
from sage.matrix.constructor import Matrix
from sage.matrix.matrix_space import MatrixSpace
from sage.structure.element cimport RingElement 
from sage.rings.homset import RingHomset_generic
#from sage.rings.morphism import RingHomomorphism

# auxiliary class from pGroupCohomology (the Cython classes are imported in cochain.pxd
from pGroupCohomology.resolution import OPTION 

#####################################################################
#####################################################################
## Cochain extension type
#####################################################################
#####################################################################

class COCH_unpickle_class:
    r"""
    Unpickling a cochain

    EXAMPLES:
    
    The example produces files. For safety reasons, we choose files
    in a temporary directory; it will be removed as soon as Sage is quit.
    ::

        sage: tmp_root = tmp_filename()
        sage: from pGroupCohomology import CohomologyRing
        sage: CohomologyRing.set_user_db(tmp_root)
        sage: H=CohomologyRing.user_db(8,3)
        sage: H.make()
        sage: C=H.2
        sage: type(C)
        <type 'pGroupCohomology.cochain.COCH'>
        sage: D=loads(dumps(C))
        sage: print C
        1-Cochain in H^*(D8; GF(2)),
        represented by
        [1 0]
        rdeg = 0
        ydeg = 0
        sage: print D
        1-Cochain in H^*(D8; GF(2)),
        represented by
        [1 0]
        rdeg = 0
        ydeg = 0

    ``C`` and ``D`` are different objects, but of course their parents
    are the same, and the cochains are equal::

        sage: D is C
        False
        sage: D.parent() is C.parent()
        True
        sage: D == C
        True

    """
    def __init__(self):
        """
        TESTS::

            sage: from pGroupCohomology.cochain import COCH_unpickle_class
            sage: CU = COCH_unpickle_class()
            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,2,websource=False)
            sage: C = CU(H,2,'foo',[0,1,1],0,1)
            sage: C
            foo, a 2-Cochain in H^*(SmallGroup(8,2); GF(2))
            sage: print C
            2-Cochain in H^*(SmallGroup(8,2); GF(2)),
            represented by
            [0 1 1]
            rdeg = 1
            ydeg = 0

        """
        self.__safe_for_unpickling__ = True

    def __call__(self, H, n, Nick, L, ydeg, rdeg):
        """
        TESTS::

            sage: from pGroupCohomology.cochain import COCH_unpickle_class
            sage: CU = COCH_unpickle_class()
            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3,websource=False)
            sage: C = CU(H,2,'foo',[0,1,1],0,1)
            sage: C
            foo, a 2-Cochain in H^*(D8; GF(2))
            sage: print C
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [0 1 1]
            rdeg = 1
            ydeg = 0
        """
        return COCH(H, n, Nick, L, ydeg, rdeg)

coch_unpickle=COCH_unpickle_class()



cdef class COCH(RingElement):
    """
    COCH extension class representing elements of cohomology rings.

    INPUT:

    - ``H`` -- a cohomology ring (:class:`~pGroupCohomology.cohomology.COHO`)
    - ``n`` -- the degree of the cochain (integer)
    - ``s`` -- name of the `n`-cochain (string)
    - ``M`` -- data describing the cochain. Either
       1. a `(1 \\times d)` :class:`~pGroupCohomology.mtx.MTX` matrix, or 
       2. a `d`-tuple of integers

      where `d` is the projective rank of the `n`-th term of the resolution
      that underlies ``H``
    - ``ydeg`` (optional, default ``None``) -- y-degree of the cochain
    - ``rdeg`` (optional, default ``None``) -- r-degree of the cochain

    OUTPUT:

    A degree `n` element of ``H`` of name s. 

    NOTE:

    - Usually, a cochain will be created by doing algebra with the generators
      of a cohomology ring.
    - In our application (see :mod:`pGroupCohomology`), ``rdeg==1`` if and only if 
      the cochain is a Duflot regular generator of a cohomology
      ring; ``ydeg==1`` if and only if the cochain is a generator
      of a cohomology ring that is neither nilpotent nor Duflot
      regular.
    - Every cochain is provided with a name. When a cochain is the result of a 
      computation then its name will by default describe its construction.
      We think that this is a very useful feature.


    EXAMPLES:

    One of our examples requires that Singular 3-1-0 be installed.
    The examples produce files. For safety reasons, we choose files
    in a temporary directory; it will be removed as soon as Sage is quit.

    First, we show how one would usually create a cochain::

        sage: tmp_root = tmp_filename()
        sage: from pGroupCohomology import CohomologyRing
        sage: CohomologyRing.set_user_db(tmp_root)
        sage: H = CohomologyRing.user_db(8,3)
        sage: H.make()
        sage: C = H.2
        sage: D = H.3
        sage: C
        b_1_0, a 1-Cochain in H^*(D8; GF(2))
        sage: D
        b_1_1, a 1-Cochain in H^*(D8; GF(2))
        sage: C+D
        b_1_0+b_1_1, a 1-Cochain in H^*(D8; GF(2))
        sage: C*D
        (b_1_0)*(b_1_1), a 2-Cochain in H^*(D8; GF(2))
        sage: print C^3
        3-Cochain in H^*(D8; GF(2)),
        represented by
        [1 0 0 0]
        sage: print D^3
        3-Cochain in H^*(D8; GF(2)),
        represented by
        [0 1 0 0]

    Next, we show a non-commutative example. Just for documentation,
    we create the cochains more directly::
    
        sage: H = CohomologyRing.user_db(27,3, websource=False)
        sage: H.resolution()
        Resolution:
        0 <- GF(3) <- GF(3)(27gp3)

    Incidentally, we know that the projective rank of the second term
    of the resolution is four. So, we can define::

        sage: from pGroupCohomology.cochain import COCH
        sage: C = COCH(H,2,'first',(1,0,1,2))

    Now, the resolution is computed out to the second term, and the 
    cochain ``C`` has the name 'first'::

        sage: H.resolution()
        Resolution:
        0 <- GF(3) <- GF(3)(27gp3) <- rank 2 <- rank 4
        sage: C
        first, a 2-Cochain  in H^*(E27; GF(3))
        sage: print C
        2-Cochain in  H^*(E27; GF(3)),
        represented by
        [1 0 1 2]

    Using :class:`~pGroupCohomology.mtx.MTX` matrices, we provide a slightly different way
    to create a cochain. The matrix must be immutable::

        sage: from pGroupCohomology.mtx import MTX
        sage: D = COCH(H,2,'second',MTX(3,[[0,1,0,1]],mutable=False))

    Now, we can add or subtract the cochains::

        sage: C+D
        first+second, a 2-Cochain in H^*(E27; GF(3))
        sage: print C+D
        2-Cochain in H^*(E27; GF(3)),
        represented by
        [1 1 1 0]
        sage: print C-D
        2-Cochain in H^*(E27; GF(3)),
        represented by
        [1 2 1 1]

    Scalar multiplication works as well::

        sage: 2*C
        (first)*2, a 2-Cochain in H^*(E27; GF(3))
        sage: print C*2
        2-Cochain in H^*(E27; GF(3)),
        represented by
        [2 0 2 1]

    But certainly, in cohomology computations the cup-product
    is the most interesting::

        sage: C*D
        (first)*(second), a 4-Cochain in H^*(E27; GF(3))
        sage: H.resolution()
        Resolution:
        0 <- GF(3) <- GF(3)(27gp3) <- rank 2 <- rank 4 <- rank 6 <- rank 7
        sage: E = C^2
        sage: E = C^2+D*C*2
        sage: E
        (first)**2+((second)*(first))*2, a 4-Cochain in H^*(E27; GF(3))
        sage: print E
        4-Cochain in H^*(E27; GF(3)),
        represented by
        [2 2 0 1 0 0 2]

    Since `p>2` in this example, the cohomology ring is non-commutative::

        sage: X = COCH(H,3,'X',(1,1,1,0,0,0))
        sage: Y = COCH(H,3,'Y',(0,0,0,1,1,1))
        sage: print X*Y
        6-Cochain in H^*(E27; GF(3)),
        represented by
        [2 1 1 0 0 0 0 0 0]
        sage: print Y*X
        6-Cochain in H^*(E27; GF(3)),
        represented by
        [1 2 2 0 0 0 0 0 0]

    The name (which by default describes the construction of the cochain)
    can be overwritten::

        sage: E.name()
        '(first)**2+((second)*(first))*2'
        sage: E.setname('foo')
        sage: E
        foo, a 4-Cochain in H^*(E27; GF(3))

    """
#################
# init, dealloc(automatic in this case), repr
    def __init__(self, PARENT, int n, char *Nick, L, ydeg=None, rdeg=None):
        from pGroupCohomology.cohomology import COHO
        if not isinstance(PARENT, COHO):
            raise TypeError, "The parent of a cochain must be a cohomology ring (type <COHO>)"
        cdef RESL R = PARENT.Resl
        if (n<0): # or (n>len(R.Diff)):
            raise IndexError, "Degree out of range"
        while n>R.deg():
            R.nextDiff()
        if isinstance(L,tuple) or isinstance(L,list):
            if (len(L)!=R.Data.projrank[n]):
                raise IndexError, "Last parameter must be of length %d"%(R.Data.projrank[n])
            self.Resl = R
            self.Deg = n
            self.Name = str(Nick)
            self.Data = MTX(R.G_Alg.Data.p, [L], mutable=False,mutable=False)
        elif isinstance(L,MTX):
            if (L.base()!=R.G_Alg.Data.p):
                raise TypeError, "MTX matrix must be defined over GF(%d)"%(R.G_Alg.Data.p)
            if (L.is_mutable()):
                raise ValueError, "MTX matrix must be immutable"
            if (L.nrows()!=1) or (L.ncols()!=R.Data.projrank[n]):
                raise TypeError, "(1 x %d) MTX matrix expected"%(R.Data.projrank[n])
            self.Resl = R
            self.Deg = n
            self.Name = Nick
            self.Data = L
        else:
            raise TypeError, "Last parameter must be a list/tuple of integers or an MTX matrix"
        self.Ydeg = ydeg
        self.Rdeg = rdeg
        RingElement.__init__(self,PARENT)
        self._latex = None
        #print self._parent

    def __copy__(self):
        """
        Return a copy of self, defined over *the same* resolution

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C = H.2+H.3
            sage: C == copy(C)
            True
            sage: C is copy(C)
            False

        """
        return COCH(self._parent, self.Deg, self.Name, copy(self.Data), self.ydeg, self.rdeg)

    def __reduce__(self):
        """
        Return data used for pickling/unpickling self

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C = H.2+H.3
            sage: C == copy(C)
            True
            sage: C is copy(C)
            False

        """
        return coch_unpickle, (self._parent, self.Deg, self.Name, self.Data, self.Ydeg, self.Rdeg)
        
    def __repr__(self):
        """
        Return a brief description of the cochain

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C = H.2+H.3
            sage: C # indirect doc test
            b_1_0+b_1_1, a 1-Cochain in H^*(D8; GF(2))

        """
        return self.name()+', a %d-Cochain in %s'%(self.Deg,self.parent().__repr__())
    
    def __str__(self):
        """
        Return a more detailed description of the cochain

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C = H.2+H.3
            sage: print C # indirect doc test
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]

        """
        s = ("%d-Cochain in %s"%(self.Deg,self.parent().__repr__()))+',\nrepresented by\n'+self.Data.__str__()
        if (self.rdeg()!=None):	
            s=s+'\nrdeg = %d'%(self.rdeg())
        if (self.ydeg()!=None):
            s=s+'\nydeg = %d'%(self.ydeg())
        return s
    
    def _singular_(self, S):
        """
        Represent self as an element of the Singular interface version of the parent cohomology ring

        TESTS::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: singular(H.1)
            c_2_2
            sage: singular(H.2*H.1)
            c_2_2*b_1_0
        
        """
        self.parent()._singular_(S).set_ring()
        try:
            return S(self.name())
        except:
            raise RuntimeError, "unable to interprete %s in Singular"%(self.name())
        raise NotImplementedError

    def set_latex_name(COCH self, s):
        """
        Declare how self should be typeset in LaTeX

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: latex(H.2)
            b_{1,0}
            sage: (H.2).set_latex_name('H_2')
            sage: latex(H.2)
            H_2

        """
        if not isinstance(s, basestring):
            raise ValueError, "String expected"
        self._latex = s

    def _latex_(COCH self):
        """
        Return a latex typeset

        TESTS::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: latex(H.2) # indirect doc test
            b_{1,0}
            
        """
        if self._latex is not None:
            return self._latex
        out = self.name()
        i = out.find('_')
        if i>=0:
            p1 = out[:i]
            p2 = out[i+1:].replace('_',',')
        return p1+'_{'+p2+'}'


################
# structural parts
    def resolution(self):
        """
        Return the underlying resolution

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)
            sage: from pGroupCohomology.cochain import COCH
            sage: C=COCH(H,2,'foo',(1,1,0))

        Note that the needed terms of the resolution are automatically
        computed::

            sage: C.resolution()
            Resolution:
            0 <- GF(2) <- GF(2)(8gp3) <- rank 2 <- rank 3
            sage: C.resolution() is H.resolution()
            True

        """
        return self.Resl

    def deg(self):
        """
        Return the degree of self

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C=H.2*H.1
            sage: C.deg()
            3

        """
        return self.Deg

    def MTX(self):
        """
        Return the :class:`~pGroupCohomology.mtx.MTX` matrix by which self is defined

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C=H.2*H.1
            sage: print C.MTX()
            [0 0 1 0]

        """
        return self.Data

    def name(self):
        """
        Return the name of self

        NOTE:

        When a :class:`~pGroupCohomology.cochain.COCH` instance is created by invoking
        the init method, a name must be given. When further :class:`~pGroupCohomology.cochain.COCH`
        instances are created by arithmetic operations, the name of the result describes its
        construction.

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C=H.2*H.1
            sage: print C.name()
            (b_1_0)*(c_2_2)
            sage: H(C.name()) == C
            True

        Note that there is no automatic simplification when the result
        represents the trivial cochain::

            sage: (C*2).name()
            '((b_1_0)*(c_2_2))*0'
            sage: print C*2
            3-Cochain in H^*(D8; GF(2)),
            represented by
            [0 0 0 0]

        """
        return self.Name

    def setname(self, s):
        """
        Set the name of self

        INPUT:

        s -- a string providing the new name

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C=H.2*H.1
            sage: C
            (b_1_0)*(c_2_2), a 3-Cochain in H^*(D8; GF(2))
            sage: C.setname('foo')
            sage: C
            foo, a 3-Cochain in H^*(D8; GF(2))

        """
        if not isinstance(s, basestring):
            raise TypeError, "String expected"
        self.Name = s

    def rdeg(self):
        """
        Return the r-degree of self

        NOTE:

        The r-degree plays an important role in the computation
        of cohomology rings. A generator of a cohomology ring
        has ``rdeg==1`` if and only if it is a Duflot regular
        generator, which means that its restriction to the 
        greatest central elementary abelian subgroup is not
        nilpotent.

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.1.rdeg()
            1
            sage: H.2.rdeg()
            0
            sage: H.3.rdeg()
            0
 
        """
        return self.Rdeg

    def ydeg(self):
        """
        Return the y-degree of self

        NOTE:

        The y-degree plays an important role in the computation
        of cohomology rings. A generator of a cohomology ring
        has ``ydeg==1`` if and only if it is nilpotent.

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,3)
            sage: H.make()
            sage: print H
            Cohomology ring of Small Group number 3 of order 16 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [b_2_1, a 2-Cochain in H^*(SmallGroup(16,3); GF(2)),
             c_2_2, a 2-Cochain in H^*(SmallGroup(16,3); GF(2)),
             c_2_3, a 2-Cochain in H^*(SmallGroup(16,3); GF(2)),
             a_1_0, a 1-Cochain in H^*(SmallGroup(16,3); GF(2)),
             b_1_1, a 1-Cochain in H^*(SmallGroup(16,3); GF(2))]
            Minimal list of algebraic relations:
            [a_1_0^2,
             a_1_0*b_1_1,
             b_2_1*a_1_0,
             b_2_1^2+c_2_2*b_1_1^2]
            sage: H.1.rdeg(),H.1.ydeg()
            (0, 0)
            sage: H.2.rdeg(),H.2.ydeg()
            (1, 0)
            sage: H.4.rdeg(),H.4.ydeg()
            (0, 1)

        """
        return self.Ydeg
    
################
# !=,==, dictionary etc.
    def __richcmp__(COCH self, COCH C,x):
        """
        Compare cochains

        NOTE:

        First, it is checked whether the underlying 
        resolutions are identical. If they are not, "False"
        is returned for "<", "<=", "==", ">" and ">=".
        Then, for "<", "<=", ">" and ">=", the degrees 
        of the cochains are compared. For "==", the answer
        is true if and only if the resolutions are identical
        and both the degrees and defining matrices are 
        equivalent.

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.1.deg()
            sage: H.2.deg()
            sage: H.1>H.2
            True
            sage: print H.2*H.3
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [0 0 0]
            sage: H.2*H.3 == H.1*0
            True

        """
        # < 0, <= 1, == 2, != 3, > 4, >= 5
        if (x==0):
            return (self.Resl.G_Alg == C.Resl.G_Alg) and (self.Resl is C.Resl) \
                   and (self.Deg<C.Deg)
        if (x==1):
            return (self.Resl.G_Alg == C.Resl.G_Alg) and (self.Resl is C.Resl) \
                   and (self.Deg<=C.Deg)
        if (x==2):
            # A more general definition would be: Test whether
            # C.Resl and self.Resl are equal (not identical) up
            # to degree self.Deg
            return (self.Resl.G_Alg == C.Resl.G_Alg) and (self.Resl is C.Resl) \
                   and (self.Deg==C.Deg) and (self.Data==C.Data)
        if (x==3):
            return (self.Resl.G_Alg != C.Resl.G_Alg) or (self.Deg!=C.Deg) or (self.Data!=C.Data) \
                   or (not (self.Resl is C.Resl))
        if (x==4):
            return (self.Resl.G_Alg == C.Resl.G_Alg) and (self.Resl is C.Resl) \
                   and (self.Deg>C.Deg)
        if (x==5):
            return (self.Resl.G_Alg == C.Resl.G_Alg) and (self.Resl is C.Resl) \
                   and (self.Deg>=C.Deg)
        raise NotImplementedError, "Last argument must range from 0 to 5"

    def __hash__(COCH self):
        r"""
Return a hash value of self

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: C = H.2
            sage: hash(C)==hash(copy(C))
            True
            sage: hash(C)==hash(loads(dumps(C)))
            True
            sage: D = {C:1, C^2:2}
            sage: D[copy(C)]
            1
            sage: D[copy(C)*C]
            2

        """
        return (self.Deg, self.Data).__hash__()
        
    
################
# Arithmetic
    def __add__(COCH self, COCH C):
        r"""
        Sum of cochains of the same degree

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.1+H.2
            Traceback (most recent call last):
            ...
            IndexError: cochains must be of the same degree
            sage: H.2+H.3
            b_1_0+b_1_1, a 1-Cochain in H^*(D8; GF(2))
            sage: print H.3+H.2
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]

        """
        if (self.Resl is C.Resl):
            if self.Deg!=C.Deg:
                raise IndexError, "cochains must be of the same degree"
            else:
                return COCH(self._parent, self.Deg, self.Name+'+'+C.Name, self.Data+C.Data)
        else:
            raise TypeError, "Cochains must be defined over a common resolution"

    def __sub__(COCH self, COCH C):
        """
        Difference of two cochains of the same degree

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.1-H.2
            Traceback (most recent call last):
            ...
            IndexError: cochains must be of the same degree
            sage: H.2-H.3
            b_1_0-(b_1_1), a 1-Cochain in H^*(D8; GF(2))
            sage: print H.3+H.2
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]

        """
        if (self.Resl is C.Resl):
            if self.Deg!=C.Deg:
                raise IndexError, "cochains must be of the same degree"
            else:
                return COCH(self._parent, self.Deg, self.Name+'-('+C.Name+')', self.Data-C.Data)
        else:
            raise TypeError, "Cochains must be defined over a common resolution"
    def __neg__(COCH self):
        r"""
        Additive inverse of a cochain

        TESTS:
        
        It is required that Singular-3-1-0 be installed.
        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()
            sage: - H.1
            (-(b_2_0)), a 2-Cochain in H^*(E27; GF(3))
            sage: print H.1
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [1 0 0 0]
            rdeg = 0
            ydeg = 0
            sage: print -H.1
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [2 0 0 0]

        """
        return COCH(self._parent, self.Deg, '(-('+self.Name+'))', -self.Data)
    
    def __mul__(left, right):
        """
        Cup product of two cochains, or scalar multiplication 

        NOTE:

        Some data needed for the construction of the cup product are 
        cached. This yields to a massive improvement of performance
        in cohomology computations.

        NOTE:

        Let ``D1,D2`` be two cochains in a cohomology ring, of
        degrees ``d1,d2``. In general, the computation of ``D1*D2``
        is faster than the computation of ``D2*D1``, if ``d1<d2``.
        Hence, for computing ``D2*D1``, we recommend to compute 
        ``D1*D2`` first.

        The cup product is graded commutative. Hence, 
        ``D2*D1 = D1*D2*(-1)^(d1*d2)``.

        TESTS:

        It is required that Singular-3-1-0 be installed.
        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()
            sage: H.gens()
            [1,
             b_2_0, a 2-Cochain in H^*(E27; GF(3)),
             b_2_1, a 2-Cochain in H^*(E27; GF(3)),
             b_2_2, a 2-Cochain in H^*(E27; GF(3)),
             b_2_3, a 2-Cochain in H^*(E27; GF(3)),
             c_6_8, a 6-Cochain in H^*(E27; GF(3)),
             a_1_0, a 1-Cochain in H^*(E27; GF(3)),
             a_1_1, a 1-Cochain in H^*(E27; GF(3)),
             a_3_4, a 3-Cochain in H^*(E27; GF(3)),
             a_3_5, a 3-Cochain in H^*(E27; GF(3))]
            sage: H.2*H.3
            (b_2_1)*(b_2_2), a 4-Cochain in H^*(E27; GF(3))
            sage: H.2*H.3 == H.3*H.2
            True
            sage: print H.7*H.9
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [1 2 1 0 0 0 0]
            sage: print H.9*H.7
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [2 1 2 0 0 0 0]
            sage: 2*H.3 == (-1)*H.3 == H.3*2 == -H.3
            True
            sage: (-1)*H.3
            (b_2_2)*2, a 2-Cochain in H^*(E27; GF(3))

        """
        cdef COCH self
        cdef COCH C
        if not isinstance(left, COCH):  # can be switched -- a little Cython oddity...
            return right.__mul__(left)
        
        self = left    # now, left is a cochain
        if not isinstance(right, COCH): # then, right is supposed to be convertible to an int
            right = int(right)
            if right==1:
                return self
            else:
                return COCH(self._parent, self.Deg, '('+self.Name+')*'+str(right%self.Data.Data.fl), self.Data._mulInt_(right))
        C = right  # now, right is supposed to be a cochain
        if C.Deg == 0:
            right = C.Data[0,0]
            if right==1:
                return self
            else:
                return COCH(self._parent, self.Deg, '('+self.Name+')*'+str(right%self.Data.Data.fl), self.Data._mulInt_(right))
        if self.Deg == 0:
            left = self.Data[0,0]
            if left==1:
                return C
            else:
                return COCH(C._parent, C.Deg, '('+C.Name+')*'+str(left%C.Data.Data.fl), C.Data._mulInt_(left))
        cdef RESL R
        cdef MTX CM, CM1,CM2,OUT
        cdef int n,n_old,n_oldmax,Cdeg,nontips
        cdef int i,j, rk, RK, n_orig
        cdef list L
        # cdef PTR src,dest
        if isinstance(C,COCH):
            if not (self.Resl is C.resolution()):
                raise TypeError, "Cochains must be defined over a common resolution"
            CM = C.MTX()
            n_orig = C.deg()
            #if (self.deg()+C.deg())>len(self.Resl.Diff):
            #    raise IndexError, "Resolution only known up to degree %d"%(len(self.Resl.Diff))
            while (self.deg()+C.deg())>len(self.Resl.Diff):
                self.Resl.nextDiff()
            R = self.Resl
            nontips = R.G_Alg.Data.nontips
            Cdeg = C.deg()
            n = self.deg()+Cdeg
            # lift to chain maps -> kG=P_0:
            # print 'starting product'
            (n1,d1,CM1) = R.CochainToChainmap(self.Deg,self.Data)
            # (n2,d2,CM2) = R.CochainToChainmap(Cdeg,CM)

            ###########
            # the lift of the second chain map 
            # is likely to be known; otherwise it is computed now.
            # if OPTION.opts['keeplifts']:
            #if R.Lifts.has_key(1):
            #    R.importLifts()
            TryLift = R.Lifts[(n,n_orig,CM)]
            if TryLift is None:
                if OPTION.opts['prot']:
                    print ((self.Resl.indent)*'::: ')+'> > We need more lifts!'
                for n_oldmax from n > n_oldmax >= Cdeg:
                    TryLift = R.Lifts[(n_oldmax,n_orig,CM)]
                    if TryLift != None:
                        break
                #TryLift = R.Lifts[(n_oldmax,n_orig,CM)]
                if TryLift is None:
                    ((n2,d2,CM2),n_max) = (R.CochainToChainmap(Cdeg,CM), 2*Cdeg)
                    R.setLift(C,2*Cdeg)
                else:
                    ((n2,d2,CM2),n_max) = TryLift
                while (n2<n):
                    (n2,d2,CM2) = R.liftChainMap((n2,d2,CM2))
                    self.Resl.Lifts[(n2,n_orig,CM)] = ((n2,d2,CM2),n_max)
            else:
                ((n2,d2,CM2),n_max) = TryLift
            # compose self with the lift of C
            # OUT = R.composeChainMaps(CM2,CM1,self.deg()+C.deg(),self.deg(),0)
            # the following should be MUCH faster:
            OUT = MTX(self.Data.Data.fl, R.Data.projrank[self.deg()+Cdeg], nontips,mutable=False)
            zsetfield(self.Data.Data.fl)
            zsetlen(nontips)
            L = self.Data._rowlist_(0)
            rk = R.Data.projrank[self.deg()]
            RK = R.Data.projrank[self.deg()+Cdeg]
            # CM2 should have rk*RK rows
            # print 'Compose Chainmaps'
            for i from 0 <= i < rk: # loop through the rows of OUT
                if L[i]:
                    for j from 0 <= j < RK: # loop through the entries of the lift of C
                        # print CM2[i+rk*j],i
                        if not (zaddmulrow(ptrPlus(OUT.Data.d,j),ptrPlus(CM2.Data.d,i+rk*j), zitof(L[i]))):
                            raise ArithmeticError, "Something went wrong"
            return COCH(self._parent, self.Deg+Cdeg, '('+self.Name+')*('+C.name()+')', \
                R.ChainmapToCochain((self.Deg+Cdeg,0,OUT)))
        else:
            raise TypeError, "Multiplication COCH*%s not defined"%(type(C))

    def __div__(COCH self, long c):
        """
        Quotient of a cochain by a field element

        TESTS:

        It is required that Singular-3-1-0 be installed.
        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()
            sage: print H.2
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [0 1 0 0]
            rdeg = 0
            ydeg = 0
            sage: print H.2/2
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [0 2 0 0]

        """
        if c == 1:
            return self
        else:
            return COCH(self._parent, self.Deg, '('+self.Name+')/'+str(c%self.Data.Data.fl), self.Data/c)
       
    def __pow__(COCH self, int n, x):
        """
        Raise a cochain to some non-negative integer power (cup product with itself)

        NOTE:

        Some data needed for the construction of the cup product are cached. This
        yields to a massive improvement of performance in cohomology computations.

        TESTS:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.3^4
            (b_1_1)**4, a 4-Cochain in H^*(D8; GF(2))
            sage: print H.3^4
            4-Cochain in H^*(D8; GF(2)),
            represented by
            [0 1 0 0 0]

        """
        if n<0: 
            raise ArithmeticError, "Exponent must be non-negative integer"
        if n==0:
            return int(1)
        #if (self.Deg*n>len(self.Resl.Diff)):
        #    raise ArithmeticError, "Resolution only known up to degree %d"%(len(self.Resl.Diff))
        while (self.Deg*n>len(self.Resl.Diff)):
            self.Resl.nextDiff()
        s=self.Name
        cdef COCH OUT
        OUT = self
        for i from 1 <= i < n:
            OUT = self*OUT
        if n>1:
            OUT.Name='('+s+')**'+str(n)
        else:
            OUT.Name=s
        return OUT

    def right_multiplication(COCH self):
        """
        Return the cohomology ring endomorphism given by right-multiplication with self

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            sage: x,a,b,c = H.gens()
            sage: B = b.right_multiplication()
            sage: C = c.right_multiplication()
            sage: B(a) == a*b
            True
            sage: A = a.right_multiplication()
            sage: BA = A(B)
            sage: BA(a) == a*a*b
            True
            sage: r = H.restriction_maps()[2][1]
            sage: rB = r(B)
            sage: r(a)*r(b) == rB(a)
            True

        """
        cdef RESL R = self.Resl
        nt = R.G_Alg.Data.nontips
        fd = R.G_Alg.Data.p
        I = MTX(fd,nt,"id")
        M = R.CochainToChainmap(self.Deg,self.Data)[2]
        name = '(.*%s)'%(self.Name)
        f = self.parent().hom(I,self.parent(),M,-self.Deg)
        f.set_name(name)
        return f

    def normalize(COCH self):
        """
        Scale self inplace so that its leading coefficient is 1 (if non-zero)

        OUTPUT:

        ``True`` if self is a null-cochain, and ``None`` otherwise

        EXAMPLES:

        It is required that Singular-3-1-0 be installed.
        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()
            sage: C=2*H.1+H.2
            sage: print C
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [2 1 0 0]
            sage: C.normalize()
            sage: print C
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [1 2 0 0]
            sage: C
            ((b_2_0)*2+b_2_1)/2, a 2-Cochain in H^*(E27; GF(3))
            sage: D = 0*H.2
            sage: D.normalize()
            True
            sage: D
            (b_2_1)*0, a 2-Cochain in H^*(E27; GF(3))

        """
        f,i = self.Data.lead()
        if f:
            if f!=1:
                matmulF(self.Data.Data,tmultinv[zitof(f)])
                self.Name = '('+self.Name+')/%d'%(f)
            return None
        else:
            return True
        

#####################################################################
#####################################################################
## Chainmap extension type (and homset)
#####################################################################
#####################################################################

class CohomologyHomset(RingHomset_generic):
    """
    Set of Homomorphisms between Modular Cohomology Rings of Finite `p`-Groups

    EXAMPLES:

    The example produces files. For safety reasons, we choose files
    in a temporary directory; it will be removed as soon as Sage is quit::

        sage: tmp_root = tmp_filename()
        sage: from pGroupCohomology import CohomologyRing
        sage: CohomologyRing.set_user_db(tmp_root)

    We consider the dihedral group of order 8 and will study the restriction
    maps to representatives for all conjugacy classes of non-trivial subgroups.
    ::

        sage: G = gap('DihedralGroup(8)')
        sage: H = CohomologyRing.user_db(G,GroupName = 'D8')
        sage: H.make()
        sage: SubG = [X.RepresentativeSmallest() for X in G.ConjugacyClassesSubgroups()]
        sage: SubG = [X.MinimalGeneratingSet().Group() for X in SubG if X.Order()>1]
        sage: for i in range(len(SubG)):
        ...       SubG[i].SetName('"U%d"'%(i+1))
        sage: HSubG = [CohomologyRing.user_db(X) for X in SubG]
        sage: for X in HSubG:
        ...       X.make()
        sage: IncG = [X.GroupHomomorphismByImages(G,X.GeneratorsOfGroup(),X.GeneratorsOfGroup()) for X in SubG]
        sage: for i in range(len(IncG)):
        ...       IncG[i].SetName('"i_%d"'%(i+1))
        sage: ResG = [H.hom(IncG[i], HSubG[i]) for i in range(len(HSubG))]

    Now, we can apply the maps to elements of ``H``::

        sage: ResG[2](H.2+H.3)
        i_3^*(b_1_0+b_1_1), a 1-Cochain in H^*(U3; GF(2))

    The image of an element of the base field is mapped to itself, while
    cochains of degree 0 are properly mapped::

        sage: ResG[2](H.0)
        1
        sage: ResG[2](H(1))
        i_3^*(1), a 0-Cochain in H^*(U3; GF(2))

    Finally, we show how the images of non-scalar generators of ``H`` under
    the various restrictions maps can be expressed as elements of the cohomology
    rings of the subgroups::

        sage: for f in ResG:
        ...       print [f.codomain().cochain_to_polynomial(f(X)).name() for X in H.gens()[1:]]
        ['c_1_0^2', '0', '0']
        ['0', 'c_1_0', '0']
        ['c_1_0^2', 'c_1_0', 'c_1_0']
        ['c_1_1^2+c_1_0*c_1_1', 'c_1_0', '0']
        ['c_2_0', '0', 'c_1_0']
        ['c_1_1^2+c_1_0*c_1_1+c_1_0^2', 'c_1_1', 'c_1_1']
        ['c_2_2', 'b_1_0', 'b_1_1']

    """
    # it inherits _domain and _codomain
    # -- option cat is not supported yet
    def __init__(self, X, Y, cat=None): #, check=True):
        from pGroupCohomology.cohomology import COHO
        if not isinstance(X,COHO):
            raise TypeError, "Domain must be a cohomology ring"
        if not isinstance(Y,COHO):
            raise TypeError, "Codomain must be a cohomology ring"
        if (X.Resl.coef()<>Y.Resl.coef()):
            raise TypeError, "Domain and Codomain must be defined over the same base field"
        self._cache = {}  # caches different maps between the same rings
        self._H0Map = MTX(X.Resl.coef(), 1, X.Resl.G_ALG().order())
        self._H0Map[0,0] = 1
        self._H0Map.set_immutable()
        RingHomset_generic.__init__(self, X, Y)
    
    def __call__(self, GMap, H0Map=None, d=0):
        """
        INPUT:

        - ``GMap`` (see below)
        - ``H0Map`` - usually ``None`` (default), but can be a 
          :class:`~pGroupCohomology.mtx.MTX` matrix providing a map from
          term ``-d`` of the resolution of the codomain to term zero of
          the resolution of the domain (d<0), respectively from term
          zero of the resolution of the codomain to term ``d`` of the
          resolution of the domain. We do not further document nor support
          this usage, so, it is very likely that it won't work.
        - ``d`` - an integer, the degree of the chain map. 
          Must be zero if ``H0Map`` is ``None``.
        
        Use cases:

        1. ``GMap`` is a :class:`~pGroupCohomology.mtx.MTX` matrix defining a 
           homeomorphism of group algebras.
        2. ``GMap`` is a group homomorphism, defined in the gap interface
        3. (not implemented yet) ``GMap`` is a subgroup, defined in the gap interface
        If ``H0Map`` is ``None``, we replace it by the matrix given by ``[[1,0,0,...]]``

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: HS = H2.Hom(H1)
            sage: HS(phi_star.G_map()) == HS(phi) == phi_star
            True

        """
        Name = None
        Src = self._codomain  # source of chain map, not of induced map
        Tgt = self._domain
        if H0Map is not None:
            if not isinstance(H0Map,MTX):
                raise TypeError, "second argument, if provided, must be an MTX matrix"
            if H0Map.is_mutable():
                H0Map.set_immutable()
                H0Mapmutable = True
            else:
                H0Mapmutable = False
        else:
            H0Map = self._H0Map
            H0Mapmutable = False

        if isinstance(GMap, MTX):
            if GMap.is_mutable():
                GMap.set_immutable()
                GMapmutable = True
            else:
                GMapmutable = False
        else: # GMap should be a group homomorphism in gap or 
              # (not yet implemented) list of elements of the group of self._domain,
              # defining the images of the generators of the group of self._codomain
            if hasattr(GMap,'parent'):
                GAP = GMap.parent()
                try:
                    Name = GAP.eval('Name(%s)'%(GMap.name()))[1:-1]
                except RuntimeError:
                    pass
                if not (GAP == Src.group().parent() == Tgt.group().parent()):
                    raise RuntimeError, "The second argument and the groups of domain and codomain must be defined in the same Gap session" 
                if GAP('IsGroupHomomorphism(%s)'%(GMap.name())):
                    # test whether source and image match Src and Tgt
                    GSrc = GMap.Source()
                    GTgt = GMap.Range()
                    GSrcAdm = GSrc.admissibleGroup()
                    if repr(GSrcAdm) == 'fail':
                        raise ValueError, "The source of the group homomorphism isn't given by minimal generators"
                    GTgtAdm = GTgt.admissibleGroup()
                    if repr(GTgtAdm) == 'fail':
                        raise ValueError, "The range of the group homomorphism isn't given by minimal generators"
                    GSrcAdmH = GSrcAdm.Range().canonicalIsomorphism(Src.group())
                    if repr(GSrcAdmH) == 'fail':
                        raise ValueError, "The source of the group homomorphism doesn't fit to the range of the induced homomorphism, %s"%repr(Src.group())
                    GTgtAdmH = GTgtAdm.Range().canonicalIsomorphism(Tgt.group())
                    if repr(GTgtAdmH) == 'fail':
                        raise ValueError, "The range of the group homomorphism doesn't fit to the source of the induced homomorphism, %s"%repr(Tgt.group())

                    # Finally, we replace GMap by a map that goes from Src.group() to Tgt.group(), by composition.
                    GMap = GAP('CompositionMapping(%s,%s,%s,InverseGeneralMapping(%s),InverseGeneralMapping(%s))'%(GTgtAdmH.name(),GTgtAdm.name(),GMap.name(),GSrcAdm.name(),GSrcAdmH.name()))
                else: # not a homomorphism
                    raise NotImplementedError, "We expected a group homomorphism"

                # Now we have a map GMap between Src.group() and Tgt.group().
                # Hence, we can produce a temporary file providing the MTX matrix of the induced group algebra map.
                IStem = tmp_filename()
                GStem = Tgt.gps_folder+Tgt.GStem
                HStem = Src.gps_folder+Src.GStem
                GAP.eval('makeInducedHomomorphismData("%s","%s","%s", %s, %d)'%(GStem,HStem,IStem,GMap.name(),Tgt.Resl.G_ALG().order()))
                GMap = MTX(IStem+'.ima')
                GMap.set_immutable()
                GMapmutable = False
            else:
                raise TypeError, "First argument must be an MTX matrix or some data defined in Gap"
        try:
            OUT = self._cache[GMap,H0Map,d]
        except KeyError:
            OUT = ChMap(self, GMap, H0Map, d)
            self._cache[GMap,H0Map,d] = OUT
        if GMapmutable:
            GMap.set_mutable()
        if H0Mapmutable:
            H0Map.set_mutable()
        if Name is not None:
            OUT.set_name(Name+'^*')
        return OUT

    def _repr_(self):
        """
        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.restriction_maps()[2][1].parent()
            Set of Homomorphisms from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))

        """
        return "Set of Homomorphisms from %s to %s"%(self._domain.__repr__(), self._codomain.__repr__())

class ChMap_unpickle_class:
    """
    Unpickling an instance of :class:`~pGroupCohomology.cochain.ChMap`, representing an induced homomorphism
    
    EXAMPLES:

    Usually, an induced homomorphism is created by defining two groups and
    a homomorphism between them, computing the cohomology rings of the groups,
    and then invoking the :meth:`~pGroupCohomology.cohomology.COHO.hom`` method
    of cohomology rings.

    We first create the cohomology rings for two different presentations of the
    dihedral group of order 8. We use a temporary directory for saving data; it
    will be removed as soon as Sage is quit.
    ::

        sage: from pGroupCohomology import CohomologyRing
        sage: tmp_root = tmp_filename()
        sage: CohomologyRing.set_user_db(tmp_root)
        sage: G1 = gap('SmallGroup(8,3)')
        sage: H1 = CohomologyRing.user_db(8,3,websource=False)
        sage: H1.make()
        sage: G2 = gap('DihedralGroup(8)')
        sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
        sage: H2.make()
        sage: phi = G1.IsomorphismGroups(G2)

    After ensuring that ``phi`` is printed nicely, we obtain the induced map and see that it is cached::

        sage: phi.SetName('"phi"')
        sage: phi_star = H2.hom(phi,H1)
        sage: phi_star is loads(dumps(phi_star))
        True


    """
    def __init__(self):
        """
        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi_star = H2.hom(phi,H1)
            sage: from pGroupCohomology.cochain import ChMap_unpickle_class
            sage: CU = ChMap_unpickle_class()
            sage: phi_star2 = CU(H2,H1,phi_star.G_map(),[phi_star[0]],0)
            sage: phi_star2 is phi_star
            True 

        """
        self.__safe_for_unpickling__ = True

    def __call__(self, Src,Tgt,GMap,Data,Deg, name = None):
        """
        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi_star = H2.hom(phi,H1)
            sage: from pGroupCohomology.cochain import ChMap_unpickle_class
            sage: CU = ChMap_unpickle_class()
            sage: phi_star2 = CU(H2,H1,phi_star.G_map(),[phi_star[0]],0)
            sage: phi_star2 is phi_star
            True

        """
        cdef ChMap OUT
        #print "chmap_unpickle for ",type(Src),type(Tgt)
        #OUT = Tgt.hom(GMap,Src,Data[0], Deg)
        OUT = Src.hom(GMap,Tgt,Data[0], Deg)
        OUT._name = name
        OUT.Data = Data
        return OUT

chmap_unpickle=ChMap_unpickle_class()

cdef class ChMap(RingHomomorphism):
    """
    Extension class representing induced homomorphisms of cohomology rings.

    EXAMPLES:

    Usually, an induced homomorphism is created by defining two groups and
    a homomorphism between them, computing the cohomology rings of the groups,
    and then invoking the :meth:`~pGroupCohomology.cohomology.COHO.hom`` method
    of cohomology rings.

    We first create the cohomology rings for two different presentations of the
    dihedral group of order 8. We use a temporary directory for saving data; it
    will be removed as soon as Sage is quit.
    ::

        sage: from pGroupCohomology import CohomologyRing
        sage: tmp_root = tmp_filename()
        sage: CohomologyRing.set_user_db(tmp_root)
        sage: G1 = gap('SmallGroup(8,3)')
        sage: H1 = CohomologyRing.user_db(8,3,websource=False)
        sage: H1.make()
        sage: G2 = gap('DihedralGroup(8)')
        sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
        sage: H2.make()

    In order to obtain reproducible doc tests, we switch to permutation
    groups that are, from the perspective of our programs, equivalent to
    ``G1`` and ``G2``, and provide an explicit group isomorphism::

        sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
        sage: H1.group()==phi.Source()
        True
        sage: H2.group()==phi.Range()
        True
        sage: phi.IsInjective()
        true
        sage: phi.IsSurjective()
        true

    After ensuring that ``phi`` is printed nicely, we obtain the induced map::

        sage: phi.SetName('"phi"')
        sage: phi_star = H2.hom(phi,H1)
        sage: phi_star
        phi^*
        sage: phi_star.domain()
        H^*(DihedralGroup(8); GF(2))
        sage: phi_star.codomain()
        H^*(D8; GF(2))
        sage: print [H1.cochain_to_polynomial(phi_star(X)) for X in H2.gens()]
        [1, a 0-Cochain in H^*(D8; GF(2)), b_1_0^2+c_2_2, a 2-Cochain in H^*(D8; GF(2)), b_1_1+b_1_0, a 1-Cochain in H^*(D8; GF(2)), b_1_0, a 1-Cochain in H^*(D8; GF(2))]

    Similarly, we get the induced map of the inverse map::

        sage: phi_inv = phi.InverseGeneralMapping()
        sage: phi_star_inv = H1.hom(phi_inv,H2)
        sage: print [H2.cochain_to_polynomial(phi_star_inv(X)) for X in H1.gens()]
        [1, a 0-Cochain in H^*(DihedralGroup(8); GF(2)), b_1_0*b_1_1+c_2_2, a 2-Cochain in H^*(DihedralGroup(8); GF(2)), b_1_1, a 1-Cochain in H^*(DihedralGroup(8); GF(2)), b_1_1+b_1_0, a 1-Cochain in H^*(DihedralGroup(8); GF(2))]

    One can compose induced maps by multiplication respectively by applying one map to the other.
    Here, we test that the composition of the induced map and its inverse is the identity::

        sage: [X == phi_star_inv(phi_star(X)) == (phi_star_inv*phi_star)(X) == phi_star_inv(phi_star)(X) for X in H2.gens()]
        [True, True, True, True]
        sage: [X == phi_star(phi_star_inv(X)) == (phi_star*phi_star_inv)(X) == phi_star(phi_star_inv)(X) for X in H1.gens()]
        [True, True, True, True]

    It is possible to convert an induced homomorphism into a map of quotient rings in the
    Singular interface. This allows for working with cohomology ring elements of very high
    degrees. However, it is always needed to take care of the ``basering`` in Singular::

        sage: S_phi_star = singular(phi_star)
        sage: singular(H2).set_ring()
        sage: I = singular.ideal(['c_2_2^50','b_1_0^50','b_1_1^50'])
        sage: singular(H1).set_ring()
        sage: S_phi_star(I)
        b_1_0^100+c_2_2^2*b_1_0^96+c_2_2^16*b_1_0^68+c_2_2^18*b_1_0^64+c_2_2^32*b_1_0^36+c_2_2^34*b_1_0^32+c_2_2^48*b_1_0^4+c_2_2^50,
        b_1_1^50+b_1_0^2*b_1_1^48+b_1_0^16*b_1_1^34+b_1_0^18*b_1_1^32+b_1_0^32*b_1_1^18+b_1_0^34*b_1_1^16+b_1_0^48*b_1_1^2+b_1_0^50,
        b_1_0^50
        
    Note that Singular does not do automatic reduction in quotient rings. So, eventually we
    do the reductions explicitly, in two ways, with the same result::

        sage: S_phi_star(I).reduce(singular('ideal(0)'))
        b_1_0^100+c_2_2^2*b_1_0^96+c_2_2^16*b_1_0^68+c_2_2^18*b_1_0^64+c_2_2^32*b_1_0^36+c_2_2^34*b_1_0^32+c_2_2^48*b_1_0^4+c_2_2^50,
        b_1_1^50+b_1_0^50,
        b_1_0^50
        sage: singular(H2).set_ring()
        sage: I = I.reduce(singular('ideal(0)'))
        sage: singular(H1).set_ring()
        sage: S_phi_star(I)
        b_1_0^100+c_2_2^2*b_1_0^96+c_2_2^16*b_1_0^68+c_2_2^18*b_1_0^64+c_2_2^32*b_1_0^36+c_2_2^34*b_1_0^32+c_2_2^48*b_1_0^4+c_2_2^50,
        b_1_1^50+b_1_0^2*b_1_1^48+b_1_0^16*b_1_1^34+b_1_0^18*b_1_1^32+b_1_0^32*b_1_1^18+b_1_0^34*b_1_1^16+b_1_0^48*b_1_1^2+b_1_0^50,
        b_1_0*b_1_1^49+b_1_0^2*b_1_1^48+b_1_0^17*b_1_1^33+b_1_0^18*b_1_1^32+b_1_0^33*b_1_1^17+b_1_0^34*b_1_1^16+b_1_0^49*b_1_1+b_1_0^50
        sage: S_phi_star(I).reduce(singular('ideal(0)'))
        b_1_0^100+c_2_2^2*b_1_0^96+c_2_2^16*b_1_0^68+c_2_2^18*b_1_0^64+c_2_2^32*b_1_0^36+c_2_2^34*b_1_0^32+c_2_2^48*b_1_0^4+c_2_2^50,
        b_1_1^50+b_1_0^50,
        b_1_0^50

    """
    
#################
## init, dealloc(automatic in this case), repr
    def __init__(self, parent, MTX m, MTX M, int d, name = None):
        RingHomomorphism.__init__(self, parent)
        if not isinstance(parent, CohomologyHomset):
            raise TypeError, "Parent must be a <CohomologyHomset>"
        # self will be induced by a chain map 
        # Here, Src denotes the source of the chain map, hence,
        # it corresponds to the codomain of the induced map.
        Src = parent._codomain
        Tgt = parent._domain # sic! We have *co*homology
        ## check admissibility of input
        cdef RESL S = Src.Resl
        cdef RESL T = Tgt.Resl
        if ((M.base()!=S.coef()) or (m.base()!=S.coef())):
            raise TypeError, "Matrices must be defined over GF(%d)"%S.coef()
        if (M.ncols() <> T.G_Alg.Data.nontips):
            raise TypeError, "Second matrix must have |G|=%d colums"%T.G_Alg.Data.nontips
        if (m.ncols() <> T.G_Alg.Data.nontips):
            raise TypeError, "First matrix must have |G|=%d colums"%T.G_Alg.Data.nontips
        if (m.nrows() <> S.G_Alg.Data.nontips):
            raise TypeError, "First matrix must have |H|=%d rows"%S.G_Alg.Data.nontips
        if (d>0):
            if (M.nrows() <> S.rank(0)*T.rank(d)):
                raise TypeError, "Second matrix must have %d rows"%(S.rank(0)*T.rank(d))
        else:
            if (M.nrows() <> T.rank(0)*S.rank(-d)):
                raise TypeError, "Second matrix must have %d rows"%(T.rank(0)*S.rank(-d))
        # self.HSrc = Src # this would be an unnecessary reference 
        # self.HTgt = Tgt # to an object that is already determined 
                          # by parent._(co)domain
        self.Src  = S
        self.Tgt  = T
        self.Deg  = d
        self.GMap = m
        self.Data = [M]
        self._name = name

    def __reduce__(self):
        """
        Return data used for pickling/unpickling self

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star is loads(dumps(phi_star)) # indirect doc test
            True

        """
        return chmap_unpickle, (self._domain,self._codomain,self.GMap,self.Data,self.Deg,self._name)

    def __repr__(self):
        """
        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star
            Induced homomorphism of degree 0 from H^*(DihedralGroup(8); GF(2)) to H^*(D8; GF(2))
            sage: phi_star.set_name('phi^*')
            sage: phi_star
            phi^*

        """
        if self._name:
            return self._name
        return "Induced homomorphism of degree %d from %s to %s"%(self.Deg,self._parent._domain.__repr__(), self._parent._codomain.__repr__())

    def __str__(self):
        """
        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: print phi_star
            Induced homomorphism of degree 0 from H^*(DihedralGroup(8); GF(2)) to H^*(D8; GF(2))
            defined by
            [1 0 0 0 0 0 0 0]
            sage: phi_star.set_name('phi^*')
            sage: print phi_star
            phi^*
            defined by
            [1 0 0 0 0 0 0 0]

        """
        OUT = self.__repr__()
        OUT = OUT + "\n"+"defined by\n"+self.__getitem__(0).__str__()
        return OUT

    def _singular_(self, S):
        """
        Represent self as a map between the Singular interface versions of domain and codomain

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: f = singular(phi_star)
            sage: f
            sage...[1]=b_1_0^2+c_2_2
            sage...[2]=b_1_1+b_1_0
            sage...[3]=b_1_0
            sage: singular(H2).set_ring()
            sage: I = singular.ideal(['b_1_1^2+c_2_2','c_2_2^20'])
            sage: singular(H1).set_ring()
            sage: f(I).reduce(singular.ideal(0))
            c_2_2,
            b_1_0^40+c_2_2^4*b_1_0^32+c_2_2^16*b_1_0^8+c_2_2^20

        """
        C = self.codomain()
        D = self.domain()
        RD = D._singular_(S)
        RC = C._singular_(S)
        Images = [C.cochain_to_polynomial(self(X)).name() for X in D.Gen]
        RC.set_ring()
        from sage.interfaces.singular import SingularElement
        return SingularElement(S, 'map', ','.join([RD.name()]+Images))

    def exportData(self, f):
        """
        Store data describing the induced map into files

        INPUT:

        ``f`` - a string, providing the path and the beginning of 
        the name of the files into which the terms of ``self`` shall be saved

        OUTPUT:

        The matrix ``self[i]`` is saved to the file ``f+repr(i)``.

        NOTE:

        This method is internally used, in order to save memory by disposing
        of data that are currently not needed. So, a user would normally not
        directly invoke the method.

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: print phi_star(H2.1)
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 0 1]
            sage: print phi_star.__getitem_name__(1)
            [1 0 0 0 0 0 0 0]
            [1 1 1 1 1 1 0 0]
            [1 0 0 0 0 0 0 0]
            [0 0 1 1 0 0 0 0]
            sage: f = tmp_filename()
            sage: phi_star.exportData(f)
            sage: phi_star.__getitem_name__(1) == f+'1'
            True

        """
        if not isinstance(f,basestring):
            raise TypeError, "File name stem expected"
        cdef int i
        cdef int M = len(self.Data)
        for i from 1 <= i < M:
            if isinstance(self.Data[i],basestring):
                if self.Data[i] != f+str(i):
                    if OPTION.opts['prot']:
                        print (self.Tgt.indent*'::: ')+'> > > export Chain Map'
                    #save(load(self.Data[i]), f+str(i))
                    save(self[i], f+str(i))
                    self.Data[i] = f+str(i)
            else:
                if OPTION.opts['prot']:
                    print (self.Tgt.indent*'::: ')+'> > > export Chain Map'
                save(self.Data[i],f+str(i))
                self.Data[i] = f+str(i)

#################
## structural parts
    def src(self):
        """
        Return the source of the underlying chain map (type :class:`~pGroupCohomology.resolution.RESL`)

        NOTE:

        Since cohomology is a contravariant functor, the output of ``src()`` is
        the resolution of the *codomain*.

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.src()
            Resolution:
            0 <- GF(2) <- GF(2)(8gp3) <- rank 2 <- rank 3 <- rank 4
            sage: phi_star.src() is phi_star.codomain().resolution()
            True
            
        """
        return self.Src

    def tgt(self):
        """
        Return the codomain of the underlying chain map (type :class:`~pGroupCohomology.resolution.RESL`)

        NOTE:

        Since cohomology is a contravariant functor, the output of ``tgt()`` is
        the resolution of the *domain*.

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.tgt()
            Resolution:
            0 <- GF(2) <- GF(2)(DihedralGroup_8_) <- rank 2 <- rank 3 <- rank 4
            sage: phi_star.tgt() is phi_star.domain().resolution()
            True
            
        """
        return self.Tgt

    def G_map(self):
        """
        Return the underlying homomorphism of group algebras (given by a :class:`~pGroupCohomology.mtx.MTX` matrix)

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: G1 = gap('SmallGroup(8,3)')
            sage: G2 = gap('DihedralGroup(8)')
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)')
            sage: H1.make()
            sage: H2.make()

        In order to obtain reproducible doc tests, we switch to permutation
        groups that are, from the perspective of our programs, equivalent to
        ``G1`` and ``G2``, and provide an explicit group isomorphism::

            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: H1.group()==phi.Source()
            True
            sage: H2.group()==phi.Range()
            True
            sage: phi.IsInjective()
            true
            sage: phi.IsSurjective()
            true
            sage: phi_star = H2.hom(phi,H1)
            sage: print phi_star.G_map()
            [1 0 0 0 0 0 0 0]
            [0 1 1 1 1 1 1 1]
            [0 1 0 0 1 1 0 0]
            [0 0 0 1 0 0 1 1]
            [0 0 0 1 1 0 0 1]
            [0 0 0 0 0 1 1 0]
            [0 0 0 0 0 1 0 1]
            [0 0 0 0 0 0 0 1]

        """
        return self.GMap
    
    def deg(self):
        """
        Return the degree of self

        NOTE:

        An induced homomorphism always is of degree zero. We provide this method since
        :class:`~pGroupCohomology.cochain.ChMap` in principle also works for more
        general maps so that the image has a different degree from the source.

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.deg()
            0

        """
        return self.Deg
    
    def knownDeg(self):
        """
        Return the degree out to which self was constructed

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()

        In order to obtain reproducible doc tests, we switch to permutation
        groups that are, from the perspective of our programs, equivalent to
        ``G1`` and ``G2``, and provide an explicit group isomorphism::

            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: H1.group()==phi.Source()
            True
            sage: H2.group()==phi.Range()
            True
            sage: phi.IsInjective()
            true
            sage: phi.IsSurjective()
            true
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.knownDeg()
            0
            sage: print phi_star(H2.1)
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 0 1]
            sage: phi_star.knownDeg()
            2

        """
        return len(self.Data)-1
    
    def name(self):
        """
        Return the name of self

        NOTE:

        Since induced homomorphisms are cached, there can not be two equal induced
        homomorphisms with a different name.

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::
        
            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi.SetName('"phi"')
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.name()
            'phi^*'

        """
        return self._name

    def set_name(self, s):
        """
        Give self a name

        NOTE:

        Since induced homomorphisms are cached, there can not be two equal induced
        homomorphisms with a different name. If the :meth:`~pGroupCohomology.cohomology.COCH.hom`
        method is called, the name will be reset.

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi.SetName('"phi"')
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.name()
            'phi^*'
            sage: phi_star.set_name('foobar')
            sage: phi_star.name()
            'foobar'
            sage: H2.hom(phi,H1).name()
            'phi^*'
            sage: phi_star.name()
            'phi^*'

        """
        if not isinstance(s, basestring):
            raise TypeError, "Name of an induced homomorphism must be a string"
        self._name = s
    
    def __getitem_name__(self,key):
        """
        Return n-th lift of C either as file name or MTX

        INPUT:

        n - an integer
        
        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: print phi_star(H2.1)
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 0 1]
            sage: print phi_star.__getitem_name__(1)
            [1 0 0 0 0 0 0 0]
            [1 1 1 1 1 1 0 0]
            [1 0 0 0 0 0 0 0]
            [0 0 1 1 0 0 0 0]
            sage: f = tmp_filename()
            sage: phi_star.exportData(f)
            sage: phi_star.__getitem_name__(1) == f+'1'
            True

        """
        return self.Data[key]

    def __getitem__(self,key):
        """
        Return n-th lift of chainmap C (type MTX)

        INPUT:

        n - integer

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: H1.make()
            sage: G = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G, GroupName = 'DihedralGroup(8)')
            sage: H2.make()
            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: phi_star = H2.hom(phi,H1)
            sage: print phi_star(H2.1)
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 0 1]
            sage: print phi_star[1] # indirect doc test
            [1 0 0 0 0 0 0 0]
            [1 1 1 1 1 1 0 0]
            [1 0 0 0 0 0 0 0]
            [0 0 1 1 0 0 0 0]
            sage: f = tmp_filename()
            sage: phi_star.exportData(f)
            sage: phi_star.__getitem_name__(1) == f+'1'
            True
            sage: print phi_star[1] # indirect doc test
            [1 0 0 0 0 0 0 0]
            [1 1 1 1 1 1 0 0]
            [1 0 0 0 0 0 0 0]
            [0 0 1 1 0 0 0 0]

        """
        if isinstance(self.Data[key],basestring):
            try:
                return load(self.Data[key])
            except IOError:
                # Something went wrong, probably when moving data.
                # Infer the correct root from the root of the domain!
                # This is expected to happen when being called for restriction maps
                # from inside COHO.reconstructSubgroups(). 
                root = self.domain().root
                if OPTION.opts['prot']:
                    print "Can not find data at", self.Data[key]
                if OPTION.opts['prot']:
                    print "Trying new location", root + '/'.join(self.Data[key].split('/')[-3:])
                try:
                    return load(root + '/'.join(self.Data[key].split('/')[-3:]))
                except IOError:
                    raise IOError, "Files on disk were moved, can not find data"
        else:
            return self.Data[key]

    def __setitem__(self,key,object L):
        "C[d]=(M(d),M(d+1),...): Set the lifts to M(d),M(d+1),... (type MTX or file name) starting in term d"
        if (isinstance(key,Integer)) or (isinstance(key,int)):
            if (key>=0) and (key<=len(self.Data)):
                self.Data[key:] = L
            else:
                raise IndexError, "index out of range"
        else:
            raise TypeError, "key must be an integer"
        

#########################
## ==, <, >
    def __richcmp__(ChMap self, ChMap S,x):
        """
M==N <=> M and N are the same chain maps,
         i.e., defined over the same source and target
         rings, have the same degree shift, and
         are described by the same matrix in the lowest degree.
        """
        # < 0, <= 1, == 2, != 3, > 4, >= 5
        if (x==0) or (x==4):
            return None
        if (x==1) or (x==2) or (x==5):
            if self is S:
                return True
            if (self.Tgt==S.Tgt) and (self.Src==S.Src) and (self.Deg==S.Deg) and (self.__getitem__(0)==S.__getitem__(0)) and (self.GMap==S.GMap):
                return True
            else:
                return False
        ## otherwise: x==3
        if (self.Tgt==S.Tgt) and (self.Src==S.Src) and (self.Deg==S.Deg) and (self.__getitem__(0)==S.__getitem__(0)) and (self.GMap==S.GMap):
            return False
        else:
            return True
        
##################
## Arithmetic -- Application to (co)chains, composition of maps, inverse maps...

    def apply_to_chain(ChMap self, int d, MTX C):
        """
        Apply the underlying chain map of self to a `d`-chain

        INPUT:

        - ``d``, the degree of the chain
        - ``C``, a `(r\\times |G|)` :class:`~pGroupCohomology.mtx.MTX` matrix, where `r` is
          the rank of the `d`-th term of the resolution, and `|G|`  is the group order.

        EXAMPLES:

        The example produces files. For safety reasons, we choose files
        in a temporary directory; it will be removed as soon as Sage is quit::

            sage: from pGroupCohomology import CohomologyRing
            sage: from pGroupCohomology.mtx import MTX
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: f = H.2.right_multiplication()
            sage: print f.apply_to_chain(1, MTX(2,[[0,1,0,1,0,1,0,1],[1,0,1,0,1,0,1,0]]))
            [0 1 0 1 0 1 0 1]

        """
        if d<0:
            raise ValueError, "There are no chains of negative degree"
        if d < -self.Deg:
            raise ValueError, "The map can't be applied to a chain of degree less than %d"%(-self.Deg)
        if self.Deg >= 0:
            while len(self.Data)<d:
                self.lift()
            selfMTX = self[d]
        else:
            while len(self.Data)<(d+self.Deg):
                self.lift()
            selfMTX = self[d+self.Deg]
        cdef RESL S = self.Src
        cdef int RK = S.Data.projrank[d]
        cdef long Snontips = S.G_Alg.Data.nontips
        e = d + self.Deg  # in this degree lives the image
        cdef RESL T = self.Tgt
        cdef int rk = T.Data.projrank[e]
        cdef long Tnontips = T.G_Alg.Data.nontips
        # Test whether C represents a d-chain in S
        if C.ncols()!=Snontips:
            raise ValueError, "A chain must be represented by a matrix with %d columns"%Snontips
        if C.nrows()!=RK:
            raise ValueError, "A %d-chain must be represented by a matrix with %d rows"%(d,RK)
        cdef MTX OUT = MTX(T.G_Alg.Data.p, rk, Tnontips, mutable=False)
        cdef int i
        for i from 0 <= i < RK:
            T_rightaction = T.G_Alg.r_action(C[i]*self.GMap)
            OUT += selfMTX[i*rk:(i+1)*rk] * T_rightaction
        return OUT

    def __mul__(ChMap self, x):
        """
        CM*Co: CM of type <ChMap>, Co of type <COCH> or <ChMap>

        If Co is a cochain (type <COCH>), CM*Co is the image of Co under the induced map of the chain map CM 

        If Co is a chain map (type <ChMap>), CM*Co is the composition of Co followed by CM.

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()
            sage: phi = G1.IsomorphismGroups(G2)
            sage: phi_star = H2.hom(phi,H1)
            sage: res = H1.restriction_maps()[2][1]
            sage: res_phi_star = res*phi_star
            sage: [res_phi_star(X) == (res*phi_star)(X) == res(phi_star)(X) for X in H2.gens()]
            [True, True, True, True] 

        """
        cdef ChMap OUT
        cdef MTX OUT_M,xMTX,selfMTX
        cdef COCH OUT_C
        cdef int i,j, t_rk,s_rk,cf,xLj, i_trk
        cdef long X
        cdef list L=[]
        cdef int d = len(self.Data)
        cdef int RK, Rk, rk
        cdef int NT, Nt, nt
        if isinstance(x, ChMap):
            if self.domain()!=x.codomain():
                raise ValueError, "Composition of induced maps is impossible - domain and codomain don't match"
            # We have a contravariant functor. Hence, composing self after x means to 
            # first apply the underlying chain map of self, followed by the underlying chain map of x
            # Notational convention: Double uppercase means stuff in the source of the underlying chain map
            # of self; upper-lowercase means stuff in the middle resolution; double lowercase denotes the
            # target resolution, the image of the underlying chain map of x. base_deg is the degree in the middle.
            NT = self.G_map().nrows()
            Nt = self.G_map().ncols() # == x.G_map().nrows()
            nt = x.G_map().ncols()
            # Will have a map of degree self.Deg+x.deg(), first self then x.
            # We must distinguish five cases:
            if (x.deg() <= 0) and (self.deg() <= 0):
                while (self.knownDeg() <= -x.deg()):
                    self.lift()
                selfMTX = self[-x.deg()]
                xMTX = x[0]
                rk = 1
                Rk = xMTX.nrows()
                RK = selfMTX.nrows()/Rk
                base_deg = -x.deg()
            if (x.deg() > 0) and (self.deg()<0):
                #xMTX = x[0]
                #selfMTX = self[0]
                raise ValueError, "We can't compose a map of negative degree with a map of positive degree"
            if (self.Deg > 0) and (x.deg()>0):
                while(x.knownDeg() <= self.deg()):
                    x.lift()
                selfMTX = self[0]
                xMTX = x[self.deg()]
                RK = 1
                Rk = selfMTX.nrows()
                rk = xMTX.nrows()/Rk
                base_deg = self.Deg
            if (x.deg() < 0) and (self.deg() > 0):
                if self.Deg+x.deg() < 0:
                    while (self.knownDeg() <= -(self.Deg+x.deg())):
                        self.lift() 
                    selfMTX = self[-(self.Deg+x.deg())]
                    xMTX = x[0]
                    rk = 1
                    Rk = xMTX.nrows()
                    RK = selfMTX.nrows()/Rk
                    base_deg = -x.deg()
                else:
                    while (x.knownDeg() <= (self.Deg+x.deg())):
                        x.lift()
                    selfMTX = self[0]
                    xMTX = x[self.Deg+x.deg()]
                    RK = 1
                    Rk = xMTX.nrows()
                    rk = selfMTX.nrows()/Rk
                    base_deg = self.deg()
            OUT_M = MTX(self.Tgt.G_Alg.Data.p, RK*rk, nt)
            for i from 0 <= i < RK:
                OUT_M[i*rk] = x.apply_to_chain(base_deg, selfMTX[i*Rk:(i+1)*Rk])
            OUT = x.domain().hom(self.G_map()*x.G_map(), self.codomain(),OUT_M,self.Deg+x.deg())
            if self.name() is not None and x.name() is not None:
                OUT.set_name(self.name()+'('+x.name()+')')
            return OUT
        elif isinstance(x, COCH):
            if (self.domain() is not x.parent()):
                raise TypeError, "Cochain must belong to the domain of the induced map"
            if (x.deg()-self.Deg < 0):
                raise IndexError, "Degree of second factor (type <COCH>) must be at least %d"%(self.Deg)
            if self.Deg > 0:
                base = x.deg() - self.Deg
            else:
                base = x.deg()
            while (base >= len(self.Data)):
                self.lift()
            selfMTX = self.__getitem__(base)
            xMTX = x.MTX()
            s_rk = self.Src.rank(x.deg()-self.Deg)
            t_rk = self.Tgt.rank(x.deg())
            if (selfMTX.Data.nor<>s_rk*t_rk):
                raise ArithmeticError, "Theoretical Error"
            if (xMTX.Data.noc<>t_rk):
                raise ArithmeticError, "Theoretical Error"
            OUT_M = MTX(self.Src.coef(), 1,s_rk)
            for i from 0 <= i < s_rk:
                cf = 0
                i_trk = i*t_rk
                for j from 0 <= j < t_rk:
                    xLj = xMTX[0,j]
                    if xLj:
                        cf = cf + xLj*int(selfMTX[i_trk,0])
                    i_trk+=1
                OUT_M[0,i]=cf
            OUT_M.set_immutable()
            if self._name is not None:
                return COCH(self._parent._codomain,x.deg()-self.Deg, self._name+'('+x.name()+')', OUT_M)
            return COCH(self._parent._codomain,x.deg()-self.Deg, '('+x.name()+")_", OUT_M)
        raise NotImplementedError, "Multiplication of ChMap with %s not implemented"%(type(x))

    def __invert__(self):
        """

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2,GroupName='Dihedral')
            sage: phi12 = G1.IsomorphismGroups(G2)
            sage: phi12.SetName('"phi12"')
            sage: Phi21 = H2.hom(phi12,H1)
            sage: phi21 = phi12.InverseGeneralMapping()
            sage: phi21.SetName('"phi21"')
            sage: Phi12 = H1.hom(phi21,H2)
            sage: Phi12
            phi21^*
            sage: Phi12 is Phi21^(-1)
            True
            sage: Phi12
            (phi12^*)^(-1)

        """
        try:
            M = (self.G_map())**(-1)
        except ArithmeticError:
            raise ArithmeticError, "Induced homomorphism is not invertible"
        OUT = self.codomain().hom(M,self.domain())
        if self.name() is not None:
            OUT.set_name('('+self.name()+')^(-1)')
        return OUT
        
    def __pow__(self, n, z):
        """
        TESTS::


        """
        if n<0:
            OUT = self.__invert__()
            n = -n
            i = True
        else:
            OUT = self
            i = False
        if n!=1:
            if not (OUT.domain() is OUT.codomain()):
                raise ArithmeticError, "Induced homomorphism can't be composed with itself"
            OUT = OUT.domain().hom((OUT.G_map())**n, OUT.codomain())
            if self.name() is not None:
                if i:
                    OUT.set_name('('+self.name()+')^(-%d)'%n)
                else:
                    OUT.set_name('('+self.name()+')^%d'%n)
        return OUT

    def __call__(self, X):
        """
        C(X): If X is a tuple (d,n), return the image of the n-th standard cochain of degree d under C. Otherwise, return C*X

        TESTS::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()

        In order to obtain reproducible doc tests, we switch to permutation
        groups that are, from the perspective of our programs, equivalent to
        ``G1`` and ``G2``, and provide an explicit group isomorphism::

            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: H1.group()==phi.Source()
            True
            sage: H2.group()==phi.Range()
            True
            sage: phi.IsInjective()
            true
            sage: phi.IsSurjective()
            true
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.lift()
            sage: phi_star.lift()
            sage: print phi_star((2,2))
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 0 1]
            sage: print phi_star(H2.1)
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 0 1]

        """
        cdef int j, t_rk,s_rk
        cdef MTX tmpM
        cdef list L=[]
        try: # is X just a scalar?
            OUT = self.codomain().base_ring()(X)
            if not isinstance(OUT,tuple):
                return OUT
        except TypeError:
            pass
        if isinstance(X,tuple) or isinstance(X,list):
            if not((isinstance(X[0],int) or isinstance(X[0],Integer)) and (isinstance(X[1],int) or isinstance(X[1],Integer))):
                raise TypeError, "pair of integers expected"
            if (X[0]-self.Deg < 0):
                raise IndexError, "Degree of standard cochain must be at least %d"%(self.Deg)
            s_rk = self.Src.rank(X[0]-self.Deg)
            t_rk = self.Tgt.rank(X[0])
            if (X[1] >= t_rk):
                raise IndexError, "Index of standard cochain must not exceed %d"%(t_rk-1)
            tmpM = self.__getitem__(X[0])
            for j from 0 <= j < s_rk:
                L.append(tmpM[j*t_rk+X[1],0])
            return COCH(self._parent._codomain, X[0]-self.Deg, "r_(%d,%d)"%(X[0],X[1]), L)
        else:
            return self.__mul__(X)
    
    def lift(ChMap self):
        """
        Compute the next unknown term of self

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()

        In order to obtain reproducible doc tests, we switch to permutation
        groups that are, from the perspective of our programs, equivalent to
        ``G1`` and ``G2``, and provide an explicit group isomorphism::

            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,7)(2,8)(3,5)(4,6), (1,6)(2,5)(3,7)(4,8) ] )')
            sage: H1.group()==phi.Source()
            True
            sage: H2.group()==phi.Range()
            True
            sage: phi.IsInjective()
            true
            sage: phi.IsSurjective()
            true
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star.lift()
            sage: phi_star.knownDeg()
            1
            sage: phi_star[1]
            (4 x 8) MTX matrix over GF(2)
            sage: print phi_star[1]
            [1 0 0 0 0 0 0 0]
            [1 1 1 1 1 1 0 0]
            [1 0 0 0 0 0 0 0]
            [0 0 1 1 0 0 0 0]

        """
        ## Next unknown term is Src[len(Data)]->Tgt[len(Data)+Deg] (if Deg>0)
        if (self.Deg > 0):
            SrcDeg = len(self.Data)
            TgtDeg = len(self.Data) + self.Deg
        else:
            TgtDeg = len(self.Data)
            SrcDeg = len(self.Data) - self.Deg
        while (TgtDeg>=self.Tgt.deg()):
            self.Tgt.nextDiff()
        while (SrcDeg>self.Src.deg()):
            self.Src.nextDiff()
        ## AIM: construct matrix describing map from Src[SrcDeg] -> Tgt[TgtDeg]
        ## 1. compose with the next differential
        if OPTION.opts['prot']:
            print ((self.Src.indent)*'::: ')+'> lift Chain Map in the source resolution'
        ct=cputime()
        wt=walltime()
        cdef MTX M1
        cdef MTX M2
        cdef int i,j,k
        M1 = self.Src[SrcDeg]*self.GMap # TgtNontips columns
        M2 = self.__getitem__(-1) # self.Data[-1]              # TgtNontips columns
        cdef MTX Compos
        cdef long SrcNontips = self.Src.G_Alg.Data.nontips
        cdef long TgtNontips = self.Tgt.G_Alg.Data.nontips
        zsetfield(self.Src.coef())
        zsetlen(TgtNontips)
        Compos = MTX(self.Src.G_Alg.Data.p, self.Src.Data.projrank[SrcDeg]*self.Tgt.Data.projrank[TgtDeg-1],TgtNontips,mutable=False)
        cdef MTX tmp
        tmp = MTX(self.Src.G_Alg.Data.p, 1, TgtNontips,mutable=False)
        cdef matrix_t *L
        cdef int RK = self.Src.Data.projrank[SrcDeg]
        cdef int Rk = self.Src.Data.projrank[SrcDeg-1]
        cdef int rk = self.Tgt.Data.projrank[TgtDeg-1]
        for i from 0 <= i < rk:
            for j from 0 <= j < Rk:
                L = leftActionMatrix(self.Tgt.G_Alg.Data, ptrPlus(M2.Data.d,j*rk+i))
                for k from 0 <= k < RK:
                    if not (zmaprow(ptrPlus(M1.Data.d,k*Rk+j), L.d, TgtNontips, tmp.Data.d)):
                        raise ArithmeticError, "multiplication failed"
                    if not (zaddrow(ptrPlus(Compos.Data.d,k*rk+i), tmp.Data.d)):
                        raise ArithmeticError, "addition of rows failed"
                matfree(L)

        if OPTION.opts['prot']:
            print ((self.Tgt.indent)*'::: ')+'> lift in the target resolution to degree %d'%(TgtDeg)
        rk   = self.Tgt.Data.projrank[TgtDeg]
        cdef rk_1 = self.Tgt.Data.projrank[TgtDeg-1]
        cdef long fl = self.Src.G_Alg.Data.p
        cdef long nt = TgtNontips
        d = TgtDeg # will lift to that degree in the target resolution
        cdef MTX OUT
        OUT = MTX(fl, RK*rk, nt)
        cdef MTX TMP, DUMMY
        cdef list Piv, Z
        cdef dict Autolift = self.Tgt.Autolift.get(d,{})
        if Autolift:
            if OPTION.opts['prot']:
                print ((self.Tgt.indent)*'::: ')+'> > use autolift method'
            Piv = Autolift['Piv']
            ##########################
            # Lift each "long row"
            for i from 0 <= i < RK:
                # print 'lift long row ',i
                Z = Compos._rowlist_(i*rk_1, (i+1)*rk_1-1)
                TMP = MTX(fl, rk, nt)
                for J in Piv:
                    if Z[J]:
                        DUMMY = Autolift[J][Z[J]]
                        if not(matadd(TMP.Data, DUMMY.Data)):
                            raise ArithmeticError, "Matrix addition failed"
                #for j from 0 <= j < rk:
                    #OUT[i*rk+j] = TMP._rowlist_(j)
                OUT[i*rk] = TMP
            OUT.set_immutable()
            self.Data.append(OUT)
            if OPTION.opts['timing']:
                ct=cputime(ct)
                wt=walltime(wt)
                print "Time for lifting chain map:"
                print "   CPU:  %.2f\n   Wall: %.2f"%(ct,wt)
            # print 'accomplished'
            #if OPTION.opts['prot']:
            #    print ""
            return

        ## otherwise: use Urbild GB        
        if OPTION.opts['prot']:
            print ((self.Tgt.indent)*'::: ')+'> > use Urbild Groebner basis'
        #cdef nRgs_t *nRgs
        self.Tgt.load_ugb(d)
        if (self.Tgt.nRgs.ngs.r!=rk_1) or (self.Tgt.nRgs.ngs.s != rk):
            print "Tgt.nRgs.r =", self.Tgt.nRgs.ngs.r
            print rk_1
            print "Tgt.nRgs.s =", self.Tgt.nRgs.ngs.s
            print rk
            raise ArithmeticError, "Theoretical error"
        innerPreimages(self.Tgt.nRgs, Compos.Data.d, RK, self.Tgt.G_Alg.Data, OUT.Data.d)
        # freeNRgs(nRgs)
        self.Data.append(OUT)
        if OPTION.opts['timing']:
            ct=cputime(ct)
            wt=walltime(wt)
            print "Time for lifting chain map:"
            print "   CPU:  %.2f\n   Wall: %.2f"%(ct,wt)

