#*****************************************************************************
#
#    Copyright (C) 2009 Simon A. King <simon.king@uni-jena.de>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 2 or later (at your choice)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext
from sage.all import SAGE_ROOT
from sage.interfaces.singular import singular
from sage.all import srange
from sage.rings.integer import Integer


import shutil
import os
import subprocess

# Copy our singular libraries to the right place

try:
    SingularLib=os.environ['SINGULARPATH']
except KeyError:
    SingularLib=singular.eval('system("SingularLib")').split(':')[0]
shutil.copy('pGroupCohomology/dickson.lib',SingularLib)
shutil.copy('pGroupCohomology/filterregular.lib',SingularLib)

PrimePowers = [n for n in srange(2,256) if Integer(n).is_prime_power()]
os.chdir('lib/')
for n in PrimePowers:
    subprocess.call(['maketab','%d'%n])
os.chdir('../')

mtx_tables = ['lib/'+s for s in os.listdir('lib/') if s.endswith('zzz')]

setup(
  name = "pGroupCohomology",
  version = "1.0.1",
  author = "Simon A. King, David J. Green",
  author_email = "simon.king@uni-jena.de, david.green@uni-jena.de",
  maintainer = "Simon A. King", 
  maintainer_email = "simon.king@uni-jena.de",
  url = "http://sage.math.washington.edu/home/SimonKing/Cohomology/",
  description = "Modular Cohomology Rings of Groups of Prime Power Order",
  packages=["pGroupCohomology"],
  data_files=[("pGroupCohomology",
              ["pGroupCohomology/GroupNames.sobj",
               "pGroupCohomology/GapMaxels",
               "pGroupCohomology/GapMB",
               "pGroupCohomology/GapSgs"]),
              ("pGroupCohomology/db", ['db/'+X for X in os.listdir('db/')]+["db/LICENCE"]),
              ("pGroupCohomology/mtx_tables", mtx_tables)],
  ext_modules=[ 
    Extension("pGroupCohomology.resolution",
              sources = ["pGroupCohomology/resolution.pyx",
                         "pGroupCohomology/c_sources/aufloesung.c",
                         "pGroupCohomology/c_sources/aufnahme.c",
                         "pGroupCohomology/c_sources/djgerr.c",
                         "pGroupCohomology/c_sources/fileplus.c",
                         "pGroupCohomology/c_sources/nBuchberger.c",
                         "pGroupCohomology/c_sources/pgroup.c",
                         "pGroupCohomology/c_sources/pincl.c",
                         "pGroupCohomology/c_sources/slice.c",
                         "pGroupCohomology/c_sources/urbild.c",
                         "pGroupCohomology/c_sources/uvr.c"],
              libraries = ["mtx"],
              include_dirs = [SAGE_ROOT+"/local/include/csage/", SAGE_ROOT+"/devel/sage/", "mtx2.2.4/src/", "pGroupCohomology/c_sources", "pGroupCohomology"]
              ),
    Extension("pGroupCohomology.cohomology",
              sources = ["pGroupCohomology/cohomology.pyx",
                         "pGroupCohomology/c_sources/aufloesung.c",
                         "pGroupCohomology/c_sources/aufnahme.c",
                         "pGroupCohomology/c_sources/djgerr.c",
                         "pGroupCohomology/c_sources/fileplus.c",
                         "pGroupCohomology/c_sources/nBuchberger.c",
                         "pGroupCohomology/c_sources/pgroup.c",
                         "pGroupCohomology/c_sources/pincl.c",
                         "pGroupCohomology/c_sources/slice.c",
                         "pGroupCohomology/c_sources/urbild.c",
                         "pGroupCohomology/c_sources/uvr.c"],
              libraries = ["mtx"],
              include_dirs = [SAGE_ROOT+"/local/include/csage/", SAGE_ROOT+"/devel/sage/", "mtx2.2.4/src/", "pGroupCohomology/c_sources", "pGroupCohomology"]
              ),
    Extension("pGroupCohomology.cochain",
              sources = ["pGroupCohomology/cochain.pyx",
                         "pGroupCohomology/c_sources/aufloesung.c",
                         "pGroupCohomology/c_sources/aufnahme.c",
                         "pGroupCohomology/c_sources/djgerr.c",
                         "pGroupCohomology/c_sources/fileplus.c",
                         "pGroupCohomology/c_sources/nBuchberger.c",
                         "pGroupCohomology/c_sources/pgroup.c",
                         "pGroupCohomology/c_sources/pincl.c",
                         "pGroupCohomology/c_sources/slice.c",
                         "pGroupCohomology/c_sources/urbild.c",
                         "pGroupCohomology/c_sources/uvr.c"],
              libraries = ["mtx"],
              include_dirs = [SAGE_ROOT+"/local/include/csage/", SAGE_ROOT+"/devel/sage/", "mtx2.2.4/src/", "pGroupCohomology/c_sources", "pGroupCohomology"]
              ),
    Extension("pGroupCohomology.mtx",
              sources = ["pGroupCohomology/mtx.pyx"],
              libraries = ["mtx"],
              include_dirs = ["mtx2.2.4/src/", "pGroupCohomology/c_sources", "pGroupCohomology"]
              ),
    Extension("pGroupCohomology.dickson", sources = ["pGroupCohomology/dickson.pyx"])
    ],
  cmdclass = {'build_ext': build_ext}
)

# now extract the tar files in the data base
data_base = SAGE_ROOT+'/local/pGroupCohomology/db/'
L = [data_base+X for X in os.listdir(data_base) if X.endswith('.tar.gz')]
import tarfile
print 'Extracting data base...'
for f in L:
    T = tarfile.open(f)
    T.extractall(path=data_base)
    T.close()
    os.unlink(f)
