#*****************************************************************************
#
#    Cohomology Rings of Finite p-Groups
#
#    Copyright (C) 2009 Simon A. King <simon.king@uni-jena.de>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 2 or later (at your choice)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
"""
Modular Cohomology Rings of Finite p-Groups

This module contains :class:`COHO`, that provides a framework for the computation of
the cohomology rings with coefficients in `\\mathbb F_p` for any finite `p`-group.
It is based on algorithms of David Green and Dave Benson. See :mod:`pGroupCohomology`
for an extensive introduction.

AUTHORS:

- Simon King (2009-06, Initial version)

"""

###########################################################
## Klassen aus anderen Teilmoduln
import os

# Sage generalities
import sage
import sage.all
from warnings import showwarning
from sage.all import cputime
from sage.all import walltime
from sage.all import save
from sage.all import dumps
from sage.all import loads
from sage.all import load
from sage.all import deepcopy
from sage.all import copy
from sage.all import add
from sage.all import mul
from sage.all import SAGE_DB
from sage.all import DOT_SAGE
from sage.all import SAGE_ROOT
from sage.interfaces.singular import singular
from sage.interfaces.gap import gap

# Sage rings etc.
from sage.matrix.constructor import Matrix
from sage.rings.ring import Ring
from sage.rings.finite_field import FiniteField as GF
from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
from sage.rings.integer_ring import ZZ
from sage.rings.rational_field import QQ
from sage.rings.infinity import infinity
from sage.rings.integer import Integer
#from sage.interfaces.singular import Singular
#singular = Singular(logfile='/home/king/Projekte/Cohom/CAS/fullBens/src/Test/singlog.txt') 

# pGroupCohomology Cython and Python types 
from pGroupCohomology.mtx cimport FEL, PTR, MTX
from pGroupCohomology.resolution cimport RESL, G_ALG
from pGroupCohomology.cochain cimport COCH
from pGroupCohomology.resolution import OPTION, resl_sparse_unpickle, makeGroupData, makeSpecialGroupData, Ordinals
from pGroupCohomology.dickson import DICKSON

## CACHE for CohomologyHomsets
from weakref import WeakValueDictionary

##########################################################################
##########################################################################
###                                                                    ###
###         Pickling                                                   ###
###                                                                    ###
##########################################################################
##########################################################################

def COHO_unpickle(GroupKey, StateFile):
    """
    Unpickling of a cohomology ring

    TESTS::

        sage: tmp_root = tmp_filename()
        sage: from pGroupCohomology import CohomologyRing
        sage: CohomologyRing.set_user_db(tmp_root)
        sage: H = CohomologyRing.user_db(8,3)
        sage: H.make()
        sage: H is loads(dumps(H)) # indirect doc test
        True
        
    """
    # Since the cache is indexed by (GroupKey, StateFile),
    # we first see if the cohomology ring is already there:
    from pGroupCohomology import _cache
    root = '/'.join(StateFile.split('/')[:-3])+'/'
    rootlength = len(root)
    while ('//' in root):
        root = root.replace('//','/')
    if root == '@user_db@/':
        StateFile = (OPTION.opts.get('@use_this_root@') or COHO.user_db) + StateFile[rootlength:]
        root = COHO.user_db
    elif root == '@public_db@/':
        StateFile = (OPTION.opts.get('@use_this_root@') or COHO.public_db) + StateFile[rootlength:]
        root = COHO.public_db
    ## Now, StateFile should point to a file providing the state of the cohomology ring.
    ## If it doesn't, then we need to find out *after* loading
    if OPTION.opts['prot']:
        print 'The state of this cohomology ring is expected to be provided at ' + StateFile + '.sobj'
    
    if not OPTION.opts.has_key('@newroot@'):
        if _cache.has_key((GroupKey, StateFile)):
            OUT = _cache[GroupKey, StateFile]
            # test if the singular subprocess is still running
            try:
                foobar = OUT.GenS.parent().eval(OUT.GenS.name())
            except RuntimeError:
                OUT = None
                _cache.__delitem__((GroupKey, StateFile))
            if OUT is not None:
                if OPTION.opts.has_key('@use_this_root@'):
                    OPTION.opts.__delitem__('@use_this_root@')
                return OUT
    OUT = COHO()
    # print 'Trying to find', StateFile
    try:
        ST = load(StateFile)
        # print 'Statefile loaded'
    except IOError:
        OUT.setprop('_need_new_root', True)
        if OPTION.opts.has_key('@use_this_root@'):
            OPTION.opts.__delitem__('@use_this_root@')
        return OUT
    # Apparently the root given by StateFile is correct. So, use it,
    # regardless what the contents of the file pointed at by StateFile say!
    OUT.__setstate__(ST, newroot=OPTION.opts.get('@use_this_root@') or root)
    _cache[GroupKey, StateFile] = OUT
    if OPTION.opts.has_key('@use_this_root@'):
        OPTION.opts.__delitem__('@use_this_root@')
    return OUT

#########################################################
## Setting Options
#########################################################

def COHO_option(*args):
    """
    Set/unset global options for cohomology computations 

    INPUT:

    A string, or a list of strings.

    OUTPUT:

    - If the input string is '', the current options are printed.
    - If the input is (or is a list containing) the name of a global option
      for cohomology computations (see below), the corresponding option is set.
    - If the input is (or is a list containing) 'no' plus the name of a global
      option for cohomology computations (see below), this option is unset.
   
    **Main options**

    - 'prot'  : protocol mode (default off)
    - 'timing': show timings (default off)
    - 'reload': if possible, reload differentials and subgroups from disk (default on)
    - 'save'  : automatically save data after each degree (default on)
    - 'sparse': export some data to the disk, when they are not needed.
      This reduces the memory consumption, so that certain huge examples become
      feasible. Moreover, it seems to improve speed as well, on average (default on).

    **Experimental options**

    - 'useMTX': let MeatAxe do the linear algebra (default on); shouldn't be changed.
    - 'liftlist': Not officially supported (default off)

    EXAMPLES::

        sage: from pGroupCohomology.cohomology import COHO
        sage: COHO.option('')
        reload: True
        liftlist: False
        useMTX: True
        prot: False
        sparse: True
        timing: False
        save: True
        sage: COHO.option('noreload','prot')
        sage: COHO.option('')
        reload: False
        liftlist: False
        useMTX: True
        prot: True
        sparse: True
        timing: False
        save: True

    In order to not break other doc tests, we switch back to the default options.
    ::

        sage: COHO.option('reload','noprot')

    """
    for s in args:
        if s=='':
            for X in OPTION.opts.items():
                print str(X[0])+': '+str(X[1])
            return
        if isinstance(s, basestring):
            OPTION(s)

####################################################################
## Auxiliary functions
####################################################################

def FilterDegreeType(dv, rt):
    """
    Compute the filter degree type

    INPUT:

    - d - list of degrees (integers) of a filter regular parameter system, and
    - r - the 'raw filter degree type' (list of integers) of these parameters.

    OUTPUT:

    The filter degree type, a list of integers

    EXAMPLES::

        sage: from pGroupCohomology.cohomology import FilterDegreeType
        sage: d=[8,4,6,4]
        sage: r=[-1,4,7,14,18]
        sage: FilterDegreeType(d,r)
        [-1, -2, -3, -4, -4]
        sage: d=[8,4,6,3]
        sage: FilterDegreeType(d,r)
        [-1, -2, -3, -3, -3]

    """
    cdef int i,j
    if not (isinstance(dv,list) and isinstance(rt,list)):
        raise TypeError, "arguments must be lists of integers"
    if not (len(rt)==len(dv)+1):
        raise TypeError, "length of second arguments must be length of first argument plus one"
    ft = [-k for k in range(1,len(rt)+1)]
    for i from 0<=i<len(rt):
        if sum(dv[:i])+ft[i] < rt[i]:
            ft[i]=ft[i]+1
            i=i-1
            continue
        if (i>0) and (ft[i]>ft[i-1]):
            ft[i-1]=ft[i-1]+1
            i=i-2
            continue
        if (ft[i]<ft[i-1]-1):
            ft[i]=ft[i]+1
            i=i-1
    return ft

def MonomialHilbert(I):
    """
    Return the first (weighted) Hilbert function of a monomial ideal

    INPUT:

    ``I`` -- a monomial ideal or its name in singular.

    OUTPUT:

    A univariate polynomial -- the first Hilbert function of ``I``, 
    using the variable weights given by singular's base ring.

    EXAMPLES::

        sage: from pGroupCohomology.cohomology import MonomialHilbert
        sage: R = singular.ring(0,'(x,y,z)','dp')
        sage: I = singular.ideal(['x^2','y^2','z^2'])
        sage: MonomialHilbert(I)
        -t^6 + 3*t^4 - 3*t^2 + 1
        sage: MonomialHilbert(I.name())
        -t^6 + 3*t^4 - 3*t^2 + 1

    """
    if isinstance(I,basestring):
        S = singular
        n = I
    else:
        S = I._check_valid()
        n = I.name()
    S.eval('%s=sort(interred(%s),"dp")[1]'%(n,n)) # hence, the decomposable monomials last
    PR = PolynomialRing(QQ,'t')
    t = PR('t')
    if S.eval('%s[1]'%(n))=='0':
        return PR(1)
    if S.eval('%s[1]'%(n))=='1':
        return PR(0)
    if S.eval('sum(leadexp(%s[ncols(%s)]))'%(n,n))=='1': # Hence, the ideal is generated by some variables
        return mul([(1-t**Integer(S.eval('deg(%s[%d])'%(n,X)))) for X in range(1,Integer(S.eval('ncols(%s)'%(n)))+1)])
    j=1
    while (S.eval('leadexp(%s[ncols(%s)])[%d]'%(n,n,j))=='0'):
        j+=1
    I1 = S('%s+var(%d)'%(n,j))
    I2 = S('ideal(%s/var(%d))+%s'%(n,j,n))
    return MonomialHilbert(I1) + t**Integer(S.eval('deg(var(%d))'%(j)))*MonomialHilbert(I2)

def Mul(L,L0):
    """
    Compute the product of a list of elements, bracketed from the right.

    INPUT:

    - ``[L_1,...,L_n]``, a list of elements that can be multiplied
    - ``L_0``, an element

    OUTPUT:

    The product ``(L_1...*(L_{n-1}*(L_n*L_0))...)``

    EXAMPLES::

        sage: from pGroupCohomology.cohomology import Mul
        sage: M=Matrix(GF(5),[[1,2],[1,1]])
        sage: M1=Matrix(GF(5),[[1,2],[1,1]])
        sage: M2=Matrix(GF(5),[[1,3],[2,1]])
        sage: M3=Matrix(GF(5),[[1,4],[3,1]])
        sage: Mul([M1,M2],M3)
        [0 0]
        [0 1]
        sage: Mul([M2,M3],M1)
        [2 2]
        [4 4]
        sage: M2*M3*M1==Mul([M2,M3],M1)
        True

    """
    OUT = L0
    cdef int i
    cdef int m = len(L)
    for i from m > i >= 0:
        OUT=L[i]*OUT
    return OUT

def str2html(s,linelength=80):
    """
    HTML representation of a string representation of a polynomial

    INPUT:

    - ``s``             - a string representing a polynomial
    - ``linelength=80`` - (optional) approximate line length of the output

    OUTPUT:

    HTML representation of the polynomial (string).
    If the polynomial is too long, there will be line breaks. 

    NOTE:

    * It is expected that all exponents are positive integer numbers,
      written without signs.
    * The multiplication sign '*' is replaced by '&middot;'
    * In order to group the terms nicely, there is a small space around
      plus and minus signs.
    * There is an indentation after a line break.
    
    EXAMPLES::
    
        sage: from pGroupCohomology.cohomology import str2html
        sage: R.<x,y>=QQ[]
        sage: s=str(x^2+2*x*y+y^4)
        sage: s
        'y^4 + x^2 + 2*x*y'
        sage: str2html(s)
        '<nobr>y<sup>4 </sup>&thinsp;+&thinsp; x<sup>2 </sup>&thinsp;+&thinsp; 2&middot;x&middot;y</nobr>'

    If we choose a very small line length, the result breaks into several lines::
    
        sage: str2html(s,linelength=15)
        '<nobr>y<sup>4 </sup>&thinsp;+&thinsp; x<sup>2 </sup></nobr><br>&nbsp;&nbsp;<nobr>&thinsp;+&thinsp; 2&middot;x&middot;y</nobr>'

    """
    cdef list L=s.replace('+','\n+').replace('-','\n-').split('\n')
    cdef list O=['<nobr>']
    cdef list mon
    cdef int i, e
    cdef int lenL = len(L)
    lenO = 0
    for i from 0<=i<lenL:
        mon = []#'&thinsp;<nobr>']
        e=0
        s = L[i]
        for c in s:
            if c=='^':
                e+=1
                mon.append('<sup>')
            elif c=='*':
                if e:
                    e=0
                    mon.append('</sup>&middot;')
                else:
                    mon.append('&middot;')
            elif c=='-':
                mon.append('&thinsp;&#8722;&thinsp;')
            elif c=='+':
                mon.append('&thinsp;+&thinsp;')
            else:
                mon.append(c)
        if e:
            mon.append('</sup>')
        #mon.append('</nobr>')
        if lenO==0:
            O.extend(mon)
            lenO=len(mon)
        elif ((lenO+len(mon)>linelength) and (linelength>0)):
            O.append('</nobr><br>&nbsp;&nbsp;<nobr>')
            O.extend(mon)
            lenO=len(mon)
        else:
            O.extend(mon)
            lenO+=len(mon)
        #O.extend(mon)
    O.append('</nobr>')
    return ''.join(O)

def HV2Poly(L):
    """
    Translate a list of integers into a univariate polynomial, using the given integers as coefficients.

    INPUT:

    ``L`` -- list of integers

    OUTPUT:

    A univariate polynomials with coefficients given by ``L``

    EXAMPLES::

        sage: from pGroupCohomology.cohomology import HV2Poly
        sage: HV2Poly([3,2,4])
        4*t^2 + 2*t + 3

    """
    R = PolynomialRing(QQ,'t')
    t = R('t')
    return sum([L[i]*t**i for i in range(len(L))])

            
####################################################################
####################################################################
## The COHO extension class
####################################################################
####################################################################

class COHO_prefix:
    """
    COHO_prefix()() returns the next safe prefix used for naming Singular interface data of a :class:`~pGroupCohomology.cohomology.COHO` instance.
        
    TESTS::

        sage: from pGroupCohomology.cohomology import COHO_prefix    
        sage: COHO_prefix.instance=0  # initialization, for avoiding other doc tests to interfere
        sage: COHO_prefix()()
        'COHO1'
        sage: COHO_prefix()()
        'COHO2'

        """
    instance = 0

    def __init__(self):
        """
        TESTS::

            sage: from pGroupCohomology.cohomology import COHO_prefix
            sage: COHO_prefix.instance=0  # initialization, for avoiding other doc tests to interfere
            sage: C = COHO_prefix()
            sage: COHO_prefix.instance
            1

        """
        self.__safe_for_unpickling__ = True # don't know if this is needed
        self.__class__.instance = self.__class__.instance + 1

    def __call__(self):
        """
        Provide the next unused prefix for internal use in cohomology computations

        TESTS::

            sage: from pGroupCohomology.cohomology import COHO_prefix
            sage: COHO_prefix.instance=0  # initialization, for avoiding other doc tests to interfere
            sage: COHO_prefix()()
            'COHO1'
            sage: COHO_prefix()()
            'COHO2'

        """
        return 'COHO%d'%(self.__class__.instance)
        
########################################
## An auxiliary class for deletion of 
## the Ring's data in the Singular interface.
## This is since COHO must not have a __del__
## method since this would prevent COHO from
## being garbage collected.

class COHO_Terminator:
    """
    This is an auxiliary class, with the purpose to remove all data from the
    Singualar interface that are related with a certain cohomology ring.

    Each cohomology ring has a member that is an instance of this class. Hence,
    when the ring is deleted then the member's ``__del__`` method is called,
    which clears the Singular interface. 

    The somewhat hidden reason for introducing an auxiliar class is that
    circular references of cohomology rings can not be avoided; hence, if they
    had a ``__del__`` method, they could never be garbage collected (the reason
    is described in the `Python references <http://www.python.org/doc/2.6.2/reference/datamodel.html#object.__del__>`_);
    hence, we moved the ``__del__`` method to here.

    TESTS::

        sage: from pGroupCohomology.cohomology import COHO_Terminator
        sage: T = COHO_Terminator(singular(1), 'MyPrefix')
        sage: singular.eval('ring MyPrefixR = 0,(a,b,c),dp')
        'ring MyPrefixR = 0,(a,b,c),dp;'
        sage: print singular.eval('MyPrefixR')
        //   characteristic : 0
        //   number of vars : 3
        //        block   1 : ordering dp
        //                  : names    a b c
        //        block   2 : ordering C
        sage: singular.eval('ideal MyPrefixI = a^2,b^2,c')
        'ideal MyPrefixI = a^2,b^2,c;'
        sage: print singular.eval('MyPrefixI')
        MyPrefixI[1]=a2
        MyPrefixI[2]=b2
        MyPrefixI[3]=c
        sage: del T
        sage: singular('MyPrefixI')
        Traceback (most recent call last):
        ...
        TypeError: Singular error:
           ? `MyPrefixI` is undefined
           ? error occurred in ...

    """
    def __init__(self, S, prefix):
        """
        TESTS::

            sage: from pGroupCohomology.cohomology import COHO_Terminator
            sage: T = COHO_Terminator(singular,'MyPrefix')
            sage: T._S is singular
            True
            sage: T._prefix
            'MyPrefix'

        """
        self._S = S
        self._prefix = prefix
        self._l = len(prefix)
    def __del__(self):
        """
        The purpose of this del-method is to kill all elements 
        in the interface ``self._S`` whose name starts with
        ``self._prefix``.
    
        TESTS::

            sage: from pGroupCohomology.cohomology import COHO_Terminator
            sage: T = COHO_Terminator(singular(1), 'MyPrefix')
            sage: singular.eval('ring MyPrefixR = 0,(a,b,c),dp')
            'ring MyPrefixR = 0,(a,b,c),dp;'
            sage: print singular.eval('MyPrefixR')
            //   characteristic : 0
            //   number of vars : 3
            //        block   1 : ordering dp
            //                  : names    a b c
            //        block   2 : ordering C
            sage: singular.eval('ideal MyPrefixI = a^2,b^2,c')
            'ideal MyPrefixI = a^2,b^2,c;'
            sage: print singular.eval('MyPrefixI')
            MyPrefixI[1]=a2
            MyPrefixI[2]=b2
            MyPrefixI[3]=c
            sage: del T
            sage: singular('MyPrefixI')
            Traceback (most recent call last):
            ...
            TypeError: Singular error:
               ? `MyPrefixI` is undefined
               ? error occurred in ...

        """
        cdef list ITEM
        if not hasattr(self._S,'_check_valid'):
            return
        try:
            S = self._S._check_valid()
        except ValueError:
            return
        try:
            L = S.eval('listvar(ring)').split('\n')
        except TypeError:
            L = []
        for X in L:
            ITEM = X.split(' ')
            if len(ITEM)>1:
                n = ITEM[1]
                # Singular data must be deleted if
                # their name begins with the right prefix.
                # But this is not enough: COHO11r(1) must not
                # be deleted when deleting COHO1...
                # But our data in Singular are named with
                # prefix + something that starts with no digit. 
                if n.startswith(self._prefix) and (not n[self._l].isdigit()):
                    S.eval('kill '+n)
        try:
            L = S.eval('listvar(intmat)').split('\n')
        except TypeError:
            L = []
        for X in L:
            ITEM = X.split(' ')
            if len(ITEM)>1:
                n = ITEM[1]
                if n.startswith(self._prefix) and (not n[self._l].isdigit()):
                    S.eval('kill '+n)
        if hasattr(self._S,'_name'):
            if S('defined('+self._S._name+')'):
                S.eval('kill '+self._S._name)


class COHO(Ring):
    """
    Modular Cohomology Rings of Finite `p`-Groups with coefficients in `\\mathbb F_p`

    AUTHORS:
    
    - Simon King (2009-06) initial version

    INPUT:

    A finite `p`-group, either given by its coordinates in the 
    SmallGroups library of Hans Ulrich Besche, Bettina Eick and Eamonn O'Brien,
    or given as an object in the Gap interface.

    OUTPUT:

    An object set up to compute the cohomology ring of the given `p`-group with
    coefficients in `\\mathbb F_p`.

    ALGORITHM:

    The methods are based on algorithms due to David Green and Dave Benson.

    Eventually, a minimal generating set for the cohomology ring and a minimal 
    set of algebraic relations is computed, together with various other 
    information (e.g., Poincare series).

    By :meth:`option` and by various parameters (see below), it can be influenced 
    by what methods the results are computed, how the computation is documented,
    and where files created during the computation are stored.

    See :mod:`pGroupCohomology` for a detailed introduction. Note that usually one
    would not construct an instance of the class COHO directly. Instead, the 
    user is advised to use the constructor :func:`pGroupCohomology.CohomologyRing`.

    EXAMPLES:
    
    First, a small example showing the protocol mode::

        sage: from pGroupCohomology.cohomology import COHO
        sage: COHO.option('noreload','prot')
        sage: tmp_root = tmp_filename()
        sage: H = COHO(8,3, root=tmp_root)
        Group data are rooted at ...
        <BLANKLINE>
        Initialize cohomology ring of D8
        Computing data for Small Group number 1 of order 2
        Computing data for Small Group number 2 of order 4
        Computing data for Small Group number 3 of order 8
        > > > export action matrices
        Inserting SmallGroup(2,1) as a subgroup
        Inserting SmallGroup(4,2) as a subgroup
        Computing Dickson invariants in elementary abelian subgroup of rank 2

        sage: H
        H^*(D8; GF(2))
        sage: H.option('timing')
        sage: H.make()
        We have no degree bound yet
        Start computation in Degree 1
        Computing next term of a resolution of GF(2)(8gp3)
        > Make degree 1 autolift data for GF(2)(8gp3)
        > > > import action matrices
        Time for makeAutolift:
           CPU:  ...
           Wall: ...
        There are new generators, we have to lift the restriction maps
        Computing next term of a resolution of GF(2)(8gp3)
        > > rk P_02 =   3 for Res8gp3
        Time for next differential:
           CPU:  ...
           Wall: ...
        <BLANKLINE>
        <BLANKLINE>
        Restriction to 1st subgroup (order 2)
        ::: > lift Chain Map in the source resolution
        > lift in the target resolution to degree 1
        > > use autolift method
        Time for lifting chain map:
           CPU:  ...
           Wall: ...
        > > > export Chain Map
        <BLANKLINE>
        Restriction to 2nd subgroup (order 4)
        ::: > lift Chain Map in the source resolution
        > lift in the target resolution to degree 1
        > > use autolift method
        Time for lifting chain map:
           CPU:  ...
           Wall: ...
        > > > export Chain Map
        <BLANKLINE>
        Restriction to 3rd subgroup (order 4)
        ::: > lift Chain Map in the source resolution
        > lift in the target resolution to degree 1
        > > use autolift method
        Time for lifting chain map:
           CPU:  ...
           Wall: ...
        > > > export Chain Map
        We have to choose 2 new generators of H^*(D8; GF(2)) in degree 1
        > There are 0 nilpotent generators of H^*(D8; GF(2)) in degree 1
        > There are 2 "boring" generators of H^*(D8; GF(2)) in degree 1
        Total time for choosing new generators:
           CPU:  ...
           Wall: ...
        Try to lift 1st power of 0th Dickson invariant
        simultaneously lifting subgroup cochains of degree 1
        <BLANKLINE>
        Restriction to 2nd subgroup (order 4)
        <BLANKLINE>
        Restriction to 3rd subgroup (order 4)
        > Evaluating restrictions of ring generators
        > > Construct MTX matrix
        > > Compute echelon form of 2x6 matrix
        > echelon is computed
        Total time for simultaneous cochain lift:
           CPU : ...
           Wall: ...
        Simultaneous lift was successful!
        > Express cochain of degree 1 as polynomial
        > Simplify Dickson invariant modulo nilpotent generators
        > Computing factors
        Degree 1 of the visible ring structure of H^*(D8; GF(2)) is computed!
        <BLANKLINE>
        Save H^*(D8; GF(2))
        Total time for computing H^*(D8; GF(2)) in degree 1
           CPU:  ...
           Wall: ...
        ########################################
        We need to find more regular generators!
        <BLANKLINE>
        Start computation in Degree 2
        Time for getting standard monomials:
           Wall: ...
        We got 3 standard monomials
        Monomial 0
        > Candidate: b_1_0^2
        > Express monomial as a Cochain
        > > We need more lifts!
        > Compose Chain Map with Differential
        > Lift with the autolift method
        Decomposable cochain found
        <BLANKLINE>
        Monomial 1
        > Candidate: b_1_0*b_1_1
        > Express monomial as a Cochain
        > > We need more lifts!
        > Compose Chain Map with Differential
        > Lift with the autolift method
        New relation found: b_1_0*b_1_1
        <BLANKLINE>
        Monomial 2
        > Candidate: b_1_1^2
        > Express monomial as a Cochain
        Decomposable cochain found
        <BLANKLINE>
        Time for extracting relations:
           CPU:  ...
           Wall: ...
        There are new generators, we have to lift the restriction maps
        Computing next term of a resolution of GF(2)(8gp3)
        > > rk P_03 =   4 for Res8gp3
        Time for next differential:
           CPU:  ...
           Wall: ...
        <BLANKLINE>
        > Make degree 2 autolift data for GF(2)(8gp3)
        > > > import action matrices
        Time for makeAutolift:
           CPU:  ...
           Wall: ...
        <BLANKLINE>
        Restriction to 1st subgroup (order 2)
        ::: > lift Chain Map in the source resolution
        > lift in the target resolution to degree 2
        > > use autolift method
        Time for lifting chain map:
           CPU:  ...
           Wall: ...
        > > > export Chain Map
        <BLANKLINE>
        Restriction to 2nd subgroup (order 4)
        ::: > lift Chain Map in the source resolution
        > lift in the target resolution to degree 2
        > > use autolift method
        Time for lifting chain map:
           CPU:  ...
           Wall: ...
        > > > export Chain Map
        <BLANKLINE>
        Restriction to 3rd subgroup (order 4)
        ::: > lift Chain Map in the source resolution
        > lift in the target resolution to degree 2
        > > use autolift method
        Time for lifting chain map:
           CPU:  ...
           Wall: ...
        > > > export Chain Map
        We have to choose 1 new generator of H^*(D8; GF(2)) in degree 2
        > There are 0 nilpotent generators of H^*(D8; GF(2)) in degree 2
        > There are 0 "boring" generators of H^*(D8; GF(2)) in degree 2
        > There is 1 Duflot regular generator of H^*(D8; GF(2)) in degree 2
        Total time for choosing new generators:
           CPU:  ...
           Wall: ...
        Degree 2 of the visible ring structure of H^*(D8; GF(2)) is computed!
        <BLANKLINE>
        Save H^*(D8; GF(2))
        Total time for computing H^*(D8; GF(2)) in degree 2
           CPU:  ...
           Wall: ...
        Testing whether it makes sense to try Benson's completeness criterion
        We expect that Benson's test will not apply before degree 3
        <BLANKLINE>
        Start computation in Degree 3
        Computing Groebner basis up to degree 3 for H^*(D8; GF(2))
        Time for computing Groebner basis
           Wall: ...
        Time for getting standard monomials:
           Wall: ...
        We got 4 standard monomials
        Monomial 0
        > Candidate: b_1_0^3
        > Express monomial as a Cochain
        > > We need more lifts!
        > Compose Chain Map with Differential
        > Lift with the autolift method
        Decomposable cochain found
        <BLANKLINE>
        Monomial 1
        > Candidate: b_1_1^3
        > Express monomial as a Cochain
        > > We need more lifts!
        > Compose Chain Map with Differential
        > Lift with the autolift method
        Decomposable cochain found
        <BLANKLINE>
        Monomial 2
        > Candidate: c_2_2*b_1_0
        > Express monomial as a Cochain
        > > We need more lifts!
        > Compose Chain Map with Differential
        > Lift with the autolift method
        Decomposable cochain found
        <BLANKLINE>
        Monomial 3
        > Candidate: c_2_2*b_1_1
        > Express monomial as a Cochain
        Decomposable cochain found
        <BLANKLINE>
        Time for extracting relations:
           CPU:  ...
           Wall: ...
        There is no new generator of H^*(D8; GF(2)) in degree 3
        Degree 3 of the visible ring structure of H^*(D8; GF(2)) is computed!
        <BLANKLINE>
        Save H^*(D8; GF(2))
        Total time for computing H^*(D8; GF(2)) in degree 3
           CPU:  ...
           Wall: ...
        Testing whether it makes sense to try Benson's completeness criterion
        It is possible that Benson's degree bound applies
        Test whether the sequence ['c_2_2', 'b_1_0 + b_1_1'] is filter regular
        Computing complete Groebner basis for H^*(D8; GF(2))
        Time for computing Groebner basis
           Wall: ...
        Time for testing filter regularity
           Wall: ...
        > Sequence is filter regular.
          Raw filter degree type: [-1, -1, 1]
          --> Filter degree type: [-1, -2, -2]
        <BLANKLINE>
        Cohomology computation for D8 is finished
        Save H^*(D8; GF(2))
        Total time for cohomology computation
           CPU:  ...
           Wall: ...
        <BLANKLINE>
        <BLANKLINE>
        sage: H.option('noprot','notiming')
        sage: print H
        Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
        <BLANKLINE>
        Computation complete
        Minimal list of generators:
        [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
         b_1_0, a 1-Cochain in H^*(D8; GF(2)),
         b_1_1, a 1-Cochain in H^*(D8; GF(2))]
        Minimal list of algebraic relations:
        [b_1_0*b_1_1]

    Now, after switching off the protocol and timing mode, an example using a group defined in Gap::

        sage: H.option('noprot','notiming')
        sage: G = gap('DihedralGroup(8)')
        sage: G
        Group( [ f1, f2, f3 ] )
        sage: gap.eval('SetName(%s,"OtherName")'%(G.name()))
        ''
        sage: G
        OtherName
        sage: H2 = COHO(G,root=tmp_root)
        sage: H2
        H^*(OtherName; GF(2))
        sage: H2.make()
        sage: print H2
        Cohomology ring of OtherName with coefficients in GF(2)
        <BLANKLINE>
        Computation complete
        Minimal list of generators:
        [c_2_2, a 2-Cochain in H^*(OtherName; GF(2)),
         b_1_0, a 1-Cochain in H^*(OtherName; GF(2)),
         b_1_1, a 1-Cochain in H^*(OtherName; GF(2))]
        Minimal list of algebraic relations:
        [b_1_1^2+b_1_0*b_1_1]

    Next, a slightly bigger example::

        sage: H4 = COHO(64,14,root=tmp_root)
        sage: H4.make()                           # about 8 seconds -- a long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1,
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                           # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']
    
    An example with `p=3`, so that the cohomology ring is non-commutative. It is required
    that Singular be installed at least in version 3-1-0::

        sage: R = COHO(27,3,root=tmp_root)
        sage: R.make()
        sage: R.gens()
        [1,
         b_2_0, a 2-Cochain in H^*(E27; GF(3)),
         b_2_1, a 2-Cochain in H^*(E27; GF(3)),
         b_2_2, a 2-Cochain in H^*(E27; GF(3)),
         b_2_3, a 2-Cochain in H^*(E27; GF(3)),
         c_6_8, a 6-Cochain in H^*(E27; GF(3)),
         a_1_0, a 1-Cochain in H^*(E27; GF(3)),
         a_1_1, a 1-Cochain in H^*(E27; GF(3)),
         a_3_4, a 3-Cochain in H^*(E27; GF(3)),
         a_3_5, a 3-Cochain in H^*(E27; GF(3))]
        sage: print R('a_1_0*a_3_4')
        4-Cochain in H^*(E27; GF(3)),
        represented by
        [1 0 0 0 0 0 0]
        sage: print R('a_3_4*a_1_0')
        4-Cochain in H^*(E27; GF(3)),
        represented by
        [2 0 0 0 0 0 0]
        sage: print R.6*R.8
        4-Cochain in H^*(E27; GF(3)),
        represented by
        [1 0 0 0 0 0 0]
        sage: c=R('a_1_0*a_1_1*b_2_0+b_2_1^2')
        sage: c.name()
        '((a_1_0)*(a_1_1))*(b_2_0)+(b_2_1)**2'
        sage: c.setname('C')
        sage: c
        C, a 4-Cochain in H^*(E27; GF(3))
        sage: R.poincare_series()
        (t^4 + 2*t^2 + 1)/(t^6 - 2*t^5 + 2*t^4 - 2*t^3 + 2*t^2 - 2*t + 1)
        sage: R.poincare_without_parameters()
        (t^4 + 2*t^2 + 1)/(t^6 - 2*t^5 + 2*t^4 - 2*t^3 + 2*t^2 - 2*t + 1)

    We return to our standard example, the cohomology of the dihedral group
    of order 8::

        sage: H = COHO(8,3,root=tmp_root)
        sage: H.make()

    First of all, the attribute 'completed' tells whether the cohomology
    computation has been successfully finished. The attribute
    'knownDeg' tells up to what degree the ring structure is explored,
    and 'lastRel' is the degree of the last found relation::

        sage: H.completed
        True
        sage: H.knownDeg
        3
        sage: H.lastRel
        2

    A cohomology ring is based on some minimal projective resolution
    (see :class:`~pGroupCohomology.resolution.RESL`)::

        sage: H.resolution()
        Resolution:
        0 <- GF(2) <- GF(2)(8gp3) <- rank 2 <- rank 3 <- rank 4

    The lists of generators and relations of a cohomology ring can be obtained with
    :meth:`gens` and :meth:`rels`.

    The attribute ``H.GenS`` provides a ring defined in the Singular interface,
    and ``H.RelG`` is a list of strings defining a Groebner basis for the
    relation ideal. The cohomology ring is isomorphic to the quotient
    of ``H.GenS`` by the ideal of ``H.RelG``. The same ring is created when
    putting ``H`` into the singular interface::

        sage: H.GenS.set_ring()
        sage: I=H.GenS.parent().ideal(H.RelG)
        sage: s=H.GenS.parent().eval('qring Q = %s'%(I.name()))
        sage: H.GenS.parent()('Q')
        //   characteristic : 2
        //   number of vars : 3
        //        block   1 : ordering M
        //                  : names    c_2_2 b_1_0 b_1_1
        //                  : weights      2     1     1
        //                  : weights     -1     0     0
        //                  : weights      0    -1     0
        //        block   2 : ordering C
        // quotient ring from ideal
        _[1]=b_1_0*b_1_1
        sage: singular(H)
        //   characteristic : 2
        //   number of vars : 3
        //        block   1 : ordering M
        //                  : names    c_2_2 b_1_0 b_1_1
        //                  : weights      2     1     1
        //                  : weights     -1     0     0
        //                  : weights      0    -1     0
        //        block   2 : ordering C
        // quotient ring from ideal
        _[1]=b_1_0*b_1_1

    The computation of a cohomology ring makes heavy use of the Singular
    interface. The names of all data in Singular that are related with a
    specific cohomology ring ``H`` have a common prefix, namely ``H.prefix``.
    The prefix is chosen automatically and is different for any instance
    of :class:`~pGroupCohomology.cohomology.COHO`::

        sage: H.prefix==H.subgps[(4,2)].prefix
        False

    In the Singular interface, a graded commutative polynomial ring is
    defined and can be addressed as follows::
    
        sage: print singular.eval('%sr(%d)'%(H.prefix,H.knownDeg))
        //   characteristic : 2
        //   number of vars : 3
        //        block   1 : ordering M
        //                  : names    c_2_2 b_1_0 b_1_1
        //                  : weights      2     1     1
        //                  : weights     -1     0     0
        //                  : weights      0    -1     0
        //        block   2 : ordering C
    
    Note that in Singular, ideals are only available if the ring
    to which the ideal belongs is Singular's current ``basering``. Currently,
    the quotient ring defined above is ``basering``. We change the base ring,
    and then a Groebner basis at least up to degree ``H.knownDeg`` is given
    as follows::
    
        sage: H.set_ring()
        sage: singular('%sI'%(H.prefix))
        b_1_0*b_1_1

    We change the base ring again,
    ::
    
        sage: H.subgroups()[(4,2)].set_ring()

    and then of course the relation ideal of ``H`` is not available anymore::

        sage: singular('%sI'%(H.prefix))
        Traceback (most recent call last):
        ...
        TypeError: Singular error:
          ? ... is undefined
          ? error occurred in STDIN line ...: `def ...;`

    **Properties: Custom attributes**

    In Python, it is possible to add attributes to any object.
    However, when adding a Python attribute to a cohomology ring,
    it would be lost when saving.

    Therefore we provide another way to add custom "attributes"
    to a cohomology ring. We refer to such attribute as a "property"
    of the ring.

    Properties can be retreived in the same way as usual attributes,
    but are preserved when the cohomology ring is saved and reloaded. 
    This is very useful for adding more information. However,
    unlike attributes, properties are not available by tab
    completion.

    When an attribute of the requested name (``Status`` in the
    following example) does not exist, it is checked whether
    a property of  that name has yet been defined. If this is
    the case, it is returned, otherwise None is returned. Note
    that *for cohomology rings you will never get an
    ``AttributeError`` except for attributes that start and end
    with an underscore*::
    
        sage: print H.Status  # not defined at that point, hence "None"
        None
        sage: H.setprop('Status','Our main example')
        sage: H.Status
        'Our main example'
        sage: HR=loads(dumps(H))
        sage: HR.Status
        'Our main example'
        sage: HR._foobar_
        Traceback (most recent call last):
        ...
        AttributeError: 'COHO' object has no attribute '_foobar_'

    For example, the (raw) filter degree type (see below), the group 
    name (that we passed on by the parameter ``GroupName``) and a more detailed
    autogenerated group description (that can be overwritten using 
    ``setprop('GroupDescr',...)`` ) are provided by custom attributes::
    
        sage: H.GroupName  # This is a name that was looked up in some list
        'D8'
        sage: H.GroupDescr # This comes from a list of known group descriptions
        'Dihedral group of order 8'
    
    The custom attribute 'Status' can be deleted with
    ::
    
        sage: H.delprop('Status')
        sage: H.Status


    TESTS:

    As mentioned above, there are various options (both in the definition
    of a cohomology ring and by using COHO.option(...)) that influence the
    choice of algorithm. For unit testing, we repeat here one of the
    examples from above.
    In order to not loose too much time, we now allow to reload data.
    ::
    
        sage: COHO.option('reload')
 
    First, we use linear algebra and not elimination for finding the Dickson classes::
    
        sage: H4 = COHO(64,14,root=tmp_root, useElimination=False)
        sage: H4.make()                           # long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1,
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                            # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']

    We repeat the example, but this time use elimination.
    ::

        sage: H4 = COHO(64,14,root=tmp_root, useElimination=True)
        sage: H4.make()                           # long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1,
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                           # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']

    Now we switch off the 'sparse' option::
    
        sage: COHO.option('nosparse')
        sage: H4 = COHO(64,14,root=tmp_root)
        sage: H4.make()                           # long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1,
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                           # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']
        sage: COHO.option('sparse')

    Next, we try to lift many chain maps in one step::
    
        sage: H4 = COHO(64,14,root=tmp_root)
        sage: H4.option('liftlist')
        sage: H4.make()                           # long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1,
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                           # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']
        sage: COHO.option('noliftlist')

    And finally, although we do not recommend it, we allow to convert to Sage matrices::
    
        sage: H4 = COHO(64,14,root=tmp_root)
        sage: H4.option('nouseMTX')
        sage: H4.make()                           # long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1, 
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)), 
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)), 
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)), 
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)), 
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)), 
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                           # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']
        sage: H4.option('useMTX')

    """

####################
## Initializing the dictionary of group names as class variable
    GroupNames = {}
    try:
        for X in load(SAGE_ROOT+"/local/pGroupCohomology/GroupNames.sobj"):
            GroupNames[tuple(X[0])] = [X[1],X[2]]
    except IOError:
        "We did not find the dictionary of custom group names."
## Determine the standard locations for the public and the private data bases
    user_db = DOT_SAGE+'/pGroupCohomology/db/'
    while ('//' in user_db):
        user_db = user_db.replace('//','/')
    public_db = SAGE_ROOT+'/local/pGroupCohomology/db/'
    while ('//' in user_db):
        user_db = user_db.replace('//','/')
    web_db = 'http://sage.math.washington.edu/home/SimonKing/Cohomology/db/'
    

####################
## init, repr, str
    def __init__(self, *args, **kwds):
        """
    TESTS::

        sage: from pGroupCohomology.cohomology import COHO
        sage: tmp_root = tmp_filename()
        sage: H4 = COHO(64,14,root=tmp_root)
        sage: H4.make()                           # about 8 seconds -- a long time...
        sage: H4.gens()                           # continuing the long time example ...
        [1,
         a_2_1, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_2_2, a 2-Cochain in H^*(SmallGroup(64,14); GF(2)),
         c_4_4, a 4-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_0, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_1_1, a 1-Cochain in H^*(SmallGroup(64,14); GF(2)),
         a_3_3, a 3-Cochain in H^*(SmallGroup(64,14); GF(2))]
        sage: H4.rels()                           # ... and finishing the long time example
        ['a_1_0^2',
         'a_1_0*a_1_1',
         'a_1_1^3',
         'a_2_1*a_1_0',
         'a_2_1^2+a_2_1*a_1_1^2',
         'a_1_1*a_3_3+a_2_1^2',
         'a_1_0*a_3_3+a_2_1^2',
         'a_2_1*a_3_3',
         'a_3_3^2']
        sage: G = gap('DihedralGroup(8)')
        sage: G
        Group( [ f1, f2, f3 ] )
        sage: H2 = COHO(G,GroupName='OtherName',root=tmp_root)
        sage: H2
        H^*(OtherName; GF(2))
        sage: H2.make()
        sage: print H2
        Cohomology ring of OtherName with coefficients in GF(2)
        <BLANKLINE>
        Computation complete
        Minimal list of generators:
        [c_2_2, a 2-Cochain in H^*(OtherName; GF(2)),
         b_1_0, a 1-Cochain in H^*(OtherName; GF(2)),
         b_1_1, a 1-Cochain in H^*(OtherName; GF(2))]
        Minimal list of algebraic relations:
        [b_1_1^2+b_1_0*b_1_1]

        """
        # argument "root" declares the name of the root directory
        self.__dict = {}  # It is important that this attribute is present!
                          # Otherwise, an error in __init__ would produce a seg fault.
        self._HomsetCache = WeakValueDictionary({})
        indent = kwds.get('indent',0)
        self.setprop('KeepBases',kwds.get('KeepBases'))
        Subgroups = kwds.get('Subgroups',True)
        root = kwds.get('root',COHO.user_db)
        if Subgroups and len(args) and OPTION.opts['prot']:
            print "Group data are rooted at "+root
        Hfinal = None
        if len(args) == 2:
            # We expect an address in the Small Groups library
            q,n = args
            q = Integer(q)
            n = Integer(n)
            if q == 1:
                raise ValueError, "We don't consider the trivial group"
            if (n<0) or (n<=0 and q>0): #(0,0) is for eating very old pickles...
                raise ValueError, "SmallGroup number must be positive"
            if (q>0) and (n > Integer(gap.eval('NumberSmallGroups(%d)'%q))): # will raise another error if SmallGroups is not installed
                raise ValueError, "(%d,%d) is not in the SmallGroups library"%(q,n)
            
            ####
            ## make up (stem) name and unique group key
            GStem = kwds.get('GStem','%dgp%d'%(q,n)) 
            # Priorities for chosing GroupName: 1. user defined, 2. known from list, 3. default: GStem
            GroupName = kwds.get('GroupName')
            if GroupName is None:
                GroupName = COHO.GroupNames.get((q,n), ('SmallGroup(%d,%d)'%(q,n),))[0]
            GroupKey = kwds.get('key',[(q,n),''])[0]


        elif len(args) == 0: # *only* used in pickling/unpickling
            q = 0
            n = 0
            GStem = ''


        elif len(args) == 1:
            ## We expect a group that is defined in Gap
            ##############
            ## It is assumed here that the group is given by minimal generators, and that there is a unique
            ## descriptor for the group provided (by the option 'key'). Moreover, GStem and GroupName must
            ## be explicitly provided. This is taken care of in
            ##       pGroupCohomology.CohomologyRing
            ## So, we see if there are proper key words (provided by CohomologyRing) and otherwise
            ## we test the input explicitly.
            if kwds.get('gap_input'):
                q = kwds['gap_input']  # it is supposed to be the group order
                GStem = kwds['GStem']  # *Must* be provided
                GroupName = kwds['GroupName']
                GroupKey = kwds['key'][0]
                Hfinal = args[0]
                n = -1
            else:
                H0 = args[0]
                if not hasattr(H0,'parent'):
                    raise TypeError, "We expected a group defined in Gap"
                GAP = H0.parent()
            
                ####
                ## make up the (stem) name
                try:
                    GroupName = GAP.eval('Name(%s)'%(H0.name()))[1:-1]
                except RuntimeError:
                    GroupName = None
                GroupName = kwds.get('GroupName', GroupName)
                GStem = kwds.get('GStem', GroupName)
                if (GStem is None):
                    raise ValueError, "Either 'GroupName' or 'GStem' must be provided"
                if not (isinstance(GStem, basestring) and GStem.isalnum()):
                    raise TypeError, "'GroupName' must be an alphanumeric string"
                if GroupName is None:
                    GroupName = GStem

                ####
                ## process the gap input:
                #   1. good group order?
                #   2. minimal generators given?
                #      if not, find them!
                #   3. transform into permutation group
                q = Integer(GAP.eval('Order(%s)'%(H0.name())))
                if not q.is_prime_power():
                    raise ValueError, "The group must be of prime power order"
                n = -1

                ## Try to make up minimal generators
                if GAP.eval('RankPGroup(%s)'%(H0.name())) == GAP.eval('Length(GeneratorsOfGroup(%s))'%(H0.name())):
                    # We already have minimal generators, and then we are supposed to use exactly *these* generators!
                    Hfinal = H0
                else:
                    if OPTION.opts['prot']:
                        print "\n"+(indent*'::: ')+"Try to find minimal generators for", H0
                    HP = GAP('Group(Image(IsomorphismPermGroup(%s),GeneratorsOfGroup(%s)))'%(H0.name(),H0.name()))
                    ## Hopefully gap can find minimal generating sets for permutation groups...
                    Hfinal = GAP('Subgroup(%s,MinimalGeneratingSet(%s))'%(HP.name(),HP.name()))
                ####
                ## Finally, make up a description of the group which preserves the generators
                ## -- unless the user wants a different key (which may fail badly, but this is not our problem... :)
                if kwds.has_key('key'): 
                    GroupKey = kwds['key'][0]
                else:
                    if not GAP('IsPermGroup(%s)'%(Hfinal.name())):
                        GroupKey = GAP.eval('Group(Image(IsomorphismPermGroup(%s),GeneratorsOfGroup(%s)))'%(Hfinal.name(),Hfinal.name()))
                    else:
                        GroupKey = str(Hfinal)
        else:
            raise TypeError, "COHO initialisation requires between 0 and 2 arguments (%d given)"%(len(args))
        
        if not isinstance(GStem, basestring):
            raise TypeError, "Group stem name must be a string"

        self.GenS = singular(0)
        if q==0:  # only used for Python style unpickling of old data (Simon King only... :)
            self.prefix = COHO_prefix()()
            self._Terminator = COHO_Terminator(self.GenS, self.prefix)
            self.suffDeg = -1
            self.completed = False
            return
        if not ((q>0) and Integer(q).is_prime_power()):
            raise TypeError, "The group order must be a prime power"

        ##################################
        ##################################
        ## Now the preparation is done, we have q, GStem, GroupName and GroupKey.
        ##################################
        ##################################
        if OPTION.opts['prot']:
            if ((q!=1) and (Hfinal is None) and (GroupName==GStem)): # hence, the name was not found on the list and was not user defined
                print "\n"+(indent*'::: ')+"Initialize cohomology ring of group number %d of order %d"%(n,q)
            else:
                print "\n"+(indent*'::: ')+"Initialize cohomology ring of %s"%(GroupName)

        #########
        ## Store group name and group description
        self.setprop('GroupName',GroupName)
        if n>0:
            GroupDescr = 'Small Group number %d of order %d'%(n,q)
        else:
            GroupDescr = GroupName
        self.setprop('GroupDescr', kwds.get('GroupDescr',COHO.GroupNames.get((q,n), ('',GroupDescr))[1]))

        #####
        ## Determine data location relative to the root directory ...
        gps_folder = GStem + '/'
        dat_folder = gps_folder+'dat/'
        res_folder = gps_folder+'dat/'  
        inc_folder = gps_folder+'sgp/'
        ## ... and now absolutely:
        if len(root) and root[-1]!='/':
            root=root+'/'
        self.setprop('root',root)
        self.gps_folder = root+gps_folder
        self.dat_folder = root+dat_folder
        self.res_folder = root+res_folder
        self.inc_folder = root+inc_folder

        # store the key for this ring and insert itself to the cache
        if kwds.has_key('key'):
            if not kwds['key'][1].startswith(self.dat_folder):
                raise ValueError, 'Invalid location %s of State descriptor'%(kwds['key'][1])
            self.setprop('_key', kwds['key'])
        else:
            self.setprop('_key', (GroupKey, self.dat_folder+'State'))
        from pGroupCohomology import _cache
        _cache[self._key] = self  # Note that there is no entry yet with this key --
                                  # provided that the ring initialisation is done with CohomologyRing!

        ## main attributes
        self.prefix = COHO_prefix()()  # that way, we can distinguish the instances of COHO in the Singular interface
        self._Terminator = COHO_Terminator(self.GenS, self.prefix) # ensures that self's data in the singular interface are killed
        self.GStem = GStem
        ###########
        ## Make the group data on disc ready for use!
        if Hfinal is None:
            try: # we assume that *if* data are present then they are alright. pGroupCohomology.CohomologyRing takes care of this.
                f = file(self.gps_folder+GStem+'.nontips','r')
                f.close()
            except IOError:
                if q==1:
                    raise ValueError, "The group data for %s are not present in folder %s, and we don't know how to create them"%(GStem,gps_folder)
                makeGroupData(q,n,folder=root)
        else:
            makeSpecialGroupData(Hfinal,GStem,folder=root)
        self.Resl  = RESL(GStem, self.gps_folder, self.res_folder)
        self.Resl.indentation(indent)
        self.Gen = [] # Gen contains the list of generators
        self.firstOdd = 0
        self.Rel = []  # shall be a list of polynomials given by strings
        self.knownDeg = 0
        self.completed = False
        self.suffDeg = -1
        self.degvec = []

        ## attributes of more internal use
        #if (not kwds.get('GStem')) and (Hfinal is None) and (n==gap('NumberSmallGroups(%d)'%(q))):
        if n==gap('NumberSmallGroups(%d)'%(q)):
            self.setprop('ElAb',kwds.get('ElAb',True))
        self.Triangular = {} # Contains (for each degree) an echelon basis for decomposable classes.
        self.NilBasis = {} # Contains (for each degree) an echelon basis for the nilpotent classes.
                           # Only makes sense under the implicit assumption that the
                           # cohomology ring is a product of polynomial and outer algebras.
        self.lastRel = 0
        self.subgps = {}
        self.RestrMaps = {}
        self.RelG = None # shall be a list of polynomials given by strings
        self.Monomials = {} # keyname of standard monomial -> COCH
        self.StdMon = {0:{'1':singular('1')}}
        self.RelGName = ''
        self.Dickson = [] # shall contain strings that eventually help to define a filter regular sequence
        self.alpha = None # used in Dickson's criterium. Is very likely to be -1, if the cohomology ring is computed
        self.setprop('DicksonExp',kwds.get('DicksonExp',3))  # up to what p-power will we try to lift Dickson classes?
        if self.ElAb:
            self.setprop('auto',2)                     # up to what degree will we use the autolift method?
        else:
            self.setprop('auto',kwds.get('auto',4))    # Creation of basic autolift data in high degrees may be expensive
        self.setprop('useFactorization', kwds.get('useFactorization',True))

        ###########
        ## Prepare Singular
        if self.Resl.coef()!=2: # non-commutative case
            singular.LIB('ncall.lib')
        singular.LIB('general.lib')
        singular.LIB('poly.lib')
        singular.LIB('dickson.lib')
        singular.LIB('filterregular.lib')
        singular.eval('option(redSB)');
        singular.eval('int i')
        self.SingularTime = singular.cputime()
        self.CElPos = 1
        self.CenterRk = None
        self.NumSubgps = 0
        self.MaxelPos = []
        self.MaxelRk  = []
        self.pRank = None
        self.__dict['useElimination'] = kwds.get('useElimination')
        if kwds.get('useSlimgb'):
            self.setprop('useSlimgb',True)
        elif kwds.get('useStd'):
            self.setprop('useStd',True)

        #############
        ## Initialisation of subgroups
        if Subgroups:
            self.InitSubgroups()
        Ring.__init__(self,GF(self.Resl.coef()))

    ## there will be no copy method
 
    def __call__(self, s):
        """
        Interprete a string as a cochain representing a cohomology class in self

        INPUT:

        s -- a string (representing a polynomial in the generators of self) or a cochain 

        OUTPUT:

        A cochain of self.

        * If the input is an element of self, it is returned without change.
        * If the input is a string, it is attempted to evaluate it
          as an algebraic expression in the generators of self.
        * Otherwise, if the input can be converted into an element of the base field
          then a cochain of degree zero is returned.

        If all of the above fails, a ``TypeError`` is raised.


        EXAMPLES:
        
        We use a temporary root directory, that will be removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.Gen
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            sage: print H('b_1_1+b_1_0')
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]
            sage: print H.2+H.3
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]
            sage: print H(H.2+H.3)
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]
            sage: H.subgps[4,2](H.2+H.3)
            Traceback (most recent call last):
            ...
            TypeError: Cochain belongs to a different cohomology ring, namely H^*(D8; GF(2))

        """
        if isinstance(s, COCH):
            if s.parent() is self:
                return s
            else:
                raise TypeError, "Cochain belongs to a different cohomology ring, namely %s"%(repr(s.parent()))
        if not isinstance(s,basestring):
            try:
                s = self.base_ring()(s)
            except TypeError:
                raise TypeError, "We don't know how to interprete %s"%(s)
            # The following is necessary, since by some oddity the above might
            # return a tuple of an error with an error message!
            if not s in self.base_ring():
                raise TypeError, "We don't know how to interprete %s"%(s)
            return COCH(self, 0, repr(s), [s])
        L = {}
        for X in self.Gen:
            L[X.name()] = X
        from sage.misc.sage_eval import sage_eval
        return sage_eval(s,locals=L)

    ## Homomorphisms -- Option "cat" is not supported yet
    def Hom(self, other, cat=None):
        """
        Return a homset containing the induced homomorphisms between two cohomology rings

        INPUT:

        - A cohomology ring that will be the codomain for all elements of the homset
        - ``cat`` (not implemented) a category of homomorphisms.

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H2 = CohomologyRing.user_db(gap('DihedralGroup(8)'), GroupName = 'DihedralGroup(8)',websource=False)
            sage: H1.Hom(H2)
            Set of Homomorphisms from H^*(D8; GF(2)) to H^*(DihedralGroup(8); GF(2))

        """
        if not isinstance(other, COHO):
            raise TypeError, "The codomain must be %s"%(self.__class__)
        try:
            return self._HomsetCache[other._key, cat]
        except KeyError:
            pass
        from pGroupCohomology.cochain import CohomologyHomset
        H = CohomologyHomset(self, other, cat=cat)
        self._HomsetCache[other._key, cat] = H
        return H

    def hom(self, m, other, M=None, d=0):
        """
        Create an induced homomorphism.

        INPUT:

        - ``m``, which usually is a group homomorphism defined in the Gap interface,
          but in principle could also be a :class:`~pGroupCohomology.mtx.MTX` matrix
          describing a homomorphism of the group algebras (not documented). The
          domain must be the underlying group of ``other``, the codomain must be
          the underlying group of ``self``.
        - ``other``, a cohomology ring
        - ``M`` (optional :class:`~pGroupCohomology.mtx.MTX` matrix): By this matrix one
          could prescribe a mapping of the first terms of the respective resolutions
          (not documented, and not needed for induced maps of group homomorphisms).
        - ``d`` (optional, default 0): Degree of the map. If it is non-zero then
          ``M`` must be provided.

        NOTE:

        The resulting homomorphism is cached. 

        EXAMPLES:
          
        We first create the cohomology rings for two different presentations of the
        dihedral group of order 8. We use a temporary directory for saving data; it
        will be removed as soon as Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: G1 = gap('SmallGroup(8,3)')
            sage: H1 = CohomologyRing.user_db(8,3,websource=False)
            sage: H1.make()
            sage: G2 = gap('DihedralGroup(8)')
            sage: H2 = CohomologyRing.user_db(G2, GroupName = 'DihedralGroup(8)',websource=False)
            sage: H2.make()

        In order to obtain reproducible doc tests, we switch to permutation
        groups that are, from the perspective of our programs, equivalent to 
        ``G1`` and ``G2``, and provide an explicit group isomorphism::

            sage: phi = gap('GroupHomomorphismByImages( Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] ), Group( [ (1,5)(2,6)(3,8)(4,7), (1,3,2,4)(5,7,6,8) ] ), [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ], [ (1,5)(2,6)(3,8)(4,7), (1,8)(2,7)(3,6)(4,5) ] )')
            sage: H1.group()==phi.Source()
            True
            sage: H2.group()==phi.Range()
            True
            sage: phi.IsInjective()
            true
            sage: phi.IsSurjective()
            true
        
        After ensuring that ``phi`` is printed nicely, we obtain the induced map::

            sage: phi.SetName('"phi"')
            sage: phi_star = H2.hom(phi,H1)
            sage: phi_star
            phi^*
            sage: phi_star.domain()
            H^*(DihedralGroup(8); GF(2))
            sage: phi_star.codomain()
            H^*(D8; GF(2))
            sage: print [H1.cochain_to_polynomial(phi_star(X)) for X in H2.gens()]
            [1, a 0-Cochain in H^*(D8; GF(2)), b_1_1^2+c_2_2, a 2-Cochain in H^*(D8; GF(2)), b_1_1+b_1_0, a 1-Cochain in H^*(D8; GF(2)), b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            sage: phi_star is H2.hom(phi,H1)
            True
            sage: phi_star is loads(dumps(phi_star))
            True

    """
        return self.Hom(other)(m,M,d)

    ###########################################################
    ###########################################################
    ## Pickling and unpickling
    #
    # We now use __reduce__, since this allows for cacheing,
    # uniqueness of parent structures, and other nice features.
    ###########################################################
    ###########################################################

    def __reduce__(self):
        """
        Serialise ``self``

        TESTS::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H is loads(dumps(H))
            True
        
        """
        # self._key provides a unique description of the p-Group,
        # namely either a pair of integers (p,q) providing an address
        # in the SmallGroups library, or (s,) with a string s providing
        # the definition of a permutation group in Gap.
        # Moreover, self._key tells where to find self's data
        if self.__dict.has_key('_dont_save_the_State'):
            self.delprop('_dont_save_the_State')
        else:
            save(self.__getstate__(), self.dat_folder+'State')
        if self.root == COHO.user_db:
            StateFile = '@user_db@/'+self.GStem+'/dat/State'
        elif self.root == COHO.public_db:
            StateFile = '@public_db@/'+self.GStem+'/dat/State'
        else:
            StateFile = self._key[1]
        #print StateFile
        return COHO_unpickle, (self._key[0], StateFile)

    def __getstate__(self):
        """
        Returns data that allow __setstate__ to reconstruct self when unpickling.

        OUTPUT:
        
        A list of length 34

        EXAMPLES:
        
        We use a temporary root directory, that will be removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: from pGroupCohomology.cohomology import COHO
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            sage: len(H.__getstate__())
            34
    
        Now, we initialize a cohomology ring "without properties"::
        
            sage: K = COHO()
            sage: print K.knownDeg
            None

        We change ``K`` into the same state as ``H``, making ``K`` a copy of ``H``::

            sage: K.__setstate__(H.__getstate__())
            sage: print K
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            
        """
        #print 'getstate'
        #########
        ## The following data need special treatment:
        ## subgroups:
        cdef COCH X
        if self.subgps:
            subgps=[(i,gp.gps_folder+'H'+gp.GStem) for i,gp in self.subgps.items()]
        else:
            subgps = []
        if self.subgpDickson:
            self.setprop('sgpDickson',[(i,[[X.deg(),X.name(),X.MTX()] for X in gpDickson]) for i,gpDickson in self.subgpDickson.items()])
        else:
            self.setprop('sgpDickson',[])
                  # new version: subgroup is already saved (as a group on its own!), and this is the location.
        ## Singular elements:
        StdMon = []
        # we need to set the singular ring of self
        if singular.eval('defined(%sr(%d))'%(self.prefix,self.knownDeg))=='1':
            singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
            for nkey in self.StdMon.keys():
                StdMon.append([nkey,[]])
                for monkey in self.StdMon[nkey].keys():
                    StdMon[-1][1].append([monkey, str(self.StdMon[nkey][monkey]).split(',\n')])
            #DG=str(singular('DG')).split(',\n')
            DG = [s.split('=')[1] for s in singular.eval('DG').split('\n')]
        else:
            DG=['0']
        #########
        ## Apparently it is a problem to pickle dictionaries.
        ## Cochains:
        Gen = [[X.Deg,X.Name,X.Data,X.Rdeg,X.Ydeg] for X in self.Gen]
        Triangular = [[i,[[X.Deg,X.Name,X.Data] for X in Tr]] for i,Tr in self.Triangular.items()]
    
        ## Chain maps:
        if self.RestrMaps:
            RestrMaps = [[i,[Restr[0], [Restr[1].G_map(),[Restr[1].__getitem_name__(k) for k in range(Restr[1].knownDeg()+1)]]]] for i,Restr in self.RestrMaps.items()]
        else:
            RestrMaps = []
        ## Save self.Monomials and self.Resl externally, instead of returning self.Resl.__reduce()[1]
        Resl = self.res_folder+'R'+self.GStem+'.sobj'
        save(self.Resl,Resl)
        self.exportMonomials()

        ######
        ## ROOT: Should be explicitly defined as a property of self.
        ## If it isn't it can still be infered.
        ## We have a special location for public and private data bases.
        ## We handle these explicitly. It is possible for the user to
        ## copy the data to a completely different location -- then
        ## we are confident that loading still works, but we can't give
        ## a guarantee since this is a difficult business...
        root = self.root
        if root is None: # try to infer the root
            root = '/'.join(self.gps_folder.split('/')[:-1])
        if len(root) and root[-1]!='/':
            root = root + '/'
        if root == COHO.user_db:
            self.setprop('root','@user_db@')
        elif root == COHO.public_db:
            self.setprop('root','@public_db@')
        else:
            self.setprop('root',root)
        __dict = self.__dict.items()
        self.setprop('root',root)
        rootlength = len(root)
        if self.Dickson:
            if isinstance(self.Dickson[0],COCH): # ... then self is a subgroup, hence, Dickson data depend on the super-group
                return (RestrMaps,self.degvec,self.CElPos,self.CenterRk,self.gps_folder[rootlength:],self.Rel,self.res_folder[rootlength:],subgps,self.dat_folder[rootlength:],Triangular,self.inc_folder[rootlength:],self.lastRel,self.MaxelPos,self.MaxelRk,self.pRank,self.knownDeg,self.RelG,Gen,StdMon,self.NilBasis,self.SingularTime,self.completed,'Monomials',self.suffDeg,self.Automatic,self.RelGName,self.NumSubgps,self.GStem,Resl[rootlength:],self.firstOdd,DG,  []  ,self.alpha, __dict)
        return (RestrMaps,self.degvec,self.CElPos,self.CenterRk,self.gps_folder[rootlength:],self.Rel,self.res_folder[rootlength:],subgps,self.dat_folder[rootlength:],Triangular,self.inc_folder[rootlength:],self.lastRel,self.MaxelPos,self.MaxelRk,self.pRank,self.knownDeg,self.RelG,Gen,StdMon,self.NilBasis,self.SingularTime,self.completed,'Monomials',self.suffDeg,self.Automatic,self.RelGName,self.NumSubgps,self.GStem,Resl[rootlength:],self.firstOdd,DG,self.Dickson,self.alpha, __dict)

    def __setstate__(self, s, newroot=None): #s = (RestrMaps,degvec,CElPos,CenterRk,gps_folder,Rel,res_folder,subgps,dat_folder,Triangular,inc_folder,lastRel,MaxelPos,MaxelRk,pRank,knownDeg,RelG,Gen,StdMon,NilBasis,SingularTime,completed,Monomials,suffDeg,Automatic,RelGName,NumSubgps,GStem,Resl,firstOdd,DG,Dickson,alpha, Dict):
        """
        Set the state of self

        INPUT:
        
        A list of length 34, as provided by the __getstate__() method of a cohomology ring.

        NOTE:
        
        In order to save time, not all data are fully reconstructed. For instance,
        the data concerning maximal elementary abelian subgroups are only fully
        reconstructed if it is really necessary for further computation.
        
        EXAMPLES:
        
        We use a temporary root directory, that will be removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: from pGroupCohomology.cohomology import COHO
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
    
        Now, we initialize a cohomology ring "without properties"::
        
            sage: K = COHO()
            sage: print K.knownDeg
            None

        We change ``K`` into the same state as ``H``, making ``K`` a copy of ``H``::

            sage: K.__setstate__(H.__getstate__())
            sage: print K
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]

        However, not all attributes are present at that point. For example,
        the attribute 'subgps' does not exist, yet::
        
            sage: K.__dict__.has_key('subgps')
            False

        Nevertheless, it is no problem to call the 'subgps' attribute. The technical
        details:
        
        - First, from the point of view of Python, 'subgps' is no attribute.
        - Hence, H.subgps results in calling H.__getattr__('subgps')
        - We implemented the __getattr__() method such that a call for 'subgps
          will reconstruct and return the subgroup data.
        - Hence, eventually, 'subgps' will be known to Python as an attribute.
 
        The following lines should explain these technical details::
        
            sage: K.__dict__.has_key('subgps')
            False
            sage: K.subgps
            {(4, 2):  H^*(SmallGroup(4,2); GF(2)), (2, 1): H^*(SmallGroup(2,1); GF(2))}
            sage: K.__dict__.has_key('subgps')
            True
        
        """
        if len(s)!=34:
            raise ValueError, "wrong number of arguments"
        RestrMaps,degvec,CElPos,CenterRk,gps_folder,Rel,res_folder,subgps,dat_folder,Triangular,inc_folder,lastRel,MaxelPos,MaxelRk,pRank,knownDeg,RelG,Gen,StdMon,NilBasis,SingularTime,completed,Monomials,suffDeg,Automatic,RelGName,NumSubgps,GStem,Resl,firstOdd,DG,Dickson,alpha, Dict = s
            
        opts = copy(OPTION.opts)
        OPTION.opts['autolift']=False
        OPTION.opts['prot']=False
        OPTION.opts['timing']=False
        OPTION.opts['save']=False
        OPTION.opts['reload']=True
        
        ## First, the "Additional Properties" are reconstructed
        for X,Y in Dict:
            self.__dict[X]=Y

        ##########
        ## Try to find the folder in which the cohomology data are rooted.
        ## We have special places for public and private data bases
        root = self.root
        if root == '@user_db@':
            root = COHO.user_db
        elif root == '@public_db@':
            root = COHO.public_db

        ## DEL
        #if root is None: # try to infer the root from gps_folder -- happens for old pickles
        #    root = '/'.join(gps_folder.split('/')[:-2])
        if len(root) and (root[-1]!='/'):
            root = root + '/'
        rootlength = len(root)
        oldroot = None
        if gps_folder.startswith(root): # this is when loading Simon King's old pickles
            gps_folder = gps_folder[rootlength:]
        if inc_folder.startswith(root):
            inc_folder = inc_folder[rootlength:]
        if res_folder.startswith(root):
            res_folder = res_folder[rootlength:]
        if dat_folder.startswith(root):
            dat_folder = dat_folder[rootlength:]
        if isinstance(Resl,basestring) and Resl.startswith(root):
                Resl = Resl[rootlength:]

        ## DEL
        #if root == '':
        #    newroot = newroot or OPTION.opts['PWD']
        if (newroot is not None) and (root!=newroot):
            oldroot = root
            root = newroot
            if len(root) and (root[-1]!='/'):
                root = root + '/'
        self.setprop('root', root)

        self.degvec = degvec
        self.CElPos = CElPos
        self.CenterRk = CenterRk
        self.gps_folder = root+gps_folder
        self.Rel = Rel
        self.res_folder = root+res_folder
        self.dat_folder = root+dat_folder
        self.inc_folder = root+inc_folder
        self.lastRel = lastRel
        self.MaxelPos = MaxelPos
        self.MaxelRk = MaxelRk
        self.pRank = pRank
        self.knownDeg = knownDeg
        self.RelG = RelG
        self.SingularTime = SingularTime
        self.completed = completed
        self.suffDeg = suffDeg
        self.RelGName = RelGName
        self.NumSubgps = NumSubgps
        self.GStem = GStem
        self.Dickson = Dickson
        self.alpha = alpha
        if isinstance(Resl,basestring):
            if (oldroot is not None):
                OPTION.opts['@oldroot@'] = oldroot
                OPTION.opts['@newroot@'] = root
            try:
                self.Resl = load(root+Resl)
            except (IOError, RuntimeError):
                raise IOError, "Unable to read resolution saved at "+root+Resl
            if OPTION.opts.has_key('@newroot@'):
                OPTION.opts.__delitem__('@newroot@')
                try:
                    if OPTION.opts['prot']:
                        print "Try to update resolution data on disk"
                    save(self.Resl, root+Resl)
                    if OPTION.opts['prot']:
                        print "> successful"
                except (IOError,RuntimeError):
                    if OPTION.opts['prot']:
                        print "WARNING: No write permissions"
            if OPTION.opts.has_key('@oldroot@'):
                OPTION.opts.__delitem__('@oldroot@')
        else:
            MaxDeg = max([0]+[X[0] for X in Gen])
            self.Resl = None
            for K in Resl[4].keys():
                self.Resl = (K[1]).resolution()
                break
            #if self.Resl is None: # This is for dealing with very old data - should be removed
            #    self.Resl = resl_sparse_unpickle(Resl[0],Resl[1],Resl[2],MaxDeg,Resl[4],Resl[5],Resl[6])
        Ring.__init__(self,GF(self.Resl.coef()))

        self.firstOdd = firstOdd
        n = self.knownDeg
        #########
        ## The following data need special treatment:
        ## Subgroups: Quick-Loading --- they will be fully reconstructed later!
        if subgps:
            if (oldroot is not None):
                oldroot = '@'+oldroot
                for i in range(len(subgps)):
                    subgps[i]=(subgps[i][0],('@'+subgps[i][1]).replace(oldroot,newroot))
            self.setprop('SUBGPS',subgps)

        ## Cochains:
        self.Gen = [COCH(self,X[0],X[1],X[2],rdeg=X[3],ydeg=X[4]) for X in Gen]
        self.Triangular = {}
        for i,Tr in Triangular:
            self.Triangular[i] = [COCH(self,X[0],X[1],X[2]) for X in Tr]
        self.NilBasis = NilBasis
        if isinstance(Monomials,basestring):
            self.Monomials = {'bla':1}
            self.importMonomials()
        else:
            self.Monomials = {}
            for i,Mo in Monomials:
                self.Monomials[i] = COCH(self,Mo[0],Mo[1],Mo[2])

        ## Chain maps: Quick-Loading --- they will be fully reconstructed later!
        if subgps:
            self.setprop('RESTRMAPS', RestrMaps)

        ###
        ## The key for uniqueness of parent structures
        if self._key is not None: # we will explicitly provide the second half of the key (file location),
                                  # but will try to re-use the first half (group description)
            self.setprop('_key', (self._key[0], root+dat_folder+'State'))
        if self._key is None:
            try:
                q,n = [Integer(nb) for nb in GStem.split('gp')]
                #print 'Found', (q,n)
                self.setprop('_key', ((q,n),self.dat_folder+'State'))
            except (TypeError, ValueError):
                print 'WARNING: No good group key found'
                print 'Please chose a string ``s`` that provides a'
                print 'definition of a finite p-Group in Gap and do'
                print "    sage: H.setprop('_key',(s,%s))"%(self.dat_folder+'State')
                print 'where ``H`` denotes the name of this cohomology ring.'
                self.setprop('_key', ((GStem,),dat_folder+'State'))
        # use the updated _key for inserting self into the cache
        from pGroupCohomology import _cache
        if not ((self._key is None) or _cache.has_key(self._key)):
            _cache[self._key] = self
        
        ################
        # Finally, we reconstruct the attributes in Singular:
        # GenS, StdMon; a ring and the relation ideal in singular.
        cdef int p = self.Resl.coef()
        if p!=2:
            singular.LIB("ncall.lib")
        singular.LIB('general.lib')
        singular.LIB('poly.lib')
        singular.LIB('dickson.lib')
        singular.LIB('filterregular.lib')
        if singular.eval('defined(i)')=='0':
            singular.eval('int i')
        else:
            if singular.eval('typeof(i)')!='int':
                OPTION.opts = copy(opts)
                raise RuntimeError, "Singular has defined that i is not an integer - but we need i as integer!"
        if Gen:
            if self.__dict.get('use_dp'):
                if len(self.degvec)==1:
                    singular.eval('ring tmp = %d,(%s),%s'%(p, ','.join([x.name() for x in self.Gen]), '(a(%d),dp)'%(self.degvec[0])))
                else:
                    singular.eval('ring tmp = %d,(%s),%s'%(p, ','.join([x.name() for x in self.Gen]), '(wp%s)'%(str(tuple(self.degvec)))))
            else:
                self._makeOrderMatrix_()
                singular.eval('ring tmp = %d,(%s),M(%sM)'%(p, ','.join([x.name() for x in self.Gen]), self.prefix))
            if self.Resl.coef()!=2:   # non-commutative case
                singular.eval('degBound = 0')
                singular.eval('def %sr(%d) = SuperCommutative(%d,%d)'%(self.prefix,n,self.firstOdd+1, len(self.Gen)))
            else:
                singular.eval('def %sr(%d) = tmp'%(self.prefix,n))
            singular.eval('setring %sr(%d)'%(self.prefix,n))
            self.GenS = singular('%sr(%d)'%(self.prefix,n))
        if self.RelG:
            singular.eval('ideal %sI = %s'%(self.prefix,','.join(self.RelG)))
        else:
            if self.Rel:
                singular.eval('ideal %sI = %s'%(self.prefix,','.join(self.Rel)))
                self.delprop('completeGroebner')
            else:
                singular.eval('ideal %sI'%(self.prefix))
    
        self.StdMon = {}
        for nkey,StdMonN in StdMon:
            self.StdMon[nkey]={}
            for monkey,STD in StdMonN:
                if monkey!='1':
                    self.StdMon[nkey][monkey] = singular.ideal(STD)
                else:
                    self.StdMon[nkey][monkey] = singular('1')
        singular.eval('ideal DG = '+','.join(DG))
        OPTION.opts = copy(opts)

#    def export(self,f, compress = None):
#        """
#        Export group data into an archive file
#
#INPUT:
#    f -- a filename (string)
#    compress [option] -- either 'gz', 'bz' or '', which means that the
#        resulting file is compressed using gzip (default), bzip2 or is
#        not compressed.
#
#When a cohomology ring ``H`` is computed, data are stored in various files.
#When saving H with ``save(H,'filename')``, then the file ``'filename.sobj'``
#will only contain data that are needed to find the files associated
#with ``H``, and to reconstruct ``H`` from these files. This is much faster
#than storing all data in a single file. However, it is not so nice
#when moving the data for ``H`` to a different location. The export method
#produces a tar file.
#
#EXAMPLES:
#We use temporary files, that will be automatically removed when quitting.
#We choose here bzip2-compression. With compress='gz' or without specifying
#'compress', gzip-compression would be used. compress='' means no compression.
#::
#
#    sage: from pGroupCohomology.cohomology import COHO
#    sage: root1 = tmp_filename()
#    sage: root2 = tmp_filename()
#    sage: f1 = tmp_filename()
#    sage: f2 = tmp_filename()
#    sage: H1 = COHO(8,3,root=root1)
#    sage: H1.make()
#    sage: H1.export(f1,compress='bz2')
#    sage: import os
#    sage: os.rename(f1,f2)
#    sage: H2 = COHO.import_archive(f2,root=root2)
#    sage: H1.root==H2.root
#    False
#    sage: H1
#    Cohomology ring of D8 with coefficients in GF(2), computation complete
#    sage: H2
#    Cohomology ring of D8 with coefficients in GF(2), computation complete
#        """
#        save(self, self.autosave_name())
#        # put the folders for self and the subgroups into a list,
#        # checking if the data are correctly rooted
#        import tarfile
#        save_opts = copy(OPTION.opts)
#        OPTION('noprot')
#        L = []
#        if (self.root is None) or (self.root+self.GStem+'/' != self.gps_folder):
#            # try to get the root right
#            if self.gps_folder.endswith(self.GStem+'/'):
#                root = self.gps_folder[:-(len(self.GStem)+1)]
#                self.setprop('root', root)
#            else:
#                raise RuntimeError, "The structure of data for %s is broken -- unable to export"%(self.GStem)
#        L.append([self.gps_folder, self.GStem])
#        #if self.subgps is None:
#        #    self.reconstructSubgroups()
#        for G in self.subgps.values():
#            L.append([G.gps_folder, G.GStem])
#
#        # create a tar file and put everything in, but with paths *relative* to root!
#        if compress is None:
#            compress = 'gz'
#        if not isinstance(compress,basestring):
#            raise TypeError, "the optional 'compress' parameter must be a string:\n    'gz' [gzip compression, the default],\n    'bz2' [bzip2 compression], or\n    '' [no compression]"
#        T = tarfile.open(f,'w:'+compress)
#        for fullname, name in L:
#            T.add(fullname,name)
#        T.close()
#        OPTION.opts = save_opts
#        if OPTION.opts['prot']:
#            print "Export successful"

    def autosave_name(self):
        """
        Return the name of the file under which self is automatically saved

        EXAMPLES:
        
        We use a temporary root directory for creating our cohomology ring.
        The directory will be removed, when Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: K=load(H.autosave_name())
            sage: print K
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            
        """
        return self.gps_folder+'H'+self.GStem+'.sobj'

####################################

    option = COHO_option

    def exportMonomials(self):
        """
        Save previously computed standard monomials in a standard location

        NOTE:

        This method is used when saving a cohomology ring. The saved standard
        monomials can be reloaded with :meth:`importMonomials`, but this requires
        that the dictionary ``self.Monomials`` has the key ``'bla'``. 

        EXAMPLES:
        
        We use a temporary root directory, that will be removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: L1 = H.Monomials.items()
            sage: L1.sort()
            sage: L1
            [('b_1_0', b_1_0, a 1-Cochain in H^*(D8; GF(2))),
             ('b_1_0b_1_0', (b_1_0)*(b_1_0), a 2-Cochain in H^*(D8; GF(2))),
             ('b_1_0b_1_0b_1_0', (b_1_0)*((b_1_0)*(b_1_0)), a 3-Cochain in H^*(D8; GF(2))),
             ('b_1_0b_1_1', (b_1_0)*(b_1_1), a 2-Cochain in H^*(D8; GF(2))),
             ('b_1_1', b_1_1, a 1-Cochain in H^*(D8; GF(2))),
             ('b_1_1b_1_1', (b_1_1)*(b_1_1), a 2-Cochain in H^*(D8; GF(2))),
             ('b_1_1b_1_1b_1_1', (b_1_1)*((b_1_1)*(b_1_1)), a 3-Cochain in H^*(D8; GF(2))),
             ('c_2_2', c_2_2, a 2-Cochain in H^*(D8; GF(2))),
             ('c_2_2b_1_0', (b_1_0)*(c_2_2), a 3-Cochain in H^*(D8; GF(2))),
             ('c_2_2b_1_1', (b_1_1)*(c_2_2), a 3-Cochain in H^*(D8; GF(2)))]
            sage: H.exportMonomials()
            sage: H.Monomials={'bla':1}
            sage: H.importMonomials()
            sage: L2 = H.Monomials.items()
            sage: L2.sort()
            sage: L1 == L2
            True
             
        """
        cdef COCH X
        if not(self.Monomials.has_key('bla')):
            save([[i,[X.Deg,X.Name,X.Data]] for i,X in self.Monomials.items()], self.dat_folder+'M'+self.GStem)

    def importMonomials(self):
        """
        Reload previously exported standard monomials

        NOTE:
        
        This method is used when reloading a cohomology ring.
        It is required that the dictionary self.Monomials has the key ``'bla'``.
        Otherwise, the monomials, that are stored in some standard location,
        will not be reloaded.

        EXAMPLES:
        
        We use a temporary root directory, that will be removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: L1 = H.Monomials.items()
            sage: L1.sort()
            sage: L1
            [('b_1_0', b_1_0, a 1-Cochain in H^*(D8; GF(2))),
             ('b_1_0b_1_0', (b_1_0)*(b_1_0), a 2-Cochain in H^*(D8; GF(2))),
             ('b_1_0b_1_0b_1_0', (b_1_0)*((b_1_0)*(b_1_0)), a 3-Cochain in H^*(D8; GF(2))),
             ('b_1_0b_1_1', (b_1_0)*(b_1_1), a 2-Cochain in H^*(D8; GF(2))),
             ('b_1_1', b_1_1, a 1-Cochain in H^*(D8; GF(2))),
             ('b_1_1b_1_1', (b_1_1)*(b_1_1), a 2-Cochain in H^*(D8; GF(2))),
             ('b_1_1b_1_1b_1_1', (b_1_1)*((b_1_1)*(b_1_1)), a 3-Cochain in H^*(D8; GF(2))),
             ('c_2_2', c_2_2, a 2-Cochain in H^*(D8; GF(2))),
             ('c_2_2b_1_0', (b_1_0)*(c_2_2), a 3-Cochain in H^*(D8; GF(2))),
             ('c_2_2b_1_1', (b_1_1)*(c_2_2), a 3-Cochain in H^*(D8; GF(2)))]
            sage: H.Monomials={'bla':1}

        Since, by default, the cohomology ring was automatically saved, the monomials
        are available in some file. So, we can import them::
        
            sage: H.importMonomials()
            sage: L2 = H.Monomials.items()
            sage: L2.sort()
            sage: L2 == L1
            True
             
        """
        cdef dict D = {}
        if self.Monomials.has_key('bla'):
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+'> > > Import monomials'
            Monomials = load(self.dat_folder+'M'+self.GStem)
            for i,Mo in Monomials:
                D[i] = COCH(self,Mo[0],Mo[1],Mo[2])
            self.Monomials = D
        

    def decomposable_classes(self, int n):
        """
        Return decomposable classes in degree n
        
        EXAMPLES:
        
        We use a temporary root directory, that will be removed when
        Sage is quit.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.decomposable_classes(3)
            [(b_1_0)*((b_1_0)*(b_1_0)), a 3-Cochain in H^*(D8; GF(2)),
             (b_1_1)*((b_1_1)*(b_1_1)), a 3-Cochain in H^*(D8; GF(2)),
             (b_1_0)*(c_2_2), a 3-Cochain in H^*(D8; GF(2)),
             (b_1_1)*(c_2_2), a 3-Cochain in H^*(D8; GF(2))]

        """
        cdef list DecGen = self.Triangular.get(n)
        if DecGen is not None:
            return DecGen
        cdef RESL R
        R = self.Resl
        DecGen=[]  # will be a triangular basis of the sub space of decomposables of H^n
        cdef list VSGen=self.Resl.rank(n)*[1]
        cdef list pivot=[]
        cdef COCH Cand # will be the 'next candidate' for a decomposable generator or a relation
        newRel = False
        
        cdef list MonExp,lastPiv, ComL
        cdef int i,k,m,f,j
        cdef int lenMonExp, lAn, lenDecGen
        if self.Gen!=[]:  # there can only be decomposables if there are generators, yet
            ####################################
            # We must lift self.RelG to the new degree.
            # First, switch to the previous ring
            self.set_ring() #singular.eval('setring %sr(%d)'%(self.prefix,n-1))
            sizeG = int(singular.eval('ncols(%sI)'%(self.prefix)))

            #######################################
            # Get standard monomials:
            self.__makeStdMon(n,"Mon")
            if singular.eval('Mon[1]')!='0':
                if OPTION.opts['prot']:
                    print (R.indent*'::: ')+"We got %s standard monomials"%(singular.eval('ncols(Mon)'))
                # get list of exponent vectors of the standard monomials
                MonExp = [[int(y) for y in x.split(',\n')] for x in \
                          (singular.eval('for (i=1;i<=ncols(Mon);i++) { print(leadexp(Mon[i]));print(\";\");}')+'\n').split('\n;\n') if x]
                lenMonExp = len(MonExp)

                ct=cputime()
                wt=walltime()
                
                #######################################
                # Perform the products
                for i from 0<= i < lenMonExp:
                    if OPTION.opts['prot']:
                        print (R.indent*'::: ')+'Monomial',i
                        print (R.indent*'::: ')+'> Candidate: '+singular.eval('Mon[%d]'%(i+1))
                        print (R.indent*'::: ')+'> Express monomial as a Cochain'
                    Cand = self.MonToProd(MonExp[i])
                    f,j = Cand.Data.lead()
                    IsMonomial = True
                    lenDecGen = len(DecGen)
                    if f:
                        for k from 0 <= k < lenDecGen:
                            if (j <= pivot[k]) and Cand.Data[0,pivot[k]]:
                                Cand = Cand - (DecGen[k])*Cand.Data[0,pivot[k]]
                                f,j = Cand.Data.lead()
                                IsMonomial = False
                    if f: # We found a decomposable generator of H^n
                        #if OPTION.opts['prot']:
                        #    print (R.indent*'::: ')+'Decomposable cochain found\n'
                        DecGen.append(Cand/f)
                        if len(DecGen[-1].name()) < 850:
                            singular.eval(('DG[%d]='%(len(DecGen)))+DecGen[-1].name())
                        else:
                            ComL = Cand.name().split('-')
                            DGstr = 'DG[%d]'%(len(DecGen)) 
                            singular.eval(DGstr+'='+ComL[0])
                            DGstr = DGstr + '=' + DGstr + '-'
                            for trm in ComL[1:]:
                                singular.eval(DGstr+trm)
                            singular.eval('DG[%d]=DG[%d]/%d'%(len(DecGen),len(DecGen),f))
                        if not IsMonomial:
                            DecGen[-1].setname('DG[%d]'%(len(DecGen)))
                        pivot.append(j)
                        VSGen[j]=0
                    else: ## We found a relation in degree n -- this must not happen,
                          ## since we assume that the ring structure is known out to degree at least n
                        raise RuntimeError, "The ring structure must be known at least out to degree %d"%(n)
                if OPTION.opts['timing']:
                    ct=cputime(ct)
                    wt=walltime(wt)
                    print "Time for extracting relations:"
                    print "   CPU:  %.2f\n   Wall: %.2f"%(ct,wt)
                    ct=cputime()
                    wt=walltime()
            else:
                if OPTION.opts['prot']:
                    print (R.indent*'::: ')+"There are no standard monomials in degree %d"%(n)
        self.Triangular[n] = DecGen
        return DecGen

    def reconstructSubgroups(self):
        """
        When loading a cohomology ring, the subgroups are not reconstructed immediately. This is done here.

        Note that it is not necessary to call this function manually. It is 
        done automaticall as soon as the subgroups are requested.
        
        EXAMPLES:
        
        We use a temporary file, that will be automatically removed when 
        Sage is quit.
        ::
        
            sage: from pGroupCohomology import CohomologyRing, _cache
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H=CohomologyRing.user_db(8,3)

        In this example, we do not want that the cohomology rings are cached.
        Hence, we destroy the relevant cache entry::
            
            sage: _cache.__delitem__(H._key)
            sage: K = loads(dumps(H))
            sage: K is H
            False
            sage: K.__dict__.has_key('subgps')
            False
            sage: K.reconstructSubgroups()
            sage: K.__dict__.has_key('subgps')
            True
            
        """
        from pGroupCohomology import CohomologyRing
        if not self.SUBGPS:
            return
        self.subgps = {}
        root = self.root
        DoProtocol = OPTION.opts['prot']
        saveopts = copy(OPTION.opts)
        for i,L in self.SUBGPS:
            if DoProtocol:
                print (self.Resl.getindent()*'::: ')+"Inserting SmallGroup(%d,%d) as a subgroup"%(i[0],i[1])
            self.subgps[i] = CohomologyRing.user_db(i[0],i[1], websource=False, options = ('noprot','notiming')) #in order to avoid problems with write permission
            self.subgps[i].setprop('ElAb', True) 
            self.subgps[i].Resl.indentation(self.Resl.getindent()+1)
            self.subgps[i].make()
            if not (self.sgpDickson or self.useElimination):
                self.dickson_in_subgroup(i)#subgps[i].dickson_in_subgroup(self.pRank,self.CenterRk)
        OPTION.opts = saveopts
        if self.degvec:
            n=max(self.degvec)
            for sgp in self.subgps.values():
                while (sgp.knownDeg < n):
                    sgp.next(Forced=True,KeepDecomposables=True,ElabSubgp=True)
        if self.sgpDickson:
            self.subgpDickson = {}
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"Reconstructing subgroup data"
            for i, DicksonList in self.sgpDickson:
                self.subgpDickson[i] = [COCH(self.subgps[i],X[0],X[1],X[2]) for X in DicksonList]
        ## Chain maps:
        self.RestrMaps = {}
        for i,Restr in self.RESTRMAPS:
            self.RestrMaps[i]=[Restr[0], self.hom(Restr[1][0],self.subgps[Restr[0]],Restr[1][1][0], 0)]  #ChMap(self.subgps[Restr[0]].Resl,self.Resl,Restr[1][0],Restr[1][1][0],0)]
            self.RestrMaps[i][1][1]=Restr[1][1][1:]    # This inserts the list of lifts;
        self.__dict.__delitem__('RESTRMAPS')
        if OPTION.opts['sparse']:
            for i,X in self.RestrMaps.items():
                X[1].exportData(self.inc_folder+self.GStem+'sg'+str(i)+'_')
        self.__dict.__delitem__('SUBGPS')
        if self.sgpDickson:
            self.__dict.__delitem__('sgpDickson')
        
    def __cmp__(self,other):
        """
        ``self`` and ``other`` are considered equal, if their unique keys
        describe the *same* group (not just isomorphic groups), and if
        the generators and relations match.

        NOTE:

        Usually, one would create a cohomology ring using :func:`~pGroupCohomology.CohomologyRing`.
        Then, the resulting cohomology rings are cached, so that two cohomology
        rings of the *same* group with data stored in the *same* location are
        not only equal but identical.

        TESTS:

        We use a temporary file, that will be automatically removed when 
        Sage is quit.
        ::
        
            sage: from pGroupCohomology.cohomology import COHO
            sage: tmp_root1 = tmp_filename()
            sage: tmp_root2 = tmp_filename()
            sage: H1 = COHO(8,3,root=tmp_root1)
            sage: H2 = COHO(8,3,root=tmp_root2)
            sage: H1.make()
            sage: H2.make()
            sage: H1 == H2
            True
            sage: H1 is H2
            False
            sage: H3 = COHO(8,2,root=tmp_root1)
            sage: H3.make()
            sage: H1 == H3
            False
            sage: H1 < H3
            True

        """
        if not isinstance(other,self.__class__):
            return -1
        if self._key[0] != other._key[0]:
            return cmp(self.group(),other.group())
        if [g.name() for g in self.Gen] != [g.name() for g in other.Gen]:
            return cmp([g.name() for g in self.Gen],[g.name() for g in other.Gen])
        return cmp(self.Rel, other.Rel)

    ######################################
    ### String representations and other information

    def __repr__(self):
        """
        Return a brief desctiption of the cohomology ring

        TESTS:
        
        We use a temporary file, that will be automatically removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H
            H^*(D8; GF(2))
            
        """
        if self.GroupName:
            GStem = self.GroupName
        else:
            GroupKey = self._key[0]
            if len(GroupKey)==1:
                GStem = GroupKey[0]
            else:
                GStem = 'SmallGroup(%d,%d)'%GroupKey
        return "H^*(%s; GF(%d))"%(GStem, self.Resl.coef())

    def __str__(self):
        """
        Return detailed information on the cohomology ring

        TESTS:
        
        We use a temporary file, that will be automatically removed when
        Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            
        """
        if self.GroupDescr:
            GStem = self.GroupDescr
        elif self.GroupName:
            GStem = self.GroupName
        else:
            GroupKey = self._key[0]
            if len(GroupKey)==1:
                GStem = GroupKey[0]
            else:
                GStem = 'SmallGroup(%d,%d)'%GroupKey
        if self.completed:
            return """
Cohomology ring of %s with coefficients in GF(%d)

Computation complete
Minimal list of generators:
%s
Minimal list of algebraic relations:
%s
            """%(GStem, self.Resl.coef(), '['+',\n '.join([X.__repr__() for X in self.Gen])+']', '['+',\n '.join([X for X in self.Rel])+']')
        else:
            return """
Cohomology ring of %s with coefficients in GF(%d)

Computed up to degree %d
Minimal list of generators:
%s
Minimal list of algebraic relations:
%s
            """%(GStem, self.Resl.coef(), self.knownDeg, '['+',\n '.join([X.__repr__() for X in self.Gen])+']', '['+',\n '.join([X for X in self.Rel])+']')

    def group(self):
        """
        Return the group over which self is defined, as a permutation group.

        OUTPUT:
        
        Let `G` be the group for which self is defined. Recall that it is assumed that the first,
        say `n`, generators of `G` form a minimal generating set. Then, the output of :meth:`group`
        is a permutation group `G_P`, defined in the Gap interface, with precisely `n` generators, so
        that mapping generator `i` of `G` to generator `i` of `G_P`, for `i=1,...,n`, results in
        a group isomorphism

        NOTE:

        ``CohomologyRing(self.group())`` is always equal to ``self``. It is even identical 
        with ``self``, unless ``self`` was defined in the form ``self = CohomologyRing(q,n)``.

        
        EXAMPLES:
        
        We use a temporary file, that will be automatically removed when
        Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource = False)
            sage: H.make()
            sage: G = H.group()
            sage: G
            Group( [ (1,2)(3,8)(4,6)(5,7), (1,3)(2,5)(4,7)(6,8) ] )
            sage: H1 = CohomologyRing.user_db(G, GroupName='NewD8', websource = False)
            sage: H1.make()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            sage: print H1
            Cohomology ring of NewD8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(NewD8; GF(2)),
             b_1_0, a 1-Cochain in H^*(NewD8; GF(2)),
             b_1_1, a 1-Cochain in H^*(NewD8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
            sage: H is H1
            False
            sage: H == H1
            True
            
        """
        if self._gap_group is not None:
            return self._gap_group
        from sage.interfaces.gap import gap
        if len(self._key[0])==1:
            self._gap_group = gap(self._key[0][0])
            if not gap('IsPermGroup(%s)'%(self._gap_group.name())):
                self._gap_group = gap('Group(verifiedMinGens(regularPermutationAction(%s: forceDefiningGenerators)))'%(self._gap_group.name()))
        else:
            self._gap_group=gap('Group(verifiedMinGens(regularPermutationAction(SmallGroup(%d,%d): forceDefiningGenerators)))'%(self._key[0][0],self._key[0][1]))
        return self._gap_group
        

###################
## Methods for setting/getting additional properties of the group
    def setprop(self,key, value):
        """
        Define a property of self

        INPUT:

        - ``key``, a string
        - ``value``, any object

        OUTPUT:

        ``value`` is now available as an attribute ``key`` of self.
        
        EXAMPLES:
        
        We use a temporary file, that will be automatically removed when
        Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource = False)
            sage: print H.foobar
            None
            sage: print H._foobar
            None
            sage: print H.foobar_
            None

        Note that if an attribute hasn't been defined, ``None`` is returned.
        The only exception are undefined attributes that start and end with
        an underscore::
        
            sage: H._foobar_
            Traceback (most recent call last):
            ...
            AttributeError: 'COHO' object has no attribute '_foobar_'

        Now, we provide ``_foobar_`` with a meaning, and then it works::

            sage: H.setprop('_foobar_', 'It works!')
            sage: print H._foobar_
            It works!
            
        """
        self.__dict[key] = value
        
    def __getattr__(self,key):
        """
        H.foo:  return the value of attribute/property 'foo' (None, if undefined)

        NOTE:

        There will only be an attribute error if foo is not an attribute and
        starts and ends with an underscore.

        TESTS:
        
        We use a temporary file, that will be automatically removed when
        Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: print H.foobar
            None
            sage: print H._foobar
            None
            sage: print H.foobar_
            None

        Note that if an attribute hasn't been defined, ``None`` is returned.
        The only exception are undefined attributes that start and end with
        an underscore::
        
            sage: H._foobar_
            Traceback (most recent call last):
            ...
            AttributeError: 'COHO' object has no attribute '_foobar_'

        Now, we provide ``_foobar_`` with a meaning, and then it works::

            sage: H.setprop('_foobar_', 'It works!')
            sage: print H._foobar_
            It works!

        """
        # If a cohomology ring is loaded from a file which was moved *together with
        # all the ring data* (!!) to a new location then an uninitialized ring is
        # returned. Hence, when working with it, very soon this __getattr__ method
        # will be called. At this point, _default_filename might tell us where the
        # file and hence the ring data are located!
        #print 'getattr',key
        if key == '_default_filename': # This ought to be a *proper* attribute, no fake attribute!
            raise AttributeError, "'pGroupCohomology.cohomology.COHO' object has no attribute '_default_filename'"
        if self.__dict.get('_need_new_root'):
            if OPTION.opts['prot']:
                print 'Files on disk were moved - trying to get things right'
            try:
                defaultname = self._default_filename
            except AttributeError:
                raise IOError, "Apparently you have moved the data for this cohomology ring.\n    The new location of the data can't be reconstructed"
            # now, infer the new root
            newroot = '/'.join(defaultname.split('/')[:-2]) + '/'
            try:
                ST = load('/'.join(defaultname.split('/')[:-1])+'/dat/State')
            except IOError:
                raise IOError, "Apparently you have moved the data for this cohomology ring.\n    The new location of the data can't be reconstructed"
            self.delprop('_need_new_root')
            self.__setstate__(ST, newroot=newroot)
            self.setprop('_dont_save_the_State', True)
            try:
                if OPTION.opts['prot']:
                    print "Try to update cohomology data on disk"
                save(self, self.autosave_name())
                if OPTION.opts['prot']:
                    print "> successful"
            except (IOError, RuntimeError):
                self.delprop('_dont_save_the_State')
                if OPTION.opts['prot']:
                    print "WARNING: No write permissions"
            return getattr(self, key)
        
        # After quickloading, the attributes "subgps" and "RestrMaps" are not defined.
        # Hence, if these attributes are required in that case, __getattr__ is called, 
        # and we reconstruct the subgroups first before returning the required data.
        if (key=="subgps") and (self.__dict.has_key('SUBGPS')):
            self.reconstructSubgroups()
            return self.subgps
        if (key=="RestrMaps") and (self.__dict.has_key('SUBGPS')):
            self.reconstructSubgroups()
            return self.RestrMaps
        try:
            return self.__dict[key]
        except KeyError:
            if key[0]==key[-1]=='_':
                raise AttributeError, "'COHO' object has no attribute '%s'"%(key)
            return None

    def delprop(self,key):
        """
        Delete a property of self.
        
        TESTS:
        
        We use a temporary file, that will be automatically removed when
        Sage is quit.
        ::

            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource = False)
            sage: H.setprop('_foobar_', 'It works!')
            sage: print H._foobar_
            It works!
            sage: H.delprop('_foobar_')
            sage: H._foobar_
            Traceback (most recent call last):
            ...
            AttributeError: 'COHO' object has no attribute '_foobar_'

        """
        self.__dict.pop(key,None)

    def properties(self):
        """
        List all property names of self

        EXAMPLES:
        
        We use a temporary root directory, that will be removed when 
        sage is quit. Also, we make sure that the ring will not be downloaded
        from the web.
        ::
        
            sage: from pGroupCohomology import CohomologyRing
            sage: tmp_root = tmp_filename()
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource = False)
            sage: H.properties()
            ['useFactorization',
             'useElimination',
             'GroupDescr',
             '_key',
             'auto',
             'GroupName',
             'DicksonExp',
             'KeepBases',
             'root']
            sage: H.make()
            sage: H.properties()
            ['ParameterDegrees',
             'sgpDickson',
             'useFactorization',
             'HilbertVectors',
             'useElimination',
             'GroupDescr',
             '_key',
             'auto',
             'rfdt',
             'GroupName',
             'DicksonExp',
             'fdt',
             'KeepBases',
             'completeGroebner',
             'root',
             'lastTested']
             
        """
        return self.__dict.keys()

    def gen(self,i):
        """
        Return generators of self.

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()

        Generator number zero is 1 in the base ring of ``H``::

            sage: H.0
            1
            sage: H.1
            c_2_2, a 2-Cochain in H^*(D8; GF(2))
            sage: H.2
            b_1_0, a 1-Cochain in H^*(D8; GF(2))
            sage: print H.1*H.2
            3-Cochain in H^*(D8; GF(2)),
            represented by
            [0 0 1 0]
            
        """
        if i:
            return self.Gen[i-1]
        else:
            return self.base_ring()(1)

    def ngens(self):
        """
        Return the number of generators of self

        NOTE:

        We consider the scalar ``1`` as a generator.

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.ngens()
            4
            sage: len(H.gens())
            4

        """
        return len(self.Gen)+1

    def gens(self):
        """
        Return the list of generators of self

        NOTE:

        We consider the scalar ``1`` as a generator. The
        list of generators may change if an incomplete
        cohomology computation is continued.

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)
            sage: H.make(1)
            sage: H.gens()
            [1, b_1_0, a 1-Cochain in H^*(D8; GF(2)), b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            sage: H.make()
            sage: H.gens()
            [1,
             c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]

        """
        return [self.base_ring()(1)]+self.Gen

    def rel(self,i):
        """
        Return a relation of self, represented by a string

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,3)
            sage: H.make()
            sage: H.rels()
            ['a_1_0^2', 'a_1_0*b_1_1', 'b_2_1*a_1_0', 'b_2_1^2+c_2_2*b_1_1^2']
            sage: H.rel(0)
            'a_1_0^2'
            sage: H.rel(2)
            'b_2_1*a_1_0'

        """
        return self.Rel[i]

    def rels(self):
        """
        Return the list of minimal relations of self, represented by strings

        NOTE:

        The list of relations may change if an incomplete
        cohomology computation is continued.

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,3, websource=False)
            sage: H.make(2)
            sage: H.rels()
            ['a_1_0^2', 'a_1_0*b_1_1']
            sage: H.make()
            sage: H.rels()
            ['a_1_0^2', 'a_1_0*b_1_1', 'b_2_1*a_1_0', 'b_2_1^2+c_2_2*b_1_1^2']

        """
        return self.Rel

    def nrels(self):
        """
        Return the minimal number of relations of self

        NOTE:

        The list of relations may change if an incomplete
        cohomology computation is continued.

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,3, websource=False)
            sage: H.make(2)
            sage: H.nrels()
            2
            sage: H.make()
            sage: H.nrels()
            4
            sage: len(H.rels())
            4

        """
        return len(self.Rel)

    def subgroups(self):
        """
        Return the subgroup dictionary of self

        OUTPUT:

        A dictionary of cohomology rings of elementary abelian groups
        that occur as maximal elementary abelian subgroups or as the
        greatest central elementary abelian subgroup of the group over
        which self is defined. The dictionary is indexed by addresses
        for the SmallGroups libarary.        

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,3)
            sage: H.subgroups()
            {(4, 2): H^*(SmallGroup(4,2); GF(2)), (8, 5): H^*(SmallGroup(8,5); GF(2))}

        """
        return self.subgps
    
    def restriction_maps(self):
        """
        Return the dictionary of restriction maps of self

        OUTPUT:

        A dictionary of induced homomorphisms, namely the restriction
        maps of self to conjugacy classes of maximal elementary abelian
        subgroups or to the greatest central elementary abelian subgroup.
        The greatest central elementary abelian subgroup is available
        with the index ``1``.

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,3)
            sage: H.restriction_maps()
            {1: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(SmallGroup(16,3); GF(2)) to H^*(SmallGroup(4,2); GF(2))],
             2: [(8, 5),
                 Induced homomorphism of degree 0 from H^*(SmallGroup(16,3); GF(2)) to H^*(SmallGroup(8,5); GF(2))]}

        """
        return self.RestrMaps
    
    def resolution(self):
        """
        Return the underlying resolution of self

        EXAMPLES:
        
        We use a temporary root directory, that will be
        removed when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(16,5, websource=False)
            sage: H
            H^*(SmallGroup(16,5); GF(2))
            sage: H.resolution()
            Resolution:
            0 <- GF(2) <- GF(2)(16gp5)
            sage: H.make()
            sage: H.resolution()
            Resolution:
            0 <- GF(2) <- GF(2)(16gp5) <- rank 2 <- rank 3 <- rank 4

        """
        return self.Resl

##########################
###   HTML             ###
##########################
    def item2html(self,x):
        """
        H.item2html(x): return a string that provides HTML code for x

        INPUT:
        
        Any object; but its intended use is for cochains or
        polynomials.

        EXAMPLES:

        We use a temporary root directory, that will be removed
        when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.item2html(H.1)
            '<nobr>c_2_2</nobr>, a Duflot regular element of degree 2'
            sage: H.item2html(H.2*H.1^2)
            '<nobr>c_2_2<sup>2</sup>&middot;b_1_0</nobr>, an element of degree 5'
            sage: H.item2html(H.poincare_series())
            '<nobr>1/(t<sup>2 </sup>&thinsp;&#8722;&thinsp; 2&middot;t &thinsp;+&thinsp; 1)</nobr>'
            
        """
        cdef list L
        if isinstance(x,int) or isinstance(x,Integer):
            return str(x%self.Resl.coef())
        singular.eval("setring %sr(%d)"%(self.prefix,self.knownDeg))
        if isinstance(x,COCH):
            if not (x.resolution() is self.Resl):
                raise TypeError, "Cochain must be defined over the same resolution"
            L=[str2html(str(singular.eval(x.name())))]
            if x.rdeg():
                L.append(', a Duflot regular ')
            elif x.ydeg():
                L.append(', a nilpotent ')
            else:
                L.append(', an ')
            L.append('element of degree %d'%(x.deg()))
            return ''.join(L)
        if isinstance(x,basestring):
            return str2html(str(singular.eval(x)))
        return str2html(str(x))

    def htmlpage(self, user=None):
        """
        Create a html page describing self

        OUTPUT:
        
        A html file describing the cohomology ring ``H`` is written
        to ``H.root+repr(H.group().Order())+'web/'+H.GStem+'.html'``

        EXAMPLES:

        We use a temporary root directory, that will be removed
        when quitting Sage.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.htmlpage()
            sage: print open(H.root+repr(H.group().Order())+'web/'+H.GStem+'.html').read()
            <html>
            <head>
            <title>Cohomology of Group number 3 of Order 8
            </title>
                <!--#config timefmt="%d.%m.%Y" -->
            </head>
            <body>
            <h2>
              Cohomology of group number 3 of order 8
            </h2>
            <p>
            <table border="1">
              <tr>
                <td><a href="#general">About the group</a></td>
                <td><a href="#generators">Ring generators</a></td>
                <td><a href="#relations">Ring relations</a></td>
                <td><a href="#benson">Completion information</a></td>
                <td><a href="#restrict">Restriction maps</a></td>
              </tr>
            </table>
            <BLANKLINE>
            <p><hr></p>
            <h3>
              General information on the group
            </h3>
            <p><ul>
              <li> The group is also known as D8, the Dihedral group of order 8.
              </li>
              <li>The group has 2 minimal generators and exponent 4.
              </li>
              <li> It is non-abelian.
              </li>
              <li> It has p-Rank 2.
              </li>
              <li> Its center has rank 1.
              </li>
              <li> It has 2 conjugacy classes of maximal elementary abelian subgroups, which are all of rank 2.
              </li>
            </ul></p>
            <p><hr></p>
            <h3>
              Structure of the cohomology ring
            </h3>
            <h4>
              General information
            </h4>
            <p><ul>
              <li> The cohomology ring is of dimension 2 and depth 2.
              </li>
              <li> The depth exceeds the Duflot bound, which is 1.
              </li>
              <li> The Poincar&eacute; series is    <table style="color:black"><tr><td align="center"><nobr>1</nobr></td></tr>      <tr><td><hr style="color:black"></td></tr><tr><td align="center"><nobr>(t &thinsp;&#8722;&thinsp; 1)<sup>2</sup></nobr></td></tr>
                </table>
              </li>  <li> The a-invariants are -&infin;,-&infin;,-2.  They were obtained using the filter regular HSOP of the <a href="#benson">Benson test</a>.
              </li>
            </ul></p>
            <BLANKLINE>
            <BLANKLINE>
              <a name = "generators"></a>
              <p>
            <table border="1">
              <tr>
                <td><a href="#general">About the group</a></td>
                <td><a href="#generators">Ring generators</a></td>
                <td><a href="#relations">Ring relations</a></td>
                <td><a href="#benson">Completion information</a></td>
                <td><a href="#restrict">Restriction maps</a></td>
              </tr>
            </table></p>
            <BLANKLINE>
            <h4>
              Ring generators
            </h4>
             <p>The cohomology ring has 3 minimal generators of maximal degree 2:
            </p>
            <ol>
              <li> <nobr>b_1_0</nobr>, an element of degree 1
              </li>
              <li> <nobr>b_1_1</nobr>, an element of degree 1
              </li>
              <li> <nobr>c_2_2</nobr>, a Duflot regular element of degree 2
              </li>
            </ol>
            <BLANKLINE>
            <a name = "relations"></a>
              <p>
            <table border="1">
              <tr>
                <td><a href="#general">About the group</a></td>
                <td><a href="#generators">Ring generators</a></td>
                <td><a href="#relations">Ring relations</a></td>
                <td><a href="#benson">Completion information</a></td>
                <td><a href="#restrict">Restriction maps</a></td>
              </tr>
            </table></p>
            <BLANKLINE>
            <h4>
              Ring relations
            </h4>
            <p>There is one minimal relation of degree 2:
            <ol>
              <li> <nobr>b_1_0&middot;b_1_1</nobr>
              </li>
            </ol></p>
            <BLANKLINE>
            <a name = "benson"><p><hr></p></a>
              <p>
            <table border="1">
              <tr>
                <td><a href="#general">About the group</a></td>
                <td><a href="#generators">Ring generators</a></td>
                <td><a href="#relations">Ring relations</a></td>
                <td><a href="#benson">Completion information</a></td>
                <td><a href="#restrict">Restriction maps</a></td>
              </tr>
            </table></p>
            <BLANKLINE>
            <h3>
              Data used for Benson&prime;s test
            </h3>
            <p>
              <ul>
                <li> Benson&prime;s completion test succeeded in degree 3.
                </li>
                <li> However, the last relation was already found in degree 2 and the last generator in degree 2.
                </li>
                <li> The following is a filter regular homogeneous system of parameters:
                <ol>
                  <li><nobr>c_2_2</nobr>, a Duflot regular element of degree 2
                  </li>
                  <li><nobr>b_1_1&thinsp;+&thinsp;b_1_0</nobr>, an element of degree 1
                  </li>
                </ol>
                </li>    <li> The Raw Filter Degree Type of that HSOP is [-1, -1, 1].
                </li>
                <li> The filter degree type of any filter regular HSOP is [-1, -2, -2].
                </li>
              </ul></p>
            <p><a name="restrict"><hr></a></p>
            <p>
            <table border="1">
              <tr>
                <td><a href="#general">About the group</a></td>
                <td><a href="#generators">Ring generators</a></td>
                <td><a href="#relations">Ring relations</a></td>
                <td><a href="#benson">Completion information</a></td>
                <td><a href="#restrict">Restriction maps</a></td>
              </tr>
            </table></p>
            <BLANKLINE>
            <h3>
              Restriction maps
            </h3>
              <h4>
                Restriction map to the greatest central el. ab. subgp., which is of rank 1
              </h4>
                <ol>
                  <li> <nobr>b_1_0</nobr> &rarr; <nobr>0</nobr>, an element of degree 1
                  </li>
                  <li> <nobr>b_1_1</nobr> &rarr; <nobr>0</nobr>, an element of degree 1
                  </li>
                  <li> <nobr>c_2_2</nobr> &rarr; <nobr>c_1_0<sup>2</sup></nobr>, an element of degree 2
                  </li>
                </ol>
            <BLANKLINE>
              <h4>
                Restriction map to a maximal el. ab. subgp. of rank 2
              </h4>
                <ol>
                  <li> <nobr>b_1_0</nobr> &rarr; <nobr>c_1_1</nobr>, an element of degree 1
                  </li>
                  <li> <nobr>b_1_1</nobr> &rarr; <nobr>0</nobr>, an element of degree 1
                  </li>
                  <li> <nobr>c_2_2</nobr> &rarr; <nobr>c_1_0&middot;c_1_1&thinsp;+&thinsp;c_1_0<sup>2</sup></nobr>, an element of degree 2
                  </li>
                </ol>
            <BLANKLINE>
              <h4>
                Restriction map to a maximal el. ab. subgp. of rank 2
              </h4>
                <ol>
                  <li> <nobr>b_1_0</nobr> &rarr; <nobr>0</nobr>, an element of degree 1
                  </li>
                  <li> <nobr>b_1_1</nobr> &rarr; <nobr>c_1_1</nobr>, an element of degree 1
                  </li>
                  <li> <nobr>c_2_2</nobr> &rarr; <nobr>c_1_0&middot;c_1_1&thinsp;+&thinsp;c_1_0<sup>2</sup></nobr>, an element of degree 2
                  </li>
                </ol>
            <BLANKLINE>
            <p><hr></p>
            <p>
            <table border="1">
              <tr>
                <td><a href="#general">About the group</a></td>
                <td><a href="#generators">Ring generators</a></td>
                <td><a href="#relations">Ring relations</a></td>
                <td><a href="#benson">Completion information</a></td>
                <td><a href="#restrict">Restriction maps</a></td>
              </tr>
            </table></p>
            <BLANKLINE>
            <BLANKLINE>
                          <br clear="all">
                          <P>
                          <HR WIDTH="100%">
                          <HR WIDTH="100%"></P>
                          Created with the <a href="http://www.sagemath.org/" style="font-variant:small-caps">Sage</a>
                          cohomology package written by <a href="http://users.minet.uni-jena.de/~king/">Simon King</a>
                          and <a href="http://users.minet.uni-jena.de/~green/">David Green</a>
                          <P>
                          <HR WIDTH="100%">
                          <HR WIDTH="100%"></P>
                  <p>
                  <i>Last change: <!--#echo var="LAST_MODIFIED" --></i>
                  <hr>
            <BLANKLINE>
                    </BODY>
                    </HTML>
            <BLANKLINE>
            <BLANKLINE>

        """
        if not self.CenterRk: # i.e., it is an abelian group
            if self.knownDeg>1:
                self.completed=True
        cdef int p = self.Resl.coef()
        self.reconstructSubgroups()
        q=self.Resl.G_ALG().order()
        if len(self._key[0])==2:
            n=self._key[0][1]
        else:
            n=None
        #############
        ## Create web folder, if it doesn't exist yet
        try:
            os.mkdir(self.root+'%dweb'%(q))
        except OSError:
            pass
        L = file(self.root+'%dweb/%s.html'%(q,self.GStem),'w')
        L.write('<html>\n<head>\n')
        if n:
            L.write('<title>Cohomology of Group number %d of Order %d\n</title>\n'%(n,q))
        else:
            if self.GroupName:
                L.write('<title>Cohomology of %s (Group of Order %d)\n</title>\n'%(self.GroupName,q))
            else:
                L.write('<title>Cohomology of %s (Group of Order %d)\n</title>\n'%(self.GStem,q))
        if user=="SimonKing":
            L.write('   <link rel="shortcut icon" type="image/x-icon" href="/~king/favicon.ico" />\n')
            L.write('\n    <link rel="stylesheet" href="/~king/design.css" type="text/css">\n')
        L.write(r'    <!--#config timefmt="%d.%m.%Y" -->')
        L.write('\n</head>\n')
        if user=="SimonKing":
            ## General layout
            L.write('<body>\n  <a name="general"></a>\n  <table CELLPADDING="10" CELLSPACING="0" BORDER="0" HEIGHT="100%" WIDTH="100%">\n')
            L.write('''<TR>
    <td valign="top" id="navi">
      <!--#include virtual="/~king/navigation.txt" -->
    </td>

    <TD VALIGN="top" WIDTH="100%" id="main">
            
      <P>
        <HR WIDTH="100%">
        <HR WIDTH="100%">
      </P>\n''')
        else:
            L.write('<body>\n')
        
    ###################
    ### The Main Data:
        if self.completed:
            L.write('<h2>\n  Cohomology of ')
        else:
            L.write('<h2>\n  Cohomology approximation of ')
        if n:
            L.write('group number %d of order %d\n</h2>\n'%(n,q))
        else:
            if self.GroupName:
                L.write('%s, a group of order %d\n</h2>\n'%(self.GroupName,q))
            else:
                L.write('%s, a group of order %d\n</h2>\n'%(self.GStem,q))
        if not self.completed:
            L.write('<h3>\n  Based on a computation out to degree %d\n</h3>\n'%(self.knownDeg))
        #########
        ## General information on the group
        L.write('<p>\n<table border="1">\n  <tr>\n')
        L.write('    <td><a href="#general">About the group</a></td>\n')
        L.write('    <td><a href="#generators">Ring generators</a></td>\n')
        L.write('    <td><a href="#relations">Ring relations</a></td>\n')
        if (self.CenterRk) and (self.pRank!=1):
            L.write('    <td><a href="#benson">Completion information</a></td>\n')
        if (self.CenterRk):
            L.write('    <td><a href="#restrict">Restriction maps</a></td>\n')
        if user == "SimonKing":
            L.write('    <td><a href="index.html">Back to groups of order %d</a></td>\n'%(q))
        L.write('  </tr>\n</table>\n\n')

        L.write('<p><hr></p>\n<h3>\n  General information on the group\n</h3>\n<p><ul>\n')
        if (self.GroupName and self.GroupName!=self.GStem) or self.GroupDescr:
            L.write('  <li> The group is ')
            if self.GroupName and self.GroupName!=self.GStem:
                L.write('also ')
            L.write('known as '+str(self.GroupName))
            if self.GroupDescr and (self.GroupDescr!=self.GroupName):
                L.write(', the '+str(self.GroupDescr)+'.\n  </li>\n')
            else:
                L.write('.\n  </li>\n')
        if n is None:
            L.write('  <li>The group is defined by %s.\n  </li>\n'%(self._key[0][0]))
        L.write('  <li>The group has %s minimal generators and exponent %s.\n  </li>\n'%(gap.eval('RankPGroup(%s)'%(self.group().name())),gap.eval('Exponent(%s)'%(self.group().name()))))
        if self.Resl.G_ALG().isAbelian():
            L.write('  <li> It is abelian, isomorphic to')
            L.write('&times;'.join(['C<sub>%d</sub>'%dd for dd in list(gap('AbelianInvariants(%s)'%(self.group().name())))]))
            L.write('.\n  </li>\n')
        else:
            if self.pRank==1:
                L.write('  <li> It is a (generalized) quaternion group, hence, is of p-Rank 1.\n  </li>\n')
            else:
                L.write('  <li> It is non-abelian.\n  </li>\n')
                L.write('  <li> It has p-Rank %d.\n  </li>\n'%(self.pRank))
                L.write('  <li> Its center has rank %d.\n  </li>\n'%(self.CenterRk))
            if len(self.MaxelRk)==1:
                L.write('  <li> It has a unique conjugacy class of maximal elementary abelian subgroups, which is of rank %d.\n  </li>\n'%(self.MaxelRk[0]))
            else:
                L.write('  <li> It has %d conjugacy classes of maximal elementary abelian subgroups, which are '%(len(self.MaxelRk)))
                if len(set(self.MaxelRk))==1:
                    L.write('all of rank %d.\n  </li>\n'%(self.MaxelRk[0]))
                else:
                    L.write('of rank %s and %d, respectively.\n  </li>\n'%(str(self.MaxelRk[:-1])[1:-1],self.MaxelRk[-1]))
        L.write('</ul></p>\n')

        #########
        ## Cohomology ring structure
        if self.completed:
            L.write('<p><hr></p>\n<h3>\n  Structure of the cohomology ring\n</h3>\n')
        else:
            L.write('<p><hr></p>\n<h3>\n  Appoximate structure of the cohomology ring\n</h3>\n')
        if self.completed:
            L.write('<h4>\n  General information\n</h4>\n')
            L.write('<p><ul>\n')
            if self.CenterRk:
                L.write('  <li> The cohomology ring is of dimension %d and depth %d.\n  </li>\n'%(self.pRank,self.rfdt.count(-1)))
                if self.rfdt.count(-1)==self.CenterRk:
                    L.write('  <li> The depth coincides with the Duflot bound.\n  </li>\n')
                else:
                    L.write('  <li> The depth exceeds the Duflot bound, which is %d.\n  </li>\n'%(self.CenterRk))
            else:
                MinimalGenerators = Integer(gap.eval('RankPGroup(SmallGroup(%s))'%(self.group().name())))
                L.write('  <li> The cohomology ring is of dimension %d and depth %d.\n  </li>\n'%(MinimalGenerators,MinimalGenerators))
            PS = self.poincare_series()
            P1,P2 = PS.numerator(),PS.denominator()
            L.write('  <li> The Poincar&eacute; series is')
            L.write('    <table style="color:black"><tr><td align="center">%s</td></tr>'%(str2html(str(P1.factor()),linelength=0)))
            L.write('      <tr><td><hr style="color:black"></td></tr><tr><td align="center">%s</td></tr>\n    </table>\n  </li>'%(str2html(str(P2.factor()),linelength=0)))
            L.write('  <li> The a-invariants are %s. '%(','.join([str(X) for X in self.a_invariants()]).replace('Infinity','&infin;')))
            if self.CenterRk:
                if self.A_INV_Expos.count(1)==len(self.A_INV_Expos):
                    L.write(' They were obtained using the filter regular HSOP of the <a href="#benson">Benson test</a>.')
                else:
                    XList = [' They were obtained using ']
                    for X in range(len(self.A_INV_Expos)):
                        if X == len(self.A_INV_Expos)-1:
                            XList.append('and ')
                        if self.A_INV_Expos[X]==1:
                            XList.append('the '+Ordinals(X+1))
                        elif self.A_INV_Expos[X]==2:
                            XList.append('the second power of the '+Ordinals(X+1))
                        elif self.A_INV_Expos[X]==3:
                            XList.append('the third power of the '+Ordinals(X+1))
                        else:
                            XList.append('the %s power of the '%(Ordinals(self.A_INV_Expos[X]))+Ordinals(X+1))
                        if X < len(self.A_INV_Expos)-1:
                            XList.append(', ')
                        else:
                            XList.append(' filter regular parameter of the <a href="#benson">Benson test</a>.')
                    L.write(''.join(XList))
            L.write('\n  </li>\n</ul></p>\n\n')
        #########
        ## Generators
        L.write('\n  <a name = "generators"></a>')
        L.write('\n  <p>\n<table border="1">\n  <tr>\n')
        L.write('    <td><a href="#general">About the group</a></td>\n')
        L.write('    <td><a href="#generators">Ring generators</a></td>\n')
        L.write('    <td><a href="#relations">Ring relations</a></td>\n')
        if self.CenterRk and (self.pRank!=1):
            L.write('    <td><a href="#benson">Completion information</a></td>\n')
        if self.CenterRk:
            L.write('    <td><a href="#restrict">Restriction maps</a></td>\n')
        if user == "SimonKing":
            L.write('    <td><a href="index.html">Back to groups of order %d</a></td>\n'%(q))
        L.write('  </tr>\n</table></p>\n\n')
        if self.completed:
            L.write('<h4>\n  Ring generators\n</h4>\n <p>The cohomology ring has %d minimal generators of maximal degree %d:\n</p>\n'%(len(self.Gen),max(self.degvec)))
        else:
            if (len([X for X in self.Gen if X.rdeg()])<self.CenterRk):
                L.write('<h4>\n  Ring generators\n</h4>\n  <p> There will be more Duflot regular generators.\n<br>    Out to degree %d, the cohomology ring has %d minimal generators of maximal degree %d:\n  </p>\n'%(self.knownDeg,len(self.Gen),max(self.degvec)))
            else:
                L.write('<h4>\n  Ring generators\n</h4>\n <p>Out to degree %d, the cohomology ring has %d minimal generators of maximal degree %d:\n</p>\n'%(self.knownDeg,len(self.Gen),max(self.degvec)))
        L.write('<ol>\n')
        GenL = copy(self.Gen)
        GenL.sort()
        if self.CenterRk: # i.e., the group is non-abelian
            for X in GenL:
                L.write('  <li> '+self.item2html(X)+'\n  </li>\n')
        else:
            for X in GenL:
                L.write('  <li> '+self.item2html(X.name())+', an element of degree %d\n  </li>\n'%(X.deg()))
        L.write('</ol>\n\n')
        
        #########
        ## Relations
        L.write('<a name = "relations"></a>')
        L.write('\n  <p>\n<table border="1">\n  <tr>\n')
        L.write('    <td><a href="#general">About the group</a></td>\n')
        L.write('    <td><a href="#generators">Ring generators</a></td>\n')
        L.write('    <td><a href="#relations">Ring relations</a></td>\n')
        if self.CenterRk and (self.pRank!=1):
            L.write('    <td><a href="#benson">Completion information</a></td>\n')
        if self.CenterRk:
            L.write('    <td><a href="#restrict">Restriction maps</a></td>\n')
        if user == "SimonKing":
            L.write('    <td><a href="index.html">Back to groups of order %d</a></td>\n'%(q))
        L.write('  </tr>\n</table></p>\n\n')
        if self.completed:
            L.write('<h4>\n  Ring relations\n</h4>\n')
        else:
            L.write('<h4>\n  Ring relations out to degree %d\n</h4>\n'%(self.knownDeg))
            if (self.expect_last_relation()>self.knownDeg):
                if (q%2):
                    L.write('  <p>\n  Note that there will be further "non-obvious" relations at least out to degree %d\n  </p>\n'%(self.expect_last_relation()))
                else:
                    L.write('  <p>\n  Note that there will be further relations at least out to degree %d\n  </p>\n'%(self.expect_last_relation()))
            else:
                L.write('  <p>\n  It is possible that there are no further relations\n  </p>\n')
        cdef int i
        cdef int lenGen = len(self.Gen)
        if (p%2) and (self.firstOdd<lenGen):
            if lenGen-self.firstOdd == 1:
                L.write('<p> There is one "obvious" relation:<br>\n  &nbsp;&nbsp;&nbsp;')
            else:
                L.write('<p> There are %d "obvious" relations:<br>\n  &nbsp;&nbsp;&nbsp;'%(lenGen-self.firstOdd))
            for i from self.firstOdd <= i < lenGen-1:
                L.write(self.item2html(self.Gen[i].name())+'<sup>2</sup>, ')
            L.write(self.item2html(self.Gen[-1].name())+'<sup>2</sup>\n</p>\n<p>Apart from that, t')
        else:
            L.write('<p>T')
        if len(self.Rel):
            if len(self.Rel)==1:
                L.write('here is one minimal relation of degree %d:\n<ol>\n'%(self.lastRel))
            else:
                L.write('here are %d minimal relations of maximal degree %d:\n<ol>\n'%(len(self.Rel),self.lastRel))
            for X in self.Rel:
                L.write('  <li> '+self.item2html(X)+'\n  </li>\n')
            L.write('</ol></p>\n\n')
        else:
            L.write('here are no relations.\n</p>\n')

        ##########
        ## Completion information
        if self.CenterRk and (self.pRank!=1):
            L.write('<a name = "benson"><p><hr></p></a>\n')
            L.write('  <p>\n<table border="1">\n  <tr>\n')
            L.write('    <td><a href="#general">About the group</a></td>\n')
            L.write('    <td><a href="#generators">Ring generators</a></td>\n')
            L.write('    <td><a href="#relations">Ring relations</a></td>\n')
            L.write('    <td><a href="#benson">Completion information</a></td>\n')
            L.write('    <td><a href="#restrict">Restriction maps</a></td>\n')
            if user == "SimonKing":
                L.write('    <td><a href="index.html">Back to groups of order %d</a></td>\n'%(q))
            L.write('  </tr>\n</table></p>\n\n')
            L.write('<h3>\n  Data used for Benson&prime;s test\n</h3>\n<p>\n  <ul>\n')
            if self.completed:
                L.write('    <li> Benson&prime;s completion test succeeded in degree %d.\n    </li>\n'%(max(self.suffDeg,self.lastRel,max(self.degvec))))
            else:
                L.write('    <li> The computation is incomplete, Benson&prime;s criterion does not apply up to degree %d.\n    </li>\n'%(self.knownDeg))
            if self.completed:
                if (self.suffDeg <= self.lastRel) or (self.suffDeg <= max(self.degvec)):
                    L.write('    <li> The completion test was perfect: It applied in the last degree in which a generator or relation was found.\n    </li>\n')
                else:
                    L.write('    <li> However, the last relation was already found in degree %d and the last generator in degree %d.\n    </li>\n'%(self.lastRel,max(self.degvec)))
                L.write('    <li> The following is a filter regular homogeneous system of parameters:\n    <ol>\n')
            else:
                if (self.Dickson.count(None)==0) and (len([X for X in self.Gen if X.rdeg()])==self.CenterRk):
                    L.write('    <li> The following will eventually be a filter regular homogeneous system of parameters:\n    <ol>\n')
                elif (self.Dickson.count(None)<len(self.Dickson)) or len([X for X in self.Gen if X.rdeg()]):
                    L.write('    <li> The following will eventually be <i>part of</i> a filter regular homogeneous system of parameters:\n    <ol>\n')
            for X in GenL:
                if X.rdeg():
                    L.write('      <li>'+self.item2html(X)+'\n      </li>\n')
            for X in self.Dickson:
                if X!=None:
                    L.write('      <li>'+self.item2html(X)+', an element of degree %d\n      </li>\n'%(Integer(singular.eval('deg('+X+')'))))
            L.write('    </ol>\n    </li>')
            if (self.Dickson.count(None)):
                L.write('    <li> We need to lift more Dickson invariants.\n    </li>\n')
            if (len([X for X in self.Gen if X.rdeg()])<self.CenterRk):
                L.write('    <li> We need to find more Duflot regular generators.\n    </li>\n')
            if self.completed:
                L.write('    <li> The Raw Filter Degree Type of that HSOP is %s.\n    </li>\n'%(str(self.rfdt)))
                L.write('    <li> The filter degree type of any filter regular HSOP is %s.\n    </li>\n'%(str(self.fdt)))
                if self.WhatFRS:
                    if self.pRank-self.WhatFRS[0] == 1:
                        L.write('    <li> We found that there exists some filter regular HSOP formed by the first term of the above HSOP, together with %d elements of degree %d.\n    </li>\n'%(self.WhatFRS[0],self.WhatFRS[1]))
                    else:
                        L.write('    <li> We found that there exists some filter regular HSOP formed by the first %d terms of the above HSOP, together with %d elements of degree %d.\n    </li>\n'%(self.pRank-self.WhatFRS[0],self.WhatFRS[0],self.WhatFRS[1]))
            L.write('  </ul></p>\n')
            ##############
            ## Restriction Maps
        if self.CenterRk:
            L.write('<p><a name="restrict"><hr></a></p>\n<p>\n<table border="1">\n  <tr>\n')
            L.write('    <td><a href="#general">About the group</a></td>\n')
            L.write('    <td><a href="#generators">Ring generators</a></td>\n')
            L.write('    <td><a href="#relations">Ring relations</a></td>\n')
            if self.pRank!=1:
                L.write('    <td><a href="#benson">Completion information</a></td>\n')
            L.write('    <td><a href="#restrict">Restriction maps</a></td>\n')
            if user == "SimonKing":
                L.write('    <td><a href="index.html">Back to groups of order %d</a></td>\n'%(q))
            L.write('  </tr>\n</table></p>\n\n')
            L.write('<h3>\n  Restriction maps\n</h3>\n')        
            for i from 1 <= i <= len(self.RestrMaps):
                if i==1:
                    L.write('  <h4>\n    Restriction map to the greatest central el. ab. subgp., which is of rank %d\n  </h4>\n'%(self.CenterRk))
                else:
                    L.write('  <h4>\n    Restriction map to a maximal el. ab. subgp. of rank %d\n  </h4>\n'%(self.MaxelRk[i-2]))
                RestL = self.restrict_to_subgroup(i)
                PairL = [[self.Gen[j],RestL[j]] for j in xrange(len(self.Gen))]
                PairL.sort()
                L.write('    <ol>\n')
                for X in PairL:
                    L.write('      <li> '+self.item2html(X[0].name())+' &rarr; ')
                    L.write(self.subgps[self.RestrMaps[i][0]].item2html(X[1]))
                    L.write('\n      </li>\n')
                L.write('    </ol>\n\n')
        ###########
        ## "Signature" of this package
        L.write('<p><hr></p>\n<p>\n<table border="1">\n  <tr>\n')
        L.write('    <td><a href="#general">About the group</a></td>\n')
        L.write('    <td><a href="#generators">Ring generators</a></td>\n')
        L.write('    <td><a href="#relations">Ring relations</a></td>\n')
        if self.CenterRk and (self.pRank!=1):
            L.write('    <td><a href="#benson">Completion information</a></td>\n')
        if self.CenterRk:
            L.write('    <td><a href="#restrict">Restriction maps</a></td>\n')
        if user == "SimonKing":
            L.write('    <td><a href="index.html">Back to groups of order %d</a></td>\n'%(q))
        L.write('  </tr>\n</table></p>\n\n')
        if user=="SimonKing":
            L.write(r'''
              <br clear="all">

              <!--#include virtual="/~king/address.txt" -->
              <P>
              <HR WIDTH="100%">
              <HR WIDTH="100%"></P>
              </TD>
            </TR>
          </TABLE>

      <p>
      <i>Last change: <!--#echo var="LAST_MODIFIED" --></i>
      <hr>

        </BODY>
        </HTML>
        ''')
        else:
            L.write(r'''
              <br clear="all">
              <P>
              <HR WIDTH="100%">
              <HR WIDTH="100%"></P>
              Created with the <a href="http://www.sagemath.org/" style="font-variant:small-caps">Sage</a>
              cohomology package written by <a href="http://users.minet.uni-jena.de/~king/">Simon King</a>
              and <a href="http://users.minet.uni-jena.de/~green/">David Green</a>
              <P>
              <HR WIDTH="100%">
              <HR WIDTH="100%"></P>
      <p>
      <i>Last change: <!--#echo var="LAST_MODIFIED" --></i>
      <hr>

        </BODY>
        </HTML>
        ''')
        L.close()

###################
## Auxiliary methods
    ####################
    ## Transform Singular-Monomials into products of cochains
    ## Use result of sub-monomials that are stored in self.Monomials
    def MonToProd(self, expV):
        """
        Return the products of cochains corresponding to a list of exponents

        INPUT:

        ``expV`` -- a list of non-negative integers (exponents of a list of monomials)

        OUTPUT:

        A cochain (:class:`~pGroupCohomology.cochain.COCH`)

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.MonToProd([0,2,1])
            (b_1_0)*((b_1_0)*(b_1_1)), a 3-Cochain in H^*(D8; GF(2))
            
        """ 
        # Assume: All lifts have been performed before
        cdef int lenoldV = len(expV)
        cdef int i, s_o, s_e, sml
        newKey = ''.join([expV[i]*(self.Gen[i].name()) for i in range(lenoldV)])
        # print newKey
        if self.Monomials.has_key(newKey):
            return self.Monomials[newKey]
        cdef list oldV = list(tuple(expV))# copy the list
        # find the factor of smallest even/odd degree
        for i from 0 <= i < self.firstOdd:
            if oldV[i]:
                break
        if (i<lenoldV) and oldV[i]:
            s_e=i
        else:
            s_e=-1

        for i from self.firstOdd <= i < lenoldV:
            if oldV[i]:
                break
        if (i<lenoldV) and oldV[i]:
            s_o=i
        else:
            s_o=-1

        # Which one has lower degree?
        if s_o==-1:
            sml = s_e
        elif s_e==-1:
            sml = s_o
        else:
            if self.degvec[s_o]<self.degvec[s_e]:
                sml = s_o
            else:
                sml = s_e

        # Compute the product Gen[sml]*(the rest)
        oldV[sml]-=1
        cdef COCH Coch
        cdef RESL R
        R=self.Resl
        if max(oldV)==0: # the monomial is a single variable
            return self.Gen[sml]
        Coch = self.Gen[sml]*self.MonToProd(oldV)
        self.Monomials[newKey] = Coch
        self.Resl.setLift(Coch, Coch.deg()+self.degvec[sml])
        return Coch  

    ####################
    ## Mark a monomial (given by exponent vector) as To-Be-Lifted
    def InsertLift(self, expV):
        """
        If option 'liftlist' prevails: Determine some lifts that are needed to express a monomial as a cochain

        This method should only be used internally. However, since a full doc test coverage is
        required, we need to give a rather lengthy explanation of this function. Sorry.

        INPUT:
        
        ``expV``  -- list or tuple of integers, providing the exponents of a monomial

        OUTPUT:
        
        The aim is to help express the monomial given by ``expV`` as a cochain. To that purpose,
        one needs to compute one more term of some previously studied chain map. This chain
        map is put on some internal list, and the items on the list are lifted simultaneously,
        which may safe resources.

        NOTE:
        
        For iteratively computing the ring structure of a cohomology ring, we need to express
        standard monomials in a given degree as cochains. So, we have to perform cup products.
        The order of factors does matter: If we have two cochains `x`, `y` and if
        `\\deg(x)>\\deg(y)` then `y*x` is easier to compute than `x*y`.

        Hence, when computing the cochain corresponding to the monomial given by ``expV``,
        it is reasonable to consider the cochain `C_0` corresponding to the variable of
        smallest degree in that monomial, and multiply it from the right by the cochain
        `C_r` corresponding to the rest of the monomial. Hence, we put `C_r` on the list of
        to-be-lifted cochains.
        
        EXAMPLES:
        
        This non-commutative example requires Singular 3-1-0 (or later versions)
        being installed.
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology.cohomology import COHO
            sage: H = COHO(27,3,root=tmp_root)
            sage: H.option('liftlist')
            sage: H.make(2)
            sage: H.gens()
            [1,
             b_2_0, a 2-Cochain in H^*(E27; GF(3)),
             b_2_1, a 2-Cochain in H^*(E27; GF(3)),
             b_2_2, a 2-Cochain in H^*(E27; GF(3)),
             b_2_3, a 2-Cochain in H^*(E27; GF(3)),
             a_1_0, a 1-Cochain in H^*(E27; GF(3)),
             a_1_1, a 1-Cochain in H^*(E27; GF(3))]

        When computing the next term of the cohomology ring, standard monomials are studied.
        But it is known from theory that ``a_1_1^2`` and ``a_1_0^2`` are zero, so, the only standard
        monomial in degree 2 formed by generators in lower degree is ``a_1_0*a_1_1``. Hence, in
        the previous computation, ``a_1_1`` was lifted, but ``a_1_0`` was not. Hence, when we now
        compute ``a_1_1*a_1_0``, it is needed to compute the lift for ``a_1_0``, as can be seen
        in the protocol::
        
            sage: H.option('prot')
            sage: H('a_1_1*a_1_0')
            > > We need more lifts!
            > Compose Chain Map with Differential
            > Lift with the autolift method
            (a_1_1)*(a_1_0), a 2-Cochain in H^*(E27; GF(3))
            sage: H.option('noprot')

        Now, we repeat the computation, but we explicitly ask for help on computing ``a_1_0^2``
        (although we know it is zero). This results in computing a lift for ``a_1_0``, and
        by consequence, that lift is already known when computing ``a_1_1*a_1_0``. This can
        be seen in the protocol, by the absence of the statement 'We need more lifts'.
        ::
        
            sage: H = COHO(27,3,root=tmp_root)
            sage: H.next()
            sage: H.gens()
            [1,
             a_1_0, a 1-Cochain in H^*(E27; GF(3)),
             a_1_1, a 1-Cochain in H^*(E27; GF(3))]
            sage: H.InsertLift([2,0])  # [2,0] stands for H.1^2, i.e., a_1_0^2
            sage: H.next()
            sage: H.option('prot')
            sage: H('a_1_1*a_1_0')
            (a_1_1)*(a_1_0), a 2-Cochain in H^*(E27; GF(3))
            sage: H.option('noprot','noliftlist')
            
        """
        cdef int lenoldV = len(expV)
        cdef int i, s_e, s_o, sml
        newKey = ''.join([expV[i]*(self.Gen[i].name()) for i in range(lenoldV)])
        if self.Monomials.has_key(newKey):
            return
        cdef list oldV = list(tuple(expV)) #copy the list
        # find the factor of smallest even/odd degree
        for i from 0 <= i < self.firstOdd:
            if oldV[i]:
                break
        if (i<lenoldV) and oldV[i]:
            s_e=i
        else:
            s_e=-1

        for i from self.firstOdd <=i < lenoldV:
            if oldV[i]:
                break
        if (i<lenoldV) and oldV[i]:
            s_o=i
        else:
            s_o=-1
        # Which one has lower degree?
        if s_o==-1:
            sml = s_e
        elif s_e==-1:
            sml = s_o
        else:
            if self.degvec[s_o]<self.degvec[s_e]:
                sml = s_o
            else:
                sml = s_e
        # -> Later, we have to compute the product Gen[sml]*(the rest)
        # Determine "the rest"!
        oldV[sml]-=1
        oldKey = ''.join([oldV[i]*(self.Gen[i].name()) for i in range(len(oldV))])
        # By induction, this is in the dictionary,
        # unless the whole monomial is just a generator
        cdef RESL R
        R=self.Resl
        if oldKey:
            # The following multipication is easy since a lift of
            # the right factor was constructed before
            Coch = self.Monomials[oldKey]
            if OPTION.opts['liftlist']:
                R.ToBeLifted.append((Coch.deg(),Coch.MTX()))
            return 

    #####################
    ## Compute the standard monomials in degree n and store them in Singular under
    ## the name s
    ## WARNING: There might be miscomputations if the ring order is changed so that
    ##          some of the variable x is of degree zero, and x>1
    def __makeStdMon(self,n,s):
        """
        Compute the standard monomials in a given degree

        INPUT:

        - ``n`` (integer): The degree of the requested standard monomials
        - ``s`` (string): Name of an ideal in the Singular interface into which the monomials are stored
        
        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)
            sage: H.make(2)
            sage: H.__makeStdMon(3,'M')
            sage: singular('M')
            b_1_0^3,
            b_1_1^3,
            c_2_2*b_1_0,
            c_2_2*b_1_1
            
        """
        if not (isinstance(n, int) or isinstance(n,Integer)):
            raise TypeError, "degree (first argument) must be an integer"
        if not (isinstance(s,str)):
            raise TypeError, "second argument must be a string"
        if OPTION.opts['timing']:
            ct = cputime()
            wt = walltime()
        # In order to make addition of ideals more efficient, estimate the length of StdMon
        if not self.Gen:
            singular.eval('ideal %s'%(s))
            return
        if n==0:
            singular.eval('ideal %s'%(s))
            self.StdMon[n] = {'1':singular.ideal(1)}
            return
        for i in range(n):
            if not self.StdMon.has_key(i):
                self.__makeStdMon(i,s)
        MaxLength = max(sum([[Integer(singular.eval('ncols(%s)'%(X.name()))) for X in self.StdMon[i].values()] for i in xrange(1,n)], [1]))
        self.StdMon[n]={}
        singular.eval('ideal '+s)
        singular.eval('%s[%d] = 0'%(s,MaxLength*len(self.Gen)))
        sCount=0
        singular.eval('ideal LeadGB = lead(%sI)'%(self.prefix))
        cdef list Vars = list(singular.maxideal(1).sort()[1])
        for x in Vars:
            if x.deg()>n:
                continue
            self.StdMon[n][str(x)]=singular.ideal(0)
            singular.eval('%s[%d]=0'%(self.StdMon[n][str(x)].name(),MaxLength*len(self.Gen)))
            MonCount = 0
            for I in self.StdMon[Integer(n-x.deg())].items():
                if singular.eval(I[0]+'<=%s'%(x._name))=='1':
                    tmp = Integer(I[1].ncols())
                    if tmp > 1:
                        # singular.eval('%s[%d..%d] = normalize(NF(ideal(%s*%s),LeadGB))'%(self.StdMon[n][str(x)].name(),MonCount+1,MonCount+tmp,I[1].name(),str(x)))
                        # working around a bug in non-commutative Singular (Normalize is now defined in filterregular.lib
                        singular.eval('%s[%d..%d] = Normalize(ideal(%s*%s),LeadGB)'%(self.StdMon[n][str(x)].name(),MonCount+1,MonCount+tmp,I[1].name(),str(x)))
                    else:
                        singular.eval('%s[%d] = normalize(NF(%s[1]*%s,LeadGB))'%(self.StdMon[n][str(x)].name(),MonCount+1,I[1].name(),str(x)))
                    MonCount=MonCount+tmp
            tmp = Integer(self.StdMon[n][str(x)].ncols())
            singular.eval('%s[%d..%d] = %s[1..%d]'%(s,sCount+1,sCount+tmp,self.StdMon[n][str(x)].name(),tmp))
            sCount = sCount + tmp
        ## cleanup
        for I in self.StdMon[n].items():
            singular.eval('%s = simplify(%s,6)'%(I[1].name(),I[1].name()))
            if I[1].size()==0:
                self.StdMon[n].__delitem__(I[0])
        singular.eval('%s = simplify(%s,6)'%(s,s))
        singular.eval('kill LeadGB')
        if OPTION.opts['timing']:
            ct=cputime(ct)
            wt=walltime(wt)
            print "Time for getting standard monomials:"
            print "   Wall: %.2f"%(wt)

    #####################
    ## Source of vector space bases for H^n
    def standardCochain(self, n, i, rdeg=None,ydeg=None,name='c'):
        """
        Standard cochain of self
        
        INPUT:
        
        - ``n`` -- degree (integer) of a cochain
        - ``i`` -- integer
        - ``rdeg``, ``ydeg`` -- (optional) integers or ``None`` (default ``None``).
        - ``name`` -- (optional) string used to give the cochain a name (default ``'c'``)

        OUTPUT:
        
        A `n`-cochain represented by ``[0,...,0,1,0,...0]`` with 1 at position `i`
        (counting from zero) and name ``'c_%d_%d'%(n,i)``. The letter ``'c'`` in
        the name can be changed with the optional parameter 'name'.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: c=H.standardCochain(3,2,rdeg=1,ydeg=0,name='X')
            sage: c
            X_3_2, a 3-Cochain in H^*(D8; GF(2))
            sage: print c
            3-Cochain in H^*(D8; GF(2)),
            represented by
            [0 0 1 0]
            rdeg = 1
            ydeg = 0
            
        """
        if not ((isinstance(n, int) or isinstance(n,Integer)) and\
                (isinstance(i, int) or isinstance(i,Integer))):
            raise TypeError, "integers expected"
        if (n<0) or (n>self.Resl.deg()):
            raise IndexError, "resolution only known up to degree %d"%(self.Resl.deg())
        if (i<0) or (i>self.Resl.rank(n)):
            raise IndexError, "The %s term of the resolution is of rank %d"%(Ordinals(n), self.Resl.rank(n))
        return COCH(self, n, name+'_%d_%d'%(n,i), i*[0]+[1]+(self.Resl.rank(n)-i-1)*[0], rdeg=rdeg, ydeg=ydeg)

    #####################
    ## Express a cochain (type COCH) as polynomial in the cohomology ring generators
    def cochain_to_polynomial(self, c):
        """
        Express a cochain as polynomial in the generators of self

        INPUT:
        
        ``C`` -- a cochain of self

        OUTPUT:
        
        ``C`` is changed in-place, so that its name represents ``C`` as a polynomial in the ring generators.
        
        NOTE:
        
        If needed, all decomposable classes in the degree of ``C`` will be 
        computed and then kept in memory. The decomposable classes need to be
        obtained anyway when computing the ring structure. By default, for
        saving resources, the memory is cleared (except for elementary abelian
        groups). If the optional parameter ``KeepBases`` is ``True``  when 
        creating the cohomology ring, the memory is not cleared.

        EXAMPLES:
            
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
            
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: c=H.standardCochain(3,3)
            sage: c
            c_3_3, a 3-Cochain in H^*(D8; GF(2))
            sage: c is H.cochain_to_polynomial(c)
            True
            sage: c
            c_2_2*b_1_1, a 3-Cochain in H^*(D8; GF(2))
            sage: H('c_2_2*b_1_1')==c
            True
            
        """
        cdef COCH C = self(c)
        #if self.Resl<>C.resolution():
        #    raise TypeError, "Cohomology ring and cochain must be defined over the same resolution"
        n = C.deg()
        if n==0:
            return C
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"> Express cochain of degree %d as polynomial"%(n)
        if singular.eval('defined(basering)')=='1':
            br = singular("basering")
        else:
            br = None
        cdef list DecGen = self.decomposable_classes(n) # triangular basis of the sub space of decomposables of H^n
        cdef list VSGen=self.Resl.rank(n)*[1]
        cdef list pivot=[]
        cdef COCH Cand # will be the 'next candidate' for a decomposable generator in Triang, if it needs to be constructed
        newRel = False
        cdef RESL R
        R=self.Resl
        cdef int f,j,k
        cdef int lenDecGen=len(DecGen)
        for k from 0 <= k < lenDecGen:
            f,j = DecGen[k].MTX().lead()
            if f:
                pivot.append(j)
            else:
                raise ArithmeticError, "One of the basis elements is zero"
        Cand = -C          # Cand is the negative of C...
        Cand.setname('0')  # ... and has the name '0'. We will subtract decomposable generators until
                           # the value of Cand is zero -- and then, its name is a polynomial expression
                           # for C and will be assigned to C's name.
        f,j = Cand.Data.lead()
        if f:
            for k from 0 <= k < lenDecGen:
                if (j <= pivot[k]) and Cand.Data[0,pivot[k]]:
                    Cand = Cand - (DecGen[k])*(Cand.Data[0,pivot[k]])
                    f,j = Cand.Data.lead()
        if f:
            raise ArithmeticError, "The set of generators of the cohomology ring is incomplete"
        singular.eval("setring %sr(%d)"%(self.prefix,self.knownDeg))
        C.setname(singular.eval(Cand.name()))
        if not (br is None):
            singular.setring(br)
        return C

    ##################
    ## Manage restrictions
    def InsertSubgroup(self, q,nr, n):
        """
        Initialize the restriction map to some subgroup

        INPUT:
        
        - ``q`` - the order of the subgroup
        - ``nr`` - the position of this group in the Small Group Library
        - ``n`` - the subgroup is addressed as subgroup number ``n``

        NOTE:
        
        * This method will normally be of internal use only. It is invoked by
          the method :meth:`InitSubgroups`
        * It reads the data for the inclusion map from a file whose name is
          determined by ``self.GStem`` and ``n``. The necessary files are
          created by :func:`~pGroupCohomology.resolution.makeGroupData` or
          or :func:`~pGroupCohomology.resolution.makeSpecialGroupData` during
          iniitialisation of the cohomology ring.
        * Normally, we only consider elementary abelian subgroups. This
          method probably also allows for insertion of other interesting 
          subgroups, provided a file with data for the inclusion map exists.
          However, **PLEASE DO NOT INSERT SUBGROUPS MANUALLY!**
          The computations crucially depend on the restrictions to certain
          elementary abelian subgroups, and it is essential that *only* 
          elementary abelian abelian subgroups are inserted, including the 
          greatest central elementary abelian (this *must* be subgroup number
          one) and one representative for each conjugacy class of maximal 
          elementary abelian subgroups. 
        * The method will try to read a file in a specific location (see example
          below). This file should provide a MeatAxe matrix that describes the
          inclusion map of the algebra of the subgroup into the algebra of the
          ambient group. 
        * When a cohomology ring is initialized, the greatest central elementary
          abelian subgroup will be inserted as special subgroup number 1. Moreover,
          representatives for the conjugacy classes of maximal elementary subgroups
          will be inserted.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
          
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.subgroups()
            {(4, 2): H^*(SmallGroup(4,2); GF(2)), (2, 1): H^*(SmallGroup(2,1); GF(2))}
            sage: H.restriction_maps()
            {1: [(2, 1),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(2,1); GF(2))],
             2: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))],
             3: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))]}

        Hence, there are 3 special subgroups, that are of two different isomorphism types. We
        now destroy them on purpose::
         
            sage: H.RestrMaps={}
            sage: H.subgps={}

        Next, we put them back manuall. Note that it is attempted
        to reload the subgroups from cache or from disk::
        
            sage: H.InsertSubgroup(2,1,1)
            sage: H.InsertSubgroup(4,2,2)

        The third subgroup has the same automorphism type as the second, so, 
        there is no need to reload it, but it doesn't hurt either::
        
            sage: H.InsertSubgroup(4,2,3)
            sage: H.subgroups()
            {(4, 2): H^*(SmallGroup(4,2); GF(2)), (2, 1): H^*(SmallGroup(2,1); GF(2))}
            sage: H.restriction_maps()
            {1: [(2, 1),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(2,1); GF(2))],
             2: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))],
             3: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))]}

        Next, we show where to find the files defining the inclusion of the second and third
        subgroup. Although they are isomorphic, the inclusions are, of course, different::
          
            sage: from pGroupCohomology.mtx import MTX
            sage: print MTX(H.inc_folder+H.GStem+'sg2.ima')
            [1 0 0 0 0 0 0 0]
            [0 0 0 1 1 1 1 1]
            [0 1 0 0 0 0 0 0]
            [0 0 0 0 0 1 0 1]
            sage: print MTX(H.inc_folder+H.GStem+'sg3.ima')
            [1 0 0 0 0 0 0 0]
            [0 0 0 1 1 1 1 1]
            [0 0 1 0 0 0 0 0]
            [0 0 0 0 0 0 1 1]

        Finally, we demonstrate how to continue the computation. After re-inserting the special
        subgroups, it is necessary to re-compute the Dickson classes in the special subgroups
        (using the method dickson_in_subgroup), or to use the elimination method. 
        We chose the second way::
          
            sage: H.setprop('useElimination',True)
            sage: H.option('noprot')
            sage: H.make()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
             b_1_0, a 1-Cochain in H^*(D8; GF(2)),
             b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]
              
        """
        cdef int p = self.Resl.coef()
        from pGroupCohomology import CohomologyRing
        if not ((isinstance(q,int) or isinstance(q,Integer)) and (isinstance(nr,int) or isinstance(nr,Integer)) and (isinstance(n,int) or isinstance(n,Integer))):
            raise TypeError, "Subgroup and imbedding have to be defined by three integers"
        if self.subgps.has_key((q,nr)): # that isomorphism type is known
            M=MTX(self.inc_folder+self.GStem+'sg'+str(n)+'.ima')
            #N = MTX(p,1,self.Resl.grouporder())
            #N[0,0] = 1
            ch = self.hom(M, self.subgps[(q,nr)]) #ChMap(self.subgps[(q,nr)].Resl,self.Resl,M,N,0)
            #for i in range(self.Resl.deg()):
            #    ch.lift()
        else:
            saveopts = copy(OPTION.opts)
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"Inserting SmallGroup(%d,%d) as a subgroup"%(q,nr)
            # we take the group from the private data base, since there might 
            # be problems with write permission, and not from the web, since
            # that really doesn't make much sense for abelian groups
            h = CohomologyRing.user_db(q,nr, Subgroups= False, websource = False, options=('noprot','notiming')) 
            h.Resl.indentation(self.Resl.getindent()+1)
            h.make()
            OPTION.opts = saveopts
            M=MTX(self.inc_folder+self.GStem+'sg'+str(n)+'.ima')
            if not (M.base()):
                raise RuntimeError, '%s subgroup is not of type (%d,%d)'%(Ordinals(n),q,nr) 
            #N = MTX(p,1,self.Resl.grouporder())
            #N[0,0] = 1
            ch = self.hom(M,h) #ChMap(h.Resl,self.Resl,M,N,0)
            CurrDeg = self.Resl.deg()
            while h.knownDeg < CurrDeg: # i in range(self.Resl.deg()):
                h.next(Forced=True,KeepDecomposables=True,ElabSubgp=True)
            #while ch.knownDeg() < CurrDeg:
            #    ch.lift()
            self.subgps[(q,nr)] = h
        # in either case, the new restriction map must be stored.
        self.RestrMaps[n] = [(q,nr),ch]

    def InitSubgroups(self):
        """
        Initialise maximal elementary abelian subgroups

        If self is the cohomology ring of a non-abelian p-group then ``InitSubgroups``
        initialises various group theoretic data, such as the `p`-rank or the rank
        of the center; it also creates or reloads the cohomology rings of the greatest 
        central and the maximal elementary abelian subgroups, and, if applicable,
        computes the Dickson classes of the maximal elementary abelian subgroups.

        NOTE:
        
        This function should only be of internal use and will be invoked when initializing
        a cohomology ring. It requires the presence of a certain Gap-readable file that is
        created during initialisation of the cohomology ring.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)

        Now we destroy and reconstruct the subgroups and the respective restriction maps::
        
            sage: H.subgps={}
            sage: H.RestrMaps={}
            sage: H.InitSubgroups()
            sage: H.subgroups()
            {(4, 2): H^*(SmallGroup(4,2); GF(2)), (2, 1): H^*(SmallGroup(2,1); GF(2))}
            sage: H.restriction_maps()
            {1: [(2, 1),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(2,1); GF(2))],
             2: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))],
             3: [(4, 2),
                 Induced homomorphism of degree 0 from H^*(D8; GF(2)) to H^*(SmallGroup(4,2); GF(2))]}
            sage: H.subgpDickson
            {(4, 2): [c_1_1, a 1-Cochain in H^*(SmallGroup(4,2); GF(2))]}
            
        Finally, we show location and content of the Gap-readable file that defines
        the subgroup structure::
        
            sage: print file(H.inc_folder+H.GStem+'.sgs').read()
            # Subgroup information for 8gp3
            local numSpecialSubgps, specialSubgpId, CrankPos, numMaxels,
              maxelRankPos, numBoltons, Bolton;
            numSpecialSubgps := 3;
            specialSubgpId := [ [ 2, 1 ], [ 4, 2 ], [ 4, 2 ] ];
            CrankPos := [ 1, 1 ];
            numMaxels := 2;
            maxelRankPos := [ [ 2, 2 ], [ 2, 3 ] ];
            numBoltons := 0;
            Bolton := [];
            return [numSpecialSubgps, specialSubgpId, CrankPos,
              numMaxels, maxelRankPos, numBoltons, Bolton];
              
        """
        ## Get information from the gap-readable sgs-file
        try:
            L = gap('ReadAsFunction("%s")()'%(self.inc_folder+self.GStem+'.sgs'))
        except TypeError:  # can't be loaded
            return
        cdef int i
        NumSubgps = Integer(L[1])
        self.CElPos = Integer(L[3][2])
        self.CenterRk = Integer(L[3][1])
        self.MaxelPos = [Integer(L[5][i][2]) for i in range(1,Integer(L[4])+1)]
        self.MaxelRk  = [Integer(L[5][i][1]) for i in range(1,Integer(L[4])+1)]
        self.pRank = max(self.MaxelRk)
        self.Dickson = (self.pRank-self.CenterRk)*[None]
        cdef int p = self.Resl.coef()
        ## insert the special subgroups
        for i in xrange(1,NumSubgps+1):
            self.InsertSubgroup(Integer(L[2][i][1]),Integer(L[2][i][2]),i)
        if self.useElimination is None: # determine it by a heuristic
            if Integer(p)**(self.pRank-self.CenterRk)*(1+ (p%2)) > 18:
                if OPTION.opts['prot']:
                    print (self.Resl.getindent()*'::: ')+"The degree of Dickson invariants is too high.\nWe will use elimination to find Dickson invariants"
                self.setprop('useElimination',True)
            else:
                self.setprop('useElimination',False)
        self.subgpDickson = {}
        for id,G in self.subgps.items():
            while G.knownDeg < 2:
                G.next(Forced=True,KeepDecomposables=True,ElabSubgp=True)
            if not self.useElimination:
                self.dickson_in_subgroup(id) #self.pRank,self.CenterRk)

    def restrict_to_subgroup (self, n):
        """
        Returns the restriction of the cohomology ring generators to the `n`-th subgroup

        SEE ALSO:

        - :meth:`restrictions_as_string`

        INPUT:
        
        - ``n`` -- integer indicating a special subgroup

          * ``n=1`` refers to the greatest central elementary abelian subgroup
          * ``n>1`` refers to a representative of conjugacy classes of maximal
            elementary abelian subgroups

        OUTPUT:

        A list of cochains of the specified subgroup, obtained by restricting
        the generators of self

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()

        H has two isomorphism types of special subgroups, but 3 conjugacy classes::
        
            sage: len(H.subgroups())
            2
            sage: len(H.restriction_maps())
            3

        We show the restriction to the three conjugacy classes. First, the greatest
        central elementary abelian subgroup::
        
            sage: H.restrict_to_subgroup(1)
            [c_1_0^2, a 2-Cochain in H^*(SmallGroup(2,1); GF(2)),
             0, a 1-Cochain in H^*(SmallGroup(2,1); GF(2)),
             0, a 1-Cochain in H^*(SmallGroup(2,1); GF(2))]

        Now the restrictions to the maximal elementary abelian subgroups::
        
            sage: H.restrict_to_subgroup(2)
            [c_1_0*c_1_1+c_1_0^2, a 2-Cochain in H^*(SmallGroup(4,2); GF(2)),
             c_1_1, a 1-Cochain in H^*(SmallGroup(4,2); GF(2)),
             0, a 1-Cochain in H^*(SmallGroup(4,2); GF(2))]
            sage: H.restrict_to_subgroup(3)
            [c_1_0*c_1_1+c_1_0^2, a 2-Cochain in H^*(SmallGroup(4,2); GF(2)),
             0, a 1-Cochain in H^*(SmallGroup(4,2); GF(2)),
             c_1_1, a 1-Cochain in H^*(SmallGroup(4,2); GF(2))]

        """
        h = self.subgps[self.RestrMaps[n][0]]
        ch = self.RestrMaps[n][1]
        return [h.cochain_to_polynomial(ch*G) for G in self.Gen]

    def restrictions_as_string (self,n):
        """
        Restriction of the cohomology ring generators to the n-th subgroup as list of strings

        SEE ALSO:

        - :meth:`restrict_to_subgroup`

        INPUT:
        
        - ``n`` -- integer indicating a special subgroup

          * ``n=1`` refers to the greatest central elementary abelian subgroup
          * ``n>1`` refers to a representative of conjugacy classes of maximal
            elementary abelian subgroups

        OUTPUT:
        
        A list of strings, representing a polynomial in the generators of the
        cohomology of the specified subgroup, yielding the restriction
        of the generators of self to that subgroup

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a
        temporary directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()

        H has two isomorphism types of special subgroups, but 3 conjugacy classes::
        
            sage: len(H.subgroups())
            2
            sage: len(H.restriction_maps())
            3

        We show the restriction to the three conjugacy classes. First, the greatest
        central elementary abelian subgroup::
        
            sage: H.restrictions_as_string(1)
            ['c_1_0^2', '0', '0']
            
        Now the restrictions to the maximal elementary abelian subgroups::
        
            sage: H.restrictions_as_string(2)
            ['c_1_0*c_1_1+c_1_0^2', 'c_1_1', '0']
            sage: H.restrictions_as_string(3)
            ['c_1_0*c_1_1+c_1_0^2', '0', 'c_1_1']
            
        """
        cdef COCH X
        L = self.__dict.get('Restriction_%d'%(n),None)
        if (L is None) or (len(L)<len(self.Gen)):
            L = [X.Name for X in self.restrict_to_subgroup(n)]
            self.__dict['Restriction_%d'%(n)] = L
        return L

    def nil_reduce(self, COCH C):
        """
        Reduce a cochain over an elementary abelian group by the nil radical.

        INPUT:

        ``C``, an element of self

        OUTPUT:

        The list of coefficients representing the reduced cochain.
        
        ASSUMPTION:

        self must be elementary abelian.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.

        We need to choose a `p`-group with `p>2`, since we would like to have an example
        where the cohomology of elementary abelian groups has non-trivial nil radical.
        However, this requires that Singular 3-1-0 be installed.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()

        Now we consider a maximal elementary abelian subgroup of ``H``. There are four
        conjugacy classes of elementary abelian subgoups of order 9, and we choose
        the restriction map to one of them.
        ::
        
            sage: A = H.subgroups()[(9,2)]
            sage: r = H.restriction_maps()[2][1]
            sage: r
            Induced homomorphism of degree 0 from H^*(E27; GF(3)) to H^*(SmallGroup(9,2); GF(3))
            
        It turns out that the restriction to ``A`` of the second generator of ``H`` is
        non-zero, but is contained in the nil radical::
        
            sage: print r(H.2)
            2-Cochain in H^*(SmallGroup(9,2); GF(3)),
            represented by
            [1 0 0]
            sage: print A.nil_reduce(r(H.2))
            [0, 0, 0]

        We verify that ``r(H.2)`` really belongs to the nil radical by expressing it as
        a product of nilpotent generators of ``A``::
        
            sage: A.cochain_to_polynomial(r*H.2)
            -a_1_0*a_1_1, a 2-Cochain in H^*(SmallGroup(9,2); GF(3))
            
        """
        if not (self.Resl is C.resolution()):
            raise TypeError, "Cochain and self must be defined over the same resolution"
        d = C.Deg
        cdef int p = self.Resl.coef()
        if p==2:
            return C.Data._rowlist_(0)
        if not self.NilBasis[d]:
            return C.Data._rowlist_(0)
        if not isinstance(self.NilBasis[d], MTX):
            self.NilBasis[d]=MTX(p, self.NilBasis[d]).echelon()
        B = self.NilBasis[d]
        M = C.Data.__copy__()
        fM,jM = M.lead()
        cdef int k,Bnrows
        if fM:
            Bnrows = B.nrows()
            for k from 0 <= k < Bnrows:
                fB,jB = B[k].lead()
                if (jM <= jB) and M[0,jB]:
                    M = M - ((B[k])._mulInt_(M[0,jB]))/fB
                    fM,jM = M.lead()
                    if not fM:
                        break
        return M._rowlist_(0)

    def nil_preimage(self,n):
        """
        Return the preimage of the nil-radical of the `n`-th special subgroup

        INPUT:

        - ``n`` -- integer indicating a special subgroup

          * ``n=1`` refers to the greatest central elementary abelian subgroup
          * ``n>1`` refers to a representative of conjugacy classes of maximal
            elementary abelian subgroups

        OUTPUT:
        
        An ideal in the Singular interface. It corresponds to the preimage
        of the nil radical of the cohomology of the n-th special subgroup.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.

        We need to choose a `p`-group with `p>2`, since we would like to have an example
        where the cohomology of elementary abelian groups has non-trivial nil radical.
        However, this requires Singular 3-1-0 being installed.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()
            sage: H.nil_preimage(2)
            a_1_0,
            a_1_1,
            b_2_1,
            b_2_2,
            b_2_3,
            a_3_4,
            a_3_5
            sage: _.parent()
            Singular
            sage: H.nil_preimage(3)
            a_1_0,
            a_1_1,
            b_2_0,
            b_2_1,
            b_2_2,
            a_3_4,
            a_3_5
            sage: H.nil_preimage(4)
            a_1_0,
            a_1_1,
            b_2_1-b_2_0,
            b_2_2,
            b_2_3-b_2_0,
            a_3_4,
            a_3_5
            sage: H.nil_preimage(5)
            a_1_0,
            a_1_1,
            b_2_1+b_2_0,
            b_2_2+b_2_0,
            b_2_3+b_2_0,
            a_3_4,
            a_3_5

        A cohomology class is nilpotent if and only if its restriction to all maximal elementary
        abelian subgroups is nilpotent. Hence, the nil radical of ``H`` is the intersection of
        the above ideals, namely::

            sage: H.nil_preimage(2).intersect(H.nil_preimage(3)).intersect(H.nil_preimage(4)).intersect(H.nil_preimage(5))
            a_1_0,
            a_1_1,
            a_3_4,
            a_3_5,
            b_2_1*a_1_0+b_2_0*a_1_0,
            b_2_2*a_1_0+b_2_0*a_1_0,
            b_2_3*a_1_0+b_2_0*a_1_0,
            b_2_1*a_1_1+b_2_0*a_1_1,
            b_2_2*a_1_1+b_2_0*a_1_1,
            b_2_3*a_1_1+b_2_0*a_1_1,
            b_2_0*b_2_3-b_2_0*b_2_1,
            b_2_1^2-b_2_0*b_2_2-b_2_0*b_2_1,
            b_2_1*b_2_2+b_2_0*b_2_2,
            b_2_1*b_2_3-b_2_0*b_2_2-b_2_0*b_2_1,
            b_2_1*b_2_2+b_2_0*b_2_2,
            b_2_2^2+b_2_0*b_2_2,
            b_2_2*b_2_3+b_2_0*b_2_2,
            b_2_1*b_2_3+b_2_0*b_2_3-b_2_0*b_2_2+b_2_0*b_2_1,
            b_2_2*b_2_3+b_2_0*b_2_3+b_2_0*b_2_2-b_2_0*b_2_1,
            b_2_1*a_3_4+b_2_0*a_3_4,
            b_2_2*a_3_4+b_2_0*a_3_4,
            b_2_3*a_3_4+b_2_0*a_3_4,
            b_2_1*a_3_5+b_2_0*a_3_5,
            b_2_2*a_3_5+b_2_0*a_3_5,
            b_2_3*a_3_5+b_2_0*a_3_5


        Indeed, the above gives nilpotent cohomology classes that aparently do not contain
        nilpotent generators in them::

            sage: print H('b_2_1^2-b_2_0*b_2_2-b_2_0*b_2_1')^2
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            sage: print H(' b_2_1*b_2_3+b_2_0*b_2_3-b_2_0*b_2_2+b_2_0*b_2_1')^2
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]

        """
        cdef COCH X
        cdef RESL Resl = self.Resl
        if OPTION.opts['prot']:
            print (Resl.indent*'::: ')+"> Lift nil radical of the %s special subgroup"%(Ordinals(n))
        G = self.subgps[self.RestrMaps[n][0]]
        L = self.restrictions_as_string(n)
        singular.eval('setring %sr(%d)'%(G.prefix,G.knownDeg))
        dgb = singular.eval('degBound')
        singular.eval('degBound = 0')
        if not Integer(singular.eval('defined(tmpNil)')):
            if Resl.G_Alg.Data.p%2:
                singular.eval('ideal tmpNil = maxideal(1)')
                singular.eval('tmpNil = groebner(ideal(tmpNil[nvars(basering)/2 +1..nvars(basering)]))')
            else:
                singular.eval('ideal tmpNil = groebner(ideal(0))')
        if not (Resl.G_Alg.Data.p%2):
            if Integer(singular.eval('defined(Rest)')):
                singular.eval('Rest = %s'%(','.join(L)))
            else:
                singular.eval('ideal Rest = %s'%(','.join(L)))
            singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
            singular.eval('degBound = '+dgb)
            return singular('kernel(%sr(%d),Rest)'%(G.prefix,G.knownDeg))
        self._makeOrderMatrix_()
        tmp_r = singular.ring(Resl.G_Alg.Data.p,'(%s)'%(','.join(['@'+X.Name for X in self.Gen])),'M(%sM)'%(self.prefix))
        if Resl.G_Alg.Data.p%2: ## we need to take care of the non-commutativity
            tmp_r = singular('SuperCommutative(%d,%d)'%(self.firstOdd+1, len(self.Gen)))
            tmp_r.set_ring()
        tmpR = singular('%sr(%d)'%(G.prefix,G.knownDeg)) + tmp_r
        tmpR.set_ring()
        if Integer(singular.eval('defined(Rest)')):
            singular.eval('Rest = %s'%(','.join(L)))
        else:
            singular.eval('ideal Rest = %s'%(','.join(L)))
        singular.eval('ideal tmpNil = imap(%sr(%d),tmpNil)'%(G.prefix,G.knownDeg))
        singular.eval('for (int tmp_i=1; tmp_i<=%d;tmp_i++){ Rest[tmp_i] = Rest[tmp_i]-var(%d+tmp_i); }'%(len(self.Gen),len(G.Gen)))
        singular.eval('tmpNil = groebner(tmpNil+Rest)')
        # Now, we need to filter all polynomials that are purely in terms of @<name>
        singular.eval('ideal Filter = maxideal(1)')
        singular.eval('Filter = groebner(ideal(Filter[1..%d]))'%(len(G.Gen)))
        singular.eval('ideal FtmpNil = NF(tmpNil,Filter)')
        singular.eval('for (tmp_i=1; tmp_i<=ncols(tmpNil); tmp_i++) { if (FtmpNil[tmp_i]<>tmpNil[tmp_i]) { tmpNil[tmp_i] = 0; }; }')
        singular.eval('tmpNil = simplify(tmpNil,6)')
        tmp_r.set_ring()
        singular.eval('ideal tmpNil = imap(%s,tmpNil)'%(tmpR.name()))
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        I = singular('fetch(%s,tmpNil)'%(tmp_r.name()))
        singular.eval('kill tmp_i')
        singular.eval('degBound = '+dgb)
        return I

    def nil_radical(self):
        """
        Compute the nil radical of self

        OUTPUT:

        An ideal (Groebner basis) defined in the singular interface, representing the
        ideal of nilpotent elements of self.

        THEORY:

        An element of a cohomology ring is nilpotent if and only if its restriction to all
        maximal elementary abelian subgroups is nilpotent. Hence, we compute the intersection
        of the preimages of the nil radicals of the cohomology rings of the maximal elementary
        abelian subgroups. Since we consider the nil radical not in a quotient ring but in
        a free graded commutative ring, we must also take into account the relation ideal of
        the cohomology ring.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.

        We need to choose a `p`-group with `p>2`, since we would like to have an example
        where the cohomology of elementary abelian groups has non-trivial nil radical.
        However, this requires Singular 3-1-0 being installed.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3, websource=False)
            sage: H.make()
            sage: N = H.nil_radical()
            sage: N
            a_1_0,
            a_1_1,
            a_3_4,
            a_3_5,
            b_2_0*b_2_3-b_2_0*b_2_1,
            b_2_1^2-b_2_0*b_2_2-b_2_0*b_2_1,
            b_2_1*b_2_2+b_2_0*b_2_2,
            b_2_1*b_2_3-b_2_0*b_2_2-b_2_0*b_2_1,
            b_2_2^2+b_2_0*b_2_2,
            b_2_2*b_2_3+b_2_0*b_2_2


        Not all elements of ``N`` are zero::

            sage: for X in N:
            ...     print H(repr(X))
            1-Cochain in H^*(E27; GF(3)),
            represented by
            [1 0]
            rdeg = 0
            ydeg = 1
            1-Cochain in H^*(E27; GF(3)),
            represented by
            [0 1]
            rdeg = 0
            ydeg = 1
            3-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 1 0]
            rdeg = 0
            ydeg = 1
            3-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 1]
            rdeg = 0
            ydeg = 1
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [1 0 0 0 0 0 0]
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0]
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [1 0 0 0 0 0 0]
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [1 1 0 0 0 0 0]
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [2 1 0 0 0 0 0]
            4-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 1 0 0 0 0]
        
        But all elements of ``N`` are nilpotent; in fact, squaring the elements
        suffices::
        
            sage: for X in N:
            ...     print H(repr(X))^2
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0]
            2-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0]
            6-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0]
            6-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0]
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]

        """
        if not self.completed:
            raise ValueError, "The cohomology computation is not complete, yet"
        self.set_ring()
        if self.group().IsAbelian():
            return singular.ideal(self.Rel)
        if self.NILRADICAL:
            return singular.ideal(self.NILRADICAL)
        OUT = self.nil_preimage(self.MaxelPos[0])
        for n in self.MaxelPos[1:]:
            OUT = OUT.intersect(self.nil_preimage(n))
        OUT = (OUT+singular.ideal(self.RelG)).groebner()
        self.setprop('NILRADICAL', [repr(X) for X in OUT])
        return OUT
        
    def _makeOrderMatrix_(self):
        """
        Create an integer matrix in Singular, defining David Green's matrix order for self

        OUTPUT:
        
        The matrix is defined in the Singular interface under the name ``self.prefix+'M'``
 
        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.
        ::
        
        sage: tmp_root = tmp_filename()
        sage: from pGroupCohomology import CohomologyRing
        sage: CohomologyRing.set_user_db(tmp_root)
        sage: H = CohomologyRing.user_db(8,3)
        sage: H.make()
        sage: H._makeOrderMatrix_()
        sage: singular('%sM'%(H.prefix))
             2     1     1
            -1     0     0
             0    -1     0
             
        """
        # The following is the raw data that we want to turn into something that Singular can understand
        if singular('defined(%sM)'%(self.prefix)):
            singular.eval('kill %sM'%(self.prefix))
        cdef list RawList = [self.degvec]
        cdef COCH X
        cdef int rk = 1
        cdef int rk_tmp = Matrix(ZZ, RawList+[[-X.Rdeg for X in self.Gen]]).rank()
        if rk_tmp > rk:
            RawList = RawList+[[-X.Rdeg for X in self.Gen]]
            rk = rk_tmp
        rk_tmp = Matrix(ZZ, RawList+[[-X.Ydeg for X in self.Gen]]).rank() 
        if rk_tmp > rk:
            RawList = RawList+[[-X.Ydeg for X in self.Gen]]
            rk = rk_tmp
        # First, we send these lines to Singular, creating the first lines of the matrix
        singular.eval('intmat %sM[%d][%d] = '%(self.prefix,len(self.Gen),len(self.Gen)) + ','.join([str(x) for x in add(RawList,[])]))
        M = Matrix(ZZ,RawList).transpose()
        # The rest of the order matrix should provide a reverse lexicographic order
        # idea:
        #     add colums (starting at the last one). If the rank increases then
        #     that place is irrelevant for (rev)lex order
        # Hence, we test, for what N.nrows()>i>=0, M[i:].rank() increases. For these i,
        # we don't need another entry.
        critrows = len(self.degvec)*[1] # will add a row with -1 at position i if critrows[i]==1
        rk_tmp = 0
        cdef int i,j
        cdef int Mnrows = M.nrows()
        for i from Mnrows > i >= 0:
            if M[i:].rank() > rk_tmp:
                rk_tmp+=1
                critrows[i]=0
            if rk_tmp==M.ncols():
                break
        j = len(RawList) # Row number j of the Singular matrix is already ok
        for i from 0 <= i < len(critrows):
            if critrows[i]:
                j=j+1
                singular.eval('%sM[%d,%d] = -1'%(self.prefix,j,i+1))

    def PrescribedRestrictions(self, L):
        """
        Try to construct a cochain of self that has given restrictions to the elementary abelian subgroups

        INPUT:
        
        - ``L`` -- a list of elements of the form ``[i,C]``, where

          * ``i`` -- the number of a special subgroup
          * ``C`` -- a cochain of the ``i``-th special subgroup

        OUTPUT:
          
        A cochain that has restrictions prescribed in ``L``,
        or ``None``, if such cochain doesn't exist.

        EXAMPLES:
          
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.
        ::
          
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()

        To simplify notation, we give names to the two isomorphism types of special subgroups::
          
            sage: Z = H.subgps[(2,1)]
            sage: M = H.subgps[(4,2)]

        We define cochains in ``Z`` and in ``M``, and try to find common lifts in ``H``.
        Note that ``M`` corresponds to two pairwise non-conjugate maximal elementary abelian
        subgroups of the dihedral group, while ``Z`` corresponds to the greatest central
        elementary abelian subgroup.
        ::

            sage: c = Z.standardCochain(2,0)
            sage: m1 = M.standardCochain(2,0)
            sage: m2 = M.standardCochain(2,1)
            sage: m3 = M.standardCochain(2,2)
            sage: print H.PrescribedRestrictions([[1,c],[2,m2]])
            None
            sage: print H.PrescribedRestrictions([[1,c],[2,m1+m2]])
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [0 0 1]

        Note that the lift is not unique. If we also prescribe a cochain in the third
        special subgroup, we still get a lift of ``[1,c]`` and ``[2,m1+m2]``, but a different
        lift::
          
            sage: print H.PrescribedRestrictions([[1,c],[2,m1+m2],[3,m1+m2+m3]])
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [0 1 1]

        Finally, we test whether we really have the restrictions that we want::

            sage: L = H.PrescribedRestrictions([[1,c],[2,m1+m2],[3,m1+m2+m3]])
            sage: H.restriction_maps()[1][1](L) == c
            True
            sage: H.restriction_maps()[2][1](L) == m1+m2
            True
            sage: H.restriction_maps()[3][1](L) == m1+m2+m3
            True
            
        """
        cdef int i,j,k
        cdef int p = self.Resl.coef()
        cdef MTX tmpM
        if not (isinstance(L,list) or isinstance(L,tuple)):
            raise TypeError, "list or tuple expected"
        d = L[0][1].deg()
        for X in L:
            if not ((isinstance(X, list) or isinstance(X,tuple)) and (len(X)==2)):
                raise TypeError, "argument must be a list/tuple of lists/tuple of length 2"
            if not ((isinstance(X[0],int) or isinstance(X[0],Integer)) and (isinstance(X[1],COCH))):
                raise TypeError, "Pairs (integer, COCH) expected"
            if not (X[1].deg()==d):
                raise TypeError, "Cochains must all have the same degree"
            if not (self.RestrMaps[X[0]][1].src() is X[1].resolution()):
                raise TypeError, "Cochain (2nd list element) must belong to the specified subgroup (1st list element)"
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"simultaneously lifting subgroup cochains of degree %d"%(d)
        if OPTION.opts['timing']:
            ct = cputime()
            wt = walltime()
        cdef int RK = self.Resl.rank(d)
        for X in L:
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"\nRestriction to %s subgroup (order %s)"%(Ordinals(X[0]),self.RestrMaps[X[0]][0][0])
            while (self.RestrMaps[X[0]][1].knownDeg()<d):
                self.RestrMaps[X[0]][1].lift()
        self.Resl.free_ugb()
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"> Evaluating restrictions of ring generators"
        cdef int sum_rk = add([self.RestrMaps[X[0]][1].src().rank(d) for X in L])
        cdef int src_rk
        cdef list ttmpL
        cdef list tmpL
        if OPTION.opts['useMTX']:
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> > Construct MTX matrix"
            tmpL = []
            for i from 0 <= i < RK:
                ttmpL = []
                for X in L:
                    tmpM = self.RestrMaps[X[0]][1][d]
                    src_rk = self.RestrMaps[X[0]][1].src().rank(d)
                    for j from 0 <= j < src_rk:
                        ttmpL.append(tmpM[j*RK+i,0])
                tmpL.append(ttmpL + i*[0]+[1]+(RK-i-1)*[0])
            M = MTX(p, tmpL)
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> > Compute echelon form of %dx%d matrix"%(M.nrows(),M.ncols())
            M = M.full_echelon()
        else:
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> Construct matrix"
            M = Matrix(GF(p), RK, sum_rk+RK, 0)
            for i from 0 <= i < RK:
                M[i] = add([(self.RestrMaps[X[0]][1]*self.standardCochain(d,i)).MTX()._rowlist_(0) for X in L], []) + i*[0]+[1]+(RK-i-1)*[0]
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> > Compute echelon form of %dx%d matrix"%(M.nrows(),M.ncols())
            M.echelonize()
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"> echelon is computed"
        cdef list Piv = M.pivots()
        cdef list RestL = add([X[1].MTX()._rowlist_(0) for X in L],[])
        cdef MTX RestM = MTX(p, 1, sum_rk)
        Lift = MTX(p,1,RK)
        cdef int lenPiv = len(Piv)
        cdef list RowL
        for i from 0 <= i < lenPiv:
            #if Piv[i]>=sum_rk:
            #    break
            if (Piv[i]<sum_rk) and RestL[Piv[i]]:
                if OPTION.opts['useMTX']:
                    RowL = M._rowlist_(i)
                    Lift  = Lift  + MTX(p,[RowL[sum_rk:]])._mulInt_(RestL[Piv[i]])
                    RestM = RestM + MTX(p,[RowL[:sum_rk]])._mulInt_(RestL[Piv[i]])
                else:
                    Lift  = Lift  + MTX(p,[M[i].list()[sum_rk:]])._mulInt_(RestL[Piv[i]])
                    RestM = RestM + MTX(p,[M[i].list()[:sum_rk]])._mulInt_(RestL[Piv[i]])
        if OPTION.opts['timing']:
            print "Total time for simultaneous cochain lift:"
            print "   CPU : %.2f"%(cputime(ct))
            print "   Wall: %.2f"%(walltime(wt))

        if RestM._rowlist_(0) == RestL:
            Lift.set_immutable()
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"Simultaneous lift was successful!"
            return COCH(self, d,'Lift',Lift)
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"There was no simultaneous lift"
        return None
    
###################################################
## Auxiliary methods for Benson's test
## -- new version: work with cochains.
    def dickson_in_subgroup(self,id):#r,z):
        """
        Compute Dickson classes for an elementary abelian group, considered as subgroup of the group of self.

        INPUT:
        
        - ``id`` -- the small groups address (pair of integers) of some elementary abelian subgroup of self

        OUTPUT:
        
        Let `G` be the finite `p`-group whose cohomology ring self represents. The attribute ``subgbDickson``
        of self is changed and contains cochains of degree `p^{r-z}-p^{r-z-i}`, where `r` is the `p`-rank of
        `G`, `z` is the rank of the center of `G`, and `i = 0,...,r-z`.
        These cochains are constructed using Dickson invariants in the polynomial part of
        self, vanishing on the first `z` generators. Note that in some cases a trivial
        cochain results, but it still has the stated degree.

        NOTE:
        
        For applying Benson's completeness criterion, one has to find Dickson classes in the
        ambient group that simultaneously restrict to (powers of) the classes in the special
        subgroups found with dickson_in_subgroup. However, if `p^{r-z}-p^{r-z-i}` is too large, one
        should use :meth:`find_dickson`, that avoids the construction of cochains of
        high degrees.

        EXAMPLES:
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit. We choose a group that
        has two different isomorphism classes of maximal elementary abelian subgroups::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(32,27, websource=False)

        The Dickson invariants for the maximal elementary abelian subgroups were already computed
        when creating ``H``. We destroy them on purpose and reconstruct them::

            sage: H.subgpDickson = {}
            sage: H.dickson_in_subgroup((8,5))
            sage: H.subgpDickson
            {(8, 5): [(c_1_2)**2, a 2-Cochain in H^*(SmallGroup(8,5); GF(2)),
                      ((c_1_0)**3)*0, a 3-Cochain in H^*(SmallGroup(8,5); GF(2))]}
            sage: H.dickson_in_subgroup((16,14))
            sage: H.subgpDickson
            {(16, 14): [(c_1_2)*(c_1_2)+(c_1_2)*(c_1_3)+(c_1_3)*(c_1_3), a 2-Cochain in H^*(SmallGroup(16,14); GF(2)), (c_1_2)*((c_1_2)*(c_1_3))+(c_1_2)*((c_1_3)*(c_1_3)), a 3-Cochain in H^*(SmallGroup(16,14); GF(2))], (8, 5): [(c_1_2)**2, a 2-Cochain in H^*(SmallGroup(8,5); GF(2)), ((c_1_0)**3)*0, a 3-Cochain in H^*(SmallGroup(8,5); GF(2))]}

        """
        if self.subgps[id].knownDeg<2:
            self.subgps[id].make(2) #raise RuntimeError, "we need to know the cohomology ring up to degree 2"
        if self.subgpDickson is None:
            self.subgpDickson = {}
        if OPTION.opts['timing']:
            ct = cputime()
        D = DICKSON(self.Resl.coef())
        r = self.pRank
        z = self.CenterRk
        if self.Resl.coef()==2:
            t = len(self.subgps[id].Gen) - z
        else:
            t = len(self.subgps[id].Gen)/2 - z
        if t<0:
            raise RuntimeError, "The p-rank of the center must not exceed the rank of a maximal elementary abelian subgroup"
        if t==0:
            return
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"Computing Dickson invariants in elementary abelian subgroup of rank", t+z
        cdef list DicksonList = []
        cdef int j,i,k
        p = self.Resl.coef()
        cdef int m = r-z
        for j from 1<=j<=m:
            if r == z+t:
                DicksonList.append(D(t,t-j))
            elif (j>t):
                DicksonList.append(0)
            else:
                DicksonList.append(D(t,t-j)**(p**(m-t)))
        ## DicksonList contains polynomial descriptions of cochains.
        ## Next, we want to turn them into cohomology classes.
        cdef list NewDicksonList = []
        for k from 0<=k<m:
            if DicksonList[k]==0: # have to append 0-cochain. self.Gen[0]**(p**(r-z)-p**(r-z-k-1)) always is of the right degree
                NewDicksonList.append(0*self.subgps[id].Gen[0]**(p**(m)-p**(m-k-1)))
            else:
                L=DicksonList[k].exponents()
                C=DicksonList[k].coefficients()
                if t>1:
                    NewDicksonList.append(add([ Mul(add([ [self.subgps[id].Gen[z+j]]*(L[i][j]) for j in xrange(t)],[]),int(C[i])) for i in xrange(1,len(L))], Mul(add([ [self.subgps[id].Gen[z+j]]*(L[0][j]) for j in xrange(t)],[]),int(C[0])) )) # Mul (in contrast to mul) collects the product from right to left
                else:
                    NewDicksonList.append(add([(self.subgps[id].Gen[z]**(L[i]))*int(C[i]) for i in xrange(1,len(L))], (self.subgps[id].Gen[z]**(L[0]))*int(C[0])))
        self.subgpDickson[id] = NewDicksonList
        if OPTION.opts['timing']:
            print 'Time for computing Dickson invariants in maximal elementary abelian subgroups'
            print "   CPU: %.2f"%(cputime(ct))


    def lift_dickson(self,i,j):
        """
        Try to lift some power of some Dickson class

        INPUT:
        
        - `i = 0, 1, 2, ...` - indicates which Dickson class of the maximal elementary abelian subgroups ought to be lifted
        - `j = 0, 1, 2, ...` - indicates which power of that Dickson class ought to be lifted

        OUTPUT:
        
        If it is possible to find a cochain of self that simultaneously restricts to the `p^j`-th power
        of the `i`-th Dickson classes of all maximal elementary abelian subgroups, potentially modified
        by nilpotent classes, then this cochain is returned. Here, `p` is the prime that divides the
        group order.

        EXAMPLES:
        
        We first create the cohomology ring of an elementary abelian group, whose data files are
        rooted in a temporary directory; it will be removed as soon as Sage is quit.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)
            sage: print H.lift_dickson(0,0)
            1-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1]
            sage: print H.lift_dickson(0,1)
            2-Cochain in H^*(D8; GF(2)),
            represented by
            [1 1 0]

        Indeed, the restrictions of these classes to the two maximal elementary abelian subgroups
        coincides with their Dickson classes, respectively the square of the Dickson classes::
            
            sage: H.subgpDickson[4,2][0] == H.restriction_maps()[2][1](H.lift_dickson(0,0))
            True
            sage: H.subgpDickson[4,2][0] == H.restriction_maps()[3][1](H.lift_dickson(0,0))
            True
            sage: H.subgpDickson[4,2][0]**2 == H.restriction_maps()[2][1](H.lift_dickson(0,1))
            True
            sage: H.subgpDickson[4,2][0]**2 == H.restriction_maps()[3][1](H.lift_dickson(0,1))
            True
            
        """
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"Try to lift %s power of %s Dickson invariant"%(Ordinals(self.Resl.coef()**j),Ordinals(i))
        p = self.Resl.coef()
        L=[[X[0],(self.subgpDickson[X[1][0]][i])**(p**j)] for X in self.RestrMaps.items() if X[0]!=1]
        return self.PrescribedRestrictions(L)
            

###################################################
## Auxiliary methods for Benson's test
##  -- find lifted Dickson invariants based on elimination
    def find_dickson(self):
        """
        Find classes of self that simultaneously restrict to the Dickson classes in special subgroups

        OUTPUT:
        
        The attribute ``Dickson`` (a list) is changed. The entries corresponding to factors of
        successfully lifted Dickson classes are strings that define a polynomial in the generators
        of self yielding the lift.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. We use the option ``useElimination``, in
        order to avoid that the Dickson classes will be lifted by a different method.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(64,33, useElimination=True, websource=False)

        The group is of `p`-rank 3 and its center has rank 1. So, we will eventually get
        two Dickson classes::
        
            sage: H.pRank
            3
            sage: H.CenterRk
            1
            sage: H.Dickson
            [None, None]

        We try whether the Dickson classes can be lifted after computing the ring structure
        out to degree 2::

            sage: H.find_dickson()
            sage: H.Dickson
            [None, None]
        
        We need to get to degree 4 in order to find a lift for the Dickson classes::
        
            sage: H.make(4)
            sage: H.find_dickson()
            sage: H.Dickson
            ['b_2_1^2 + b_4_6', 'b_3_2 + b_3_4']
            
        """
        if self.pRank == self.CenterRk:
            return
        if OPTION.opts['timing']:
            wt = walltime()
        cdef int i
        if not self.Gen:
            return
        if self.knownDeg<2:
            return
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"Try to lift Dickson invariants using elimination methods"

        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        ######################
        ## This shall become the intersection of lifts of the Nil radicals of the special subgroups
        DicksonLift,NilRad = self.find_dickson_in_subgroup(self.MaxelPos[0])
        if DicksonLift is None:
            return
        for Sg in self.MaxelPos[1:]:
            NewDicksonLift, NewNilRad = self.find_dickson_in_subgroup(Sg)
            if NewDicksonLift is None:
                return
            ## Make sure that the items of DicksonLift and NewDicksonLift have the same degree
            singular.eval('intvec %sdvOld = degvec(%s)'%(self.prefix,DicksonLift.name()))
            singular.eval('intvec %sdvNew = degvec(%s)'%(self.prefix,NewDicksonLift.name()))
            singular.eval('for (int tmp_i=1;tmp_i<=nrows(%sdvOld);tmp_i++){ if (%sdvOld[tmp_i]>%sdvNew[tmp_i]) { %s[1,tmp_i]=%s[1,tmp_i]**(%sdvOld[tmp_i]/%sdvNew[tmp_i]); } else {  %s[1,tmp_i]=%s[1,tmp_i]**(%sdvNew[tmp_i]/%sdvOld[tmp_i]); } }'%(self.prefix,self.prefix,self.prefix,NewDicksonLift.name(),NewDicksonLift.name(), self.prefix,self.prefix, DicksonLift.name(),DicksonLift.name(),self.prefix,self.prefix))
            singular.eval('kill tmp_i')
            singular.eval('kill %sdvOld'%(self.prefix))
            singular.eval('kill %sdvNew'%(self.prefix))
            DicksonLift = DicksonLift.cosetIntersect(NilRad,NewDicksonLift,NewNilRad,self.__dict.get('DicksonExp',3))
            if str(DicksonLift.typeof())=='int':
                if OPTION.opts['prot']:
                    print (self.Resl.getindent()*'::: ')+"> simultaneous lift failed at %s subgroup"%(Ordinals(Sg))
                return
            NilRad=NilRad.intersect(NewNilRad)
        #############
        ## Now, DicksonLift contains simultaneous lifts of the Dickson invariants.
        ## Try to optimize it, by factorizing the Dickson-lifts
        cdef list L=[]
        if self.useFactorization:
            R = PolynomialRing(GF(self.Resl.coef()), [X.name() for X in self.Gen])
            singular.eval('attrib(%sI,"isSB",1)'%(self.prefix))
            for i in range(1,DicksonLift.ncols()+1):
                # F=(R(str(DicksonLift[1,i])).factor())
                F=(R(str(DicksonLift[1,i])).factor(proof=False))
                mind = min([Integer(singular("deg(%s)"%(str(X[0])))) for X in F  if (Integer(singular("deg(%s)"%(str(X[0])))) > 0)])
                for X in F:
                    if (Integer(singular("deg(%s)"%(str(X[0]))))==mind):
                        L.append(str(X[0]))
                        break
        else:
            for i in range(1,DicksonLift.ncols()+1):
                L.append(singular.eval('%s[1,%d]'%(DicksonLift.name(),i)))
        self.Dickson = L
        if OPTION.opts['prot']:
            print 'We succeeded to lift Dickson invariants!'
        if OPTION.opts['timing']:
            print 'Time for the attempt to lift Dickson invariants'
            print "   Wall: %.2f"%(walltime(wt))

    def find_dickson_in_subgroup(self,n):
        """
        If possible, return a lift of the Dickson classes (type <SingularElement> matrix)
        and the nil-radical (type <SingularElement> ideal) of the `n`-th special subgroup. Result is cached.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. We use the option ``useElimination``, in
        order to avoid that the Dickson classes will be lifted by a different method.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(64,33, useElimination=True, websource=False)
            sage: H.make(3)
            sage: H.find_dickson_in_subgroup(2)
            (None, None)
            sage: H.find_dickson_in_subgroup(3)
            (None, None)
            sage: H.make(4)
            sage: H.find_dickson_in_subgroup(2)
            (b_4_6,b_3_4+b_3_2,
             a_1_0,
            a_1_1,
            b_2_1,
            b_3_3+b_3_2,
            b_4_4,
            b_3_2^2+b_2_2^3,
            b_3_4^2+b_2_2*b_4_6)
            sage: H.find_dickson_in_subgroup(3)
            (b_4_6+b_2_1^2,b_3_3,
             a_1_0,
            a_1_1,
            b_2_2,
            b_3_2,
            b_3_4+b_3_3,
            b_3_3^2+b_2_1*b_4_6,
            b_4_4^2+b_2_1^2*b_4_6)

        """
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        cdef int i
        #### Probably the following cacheing is incorrect 
        ## Namely, if the lift succeeds for different subgroups in different degrees
        ## then the lifts of one of the Nil radicals may be too small
        L = self.restrictions_as_string(n)
        G = self.subgps[self.RestrMaps[n][0]]
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"> Lift Dickson invariants of the %s special subgroup"%(Ordinals(n))
        singular.eval('setring %sr(%d)'%(G.prefix,G.knownDeg))
        singular.eval('int tmp_i')
        dgb = singular.eval('degBound')
        singular.eval('degBound = 0')
        #########
        ## create Dickson invariants in subgroup
        if not Integer(singular.eval('defined(%sDI)'%(self.prefix))):
            singular.eval('ideal %sDI'%(self.prefix))
            singular.eval('for (tmp_i=%d;tmp_i>=1;tmp_i--) { %sDI[tmp_i] = Delta(%d,%d,%d,tmp_i); }'%(self.pRank-self.CenterRk,self.prefix,self.pRank,self.CenterRk,self.MaxelRk[self.MaxelPos.index(n)]-self.CenterRk))
        if not Integer(singular.eval('defined(%stmpNil)'%(self.prefix))):
            if self.Resl.coef()%2:
                singular.eval('ideal %stmpNil = maxideal(1)'%(self.prefix))
                singular.eval('%stmpNil = groebner(ideal(%stmpNil[nvars(basering)/2 +1..nvars(basering)]))'%(self.prefix,self.prefix))
            else:
                singular.eval('ideal %stmpNil = groebner(ideal(0))'%(self.prefix))
        self._makeOrderMatrix_()
        tmp_r = singular.ring(self.Resl.coef(),'(%s)'%(','.join(['@'+X.name() for X in self.Gen])),'M(%sM)'%(self.prefix))
        if self.Resl.coef()%2: ## we need to take care of the non-commutativity
            tmp_r = singular('SuperCommutative(%d,%d)'%(self.firstOdd+1, len(self.Gen)))
            tmp_r.set_ring()
        tmpR = singular('%sr(%d)'%(G.prefix,G.knownDeg)) + tmp_r
        tmpR.set_ring()
        if Integer(singular.eval('defined(%sRest)'%(self.prefix))):
            singular.eval('%sRest = %s'%(self.prefix, ','.join(L)))
        else:
            singular.eval('ideal %sRest = %s'%(self.prefix, ','.join(L)))
        lastLifted, LiftGB = self.__dict.get('LiftGB_%d'%(n), (0,0))
        
        if not lastLifted:
            singular.eval('ideal %stmpNil = imap(%sr(%d),%stmpNil)'%(self.prefix, G.prefix,G.knownDeg, self.prefix))
            singular.eval('for (tmp_i=1; tmp_i<=%d;tmp_i++){ %sRest[tmp_i] = %sRest[tmp_i]-var(%d+tmp_i); }'%(len(self.Gen),self.prefix,self.prefix,len(G.Gen)))
            #########
            ## this can be used to lift polynomials modulo nil radical and modulo regular elements:
            singular.eval('%stmpNil = groebner(%stmpNil+%sRest+ideal(%s))'%(self.prefix,self.prefix,self.prefix, ','.join(['0']+[L[X] for X in range(len(self.Gen)) if self.Gen[X].rdeg()]))) 
        else:
            singular.eval('ideal %stmpNil = %s'%(self.prefix,LiftGB))
            singular.eval('attrib(%stmpNil,"isSB",1)'%(self.prefix))  ## Hopefully cacheing helps...
            for i in range(len(self.Gen)):
                if self.Gen[i].deg() > lastLifted:
                    singular.eval('%stmpNil = std(%stmpNil,%s-@%s)'%(self.prefix,self.prefix, L[i],self.Gen[i].name()))
        self.setprop('LiftGB_%d'%(n), (self.knownDeg,','.join([singular.eval('%stmpNil[%d]'%(self.prefix,X)) for X in xrange(1,Integer(singular.eval('ncols(%stmpNil)'%(self.prefix)))+1)])))
        singular.eval('ideal %sFilter = maxideal(1)'%(self.prefix))
        singular.eval('%sFilter = groebner(ideal(%sFilter[1..%d]))'%(self.prefix,self.prefix, len(G.Gen)))
        #########
        ## try to lift Dickson invariants:
        singular.eval('ideal %sDI = imap(%sr(%d),%sDI)'%(self.prefix,G.prefix,G.knownDeg,self.prefix))
        singular.eval('ideal %sFDI,%sDILift'%(self.prefix,self.prefix))
        NrDickson = Integer(singular.eval('size(%sDI)'%(self.prefix)))  # the number of non-zero invariants
        singular.eval('%sDILift[ncols(%sDI)]=0'%(self.prefix,self.prefix)) ## ??%(NrDickson)) # make sure that ncols(DI)==ncols(DILift)
        for i in range(self.__dict.get('DicksonExp',3)):
            singular.eval('%sDI = NF(%sDI,%stmpNil)'%(self.prefix,self.prefix,self.prefix))
            singular.eval('for (tmp_i=1;tmp_i<=ncols(%sDI);tmp_i++){ if ((%sDI[tmp_i]<>0) and (NF(%sDI[tmp_i],%sFilter)==%sDI[tmp_i])) { %sDILift[tmp_i] = %sDI[tmp_i]; %sDI[tmp_i] = 0; } else { %sDI[tmp_i]=%sDI[tmp_i]**char(basering); } }'%(self.prefix,self.prefix,self.prefix,self.prefix,self.prefix,self.prefix,self.prefix,self.prefix,self.prefix,self.prefix))
            #print singular.eval('DI')
            if Integer(singular.eval('size(%sDILift)==%d'%(self.prefix,NrDickson))):
                break
        if not Integer(singular.eval('size(%sDILift)==%d'%(self.prefix,NrDickson))):
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> Lift failed"
            return (None,None)

        ############
        ## Next, we care about the lift of the Nil radical
        # Now, we need to filter all polynomials that are purely in terms of @<name>
        singular.eval('ideal %sFtmpNil = NF(%stmpNil,%sFilter)'%(self.prefix,self.prefix,self.prefix))
        singular.eval('for (tmp_i=1; tmp_i<=ncols(%stmpNil); tmp_i++) { if (%sFtmpNil[tmp_i]<>%stmpNil[tmp_i]) { %stmpNil[tmp_i] = 0; }; }'%(self.prefix,self.prefix,self.prefix,self.prefix))
        singular.eval('%stmpNil = simplify(%stmpNil,6)'%(self.prefix,self.prefix))
        tmp_r.set_ring()
        singular.eval('ideal %stmpNil = imap(%s,%stmpNil)'%(self.prefix,tmpR.name(),self.prefix))
        singular.eval('ideal %sDILift = imap(%s,%sDILift)'%(self.prefix,tmpR.name(),self.prefix))
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        I = singular('fetch(%s,%stmpNil)'%(tmp_r.name(),self.prefix))
        DILift = singular('matrix(fetch(%s,%sDILift))'%(tmp_r.name(),self.prefix))
        ###### one might consider cacheing NilLift, in order to hopefully simplify the Lifting next time
        ## However, the current setting is to wait with Lifting until very probably the lift is possible
        ## Hence, we do without cacheing.
        singular.eval('kill tmp_i')
        singular.eval('degBound = '+dgb)
        #print "DILift", DILift
        #print "I", I
        return (DILift,I)
        
    def factorise(self,C):
        """
        Factorisation modulo nil-radical of a cochain that is expressed as polynomial in the generators of self

        INPUT:
        
        - A cochain ``C``. ``C.name()`` should express ``C`` as polynomial in the generators of self

        NOTE:

        In order to express ``C`` as a polynomial in the generators of self, use
        :meth:`cochain_to_polynomial`.

        OUTPUT:
        
        A string, providing the smallest found non-scalar factor of ``C``, potentially after modifying
        ``C`` by an element of the nil-radical.

        NOTE:

        If the method does not succeed to find a factor, it does not mean that ``C`` can't be factorised
        at all. This method is just a heuristics, that may be useful to simplify some computations.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. We use a non-commutative example, which
        requires that Singular 3-1-0 be installed.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3)
            sage: H.make()
            sage: H.gens()
            [1,
             b_2_0, a 2-Cochain in H^*(E27; GF(3)),
             b_2_1, a 2-Cochain in H^*(E27; GF(3)),
             b_2_2, a 2-Cochain in H^*(E27; GF(3)),
             b_2_3, a 2-Cochain in H^*(E27; GF(3)),
             c_6_8, a 6-Cochain in H^*(E27; GF(3)),
             a_1_0, a 1-Cochain in H^*(E27; GF(3)),
             a_1_1, a 1-Cochain in H^*(E27; GF(3)),
             a_3_4, a 3-Cochain in H^*(E27; GF(3)),
             a_3_5, a 3-Cochain in H^*(E27; GF(3))]

        We construct a cochain and find a non-trivial factor, although it is not obvious
        from the definition that a factorisation modulo nilpotent classes exists::
        
            sage: c = H('-b_2_2^2+b_2_1*b_2_3-b_2_2^2-b_2_1^2+b_2_0*b_2_1')
            sage: H.factorise(c)
            '-b_2_1 + b_2_2'

        We verify that indeed we obtained a factor of `c` modulo nilpotent classes::

            sage: H('-b_2_1 + b_2_2')*H('-b_2_1+b_2_2-b_2_2') == c
            False
            sage: print (H('-b_2_1 + b_2_2')*H('-b_2_1+b_2_2-b_2_2')-c)^2
            8-Cochain in H^*(E27; GF(3)),
            represented by
            [0 0 0 0 0 0 0 0 0 0 0 0]
            
        """
        if C is None:
            return None
        cdef COCH X
        cdef RESL Resl = self.Resl
        R = PolynomialRing(GF(Resl.G_Alg.Data.p),[X.Name for X in self.Gen])
        #singular.eval('attrib(%sI,"isSB",1)'%(self.prefix))
        cdef list L
        if isinstance(C,COCH):
            s = C.name()
            ## try to simplify, by killing nilpotent generators
            dgb = singular("degBound")
            singular.eval("degBound = %d"%(C.deg()))
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> Simplify Dickson invariant modulo nilpotent generators"
            L = [X.Name for X in self.Gen if X.Ydeg]
            if L:
                I = (singular("%sI"%(self.prefix))+singular.ideal(L)).groebner()
                s = str(singular("NF(%s,%s)"%(s,I.name())))
            else:
                s = str(singular("NF(%s,%sI)"%(s,self.prefix))) 
        else:
            raise TypeError, "Cochain expected"
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"> Computing factors"
        if s=='0':
            return '0'
        # F=R(s).factor()
        F=R(s).factor(proof=False)
        mindeg = min([Integer(singular("deg(%s)"%(str(Nr[0])))) for Nr in F])
        for Nr in F:
            if (Integer(singular("deg(%s)"%(str(Nr[0]))))==mindeg):
                return str(Nr[0])

    def set_ring(self):
        """
        The underlying free graded commutative ring of self in the Singular interface becomes the base ring

        NOTE:

        This ring is, in general, not isomorphic to the cohomology ring, since
        the relations are disregarded. You can obtain a ring in the Singular
        interface isomorphic to a cohomology ring ``H`` by simply doing ``singular(H)``.

        EXAMPLES:
        
        We first create the cohomology ring of an elementary abelian group, whose data files are
        rooted in a temporary directory; it will be removed as soon as Sage is quit.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.subgroups()[4,2].set_ring()
            sage: print singular.eval('basering')
            //   characteristic : 2
            //   number of vars : 2
            //        block   1 : ordering M
            //                  : names    c_1_0 c_1_1
            //                  : weights      1     1
            //                  : weights     -1     0
            //        block   2 : ordering C
            sage: H.set_ring()
            sage: print singular.eval('basering')
            //   characteristic : 2
            //   number of vars : 3
            //        block   1 : ordering M
            //                  : names    c_2_2 b_1_0 b_1_1
            //                  : weights      2     1     1
            //                  : weights     -1     0     0
            //                  : weights      0    -1     0
            //        block   2 : ordering C

        """
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        return

    def _singular_(self, S):
        """
        Create a representation of self as a quotient ring in the Singular interface

        TESTS::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: singular(H)
            //   characteristic : 2
            //   number of vars : 3
            //        block   1 : ordering M
            //                  : names    c_2_2 b_1_0 b_1_1
            //                  : weights      2     1     1
            //                  : weights     -1     0     0
            //                  : weights      0    -1     0
            //        block   2 : ordering C
            // quotient ring from ideal
            _[1]=b_1_0*b_1_1
        
        """
        if not (S is self.GenS.parent()):
            raise TypeError, "This singular session is not the one which is used in %s"%(repr(self))
        if hasattr(self,'_SINGULAR_'):
            return self._SINGULAR_
        self.set_ring()
        self.make_groebner()
        S.eval('qring %sQ = %sI'%(self.prefix,self.prefix))
        self._SINGULAR_ = S('%sQ'%(self.prefix))
        return self._SINGULAR_

    def make_groebner(self,d=0):
        """
        Compute a Groebner basis for the relation ideal

        INPUT:
        
        ``d`` (optional parameter) -- degree bound. Default: no degree bound

        NOTE:
        
        We keep track up to what degree a Groebner basis for the
        relation ideal is known. Hence, calling ``make_groebner()`` twice
        would only result in a re-computation if the relation ideal
        has changed in the meantime.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. We switch the protocol mode on. Since we
        use a non-commutative example, it is required that Singular 3-1-0 be installed.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3, websource=False)
            sage: H.make(3)
            sage: H.option('prot')
            sage: len(H.RelG)
            5
            sage: H.make_groebner(5)
            Computing Groebner basis up to degree 5 for H^*(E27; GF(3))
            sage: len(H.RelG)
            8
            sage: H.make_groebner()
            Computing complete Groebner basis for H^*(E27; GF(3))
            sage: H.make_groebner(5)
            Complete Groebner basis is already known
            sage: H.option('noprot')
    
        """
        if self.completeGroebner:
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"Complete Groebner basis is already known"
            return
        if OPTION.opts['prot']:
            if d==0:
                print (self.Resl.getindent()*'::: ')+"Computing complete Groebner basis for %s"%(self.__repr__())
            else:
                print (self.Resl.getindent()*'::: ')+"Computing Groebner basis up to degree",d,"for %s"%(self.__repr__())
            if self.useSlimgb:
                print (self.Resl.getindent()*'::: ')+"> using slimgb"
            elif self.useStd:
                print (self.Resl.getindent()*'::: ')+"> using std"
        if OPTION.opts['timing']:
            wt = walltime()
        dgb = singular('degBound')
        singular.eval('degBound = %d'%(d))
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        if self.useSlimgb:
            singular.eval('%sI=slimgb(%sI)'%(self.prefix, self.prefix))
        elif self.useStd:
            singular.eval('%sI=std(%sI)'%(self.prefix, self.prefix))
        else:
            singular.eval('%sI=groebner(%sI)'%(self.prefix, self.prefix))
        singular.eval('degBound = '+str(dgb))
        self.RelG = str(singular('%sI'%(self.prefix))).split(',\n')
        if d==0:
            self.setprop('completeGroebner',True)
        else:
            self.setprop('completeGroebner',False)
        if OPTION.opts['timing']:
            print "Time for computing Groebner basis"
            print "   Wall: %.2f"%(walltime(wt))

    def groebner_basis(self, d=0):
        """
        Return a Groebner basis (type <Sequence>)

        INPUT:
        
        - `d` (optional integer) - compute Groebner basis out to degree `d`
          default: `d=0` (complete Groebner basis)

        OUTPUT:

        A Groebner basis (Sequence of elements of self) out to degree `d` of the current relation ideal for self.

        EXAMPLES:

        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. We switch the protocol mode on. Since we
        use a non-commutative example, it is required that Singular 3-1-0 be installed.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3, websource=False)
            sage: H.make(3)
            sage: L4 = H.groebner_basis(4)
            sage: len(L4)
            5
            sage: L = H.groebner_basis()
            sage: len(L)
            8
            sage: L4
            [(a_1_0)*(a_1_1), a 2-Cochain in H^*(E27; GF(3)), (b_2_1)*(a_1_0)-((b_2_0)*(a_1_1)), a 3-Cochain in H^*(E27; GF(3)), (b_2_2)*(a_1_0)-((b_2_1)*(a_1_1))+(b_2_0)*(a_1_1), a 3-Cochain in H^*(E27; GF(3)), (b_2_2)*(a_1_1)+(b_2_1)*(a_1_1)-((b_2_0)*(a_1_1)), a 3-Cochain in H^*(E27; GF(3)), (b_2_3)*(a_1_0)-((b_2_0)*(a_1_1)), a 3-Cochain in H^*(E27; GF(3))]
            sage: L
            [(a_1_0)*(a_1_1), a 2-Cochain in H^*(E27; GF(3)), (b_2_1)*(a_1_0)-((b_2_0)*(a_1_1)), a 3-Cochain in H^*(E27; GF(3)), (b_2_2)*(a_1_0)-((b_2_1)*(a_1_1))+(b_2_0)*(a_1_1), a 3-Cochain in H^*(E27; GF(3)), (b_2_2)*(a_1_1)+(b_2_1)*(a_1_1)-((b_2_0)*(a_1_1)), a 3-Cochain in H^*(E27; GF(3)), (b_2_3)*(a_1_0)-((b_2_0)*(a_1_1)), a 3-Cochain in H^*(E27; GF(3)), ((b_2_0)*(b_2_3))*(a_1_1)-(((b_2_0)*(b_2_1))*(a_1_1)), a 5-Cochain in H^*(E27; GF(3)), ((b_2_1)**2)*(a_1_1)-(((b_2_0)**2)*(a_1_1)), a 5-Cochain in H^*(E27; GF(3)), ((b_2_1)*(b_2_3))*(a_1_1)-(((b_2_0)**2)*(a_1_1)), a 5-Cochain in H^*(E27; GF(3))]

        """
        from sage.structure.sequence import Sequence
        self.make_groebner(d)
        return Sequence([self(X) for X in self.RelG],self)

#####################################################################
## Ring theoretic Invariants of the cohomology ring
#####################################################################

    def dimension(self):
        """
        Return the dimension of self

        NOTE:

        It is known that the dimension of self equals the `p`-rank of the underlying group.
        
        EXAMPLES:
        
        We first create the cohomology ring of an elementary abelian group, whose data files are
        rooted in a temporary directory; it will be removed as soon as Sage is quit.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.dimension()
            2

        """
        if self.pRank:
            return self.pRank
        return Integer(self.group().RankPGroup())

    def depth(self):
        """
        Return the depth of self, i.e., the length of a maximal regular sequence

        NOTE:

        By theorems of Duflot and Kuhn, the rank of the center of the underlying group
        is a lower bound for the depth. The `p`-rank of the group is equal to the
        dimension of the cohomology ring, and thus is an upper bound.

        EXAMPLES:
        
        We first create the cohomology ring of an elementary abelian group, whose data files are
        rooted in a temporary directory; it will be removed as soon as Sage is quit.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(32,27)
            sage: H.make()
            sage: H.dimension()
            4
            sage: H.CenterRk
            2
            sage: H.depth()
            3
        
        """
        if not self.completed:
            raise ValueError, "The ring structure is not completely known yet"
        if self.rfdt:
            return self.rfdt.count(-1)
        from sage.rings.infinity import Infinity
        return self.a_invariants().count(-Infinity)

    def filter_degree_type(self):
        """
        Return the filter degree type of self

        OUTPUT:

        - The filter degree type (list of integers) of a filter regular system of parameters of self, or
        - ``None``, if the underlying group is abelian or a generalised quaternion group, or if the
          cohomology computation is not finished yet.

        EXAMPLES:
       
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, websource=False)
            sage: print H.filter_degree_type()
            None
            sage: H.make()
            sage: print H.filter_degree_type()
            [-1, -2, -2]

        """
        return self.fdt

    def poincare_series(self):
        """
        Return the Poincare series of self, a univariate rational function.

        ALGORITHM:
        
        For Benson's completeness criterion, we need to verify that the Duflot
        regular generators together with the lifts of the Dickson classes yield a filter
        regular HSOP. This involves computing kernel and quotients of certain
        multiplication maps. These data are saved, and now used for the
        computation of the Poincare series.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.poincare_series()
            1/(t^2 - 2*t + 1)
            
        """
        update = True
        if not self.completed:
            raise ValueError, "Cohomology computation is not finished yet"
        if not self.CenterRk:
            return self.poincare_without_parameters()
        if not self.HilbertVectors:
            self.delprop('lastTested')
            self.completed = False
            self.BensonTest()
            self.completed = True
            if update:
                if OPTION.opts['prot']:
                    print (self.Resl.getindent()*'::: ')+"Save update of %s"%(self.__repr__())
                save(self, self.gps_folder+'H'+self.GStem)
        cdef int lenHV = len(self.HilbertVectors)-1
        cdef int i
        PS = HV2Poly(self.HilbertVectors[-1])
        R = PolynomialRing(QQ,'t')
        t = R('t')
        Xtra = R(1)
        cdef list DV = self.ParameterDegrees
        for i from lenHV > i >= 0:
            PS = PS - (t**DV[i])*Xtra*HV2Poly(self.HilbertVectors[i])
            Xtra = Xtra*(1-t**DV[i])
        if Xtra.leading_coefficient() < 0:
            return (-PS)/(-Xtra)
        return PS/Xtra

    def poincare_without_parameters(self):
        """
        Return the Poincare series of self, a univariate rational function.

        ALGORITHM:
        
        Standard methods: Replace the relation ideal by its leading ideal, which
        is a monomial ideal. For a monomial ideal, the Poincare series is
        computed with an easy induction.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3)
            sage: H.make()
            sage: H.poincare_series()
            1/(t^2 - 2*t + 1)
            
        """
        if not self.completed:
            raise RuntimeError, "Computation isn't finished yet"
        R = PolynomialRing(ZZ,'t')
        t = R('t')
        # Since the computation is complete, it is known that
        # we have a complete Groebner basis
        dgb = singular('degBound')
        singular.eval('degBound = 0')
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        singular.eval('attrib(%sI,"isSB",1)'%(self.prefix))
        # Something goes wrong in the non-commutative case.
        # Hence, we have to form a commutative version of basering
        L = singular.ringlist('basering')
        tmpI = L[4]
        tmpR = singular('ring(list(%s[1..3],ideal(0)))'%(L.name()))
        tmpR.set_ring()
        HP = MonomialHilbert(singular('lead(fetch(%sr(%d),%sI))+fetch(%sr(%d),%s)'%(self.prefix,self.knownDeg,self.prefix,self.prefix,self.knownDeg,tmpI.name())))
        singular.eval('degBound = '+str(dgb))
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))

        #HP = R(sum([hp[i]*t**i for i in range(len(hp))]))
        HS = HP/mul([(1-t**X) for X in self.degvec])
        if HS.denominator().leading_coefficient()<0:
            return (-HS.numerator()/(-HS.denominator()))
        return HS

#######
## Some methods of more internal use
#######

#    def sortCoch(self,Cch):
#        "@@Only of internal use: Sort an 'interreduced' list of cochains of the same degree by their leading coefficients"
#        cdef list L = len(Cch)*[None]
#        cdef int f,j
#        cdef COCH X
#        for X in Cch:
#            f,j = X.Data.lead()
#            if f:
#                L[j] = X
#            else:
#                raise RuntimeError, "Got trivial cochain"
#        return L

    def expect_last_relation (self):
        """
        Returns a degree up to which there will provably be further non-obvious relations

        ALGORITHM:
        
        1. If there are two different generators of degree `d_1` and `d_2` that are both not
           Duflot regular, then we expect a relation at least in degree `d_1+d_2`.
        2. If there is a nilpotent generator in degree `d`, then there will certainly
           be a relation in degree `2d`. In the case of `p`-groups with `p>0`, that relation
           is obvious for `d` odd and is not counted, and only part 1. applies.


        EXAMPLES:
          
        We first create a cohomology ring, whose data files are rooted in a temporary directory;
        it will be removed as soon as Sage is quit. As the example is non-commutative, we
        require that Singular 3-1-0 be installed.
        ::
          
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3, websource=False)
            sage: H.make(3)
            sage: H.expect_last_relation()
            6
              
        Indeed, there are two different non-regular generators of degree 3::
         
            sage: H.7
            a_3_4, a 3-Cochain in H^*(E27; GF(3))
            sage: H.8
            a_3_5, a 3-Cochain in H^*(E27; GF(3))
              
        """
        cdef COCH X
        cdef RESL R = self.Resl
        cdef list L   = [X.Deg for X in self.Gen if not X.Rdeg]
        L.sort()
        if not L: # all cohomology rings should be at least computed out to degree 2
            return 2

        if R.G_Alg.Data.p!=2: # In this case, we can slightly improve the degree bound
            if L[-1]%2: # Hence, we have a nilpotent generator, but consider its relation automatically
                if len(L)==1:
                    return 2
                return L[-1]+L[-2]
        return 2*L[-1]

    def potential_degree_bound(self,FRS,dv, forced = False):
        """
        Try to prove the existence of a filter regular HSOP of small degrees, and return the resulting degree bound

        INPUT:
        
        - ``FRS`` -- a list of polynomials (given by strings)
        - ``DV``  -- the list of degrees of the polynomials from ``FRS``

        ASSUMPTION:
        
        ``FRS`` were a filter regular HSOP, provided self was completely computed

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit. For constructing Dickson
        classes, we use the linear algebra method. The example is non-commutative, so,
        it is required that Singular 3-1-0 be installed.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(27,3, useElimination=False, websource=False)

        The computational steps in this example will internally be done by H.make(). Here, we
        do them step by step.
        ::
        
            sage: H.next()
            sage: H.next()
            sage: H.next()
            sage: H.next()
            sage: H.next()
            sage: H.next()
            sage: H.next()
            sage: H.gens()
            [1,
             b_2_0, a 2-Cochain in H^*(E27; GF(3)),
             b_2_1, a 2-Cochain in H^*(E27; GF(3)),
             b_2_2, a 2-Cochain in H^*(E27; GF(3)),
             b_2_3, a 2-Cochain in H^*(E27; GF(3)),
             c_6_8, a 6-Cochain in H^*(E27; GF(3)),
             a_1_0, a 1-Cochain in H^*(E27; GF(3)),
             a_1_1, a 1-Cochain in H^*(E27; GF(3)),
             a_3_4, a 3-Cochain in H^*(E27; GF(3)),
             a_3_5, a 3-Cochain in H^*(E27; GF(3))]

        During these computations, Dickson classes have been computed. It was found::
        
            sage: H.Dickson
            ['b_2_0^2 - b_2_0*b_2_1 - b_2_0*b_2_2 + b_2_3^2']

        It is known that the Duflot regular generator ``c_6_8`` and the Dickson class form a
        filter regular homogeneous HSOP of ``H``, provided the computation is complete.
        However, Benson's criterion would only apply in degree 5+3+1=9. Now we
        test whether it is possible to replace the Dickson class by a parameter of
        smaller degree. If there is more than one Dickson class, we would potentially
        replace several (but not necessarily all) of them. Note that our method is 
        not constructive -- we just show the *existence* of a filter regular HSOP!
        ::
        
            sage: H.potential_degree_bound(['c_6_8','b_2_0^2 - b_2_0*b_2_1 - b_2_0*b_2_2 + b_2_3^2'],[6,4])
            7

        The previous result means: *If* ``['c_6_8','b_2_0^2 - b_2_0*b_2_1 - b_2_0*b_2_2 + b_2_3^2']``
        gives rise to a filter regular HSOP and *if* the `n`-th term of the resulting filter degree type
        is not greater than `-n` for all but the last term, then Benson's criterion applies already in
        degree 7. Indeed::
        
            sage: H.is_filter_regular(['c_6_8','b_2_0^2 - b_2_0*b_2_1 - b_2_0*b_2_2 + b_2_3^2'],[6,4])
            True
            sage: H.filter_degree_type()
            [-1, -2, -2]

        So, we have shown that the ring structure is completely known. 
        """
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"Testing whether it makes sense to try Benson's completeness criterion" 
        cdef int i,N,d,D,tai
        cdef list DV = [max(2,NR) for NR in dv]
        cdef int lenDV = len(dv)
        cdef int natBound,potBound
        cdef COCH X
        N = self.pRank - self.CenterRk
        if (self.suffDeg>0) and (self.suffDeg<self.knownDeg) and (not forced):
            return self.suffDeg
        if self.CenterRk>1:
            natBound = sum(DV)-lenDV
            potBound = sum(DV[:-N])+2*N - lenDV
        else:
            natBound = sum(DV)-lenDV + 1
            potBound = sum(DV[:-N])+2*N - lenDV + 1 
        if (natBound <= self.knownDeg) and not forced: # makes no sense to try and improve
            return natBound
        if potBound > self.knownDeg: # makes no sense to struggle
            return potBound
        ## Strategy: Keep all but n items of FRS and see whether the
        ## remaining can be replaced by parameters of degree d.
        ## Optimize n and d!

        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        dgb=str(singular("degBound"))
        singular.eval("degBound = 0")
        if self.completeGroebner:
            singular.eval('attrib(%sI,"isSB",1)'%(self.prefix))
            I0 = singular('%sI'%(self.prefix))
            for PL in FRS[:-N]:
                singular.eval('%s = std(%s,%s)'%(I0.name(),I0.name(),PL))
        else:
            I0 = (singular('%sI'%(self.prefix))+singular.ideal(['0']+FRS[:-N])).groebner()
        if self.Resl.coef()==2 and I0.dim()>N:
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"There will be more relations"
            return self.knownDeg + 1
        cdef int lastBoundFound = natBound
        for n from N >= n >= 1:
            tai = sum(DV[-n:])
            D = int(tai/n)  # replacements only make sense out to degree D
            for d from 2 <= d <= D:  # kill standard monomials of degree d
                                     # and generators whose degree devides d
                potBound = natBound - tai + d*n
                if potBound <= self.knownDeg: # it makes sense to try
                    J0 = singular.ideal(['0'])
                    for divis in Integer(d).divisors():
                        L=[X.Name for X in self.Gen if X.Deg==divis]
                        if L:
                            J0 = J0 + singular.ideal(L)
                    I = I0 + J0   # now, the generators will be killed, and we proceed to add the standard monomials
                    for J in self.StdMon[d].values():
                        I = I+J
                    G = I.groebner()
                    if G.vdim()>=0: #I.groebner().vdim()>=0: # we proved the existens of frHSOP
                        if not forced:
                            singular.eval('degBound = %s'%(dgb))
                            self.setprop('WhatFRS',(n,d))
                            return potBound
                        else:
                            if potBound<lastBoundFound:
                                lastBoundFound = potBound
                                self.setprop('WhatFRS',(n,d))
            singular.eval("%s = %s,%s"%(I0.name(),I0.name(),FRS[-n])) # we can not replace FRS[-n], hence, it will be killed now
        singular.eval('degBound = %s'%(dgb))
        #self.setprop('WhatFRS',(n,d))
        return lastBoundFound  # sorry, but we found nothing better...
                
            
    def is_filter_regular(self,FRS,DV):
        """
        Test whether a given list of elements forms a filter-regular HSOP

        INPUT:
        
        - ``FRS`` -- list of strings defining elements of the cohomology ring
        - ``DV``  -- the corresponding list of degrees

        OUTPUT:
        
        - True/False -- answers whether the input is a filter regular HSOP
        - If FRS does define a filter regular HSOP, its (raw) filter degree type
          and Benson's parameter alpha are saved as attributes of self.

        NOTE:
        
        It is assumed that there is only one list of elements that will ever
        be tested for being a filter regular HSOP -- namely the one given by
        Duflot regular generators and Dickson classes. To save resources, we
        therefore cache the result.
        In case the user wants to test a different list of elements, the
        property ``lastTested`` should be deleted with ``self.delprop('lastTested')``

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit. For constructing Dickson
        classes, we use the elimination method.
        ::

            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, useElimination=True, websource=False)
            sage: H.next()
            sage: H.next()
            sage: H.next()

        Of course, Dickson classes are not there, yet, so we construct them.
        ::

            sage: H.Dickson
            [None]
            sage: H.find_dickson()
            sage: H.Dickson
            ['b_1_0 + b_1_1']

        There is one Duflot regular class::

            sage: [X for X in H.Gen if X.rdeg()]
            [c_2_2, a 2-Cochain in H^*(D8; GF(2))]

        So, if the computation was complete, ``c_2_2`` and ``b_1_0+b_1_1`` were a filter
        regular HSOP in degrees 2 and 1. This is indeed the case, as we verify now::

            sage: H.is_filter_regular(['c_2_2','b_1_0+b_1_1'],[2,1])
            True

        The raw filter degree type and the filter degree type of that HSOP are
        now available::

            sage: H.rfdt
            [-1, -1, 1]
            sage: H.filter_degree_type()
            [-1, -2, -2]

        Now, after deleting the cache, we try a slightly different sequence of elements::

            sage: H.delprop('lastTested')
            sage: H.is_filter_regular(['c_2_2^2','b_1_0^2+b_1_1^2'],[4,2])
            True

        It is known that the filter degree type is the same for any filter regular
        HSOP. But the *raw* filter degree type may change::

            sage: H.rfdt
            [-1, -1, 4]
            sage: H.filter_degree_type()
            [-1, -2, -2]
        
        """
        if self.lastTested:
            if self.lastRel<=self.lastTested and max(self.degvec)<=self.lastTested: # a new test makes no sense
                if OPTION.opts['prot']:
                    print (self.Resl.getindent()*'::: ')+"Filter regularity was already tested"
                return bool(self.fdt)
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"Test whether the sequence", FRS, "is filter regular"
        if OPTION.opts['timing']:
            wt = walltime()
        dgb=str(singular("degBound"))
        singular.eval("degBound = 0")
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        self.make_groebner()
        DRaw=singular("is_fregs(ideal(%s),%sI)"%(','.join(FRS),self.prefix)).sage_structured_str_list()
        if OPTION.opts['timing']:
            print "Time for testing filter regularity"
            print "   Wall: %.2f"%(walltime(wt))
        self.setprop('lastTested',self.knownDeg)
        if DRaw=='0':
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"> Sequence is NOT filter regular. Sorry."
            return False
        # Extract Hilbert vectors
        HV = [[Integer(Y) for Y in X.split(',\n')] for X in DRaw]
        # rfdt = vector of maximal degrees in the kernel resp. quotient
        rfdt = []
        for X in HV:
            if X==[0]:
                rfdt.append(-1)
            else:
                rfdt.append(len(X)-1)
        self.setprop('rfdt',rfdt)
        fdt = FilterDegreeType(DV,rfdt)
        self.setprop('fdt',fdt)
        self.setprop('HilbertVectors',HV)
        self.setprop('ParameterDegrees',DV)
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"> Sequence is filter regular."
            print (self.Resl.getindent()*'::: ')+"  Raw filter degree type:",rfdt
            print (self.Resl.getindent()*'::: ')+"  --> Filter degree type:",fdt
        singular.eval("degBound = "+dgb)
        self.alpha = max([fdt[i]+i for i in range(len(fdt)-1)])
        return True
        
    def a_invariants(self):
        """
        Return the a-invariants of self.

        ASSUMPTION:
        
        The cohomology ring is completely computed, and the last usage of
        :meth:`is_filter_regular` was on the filter regular HSOP provided by
        the Duflot regular generators and the Dickson classes.        

        THEORY:
        
        A filter regular homogeneous system of paramaters can be used for an iterative
        computation of the a-invariants, provided the parameters are of sufficiently
        high degree. Often the data obtained by verifying Benson's completeness criterion
        are sufficient for the computation of a-invariants. However, in some cases it
        is needed to raise the parameters occuring in Benson's criterion to some power.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, useElimination=True, websource=False)
            sage: H.make()
            sage: H.a_invariants()
            [-Infinity, -Infinity, -2]

        The result is cached in ``H.A_INV``, and ``H.A_INV_Expos`` tells whether it was needed
        to raise the parameters to some power (this is not the case here)::
        
            sage: H.A_INV
            [-Infinity, -Infinity, -2]
            sage: H.A_INV_Expos
            [1, 1]
            
        """
        if not self.completed:
            return
        if self.A_INV:
            return self.A_INV
        if not self.CenterRk:
            TotalRel = ''.join(self.Rel)
            if self.Resl.coef()!=2:
                PAR = []
                ParDeg = []
            else:
                PAR = [X.name() for X in self.Gen if ((X.deg()==1) and (not TotalRel.__contains__(X.name())))]
                ParDeg = [1]*len(PAR)
            PAR = PAR + [X.name() for X in self.Gen if X.deg()==2]
            ParDeg = ParDeg + ([2]*len([X.name() for X in self.Gen if X.deg()==2]))
            if not self.is_filter_regular(PAR, ParDeg):
                raise RuntimeError, 'theoretical error'
        else:
            PAR = [X.name() for X in self.Gen if X.rdeg()] + self.Dickson
        if OPTION.opts['prot']:
            print (self.Resl.getindent()*'::: ')+"Computation of a-invariants"
        rfdt_bak = copy(self.rfdt)
        fdt_bak = copy(self.fdt)
        HV_bak = copy(self.HilbertVectors)
        ParDeg = copy(self.ParameterDegrees)
        DEG = copy(self.ParameterDegrees)
        Expos = [1]*len(ParDeg)
        cdef int m,n
        # H_m shall be H modulo the first m parameters
        # a^n_m shall be the n-th a-invariant of H_m
        while (1):
            NeedSquaring=False
            A={}
            for m from len(PAR)>=m>=0:
                if self.rfdt[m]==0:
                    A[m,0]=0
                else:
                    A[m,0] = min(self.rfdt[m]*infinity, self.rfdt[m])
                for n from 0<n<=len(PAR)-m:
                    if (A[m+1,n-1]>A[m,n-1]) or (A[m+1,n-1]==-infinity): # if recursion formula works
                        A[m,n] = A[m+1,n-1] - DEG[m]
                    else:
                        NeedSquaring = True
                        break
                if NeedSquaring:
                    if OPTION.opts['prot']:
                        print (self.Resl.getindent()*'::: ')+'> Squaring %s parameter'%(Ordinals(m+1))
                    PAR[m]='('+PAR[m]+')**2'
                    DEG[m]=2*DEG[m]
                    Expos[m]=2*Expos[m]
                    break
            if NeedSquaring:
                self.delprop('lastTested')
                self.is_filter_regular(PAR,DEG) # now, the raw filter degree type might suffice to get the a-invariants.
            else:
                break
        self.setprop('rfdt',rfdt_bak)
        self.setprop('fdt', fdt_bak)
        self.setprop('HilbertVectors',HV_bak)
        self.setprop('ParameterDegrees',ParDeg)
        self.setprop('A_INV',[A[0,n] for n in range(len(self.rfdt))])
        self.setprop('A_INV_Expos',Expos)        
        return self.A_INV               
    
    def BensonTest(self,forced = False):
        """
        Benson's completeness criterion, based on Dickson invariants.

        INPUT:

        ``forced`` [optional, default = ``False``]: If ``True``, perform Benson's test even if it seems to be too low degree

        OUTPUT:
        
        None. But the attribute ``suffDeg`` may be modified, and there is protocol output.

        EXAMPLES:
        
        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(64,6, useElimination=True, websource=False)
            sage: H.next()

        We switch the protocol mode on, in order to see why our first attempts
        with :meth:`BensonTest` fail. We switch it off, if we compute the next term
        of ``H``::
        
            sage: H.option('prot')
            sage: H.BensonTest()
            ########################################
            We need to find more regular generators!
            sage: H.option('noprot')
            sage: H.next()
            sage: H.option('prot')
            sage: H.BensonTest()
            Try to lift Dickson invariants using elimination methods
            ::: > Express cochain of degree 2 as polynomial
            ::: > Express cochain of degree 2 as polynomial
            ::: > Express cochain of degree 2 as polynomial
            ::: > Express cochain of degree 1 as polynomial
            ::: > Express cochain of degree 1 as polynomial
            > Lift Dickson invariants of the 2nd special subgroup
            We succeeded to lift Dickson invariants!
            #####################################
            We expect a relation in degree at least 4

        Since there will be a relation in degree 4, we continue up to that degree::
        
            sage: H.option('noprot')
            sage: H.next()
            sage: H.next()
            sage: H.option('prot')
            sage: H.BensonTest()
            Testing whether it makes sense to try Benson's completeness criterion
            It is possible that Benson's degree bound applies
            Test whether the sequence ['c_2_2', 'c_2_3', 'b_1_1'] is filter regular
            Computing complete Groebner basis for H^*(SmallGroup(64,6); GF(2))
            > Sequence is filter regular.
            Raw filter degree type: [-1, -1, 1, 2]
            --> Filter degree type: [-1, -2, -3, -3]

        According to the protocol, the test succeeded. Indeed, now the 'sufficient
        degree' is less than or equal to the known degree::
        
            sage: H.suffDeg
            3
            sage: H.knownDeg
            4
            sage: H.option('noprot')
        
        """
        if not (isinstance(self.Dickson,list)):
            return
        cdef COCH X
        #########
        ## try to form sequence that is known to be filter regular, provided the ring is complete
        cdef list FRS = []
        for X in self.Gen:
            if X.Rdeg:
                FRS.append(X.Name)
        if len(FRS)<self.CenterRk:
            if OPTION.opts['prot']:
                print "########################################\nWe need to find more regular generators!\n"
            return
        if self.useElimination and self.Dickson.count(None):
            self.find_dickson()
        if (self.expect_last_relation() > self.knownDeg) and not forced:
            if OPTION.opts['prot']:
                print "#####################################\nWe expect a relation in degree at least",self.expect_last_relation()
            return
        if self.Dickson.count(None):
            if OPTION.opts['prot']:
                print "#####################################\nNot all Dickson invariants are found!\n"
            return
        FRS.extend(self.Dickson)
        #########
        ## the degree vector of FRS (lower bound 2)
        singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg))
        dv = [Integer(singular("deg(%s)"%(x))) for x in FRS]  # this is the real degree vector
        DV = [max(2,Nr) for Nr in dv]                           # this is used for the degree bound

        #########
        ## Test whether Benson's criterion might apply yet
        potBound = self.potential_degree_bound(FRS,dv)
        if (potBound > self.knownDeg): # it makes no sense to test filter regularity right now
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"We expect that Benson's test will not apply before degree %d\n"%(potBound)
            if forced:
                if OPTION.opts['prot']:
                    print (self.Resl.getindent()*'::: ')+"However, we perform Benson's test"
            else:
                return
        else:
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"It is possible that Benson's degree bound applies"
        ######
        ## Now, either we are forced to do the test or it actually makes sense to do so...
        if self.is_filter_regular(FRS,dv):
            self.suffDeg = max([0,self.alpha]) + potBound

###################################################
###################################################
## Compute the visible ring structure
###################################################
###################################################

    ###########################
    ## get next degree:
    def next(self, Forced=False, KeepDecomposables=False, ElabSubgp=False):
        """
        Compute generators and relations up to the next unknown degree

        TESTS:

        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(8,3, useElimination=True, websource=False)
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computed up to degree 0
            Minimal list of generators:
            []
            Minimal list of algebraic relations:
            []
            sage: H.next()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computed up to degree 1
            Minimal list of generators:
            [b_1_0, a 1-Cochain in H^*(D8; GF(2)),
            b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            []
            sage: H.next()
            sage: print H
            Cohomology ring of Dihedral group of order 8 with coefficients in GF(2)
            <BLANKLINE>
            Computed up to degree 2
            Minimal list of generators:
            [c_2_2, a 2-Cochain in H^*(D8; GF(2)),
            b_1_0, a 1-Cochain in H^*(D8; GF(2)),
            b_1_1, a 1-Cochain in H^*(D8; GF(2))]
            Minimal list of algebraic relations:
            [b_1_0*b_1_1]

        """
        ###########
        ## In order to have 100 percent doctest coverage
        ## for resolution.pyx, I remove liftAll from there
        ## and insert it as external function.
        cdef COCH X
        if self.SUBGPS:
            self.reconstructSubgroups()
        if self.completed and (not Forced) and (not ElabSubgp):
            if OPTION.opts['prot']:
                print (self.Resl.getindent()*'::: ')+"Cohomology ring is completely known"
            return
        if self.ElAb: # in order to avoid that later another group will reload incomplete data
            KeepDecomposables = True
            ElabSubgp = True 
        TCT = cputime()
        TWT = walltime()
        cdef RESL R
        R = self.Resl
        cdef list NEW_KEYS = []
        cdef list Liftables = []
        cdef list MaxDegs = []
        cdef int lenNEW_KEYS
        
        ############
        ## Compute next term of resolution, if necessary
        while R.Data.numproj <= self.knownDeg:
            R.nextDiff()
            if R.Data.numproj<=min(self.__dict['auto'],max([1]+[X.deg() for X in self.Gen if X.rdeg()==0])):
                R.makeAutolift(R.Data.numproj)
        n = self.knownDeg + 1
        ct=cputime()
        wt=walltime()
        cdef int i,k,m,f,j
        cdef int lenDecGen,lenMonExp
        cdef int loopbound
        
        ######################################
        ## Main Procedure
        ######################################
        if ElabSubgp:
            self.NilBasis[n]=[]
        ## Find decomposables and relations in degree n

        # The following keeps track about decomposable vector space generators of H^n
        cdef list VSGen=R.Data.projrank[n]*[1]
        cdef list DecGen=[]  # will be a triangular basis of the sub space of decomposables of H^n
        cdef list pivot=[]
        cdef COCH Cand # will be the 'next candidate' for a decomposable generator or a relation
        newRel = False
        
        cdef list MonExp,lastPiv
        if self.Gen!=[]:  # there can only be decomposables if there are generators, yet
            ####################################
            # We must lift self.RelG to the new degree.
            # First, switch to the previous ring
            singular.eval('setring %sr(%d)'%(self.prefix,n-1))
            if n==2:
                singular.eval('ideal %sI = std(0)'%(self.prefix))
            else:
                if (self.lastRel < n-1) and (self.expect_last_relation() < n):
                    self.make_groebner()
                else:
                    self.make_groebner(n)
            sizeG = int(singular.eval('ncols(%sI)'%(self.prefix)))

            #######################################
            # Get standard monomials:
            self.__makeStdMon(n,"Mon")

            if OPTION.opts['timing']:
                ct=cputime()
                wt=walltime()

            if singular.eval('Mon[1]')!='0':
                if OPTION.opts['prot']:
                    print (R.indent*'::: ')+"We got %s standard monomials"%(singular.eval('ncols(Mon)'))
                # get list of exponent vectors of the standard monomials
                MonExp = [[int(y) for y in x.split(',\n')] for x in \
                          (singular.eval('for (i=1;i<=ncols(Mon);i++) { print(leadexp(Mon[i]));print(\";\");}')+'\n').split('\n;\n') if x]
                lenMonExp = len(MonExp)

                #######################################
                # Do all necessary lifts 
                if OPTION.opts['liftlist']:
                    for i from 0<= i < lenMonExp:
                        self.InsertLift(MonExp[i])
                    #R.liftAll()
                    lAn=len(R.Diff) # the degree to which we will lift
                    # filter the to-be-lifted cochains
                    for I in list(set(R.ToBeLifted)):
                        TestLift = R.Lifts[(lAn-1,I[0],I[1])]
                        if TestLift != None: # i.e., it is lifted to the previous degree
                            NEW_KEYS.append((lAn,I[0],I[1]))
                            Liftables.append(TestLift[0])
                            MaxDegs.append(TestLift[1])
                    R.ToBeLifted=[]
                    New_Lifts = R.liftListOfMaps(Liftables)
                    lenNEW_KEYS = len(NEW_KEYS)
                    for i from 0 <= i < lenNEW_KEYS:
                        R.Lifts[NEW_KEYS[i]] = (New_Lifts[i],MaxDegs[i])
                    if OPTION.opts['timing']:
                        ct=cputime(ct)
                        wt=walltime(wt)
                        print "Time for lifting standard monomials:"
                        print "   CPU:  %.2f\n   Wall: %.2f"%(ct,wt)
                ct=cputime()
                wt=walltime()
                
                #######################################
                # Perform the products
                for i from 0<= i < lenMonExp:
                    if OPTION.opts['prot']:
                        print (R.indent*'::: ')+'Monomial',i
                        print (R.indent*'::: ')+'> Candidate: '+singular.eval('Mon[%d]'%(i+1))
                        print (R.indent*'::: ')+'> Express monomial as a Cochain'
                    Cand = self.MonToProd(MonExp[i])
                    if ElabSubgp and (R.G_Alg.Data.p!=2) and (MonExp[i][self.firstOdd:].count(0) < (len(MonExp[i])-self.firstOdd)):
                        self.NilBasis[n].append(Cand.MTX()._rowlist_(0))
                    f,j = Cand.Data.lead()
                    IsMonomial = True
                    lenDecGen = len(DecGen)
                    if f:
                        for k from 0 <= k < lenDecGen:
                            if (j <= pivot[k]) and Cand.Data[0,pivot[k]]:
                                Cand = Cand - (DecGen[k])*Cand.Data[0,pivot[k]]
                                f,j = Cand.Data.lead()
                                IsMonomial = False
                    if f: # We found a decomposable generator of H^n
                        if OPTION.opts['prot']:
                            print (R.indent*'::: ')+'Decomposable cochain found\n'
                        DecGen.append(Cand/f)
                        if len(DecGen[-1].name()) < 850:
                            singular.eval(('DG[%d]='%(len(DecGen)))+DecGen[-1].name())
                        else:
                            ComL = Cand.name().split('-')
                            DGstr = 'DG[%d]'%(len(DecGen)) 
                            singular.eval(DGstr+'='+ComL[0])
                            DGstr = DGstr + '=' + DGstr + '-'
                            for trm in ComL[1:]:
                                singular.eval(DGstr+trm)
                            singular.eval('DG[%d]=DG[%d]/%d'%(len(DecGen),len(DecGen),f))
                        if not IsMonomial:
                            DecGen[-1].setname('DG[%d]'%(len(DecGen)))
                        pivot.append(j)
                        VSGen[j]=0
                    else: # We found a relation in degree n
                        newRel = True
                        self.lastRel = n
                        self.setprop('completeGroebner',False)
                        sizeG+=1
                        if len(Cand.name()) < 850:
                            singular.eval(('%sI[%d] = '%(self.prefix,sizeG))+Cand.name())
                        else:
                            ComL = Cand.name().split('-')
                            Idstr = '%sI[%d]'%(self.prefix,sizeG)
                            singular.eval(Idstr+'='+ComL[0])
                            Idstr = Idstr + '=' + Idstr + '-'
                            for trm in ComL[1:]:
                                singular.eval(Idstr+trm)
                        if OPTION.opts['prot']:
                            print (R.indent*'::: ')+'New relation found: '+singular.eval('%sI[%d]'%(self.prefix,sizeG))+'\n'
                        self.Rel.append(singular.eval('%sI[%d]'%(self.prefix,sizeG)))
                if OPTION.opts['timing']:
                    ct=cputime(ct)
                    wt=walltime(wt)
                    print "Time for extracting relations:"
                    print "   CPU:  %.2f\n   Wall: %.2f"%(ct,wt)
                    ct=cputime()
                    wt=walltime()
            else:
                if OPTION.opts['prot']:
                    print (R.indent*'::: ')+"There are no standard monomials in degree %d"%(n)
        ###################
        ## RESULT:
        ## I. COCH types
        # print 'PR-> are there new ring generators?'
        self.knownDeg=n
        ############
        ## lift restrictions
        ## Unfortunately, self.Resl must be lifted one step further
        ## in order to lift the restriction maps.
        cdef list NewGen = []
        cdef list CandL
        NrNewGen = VSGen.count(1)
        if VSGen.count(1)==0 and OPTION.opts['prot']:
            print (R.indent*'::: ')+"There is no new generator of %s in degree %d"%(self.__repr__(),n)
        if NrNewGen>0:  # there are new ring generators, hence, we must lift the restriction maps
            if singular("defined(%sQ)"%(self.prefix)):
                singular.eval("kill %sQ"%(self.prefix))
            if OPTION.opts['prot'] and self.RestrMaps:
                print (R.indent*'::: ')+"There are new generators, we have to lift the restriction maps"
            for sgp in self.subgps.values():
                while (sgp.knownDeg < n):#(sgp.Resl.deg()<n):
                    sgp.next(Forced=True,KeepDecomposables=True,ElabSubgp=True)
            while (R.Data.numproj <=n):
                R.nextDiff()
                # it seems that it doesn't help to make Autolift if there are many generators.
                # Degree 4 seems to be best, at least for groups of order 64
            # For technical reasons, we need to lift R out to degree n+1 if we want to
            # obtain new generators in degree n. But we only need the autolift out to
            # a degree in which generators live -- therefore R.Data.numproj-1
            if (R.Data.numproj<=self.__dict['auto']+1) and (not R.Autolift.has_key(R.Data.numproj-1)):
                R.makeAutolift(R.Data.numproj-1)
            for NR, chm in self.RestrMaps.items():
                if OPTION.opts['prot']:
                    print (R.indent*'::: ')+"\nRestriction to %s subgroup (order %s)"%(Ordinals(NR),chm[0][0])
                while(chm[1].knownDeg()<n):
                    #print 'try and lift', chm
                    chm[1].lift()
                if OPTION.opts['sparse']:
                    chm[1].exportData(self.inc_folder+self.GStem+'sg'+str(NR)+'_')
            R.free_ugb()
        ###################
        ## CHOSING NEW RING GENERATORS
        ##################
        ##################
            if OPTION.opts['prot']:
               if VSGen.count(1)==1:
                  print (R.indent*'::: ')+"We have to choose 1 new generator of %s in degree %d"%(self.__repr__(),n)
               else:
                  print (R.indent*'::: ')+"We have to choose %d new generators of %s in degree %d"%(VSGen.count(1),self.__repr__(),n)
        ## 1. New generators with nilpotent restriction on all
        ## maximal elementary abelian subgroups
        ## --> These are nilpotent!
            # VSGen: coeff == 0 => pivot of some decomposable class
            ZN_comp = len(VSGen)*[1] # eventually coeff == 0 => pivot of a decomposable nilpotent
                                    ## and ==2 => pivot of a new nilpotent generator
            if self.MaxelPos:
                # The following is a basis in (semi)echelon form of the subspace of decomposable nilpotent classes of degree n
                if DecGen: # there are decomposables, so let's intersect with the nilpotents:
                    NilDec = (MTX(R.G_Alg.Data.p,[add([self.subgps[self.RestrMaps[mpos][0]].nil_reduce(self.RestrMaps[mpos][1]*X) for mpos in self.MaxelPos],[]) for X in DecGen],mutable=False).nullspace() * MTX(R.G_Alg.Data.p, [X.Data._rowlist_(0) for X in DecGen])).echelon()
                else:
                    NilDec = MTX('',mutable=False)
                loopbound = NilDec.nrows()
                for i from 0 <= i < loopbound:
                    f,j = NilDec[i].lead()
                    ZN_comp[j]=0
                ## Now we compute a basis in (semi)echelon form of a complement of decomposable nilpotent classes in the nilpotent classes
                NilNonDec = MTX(R.G_Alg.Data.p,[add([self.subgps[self.RestrMaps[mpos][0]].nil_reduce(self.RestrMaps[mpos][1]([n,i])) for mpos in self.MaxelPos],[]) for i in range(len(ZN_comp)) if ZN_comp[i]],mutable=False).nullspace()
                # The rows of this matrix yield (ydeg=1,rdeg=0)-generators (i.e., nilpotent but nondecomposable classes).
                # We need to insert the normalised entries of NilNonDec at the places where ZN_comp has coefficient 1.
                loopbound = NilNonDec.nrows()
                for i from 0 <= i < loopbound:
                    CandL = []
                    NilL = NilNonDec._rowlist_(i)
                    for j in ZN_comp:
                        if j:
                            CandL.append(NilL.pop(0))
                        else:
                            CandL.append(0)
                    CandM = MTX(R.G_Alg.Data.p, [CandL],mutable=False)    
                    f,j = CandM.lead()
                    NewGen.append(COCH(self, n, 'a_%d_%d'%(n,j), CandM/f, ydeg=1,rdeg=0))  # a new normalised generator with nilpotent restriction
                    ZN_comp[j] = 2 # hence, the coeffs of NilL are still inserted in the right place, but it is clear
                                   # for later that there will be no new generator with that pivot
                NrNewGen = NrNewGen-ZN_comp.count(2)
                if ZN_comp.count(2)!=len(NewGen):
                    raise ArithmeticError, "%d==%d -- that seems strange"%(ZN_comp.count(2),len(NewGen))
                if OPTION.opts['prot']:
                    if len(NewGen)==1:
                        print (R.indent*'::: ')+"> There is 1 nilpotent generator of %s in degree %d"%(self.__repr__(), n)
                    else:
                        print (R.indent*'::: ')+"> There are %d nilpotent generators of %s in degree %d"%(len(NewGen),self.__repr__(),n)
            #print 'Nil', NewGen

        ###########
        ## 2. New generators with nilpotent restriction on the
        ## greatest elementary abelian central subgroup 
            RdegPiv = len(VSGen)*[1] # eventually, coeff will be 2 if there is a regular generator with that pivot
            if (self.RestrMaps.has_key(self.CElPos) and (NrNewGen)):
                L = [self.subgps[self.RestrMaps[self.CElPos][0]].nil_reduce(self.RestrMaps[self.CElPos][1]*X) for X in DecGen+NewGen]
                # The following is a basis in (semi)echelon form of the subspace of decomposable or nilpotent classes of degree n 
                # which in addition have nilpotent restriction to the greatest central elementary abelian subgroup
                if L:
                    NilDec = (MTX(R.G_Alg.Data.p,L).nullspace() * MTX(R.G_Alg.Data.p, [X.Data._rowlist_(0) for X in DecGen+NewGen],mutable=False)).echelon()
                else:
                    NilDec = MTX('',mutable=False)
                loopbound = NilDec.nrows()
                for i from 0 <= i < loopbound:
                    f,j = NilDec[i].lead()
                    RdegPiv[j] = 0 # no regular element has that pivot, because some decomposables or nilpotents have
                #StandardGen = [self.standardCochain(n,i) for i in range(len(ZN_comp)) if RdegPiv[i]] # generate the complement
                #NilNonDec = MTX(R.G_Alg.Data.p,[self.subgps[self.RestrMaps[self.CElPos][0]].nil_reduce(self.RestrMaps[self.CElPos][1]*X) for X in StandardGen],mutable=False).nullspace()
                NilNonDec = MTX(R.G_Alg.Data.p,[self.subgps[self.RestrMaps[self.CElPos][0]].nil_reduce(self.RestrMaps[self.CElPos][1]([n,i])) for i in range(len(ZN_comp)) if RdegPiv[i]],mutable=False).nullspace()               
                # The rows of this matrix yield (ydeg=rdeg=0)-generators (i.e., nondecomposable non-nilpotent classes with nilpotent restr. on CElAb).
                # We need to insert the normalised entries of NilNonDec at the places where RdegPiv has coefficient 1.
                loopbound = NilNonDec.nrows()
                for i from 0 <= i < loopbound:
                    CandL = []
                    NilL = NilNonDec._rowlist_(i)
                    for j in RdegPiv:
                        if j:
                            CandL.append(NilL.pop(0))
                        else:
                            CandL.append(0)
                    CandM = MTX(R.G_Alg.Data.p, [CandL],mutable=False)
                    f,j = CandM.lead()
                    NewGen.append(COCH(self, n, 'b_%d_%d'%(n,j), CandM/f, ydeg=0,rdeg=0))  # a new normalised generator with nilpotent restriction on CElAb
                    RdegPiv[j] = 2 # hence, the coeffs of NilL are still inserted in the right place, but it is clear
                                   # for later that there will be no regular generator with that pivot
                if OPTION.opts['prot']:
                    if RdegPiv.count(2)==1:
                        print (R.indent*'::: ')+'> There is 1 "boring" generator of %s in degree %d'%(self.__repr__(),n)
                    else:
                        print (R.indent*'::: ')+'> There are %d "boring" generators of %s in degree %d'%(RdegPiv.count(2),self.__repr__(),n)
                NrNewGen = NrNewGen - RdegPiv.count(2)
        ###########
        ## 3. Regular generators (rdeg=1, ydeg=0)
            if NrNewGen:
                LastPiv = len(VSGen)*[1] # coeff will remain 0 for the pivots of regular generators
                L = [X.Data._rowlist_(0) for X in DecGen+NewGen]
                if L:
                    NilDec = MTX(R.G_Alg.Data.p, L,mutable=False).echelon()  # These are the vector space generators obtained so far
                else:
                    NilDec = MTX('',mutable=False)
                loopbound = NilDec.nrows()
                for i from 0 <= i < loopbound:
                    f,j = NilDec[i].lead()
                    LastPiv[j] = 0
                if (R.G_Alg.Data.p!=2) and (n%2): # this can only happen for abelian groups
                    NewGen.extend([self.standardCochain(n, i, ydeg=1,rdeg=0,name='a') for i in range(len(LastPiv)) if LastPiv[i]])
                    if LastPiv.count(1)!=NrNewGen:
                        raise ArithmeticError, "%d==%d -- that seems strange"%(LastPiv.count(1),NrNewGen)
                    if OPTION.opts['prot']:
                        if LastPiv.count(1)==1:
                            print (R.indent*'::: ')+"> There is 1 nilpotent generator or %s in degree %d"%(self.__repr__(),n)
                        else:
                            print (R.indent*'::: ')+"> There are %d nilpotent generators of %s in degree %d"%(LastPiv.count(1),self.__repr__(),n)
                else:
                    NewGen.extend([self.standardCochain(n, i, ydeg=0, rdeg=1) for i in range(len(LastPiv)) if LastPiv[i]])
                    if LastPiv.count(1)!=NrNewGen:
                        raise ArithmeticError, "%d==%d -- that seems strange"%(LastPiv.count(1),NrNewGen)
                    if OPTION.opts['prot']:
                        if LastPiv.count(1)==1:
                            print (R.indent*'::: ')+"> There is 1 Duflot regular generator of %s in degree %d"%(self.__repr__(),n)
                        else:
                            print (R.indent*'::: ')+"> There are %d Duflot regular generators of %s in degree %d"%(LastPiv.count(1),self.__repr__(),n)
            if OPTION.opts['timing']:
               ct=cputime(ct)
               wt=walltime(wt)
               print "Total time for choosing new generators:"
               print "   CPU:  %.2f\n   Wall: %.2f"%(ct,wt)
               ct=cputime()
               wt=walltime()
        
        ## Insert the new generators:
        if ElabSubgp and (R.G_Alg.Data.p!=2) and (n%2):
            self.NilBasis[n].extend([Ca.MTX()._rowlist_(0) for Ca in NewGen])
        if n%2:
            firstNew = len(self.Gen)
            self.Gen.extend(NewGen)
            self.degvec.extend(len(NewGen)*[n])
        else:
            firstNew = self.firstOdd
            for CO in NewGen:
                self.Gen.insert(self.firstOdd, CO)
                self.degvec.insert(self.firstOdd, n)
                self.firstOdd+=1
        
        if (KeepDecomposables) and (self.knownDeg > 1): # and (self.knownDeg <= OPTION.maxkept): #(a deprecated option)
            singular.eval('setring %sr(%d)'%(self.prefix,self.knownDeg-1))
                              # The matrix DG in singular was to save name length. But we wouldn't save DG
            for X in DecGen:  # Hence: We need to change the names into clear names.
                X.setname(singular.eval(X.Name))
        ###################
        ## We can not simply extend self.Triangular by NewGen, since they are in general not triangular
        if (VSGen.count(1)>0):
            for Cand in NewGen:
                f,j = Cand.Data.lead()
                lenDecGen = len(DecGen)
                if f:
                    for k from 0 <= k < lenDecGen:
                        if (j <= pivot[k]) and Cand.Data[0,pivot[k]]:
                            Cand = Cand - (DecGen[k])*Cand.Data[0,pivot[k]]
                            f,j = Cand.Data.lead()
                    DecGen.append(Cand/f)
                    pivot.append(j)
            #DecGen.extend(self.Gen[firstNew:firstNew+len(NewGen)])
        #if KeepDecomposables or (not ElabSubgp):
        self.Triangular[n]=DecGen
        ####################
        ## II. put the new generators into the dictionary of monomials and lifts
        for i from 0 <= i < len(self.Gen):
            self.Monomials[self.Gen[i].name()] = self.Gen[i]
            R.setLift(self.Gen[i],2*(self.Gen[i].deg()))
        
        ####################
        ## III. Update Singular
        if self.__dict.get('use_dp'):
            if len(self.degvec)==1:
                singular.eval('ring tmp = %d,(%s),%s'%(R.G_Alg.Data.p, ','.join([x.name() for x in self.Gen]), '(a(%d),dp)'%(self.degvec[0])))
            else:
                singular.eval('ring tmp = %d,(%s),%s'%(R.G_Alg.Data.p, ','.join([x.name() for x in self.Gen]), '(wp%s)'%(str(tuple(self.degvec)))))
        else:
            self._makeOrderMatrix_()
            singular.eval('ring tmp = %d,(%s),M(%sM)'%(R.G_Alg.Data.p, ','.join([x.name() for x in self.Gen]), self.prefix))
        if R.G_Alg.Data.p!=2:   # non-commutative case
            singular.eval('degBound = 0')
            singular.eval('def %sr(%d) = SuperCommutative(%d,%d)'%(self.prefix,n,self.firstOdd+1, len(self.Gen)))
        else:
            singular.eval('def %sr(%d) = tmp'%(self.prefix,n))
        singular.eval('setring %sr(%d)'%(self.prefix,n))
        singular.eval('kill tmp')
        # keep standard monomials of lower degrees:
        if not self.StdMon.has_key(n):
            self.StdMon[n]={}
        for i from 0<i<=n:
            for ITEM in self.StdMon[i].items():
                self.StdMon[i][ITEM[0]]=singular('%sr(%d)'%(self.prefix,n-1)).imap(ITEM[1])
        # ... and don't forget that the new generators are standard monomials!
        for X in self.Gen:
            if X.Deg==n:
                self.StdMon[n][X.Name]=singular.ideal(X.Name)
        if n>1:
            singular.eval('ideal %sI = imap(%sr(%d),%sI)'%(self.prefix,self.prefix,n-1,self.prefix))
            # keep track of decomposable generators --
            # We need them for lifting the Dickson invariants!
            singular.eval('ideal DG = imap(%sr(%d),DG)'%(self.prefix,n-1))
        else:
            singular.eval('ideal %sI'%(self.prefix))
        if KeepDecomposables:
            self.GenS = singular('%sr(%d)'%(self.prefix,n))
            self.RelG = str(singular('%sI'%(self.prefix))).split(',\n')

        #####################################
        ## try and lift Dickson invariants...
        if (not ElabSubgp) and (self.pRank) and (self.CenterRk) and (not self.useElimination):
            m = self.pRank-self.CenterRk
            Ddg = Integer(R.G_Alg.Data.p)**m
            if R.G_Alg.Data.p==2:
                switch=1
            else:
                switch=2
            for i from 0<i<=m:
                if self.Dickson[i-1] is None:
                    j=0
                    while(1):
                        if self.knownDeg < (switch*(Ddg - Integer(R.G_Alg.Data.p)**(m-i)))*(Integer(R.G_Alg.Data.p)**j):
                            break
                        if self.knownDeg == (switch*(Ddg - Integer(R.G_Alg.Data.p)**(m-i)))*(Integer(R.G_Alg.Data.p)**j):
                            PotentialDicksonLift = self.lift_dickson(i-1,j)
                            if isinstance(PotentialDicksonLift,COCH):
                                if self.useFactorization:
                                    self.Dickson[i-1] = self.factorise(self.cochain_to_polynomial(PotentialDicksonLift))
                                else:
                                    self.Dickson[i-1] = self.cochain_to_polynomial(PotentialDicksonLift).name()
                            break
                        j+=1
        singular.eval('ideal DG') # because previously, DG was a copy of the decomposable classes from r(n-1)
        #####################
        ## Empty the dustbin!
        if  not KeepDecomposables:
            if n>1:
                singular.eval('setring %sr(%d)'%(self.prefix,n-1))
                singular.eval('kill %sI'%(self.prefix))
                singular.eval('kill DG')
                singular.eval('kill Mon')
            singular.eval('setring %sr(%d)'%(self.prefix,n))
            #singular.eval('kill %sr(%d)'%(self.prefix,n-1))
            self.Triangular = {}

        if OPTION.opts['prot']:
            print (R.indent*'::: ')+"Degree %d of the visible ring structure of %s is computed!\n"%(n,self.__repr__())
        if OPTION.opts['save']:
            if OPTION.opts['prot']:
                print (R.indent*'::: ')+"Save %s"%(self.__repr__())
            save(self, self.autosave_name())
        if OPTION.opts['timing']:
            print "Total time for computing "+self.__repr__()+" in degree",self.knownDeg
            print "   CPU:  %.2f\n   Wall: %.2f"%(cputime(TCT),walltime(TWT))
 
    #########################
    ## make the complete ring
    ## resp. up to a certain degree
    def make (self, max_deg=-1):
        """
        Construct the visible ring structure

        INPUT:
        
        - ``max_deg`` (optional integer) -- compute the complete ring structure if ``max_deg==-1``
          and compute at most up to degree ``max_deg`` if it is positive.

        TESTS:

        We first create a cohomology ring, whose data files are rooted in a temporary
        directory; it will be removed as soon as Sage is quit.
        ::
        
            sage: tmp_root = tmp_filename()
            sage: from pGroupCohomology import CohomologyRing
            sage: CohomologyRing.set_user_db(tmp_root)
            sage: H = CohomologyRing.user_db(32,5, websource=False)
            sage: H.make(2)
            sage: print H
            Cohomology ring of Small Group number 5 of order 32 with coefficients in GF(2)
            <BLANKLINE>
            Computed up to degree 2
            Minimal list of generators:
            [a_2_1, a 2-Cochain in H^*(SmallGroup(32,5); GF(2)),
             c_2_2, a 2-Cochain in H^*(SmallGroup(32,5); GF(2)),
             c_2_3, a 2-Cochain in H^*(SmallGroup(32,5); GF(2)),
             a_1_0, a 1-Cochain in H^*(SmallGroup(32,5); GF(2)),
             b_1_1, a 1-Cochain in H^*(SmallGroup(32,5); GF(2))]
            Minimal list of algebraic relations:
            [a_1_0^2,
             a_1_0*b_1_1]
            sage: H.make()
            sage: print H
            Cohomology ring of Small Group number 5 of order 32 with coefficients in GF(2)
            <BLANKLINE>
            Computation complete
            Minimal list of generators:
            [a_2_1, a 2-Cochain in H^*(SmallGroup(32,5); GF(2)),
             c_2_2, a 2-Cochain in H^*(SmallGroup(32,5); GF(2)),
             c_2_3, a 2-Cochain in H^*(SmallGroup(32,5); GF(2)),
             a_1_0, a 1-Cochain in H^*(SmallGroup(32,5); GF(2)),
             b_1_1, a 1-Cochain in H^*(SmallGroup(32,5); GF(2))]
            Minimal list of algebraic relations:
            [a_1_0^2,
             a_1_0*b_1_1,
             a_2_1*a_1_0,
             a_2_1^2]
            
        """
        TCT = cputime()
        TWT = walltime()
        cdef RESL R = self.Resl
        if (not (isinstance(max_deg, int) or isinstance(max_deg, Integer))) or \
           (max_deg==0) or (max_deg<-1):
            raise IndexError, "The degree bound must be a positive integer"
        if ((self.suffDeg>-1) and (self.knownDeg >= self.suffDeg)) or self.completed:
            if OPTION.opts['prot']:
                print "This cohomology computation has already been finished."
                return
        if (max_deg!=-1) and (max_deg <= self.knownDeg):
            print 'The ring is already known up to degree %d'%(self.knownDeg)
            return
        if ((max_deg > 1) or (max_deg==-1)) and R.G_Alg.isAbelian():
            self.suffDeg = 2
        elif ((max_deg > 3) or (max_deg==-1)) and (self.pRank==1):
            self.suffDeg = 4
        if OPTION.opts['prot']:
            if self.suffDeg == -1:
                print (R.indent*'::: ')+"We have no degree bound yet"
            else:
                print (R.indent*'::: ')+"We have the degree bound %d"%(self.suffDeg)
            if max_deg>0:
                print "We will compute at most up to degree %d"%(max_deg)
        #if (self.GenS!=None):
        #    singular.setring(self.GenS)
        while (1):
            if OPTION.opts['timing'] or OPTION.opts['prot']:
                print (R.indent*'::: ')+'Start computation in Degree %d'%(self.knownDeg+1)
            if self.ElAb:
                self.next(ElabSubgp = True)
            else:
                self.next(KeepDecomposables = self.KeepBases)
            if (self.MaxelPos) and (self.pRank!=1):
                self.BensonTest()
            if (self.suffDeg!=-1) and (self.knownDeg >= self.suffDeg):
                self.completed = True
            if (self.completed) or ((max_deg!=-1) and (self.knownDeg>=max_deg)):
                break
        # Catch the case of generalized quaternion groups:
        if self.completed and self.pRank==1:
            if not self.is_filter_regular([X.name() for X in self.Gen if X.rdeg()],[X.deg() for X in self.Gen if X.rdeg()]):
                raise RuntimeError, "theoretical error"
        self.GenS = singular('%sr(%d)'%(self.prefix,self.knownDeg))
        # singular.clear(self.GenS.name())
        # self.GenS._name='%sr(%d)'%(self.prefix,self.knownDeg)
        singular.eval("setring %sr(%d)"%(self.prefix,self.knownDeg))
        if not self.MaxelPos:
            self.make_groebner()
        self.RelG = str(singular('%sI'%(self.prefix))).split(',\n')
        # singular.clear(self.RelG.name())
        # self.RelG._name='%sI'%(self.prefix)
        self.SingularTime = singular.cputime(self.SingularTime)
        if OPTION.opts['prot']:
            print "\n"+(R.indent*'::: ')+"Cohomology computation for",self.GroupName,"is finished"
        fdt = self.fdt
        if fdt:
            alpha = max([fdt[i]+i for i in range(len(fdt)-1)])
            if self.alpha != -1:
                print "##############################\n## COUNTEREXAMPLE FOR BENSON'S CONJECTURE!!! ##\n##############################\n Please inform the author"
        if OPTION.opts['save']:
            if OPTION.opts['prot']:
                print (R.indent*'::: ')+"Save %s"%(self.__repr__())
            save(self, self.autosave_name())
        if OPTION.opts['timing']:
            print "Total time for cohomology computation"
            print "   CPU:  %.2f\n   Wall: %.2f"%(cputime(TCT),walltime(TWT))
        if OPTION.opts['prot'] or OPTION.opts['timing']:
            print "\n"
